#include "cpp-client.h"
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <netdb.h>
#include <time.h>
#include <sys/time.h>
#include <sys/wait.h>
#include <unistd.h>
#include <errno.h>

//we need the proper definition here
#ifdef TCPSTREAM
#undef TCPSTREAM
#endif
#include "imap-2000/c-client/tcp_unix.h"

//need this to initialize the ssl handling
extern "C" {
	void ssl_onceonlyinit (void);
}

struct callStack {
	callbackFunctions *topCallback;
	struct callStack *pred;
};
struct callStack *currentCallStack=NULL;

callbackFunctions *currentCallback=NULL;

void *my_blocknotify (int reason,void *data)
{
	int ret_val = 1;
	
	if(currentCallback)
	{
  	char *clear_txt = NULL;
  	switch(reason) {
  		case BLOCK_DNSLOOKUP:
  				ret_val = currentCallback->progressNotify(mpSET_STATUS,0,"DNS Lookup");
  				clear_txt = "BLOCK_DNSLOOKUP";
  				break;		
  		case BLOCK_TCPOPEN:
  				ret_val = currentCallback->progressNotify(mpSET_STATUS,0,"Connecting");
  				clear_txt = "BLOCK_TCPOPEN";
  				break;		
  		case BLOCK_TCPCLOSE:
  				ret_val = currentCallback->progressNotify(mpSET_STATUS,0,"Closing");
  				clear_txt = "BLOCK_TCPCLOSE";
  				break;		
  		case PROGRESS_START:
  				ret_val = currentCallback->progressNotify(mpSTART,0,(char *)data);
  				clear_txt = "PROGRESS_START";
  				break;
  		case PROGRESS_STOP:
  				ret_val = currentCallback->progressNotify(mpSTOP,0,(char *)data);
  				clear_txt = "PROGRESS_STOP";
  				break;
  		case PROGRESS_SET_MAX:
  				ret_val = currentCallback->progressNotify(mpSET_TOTAL,(int)data,NULL);
  				clear_txt = "PROGRESS_SET_MAX";
  				break;
  		case PROGRESS_SET_CUR:
  				ret_val = currentCallback->progressNotify(mpSET_CURRENT,(int)data,NULL);
  				clear_txt = "PROGRESS_SET_CUR";
  				break;
  		case PROGRESS_SET_ADD:
  				ret_val = currentCallback->progressNotify(mpSET_DELTA,(int)data,NULL);
  				clear_txt = "PROGRESS_SET_ADD";
  				break;
  		default:
  				clear_txt = "unknown";
  				break;
  			
  	};
  } else return (void *)1;
//	if(reason > 13) fprintf(stderr,"blocknotify reason %s data %p\n",clear_txt,data);
 return (void *)ret_val;
}

TCPSTREAM *pipe_open (char *host,char *service,unsigned long port)
{
	#define MAXARGV 10
  TCPSTREAM *stream = NIL;
  char tmp[MAILTMPLEN];

  int i;
  int pipei[2],pipeo[2];
  struct timeval tmo;
  fd_set fds,efds;

  tmo.tv_usec = 0;
  FD_ZERO (&fds);		/* initialize selection vector */
  FD_ZERO (&efds);		/* handle errors too */

	strncpy(tmp,host,MAILTMPLEN-1);

				/* make command pipes */
  if (pipe (pipei) < 0) return NIL;
  if (pipe (pipeo) < 0) {
    close (pipei[0]); close (pipei[1]);
    return NIL;
  }
	errno = 0;
  if ((i = fork ()) < 0) {	/* make inferior process */
    close (pipei[0]); close (pipei[1]);
    close (pipeo[0]); close (pipeo[1]);
    return NIL;
  }
  if (!i) {			/* if child */
    if (!fork ()) {		/* make grandchild so it's inherited by init */
      int maxfd = max (20,max (max(pipei[0],pipei[1]),max(pipeo[0],pipeo[1])));
      dup2 (pipei[1],1);	/* parent's input is my output */
      dup2 (pipei[1],2);	/* parent's input is my error output too */
      dup2 (pipeo[0],0);	/* parent's output is my input */
				/* close all unnecessary descriptors */
      for (i = 3; i <= maxfd; i++) close (i);
      setpgrp (0,getpid ());	/* be our own process group */
	    execl("/bin/sh","sh","-c",tmp,NULL);
    }
    _exit (1);			/* child is done */
  }
  grim_pid_reap (i,NIL);	/* reap child; grandchild now owned by init */
  close (pipei[1]);		/* close child's side of the pipes */
  close (pipeo[0]);
				/* create TCP/IP stream */
  stream = (TCPSTREAM *) memset (fs_get (sizeof (TCPSTREAM)),0,
				 sizeof (TCPSTREAM));
				/* copy remote host name from argument */
  stream->remotehost = cpystr (stream->host = cpystr (host));
  stream->tcpsi = pipei[0];	/* init sockets */
  stream->tcpso = pipeo[1];
  stream->ictr = 0;		/* init input counter */
  stream->port = 0xffffffff;	/* no port number */
  FD_SET (stream->tcpsi,&fds);	/* set bit in selection vector */
  FD_SET (stream->tcpsi,&efds);	/* set bit in error selection vector */
  while (((i = select (stream->tcpsi+1,&fds,0,&efds,&tmo)) < 0) &&
	 true) fprintf(stderr,"%d ",i);
//	 (errno == EINTR)) fprintf(stderr,"%d ",i);
  if (i < 0) {			/* timeout or error? */
		snprintf(tmp,MAILTMPLEN-1,"error in [%s] : %s\n",host,strerror(errno));
		mm_log(tmp,WARN);
    tcp_close (stream);		/* punt stream */
    stream = NIL;
  }
  return stream;		/* return success */
}


static NETDRIVER pipedriver = {
  (void * (*)(char *,char *,unsigned long))pipe_open,			/* open connection */
  NULL,			/* open preauthenticated connection */
  tcp_getline,			/* get a line */
  tcp_getbuffer,		/* get a buffer */
  tcp_soutr,			/* output pushed data */
  tcp_sout,			/* output string */
  tcp_close,			/* close connection */
  tcp_host,			/* return host name */
  tcp_remotehost,		/* return remote host name */
  tcp_port,			/* return port number */
  tcp_localhost			/* return local host name */
};

long pipe_mail (NETDRIVER *aDriver,char **aCmdList,ENVELOPE *env,BODY *body)
{
  /* Note: This assumes that the envelope will never generate a header of
   * more than 8K.  If your client generates godzilla headers, you will
   * need to install your own rfc822out_t routine via SET_RFC822OUTPUT
   * to use in place of this.
   */
  blocknotify_t bn = (blocknotify_t) mail_parameters (NIL,GET_BLOCKNOTIFY,NIL);

	NETSTREAM *netstream;
  char tmp[8*MAILTMPLEN];
  long error = NIL;
  if (!(env->to || env->cc || env->bcc)) {
  				/* no recipients in request */
//    smtp_fake (stream,SMTPHARDERROR,"No recipients specified");
    return NIL;
  }
	strncpy(tmp,aCmdList[0],8*MAILTMPLEN-1);
//	strncpy(tmp,"/usr/sbin/sendmail",8*MAILTMPLEN-1);
	
    netstream = (NETSTREAM *) fs_get (sizeof (NETSTREAM));
    netstream->stream = aDriver->open(tmp,NULL,0);
    netstream->dtb = aDriver;
	if(netstream->stream)
	{	
  //	netstream = aDriver->open(tmp,NULL,0);	

  //  smtp_send (stream,"RSET",NIL);/* make sure stream is in good shape */
  //  strcpy (tmp,"FROM:<");	/* compose "MAIL FROM:<return-path>" */
  /*  if (env->return_path && env->return_path->host &&
        !((env->return_path->adl &&
  	 (strlen (env->return_path->adl) > SMTPMAXPATH)) ||
  	(strlen (env->return_path->mailbox) > SMTPMAXLOCALPART) ||
  	(strlen (env->return_path->host) > SMTPMAXDOMAIN))) */
  //    rfc822_address (tmp,env->return_path);
  //  strcat (tmp,">");
  /*  if (ESMTP.ok) {
      if (ESMTP.eightbit.ok && ESMTP.eightbit.want) strcat(tmp," BODY=8BITMIME");
      if (ESMTP.dsn.ok && ESMTP.dsn.want)
        strcat (tmp,ESMTP.dsn.full ? " RET=FULL" : " RET=HDRS");
    } */
  				/* send "MAIL FROM" command */
  //  if (!(smtp_send (stream,type,tmp) == SMTPOK)) return NIL;
  				/* negotiate the recipients */
  //  if (env->to) smtp_rcpt (stream,env->to,&error);
  //  if (env->cc) smtp_rcpt (stream,env->cc,&error);
  //  if (env->bcc) smtp_rcpt (stream,env->bcc,&error);
  //  if (error) {			/* any recipients failed? */
        				/* reset the stream */
  //    smtp_send (stream,"RSET",NIL);
  //    smtp_fake (stream,SMTPHARDERROR,"One or more recipients failed");
  //    return NIL;
  //  }
  				/* negotiate data command */
  //  if (!(smtp_send (stream,"DATA",NIL) == SMTPREADY)) return NIL;
  				/* set up error in case failure */
  //  smtp_fake (stream,SMTPSOFTFATAL,"SMTP connection went away!");
  				/* output data, return success status */
    (*bn) (PROGRESS_START,(void *)"Sending");
    (*bn) (PROGRESS_SET_MAX,(void *)body->size.bytes);
    error = rfc822_output (tmp,env,body,smtpdata_soutr,netstream,1);
    (*bn) (PROGRESS_STOP,(void *)"Sending");
  	aDriver->close(netstream->stream);
	}
	fs_give((void **)&netstream);
  return error;
}

void mm_overview_header (MAILSTREAM *,unsigned long,OVERVIEW *);
void mFullFetch (MailStream *my_client,unsigned long curMsg,mailBody *in_body,char *pfx,long i);

//empty constructor
//constructor which accepts a standard mailbox
//specification 
MailStream::MailStream(const char *box)
{
	fprintf(stderr,"MailStream::MailStream()\n");
	static bool initDone = false;
	if(!initDone) {
//		fprintf(stderr,"MailStream::MailStream() - processing linkage\n");
#include "imap-2000/c-client/linkage.c"
		struct servent *service;
		int port;
		initDone = true;

//		fprintf(stderr,"MailStream::MailStream() - setting timeout handler\n");
		mail_parameters(NULL,SET_BLOCKNOTIFY,(void *)my_blocknotify );
		tcp_parameters(SET_TIMEOUT,(void *)mm_timeout);
//		ssl_parameters(SET_TIMEOUT,(void *)mm_timeout);
		tcp_parameters(SET_OPENTIMEOUT,(void *)120);
//		ssl_parameters(SET_OPENTIMEOUT,(void *)120);

//		fprintf(stderr,"MailStream::MailStream() - setting imap port\n");
		service = getservbyname("imap",NULL);
		if(service)
		{
			port = service->s_port;
			port = ((0xff & port) << 8) | ((0xff00 & port) >> 8);
		}
		else port = 143;
		mail_parameters(NULL,SET_IMAPPORT,(void *)port );

		service = getservbyname("imaps",NULL);
		if(service)
		{
			port = service->s_port;
			port = ((0xff & port) << 8) | ((0xff00 & port) >> 8);
		}
		else port = 993;
		mail_parameters(NULL,SET_ALTIMAPPORT,(void *)port );

//		fprintf(stderr,"MailStream::MailStream() - setting pop3 port\n");
		service = getservbyname("pop3",NULL);
		if(service)
		{
			port = service->s_port;
			port = ((0xff & port) << 8) | ((0xff00 & port) >> 8);
		} else port = 110;
		mail_parameters(NULL,SET_POP3PORT,(void *)port );

		service = getservbyname("pop3s",NULL);
		if(service)
		{
			port = service->s_port;
			port = ((0xff & port) << 8) | ((0xff00 & port) >> 8);
		} else port = 995;
		mail_parameters(NULL,SET_ALTPOPPORT,(void *)port );

//		fprintf(stderr,"MailStream::MailStream() - setting nntp port\n");
		service = getservbyname("nntp",NULL);
		if(service)
		{
			port = service->s_port;
			port = ((0xff & port) << 8) | ((0xff00 & port) >> 8);
		} else port = 119;
		mail_parameters(NULL,SET_NNTPPORT,(void *)port );

//		fprintf(stderr,"MailStream::MailStream() - setting smtp port\n");
		service = getservbyname("smtp",NULL);
		if(service)
		{
			port = service->s_port;
			port = ((0xff & port) << 8) | ((0xff00 & port) >> 8);
		} else port = 25;
		smtp_parameters(SET_SMTPPORT,(void *)port );
//		fprintf(stderr,"MailStream::MailStream() - init done\n");
	}
//	fprintf(stderr,"MailStream::MailStream() - clearing stream pointer\n");
	my_stream = NULL;
//	fprintf(stderr,"MailStream::MailStream() - clearing current box\n");
	currentBox[0] = 0x00;
//	fprintf(stderr,"MailStream::MailStream() - clearing retry\n");
	Retry = true;
//	fprintf(stderr,"MailStream::MailStream() - about to set box\n");
	
//	fprintf(stderr,"MailStream::MailStream() - Setting alternate driver\n");
//	mail_parameters(NULL,SET_ALTDRIVER,(void *)&ssldriver);
/*	mail_parameters(NULL,SET_TRYALTFIRST,(void *)1); */
	if(box) setBox(box);
//	fprintf(stderr,"MailStream::MailStream() - done\n");
}
	
//destructor
MailStream::~MailStream()
{
	fprintf(stderr,"MailStream::~MailStream()\n");
	if(my_stream)
	{
		closeBox();
	}
}

unsigned long MailStream::numMsgs()
{
	if(my_stream) return my_stream->nmsgs;
	return 0;
}

void MailStream::setRetry(bool newtry)
{
	Retry = newtry;
}

bool MailStream::isOpen()
{
	return (my_stream != NULL);
}

void MailStream::setTimeOut(unsigned long timeout)
{
	tcp_parameters(SET_READTIMEOUT,(void *)timeout);
	tcp_parameters(SET_WRITETIMEOUT,(void *)timeout);
//	ssl_parameters(SET_READTIMEOUT,(void *)timeout);
//	ssl_parameters(SET_WRITETIMEOUT,(void *)timeout);
}

unsigned long MailStream::getTimeOut()
{
	return (unsigned long)tcp_parameters(GET_READTIMEOUT,NULL);
}

void MailStream::flagMsg(int msg,const char *flag)
{
	char msgno[16];
	snprintf(msgno,sizeof(msgno),"%d",msg);
	flagMsg(msgno,flag);
}

void MailStream::flagMsg(const char *seq,const char *flag)
{
	if(my_stream && blockCallback())
	{
		mail_setflag(my_stream,(char *)seq,(char *)flag);
		releaseCallback();
	}
}

void MailStream::unflagMsg(int msg,const char *flag)
{
	char msgno[16];
	snprintf(msgno,sizeof(msgno),"%d",msg);
	unflagMsg(msgno,flag);
}

void MailStream::unflagMsg(const char *seq,const char *flag)
{
	if(my_stream && blockCallback())
	{
		mail_clearflag(my_stream,(char *)seq,(char *)flag);
		releaseCallback();
	}
}

void MailStream::toggleflagMsg(int msgno,const char *flag)
{
	char buffer[16];
	unsigned long uf;

	if(my_stream)
	{
		snprintf(buffer,sizeof(buffer)-1,"%d",msgno);
		if( getflagMsg(msgno) & mail_parse_flags(my_stream,(char *)flag,&uf) )
			unflagMsg(buffer,flag);
		else
			flagMsg(buffer,flag);
	}
}

int MailStream::getflagMsg(int msgno)
{
	int ret = 0;
  MESSAGECACHE *elt;

	if(blockCallback())
	{
		elt = mail_elt(my_stream,msgno);
		if(elt && elt->valid) {
			if( elt->deleted ) ret |= fDELETED;
			if( elt->flagged ) ret |= fFLAGGED;
			if( elt->answered ) ret |= fANSWERED;
			if( elt->draft ) ret |= fDRAFT;
			if( elt->seen ) ret |= fSEEN;
			if( !elt->recent ) ret |= fOLD;
		}
		releaseCallback();
	}
	return ret;
}

//get a Cache entry for a message
MESSAGECACHE *MailStream::getCache(unsigned long which)
{
	if(my_stream) return mail_elt(my_stream,which);
	return NULL;
}

//perform a check
void MailStream::Check()
{
	if(my_stream && blockCallback())
	{
		mail_check(my_stream);
		releaseCallback();
	}
}

//get a status
void MailStream::Ping()
{
	long ret = 0;
	if(blockCallback())
	{
		if(my_stream && currentBox && *currentBox)
		{
			ret = mail_ping(my_stream);
		}
		releaseCallback();
	}
}

//get a status
long MailStream::Status(const char *folder)
{
	long ret = 0;
	if(blockCallback())
	{
		if(folder)
		{
			char tmp[MAXBOXLEN];
//			if(my_stream) mail_close(my_stream);
//			my_stream = NULL;
//			setBox(folder);
			firstTry = true;
			strncpy(tmp,currentBox,MAXBOXLEN);
			strncpy(currentBox,folder,MAXBOXLEN);
			ret = mail_status(NULL,(char *)folder,SA_MESSAGES|SA_RECENT|SA_UNSEEN);
			strncpy(currentBox,tmp,MAXBOXLEN);
		} else {
			firstTry = true;
			if(my_stream) ret = mail_status(my_stream,currentBox,SA_MESSAGES|SA_RECENT|SA_UNSEEN);
		}
		releaseCallback();
	}
	return ret;
}

void MailStream::listFolders(const char *what)
{
	char buffer[MAXBOXLEN];

	strncpy(buffer,what,MAXBOXLEN-1);
	if(blockCallback()) {
		my_blocknotify(PROGRESS_START,(void *)"Listing Folders");
		firstTry = true;
		mail_list(my_stream,NULL,buffer);
		my_blocknotify(PROGRESS_STOP,(void *)"Listing Folders");
		releaseCallback();
	}
}

//expunge current mailbox
void MailStream::expungeBox()
{
	if(my_stream && blockCallback() ) {
		unSeen = 0;
		mail_expunge(my_stream);
		releaseCallback();
	}
}

void *MailStream::getParameter(unsigned long parm,void *val)
{
	return mail_parameters(my_stream,parm,val);
}

bool MailStream::appendMessage(const char *box,const char *msg,unsigned long len,const char *flag,const char *date)
{
	bool retval=false;

	if(blockCallback() ) {
		STRING str;
		firstTry = true;
		Retry = true;

		INIT(&str,mail_string,(void *)msg,len);

		retval = (mail_append_full(NULL,(char *)box,(char *)flag,(char *)date,&str));
		releaseCallback();
	}
	return retval;
}

bool MailStream::moveMessage(int msgno,const char *box)
{
	bool retval=false;
	char msg[16];


	if(blockCallback() ) {
		firstTry = true;
		Retry = true;
		snprintf(msg,sizeof(msg)-1,"%d",msgno);
		retval = mail_move(my_stream,msg,(char *)box);
		releaseCallback();
	}
	return retval;
}

void MailStream::setBox(const char *box,bool readOnly)
{
	char tmp[MAXBOXLEN];
	blocknotify_t bn = (blocknotify_t) mail_parameters (NIL,GET_BLOCKNOTIFY,NIL);

	strncpy(tmp,currentBox,MAXBOXLEN);
	strncpy(currentBox,box,MAXBOXLEN);
//close old stream if any
	if(blockCallback()) {
		if(my_stream) {
  			closeBox();
		}
		firstTry = true;
		Retry = true;
		setError("");
		unSeen = 0;
		fprintf(stderr,"MailStream::setBox( %s )",currentBox);
		(*bn) (PROGRESS_START,(void *)"Opening");
		while(Retry && !my_stream && !getError())
		{
			if(!currentBox || currentBox[0] != '{') Retry = false;
			my_stream = mail_open(my_stream,currentBox,readOnly ? OP_READONLY : 0 );
		}
		(*bn) (PROGRESS_STOP,(void *)"Opening");
		releaseCallback();
	}
}

void MailStream::closeBox()
{
	blocknotify_t bn = (blocknotify_t) mail_parameters (NIL,GET_BLOCKNOTIFY,NIL);
	fprintf(stderr,"MailStream::closeBox()\n");
	if(my_stream && blockCallback())
	{
			MAILSTREAM *close_this = my_stream;
//			int status;
			my_stream = NULL;
//      if(!(fork())) {
        /*We're the child.*/
				(*bn) (PROGRESS_START,(void *)"Closing");
//  			mm_progress(START,0,"Closing");
  			mail_close(close_this);
				(*bn) (PROGRESS_STOP,(void *)"Closing");
//  			mm_progress(STOP,0,"Closing");
 		
//        _exit(127);
//      }
//		  fprintf(stderr,"waitpid : %d",waitpid(0/*child_pid*/, &status, 0));
				
		releaseCallback();
	}
}

void MailStream::reOpen(bool readOnly)
{
	if(blockCallback()) {
		if(my_stream) {
			blocknotify_t bn = (blocknotify_t) mail_parameters (NIL,GET_BLOCKNOTIFY,NIL);
			(*bn) (PROGRESS_START,(void *)"Closing");
//			mm_progress(START,0,"Closing");
			mail_close(my_stream);
			my_stream = NULL;
			(*bn) (PROGRESS_STOP,(void *)"Closing");
//			mm_progress(STOP,0,"Closing");
		}
		firstTry = true;
		Retry = true;
		setError("");
		unSeen = 0;
		while(Retry && !my_stream && !getError())
		{
			if(!currentBox || currentBox[0] != '{') Retry = false;
			my_stream = mail_open(my_stream,currentBox,readOnly ? OP_READONLY : 0);
		}
		releaseCallback();
	}
}

//return the R/W status of the mailbox
//or read only if no box open
//-----------------------------------------------------------------
bool MailStream::getReadWrite()
{
	if(my_stream) return (my_stream->rdonly != 0);
	return true;
}

bool MailStream::isLocked()
{
	return (my_stream && my_stream->lock == T);
}

void MailStream::setUnSeen(unsigned long in_unseen)
{
	unSeen = in_unseen;
}

unsigned long MailStream::getUnSeen(void)
{
	return unSeen;
}
//fetch the overview structure on a given
//message number 
//-----------------------------------------------------------------
ENVELOPE *MailStream::fetchStructure (long msgno,BODY **body)
{
	ENVELOPE *retval = NULL;
	if(blockCallback()) {
  	if(my_stream && body) {
  		retval = mail_fetchstructure(my_stream,msgno,body);
		}
 		releaseCallback();
  }
	return retval;
}
mailMessage *MailStream::fetchStructure (long msgno)
{
	ENVELOPE *env;
	BODY *body;

	env = fetchStructure(msgno,&body);
	return new mailMessage(env,body);
}

//fetch the FROM/SUBJECT line of a message
//if you dont specify where to put it
//these will use a static buffer
//-----------------------------------------------------------------
const char *MailStream::fetchFrom (long msgno,char *str=0,long size=0){return NULL;};
const char *MailStream::fetchSubject (long msgno,char *str=0,long size=0){return NULL;};

//fetch the actual body-content of a specified part
//-----------------------------------------------------------------
void MailStream::fetchMessage(long msgno,MESSAGE *msg)
{
	MESSAGECACHE *elt;
	char *str;
	unsigned long len;

	if(msg && blockCallback()) {
		elt = mail_elt(my_stream,msgno);
  	str = mail_fetch_message(my_stream,msgno,&len,0);
    releaseCallback();
		if(!msg->full.text.data )
  	{
			msg->full.text.size = len;
			msg->full.text.data = (unsigned char *)malloc(len);
			memcpy(msg->full.text.data,str,len);
  	}
  }
}

//fetch the actual body-content of a specified part
//-----------------------------------------------------------------
void MailStream::fetchBodyContent(long msgno,const char *prefix)
{
	BODY *body;
	char *str;
	unsigned long len;

	if(msgno && blockCallback()) {
  	body = mail_body(my_stream,msgno,(char *)prefix);
		if(body && !body->contents.text.data )
		{
			blocknotify_t bn = (blocknotify_t) mail_parameters (NIL,GET_BLOCKNOTIFY,NIL);
			(*bn) (PROGRESS_START,(void *)"Message");
//			mm_progress(START,0,"Message");
			(*bn) (PROGRESS_SET_MAX,(void *)body->size.bytes);
//			mm_progress(SET_MSG,body->size.bytes,NULL);
	  	str = mail_fetch_body(my_stream,msgno,(char *)prefix,&len,0);

// body structure is NOT filled if we have a local box
			if(body->contents.text.data == NULL) {
				body->contents.text.size = len;
				body->contents.text.data = (unsigned char *)malloc(len+1);
				memcpy(body->contents.text.data,str,len);
				body->contents.text.data[len] = 0x00;
			}

	  	str = mail_fetch_mime(my_stream,msgno,(char *)prefix,&len,0);
			if(body->mime.text.data == NULL) {
				body->mime.text.size = len;
				body->mime.text.data = (unsigned char *)malloc(len+1);
				memcpy(body->mime.text.data,str,len);
				body->mime.text.data[len] = 0x00;
			}
			(*bn) (PROGRESS_STOP,(void *)"Message");
//			mm_progress(STOP,0,NULL);
		}
    releaseCallback();
  }
}

//fetch the body structure of a specified part
//-----------------------------------------------------------------
BODY *MailStream::fetchBody(long msgno,const char *prefix)
{
	BODY *ret = NULL;

	if(msgno && blockCallback()) {
  	ret = mail_body(my_stream,msgno,(char *)prefix);
    releaseCallback();
  }
	return ret;
}

//fetch the actual header-content of a specified part
//-----------------------------------------------------------------
char *MailStream::fetchHeaderContent(long msgno,const char *prefix,unsigned long *len)
{
	char *ret=NULL;
	char *str;
	BODY *body;
	
	if(my_stream && msgno && blockCallback()) {
  	fprintf(stderr,"fetchHeaderContent(%ld#%s)\n",msgno,prefix);
  	str = mail_fetch_header(my_stream,msgno,(char *)prefix,0,len,0);
  	body = mail_body(my_stream,msgno,(char *)prefix);
		ret = (char *)malloc((*len) + 1);
		if(ret)
		{
			memcpy(ret,str,*len);
			ret[*len] = 0x00;
			if(body)
			{
  			body->nested.msg->header.text.data = (unsigned char *)ret;
  			body->nested.msg->header.text.size = *len;
  			body->nested.msg->header.offset = 0;
  			
			}
		}
		releaseCallback();
	}
	return ret;
};

//give us an overview of the mailbox in the selected range
//calls overviewHeader for each entry
//-----------------------------------------------------------------
void MailStream::fetchOverview(char *range)
{
	if(blockCallback()) {
		if(my_stream) mail_fetch_overview(my_stream,range,mm_overview_header);
		releaseCallback();
	}
}

//parse a date into an MESSAGECACHE structure
//-----------------------------------------------------------------
MESSAGECACHE *MailStream::parseDate(char *date)
{
	static MESSAGECACHE selt;
	mail_parse_date(&selt,date);
	return &selt;
}

void MailStream::listNotify (int delimiter,char *mailbox,long attributes)
{
  fputs("mm_list\n",stderr);
  fputs(" ",stderr);
  if (delimiter) fprintf (stderr,"%c",delimiter);
  else fputs ("NIL",stdout);
  fputs (" ",stderr);
  fputs (mailbox,stderr);
  if (attributes & LATT_NOINFERIORS) fputs (", no inferiors",stderr);
  if (attributes & LATT_NOSELECT) fputs (", no select",stderr);
  if (attributes & LATT_MARKED) fputs (", marked",stderr);
  if (attributes & LATT_UNMARKED) fputs (", unmarked",stderr);
  fputs("\n",stderr);
}

//-----------------------------------------------------------------
// Here are our Callback functions overrides
//-----------------------------------------------------------------

void MailStream::notifyNotify (char *string,long errflg)
{
	char *unseen;

	unseen = strchr(string,'[');
	if(unseen && !strncmp(unseen,"[UNSEEN",6))
	{
		char buffer[16];
		strncpy(buffer,unseen+8,15);
		unseen = strchr(buffer,']');
		if(unseen) *unseen = 0x00;
		setUnSeen(atol(buffer));
	}
	fprintf(stderr,"mm_notify - ");
	logNotify(string,errflg);
}

void MailStream::lsubNotify(int delimiter,char *mailbox,long attributes)
{
  fprintf(stderr,"mm_lsub\n");
  putchar (' ');
  if (delimiter) putchar (delimiter);
  else fputs ("NIL",stdout);
  putchar (' ');
  fputs (mailbox,stdout);
  if (attributes & LATT_NOINFERIORS) fputs (", no inferiors",stdout);
  if (attributes & LATT_NOSELECT) fputs (", no select",stdout);
  if (attributes & LATT_MARKED) fputs (", marked",stdout);
  if (attributes & LATT_UNMARKED) fputs (", unmarked",stdout);
  putchar ('\n');
}

void MailStream::statusNotify (char *mailbox,MAILSTATUS *status)
{
  fprintf(stderr,"mm_status\n");
  fprintf (stderr," Mailbox %s",mailbox);
  if (status->flags & SA_MESSAGES) fprintf (stderr,", %lu messages",status->messages);
  if (status->flags & SA_RECENT) fprintf (stderr,", %lu recent",status->recent);
  if (status->flags & SA_UNSEEN) fprintf (stderr,", %lu unseen",status->unseen);
  if (status->flags & SA_UIDVALIDITY) fprintf (stderr,", %lu UID validity",
					      status->uidvalidity);
  if (status->flags & SA_UIDNEXT) fprintf (stderr,", %lu next UID",status->uidnext);
  printf ("\n");
}

void MailStream::loginNotify (NETMBX *mb,char *user,char *pwd,long trial)
{
  char tmp[MAILTMPLEN];

  if (*mb->user) {
    strcpy (user,mb->user);
    sprintf (tmp,"{%s/%s/user=%s} password: ",mb->host,mb->service,mb->user);
  }
  else {
    sprintf (tmp,"{%s/%s} username: ",mb->host,mb->service);
	  puts(tmp);
  	scanf("%64s",user);
    strcpy (tmp,"Password: ");
  }
#if UNIXLIKE
  strcpy (pwd,getpass (tmp));
#else
	  puts(tmp);
  	scanf("%64s",pwd);
#endif
	if(!*pwd) Retry = false;
}

void MailStream::overviewHeader(unsigned long uid,OVERVIEW *ov)
{
  unsigned long i;
  char *t,tmp[MAILTMPLEN];
  ADDRESS *adr;
  unsigned long msgno;
  MESSAGECACHE *elt;
  MESSAGECACHE selt;

  if(!my_stream) return;

  msgno = mail_msgno (my_stream,uid);
  elt = mail_elt (my_stream,msgno);
  tmp[0] = elt->recent ? (elt->seen ? 'R': 'N') : ' ';
  tmp[1] = (elt->recent | elt->seen) ? ' ' : 'U';
  tmp[2] = elt->flagged ? 'F' : ' ';
  tmp[3] = elt->answered ? 'A' : ' ';
  tmp[4] = elt->deleted ? 'D' : ' ';
  mail_parse_date (&selt,ov->date);
  sprintf (tmp+5,"%4ld) ",elt->msgno);
  mail_date (tmp+11,&selt);
  tmp[17] = ' ';
  tmp[18] = '\0';
  memset (tmp+18,' ',(size_t) 20);
  tmp[38] = '\0';		/* tie off with null */
				/* get first from address from envelope */
  for (adr = ov->from; adr && !adr->host; adr = adr->next);
  if (adr) {			/* if a personal name exists use it */
    if (!(t = adr->personal))
      sprintf (t = tmp+400,"%s@%s",adr->mailbox,adr->host);
    memcpy (tmp+18,t,(size_t) min (20,(long) strlen (t)));
  }
  strcat (tmp," ");
  if ((i = elt->user_flags)) {
    strcat (tmp,"{");
    while (i) {
      strcat (tmp,my_stream->user_flags[find_rightmost_bit (&i)]);
      if (i) strcat (tmp," ");
    }
    strcat (tmp,"} ");
  }
  sprintf (tmp + strlen (tmp),"%.25s (%ld chars)",
	   ov->subject ? ov->subject : " ",ov->optional.octets);
  puts (tmp);
}

unsigned long MailStream::uid2msgno(unsigned long uid)
{
	unsigned long retval = 0;

	if(my_stream && blockCallback()) {
		retval = mail_msgno(my_stream,uid);
		releaseCallback();
	}
	return retval;
}

unsigned long MailStream::msgno2uid(unsigned long msgno)
{
	unsigned long retval = 0;

	if(my_stream && blockCallback()) {
		retval = mail_uid(my_stream,msgno);
		releaseCallback();
	}
	return retval;
}

void MailStream::searchSimple(const char *what)
{
	if(my_stream && blockCallback()) {
		char *tmp;
		tmp = cpystr (what);
//		fprintf(stderr,"Searching for '%s'\n",what);
		blocknotify_t bn = (blocknotify_t) mail_parameters (NIL,GET_BLOCKNOTIFY,NIL);
		(*bn) (PROGRESS_START,(void *)"Searching");
//		mm_progress(START,0,"Searching messages");
		mail_search(my_stream,(char *)tmp);
		(*bn) (PROGRESS_STOP,(void *)"Searching");
//		mm_progress(STOP,0,"Searching messages");
		fs_give((void **) &tmp );
		releaseCallback();
	}
}

//-----------------------------------------------------------------
// Here are the Callback functions of the C-Client
//-----------------------------------------------------------------

//timeout handler defaults to stop
int mm_timeout( time_t t1,time_t t2)
{ if(currentCallback) return currentCallback->timeoutHandler(t1,t2); else return 0;}

//gets called if a message was searched with a positive result
void mm_searched (MAILSTREAM *stream,unsigned long number)
{ if(currentCallback) currentCallback->searchedNotify(number); else fprintf(stderr,"IGNORING : searched\n");}

//gets called if we get a message on how many messages exist in the box
void mm_exists (MAILSTREAM *stream,unsigned long number)
{ if(currentCallback) currentCallback->existsNotify(stream,number);  else fprintf(stderr,"IGNORING : exists\n");}

//gets called if a message is expunged, all other messages are renumbered
void mm_expunged (MAILSTREAM *stream,unsigned long number)
{ if(currentCallback) currentCallback->expungedNotify(number);  else fprintf(stderr,"IGNORING : expunged\n");}

//gets called if a message changes its flags
void mm_flags (MAILSTREAM *stream,unsigned long number)
{ if(currentCallback) currentCallback->flagsNotify(number);  else fprintf(stderr,"IGNORING : flags\n");}

//duno
void mm_notify (MAILSTREAM *stream,char *string,long errflg)
{
	if(currentCallback) {
		if(!errflg && string && string[0] == '[')
		{
			if(!strncmp(&string[1],"WARN",4)) { errflg |= WARN;}
			else if(!strncmp(&string[1],"ERROR",5)) { errflg |= ERROR; }
			else if(!strncmp(&string[1],"PARSE",5)) { errflg |= PARSE; }
			else if(!strncmp(&string[1],"BYE",5)) { errflg |= BYE; }
//			fprintf(stderr,"reconstructed flags %ld with %s\n",errflg,&string[1]);
		}
		if(errflg == ERROR || errflg == WARN || errflg == PARSE) currentCallback->setError(string);
		currentCallback->notifyNotify(string,errflg);
	} else
		fprintf(stderr,"IGNORING : notify : [%ld] %s\n",errflg,string);
}

//mail_list callback function
void mm_list (MAILSTREAM *stream,int delimiter,char *mailbox,long attributes)
{ if(currentCallback) currentCallback->listNotify(delimiter,mailbox,attributes);  else fprintf(stderr,"IGNORING : list\n");}

//mail_lsub callback function
void mm_lsub (MAILSTREAM *stream,int delimiter,char *mailbox,long attributes)
{ if(currentCallback) currentCallback->lsubNotify(delimiter,mailbox,attributes);  else fprintf(stderr,"IGNORING : lsub\n");}

//duno
void mm_status (MAILSTREAM *stream,char *mailbox,MAILSTATUS *status)
{ if(currentCallback) currentCallback->statusNotify(mailbox,status); else fprintf(stderr,"IGNORING : status\n");}
//{ if(currentCallback) { if(!currentCallback->skipStatus) currentCallback->statusNotify(mailbox,status); } else fprintf(stderr,"IGNORING : status\n");}

//normal log function
void mm_log (char *string,long errflg)
{
	if(currentCallback)
	{
		if(!errflg && string && string[0] == '[')
		{
			if(!strncmp(&string[1],"WARN",4)) { errflg |= WARN;}
			else if(!strncmp(&string[1],"ERROR",5)) { errflg |= ERROR; }
			else if(!strncmp(&string[1],"PARSE",5)) { errflg |= PARSE; }
			else if(!strncmp(&string[1],"BYE",5)) { errflg |= BYE; }
//			fprintf(stderr,"reconstructed flags %ld with %s\n",errflg,&string[1]);
		}
		if(errflg == ERROR || errflg == WARN) currentCallback->setError(string);
		currentCallback->logNotify(string,errflg);
	}
	else fprintf(stderr,"IGNORING : log : [%ld] %s\n",errflg,string);
}

//debug log function
void mm_dlog (char *string)
{ if(currentCallback) currentCallback->dlogNotify(string);  else fprintf(stderr,"IGNORING : dlog : %s\n",string);}

//login function
void mm_login (NETMBX *mb,char *user,char *pwd,long trial)
{ if(currentCallback) currentCallback->loginNotify(mb,user,pwd,trial);  else fprintf(stderr,"IGNORING : login\n");}


void mm_critical (MAILSTREAM *stream)
{ if(currentCallback) currentCallback->criticalNotify();  else fprintf(stderr,"IGNORING : critical\n");}


void mm_nocritical (MAILSTREAM *stream)
{ if(currentCallback) currentCallback->nocriticalNotify();  else fprintf(stderr,"IGNORING : nocritical\n");}


long mm_diskerror (MAILSTREAM *stream,long errcode,long serious)
{ if(currentCallback) currentCallback->diskerrorNotify(errcode,serious); else fprintf(stderr,"IGNORING : diskerror\n");return 0;}


void mm_fatal (char *string)
{ if(currentCallback) currentCallback->fatalNotify(string);  else fprintf(stderr,"IGNORING : fatal\n");}

void mm_overview_header (MAILSTREAM *stream,unsigned long uid,OVERVIEW *ov)
{ if(currentCallback) currentCallback->overviewHeader(uid,ov);  else fprintf(stderr,"IGNORING : overview\n");}

//-----------------------------------------------------------------
//here is our virtual class implementation of that
//shameless stolen from mtest.c (mostly)
//-----------------------------------------------------------------
callbackFunctions::callbackFunctions()
{
	currentUsage = 0;
	myPredecessor = NULL;
	lastError[0] = 0x00;
}

//return the last error message
//-----------------------------------------------------------------
const char *callbackFunctions::getError()
{
	if(*lastError) return lastError;
	return NULL;
};
	
//set the last error message
//-----------------------------------------------------------------
void callbackFunctions::setError(const char *error)
{
	if(error) strncpy(lastError,error,MAXERROR-1);
};
	
//try to get the callback functions exclusively
bool callbackFunctions::blockCallback(){
/*	if(currentCallback && currentCallback != this) {
		fprintf(stderr,"Callback functions in use!\n");
		return false;
	}
	setError("");
	currentCallback = this;
	currentUsage++;
	return true; */
	struct callStack *myEntry = (struct callStack *)malloc(sizeof(struct callStack));

	myEntry->pred = currentCallStack;
	myEntry->topCallback = currentCallback;
	currentCallStack = myEntry;

	currentUsage++;
//	myPredecessor = currentCallback;
	currentCallback = this;
	setError("");
//	fprintf(stderr,"Setting cCb to : %p [%d]\n",currentCallback,currentUsage);	
	return true;
}

//release the callback functions
bool callbackFunctions::releaseCallback()
{
	struct callStack *myEntry = currentCallStack;

	if(currentCallback != this)
	{
		fprintf(stderr,"releaseCallback called out of Line: this can be fatal\n");
		return false;
	}
	if(currentCallStack == NULL)
	{
		fprintf(stderr,"stack is EMPTY: this must be fatal\n");
		return false;
	}
	currentCallback = myEntry->topCallback;
	currentCallStack = myEntry->pred;
	free(myEntry);

//	myPredecessor = NULL;
	currentUsage--;
//	fprintf(stderr,"reSetting cCb to : %p [%d]\n",currentCallback,currentUsage);	
	return true;
/*	if(currentCallback != this) return false;
	currentUsage--;
	if(!currentUsage) currentCallback = NULL;
 	return true; */
}

int callbackFunctions::timeoutHandler(time_t t1,time_t t2)
{
	fprintf(stderr,"TIMEOUT %ld - %ld\n",t1,t2);
	return 1;
}

int callbackFunctions::progressNotify(enum mailProgress function,unsigned long value,const char *txt)
{
	static unsigned long total = 0;
	static unsigned long current = 0;
	switch(function)
	{
		case mpSET_TOTAL:
			total = value;
			break;
		case mpSET_DELTA:
			value += current;
		case mpSET_CURRENT:
			current = value;
			if(total > 0)
				fprintf(stderr,"|%d\n",(int)(100 * current / total));
			break;
		case mpSTART:
			break;
		case mpSTOP:
			break;
		case mpTIMEOUT:
			break;
		case mpSET_STATUS:
			if(txt) fprintf(stderr,"|%s\n",txt);
			break;
	}
	return 1;
}

void callbackFunctions::searchedNotify (unsigned long number)
{
	fprintf(stderr,"mm_searched %ld\n",number);
}

void callbackFunctions::existsNotify (MAILSTREAM *stream,unsigned long number)
{
	fprintf(stderr,"mm_exists: %p\n",stream);
}

void callbackFunctions::expungedNotify (unsigned long number)
{
	fprintf(stderr,"mm_expunged: %ld\n",number);
}

void callbackFunctions::flagsNotify (unsigned long number)
{
	fprintf(stderr,"mm_flags: %ld\n",number);
}

void callbackFunctions::notifyNotify (char *string,long errflg)
{
	logNotify(string,errflg);
}

void callbackFunctions::listNotify (int delimiter,char *mailbox,long attributes)
{
	fprintf(stderr,"mm_list: [%ld] : %.80s\n",attributes,mailbox);
}

void callbackFunctions::lsubNotify(int delimiter,char *mailbox,long attributes)
{
	fprintf(stderr,"mm_lsub: [%ld] : %.80s\n",attributes,mailbox);
}

void callbackFunctions::statusNotify (char *mailbox,MAILSTATUS *status)
{
  fprintf(stderr,"mm_status: %p\n",status);
}

void callbackFunctions::logNotify (char *string,long errflg)
{
  fprintf(stderr,"mm_log\n");
  switch ((short) errflg) {
  case NIL:
    fprintf (stderr,"[%s]\n",string);
    break;
  case PARSE:
  case WARN:
    fprintf (stderr,"%%%s\n",string);
    break;
  case ERROR:
    fprintf (stderr,"?%s\n",string);
    break;
	default:
    fprintf (stderr,"DUNO %s\n",string);
		break;
  }
}

void callbackFunctions::dlogNotify (char *string)
{
  fprintf(stderr,"mm_dlog : %s\n",string);
}

void callbackFunctions::loginNotify (NETMBX *mb,char *user,char *pwd,long trial)
{
	*pwd = 0x00;
}

void callbackFunctions::criticalNotify ()
{
  fprintf(stderr,"mm_critical\n");
}

void callbackFunctions::nocriticalNotify ()
{
	fprintf(stderr,"mm_nocritical\n");
}

void callbackFunctions::diskerrorNotify (long errcode,long serious)
{
  fprintf(stderr,"mm_diskerror\n");
#if UNIXLIKE
  kill (getpid (),SIGSTOP);
#else
  abort ();
#endif
}

void callbackFunctions::fatalNotify (char *string)
{
  fprintf(stderr,"mm_fatal : %s\n",string);
}

void callbackFunctions::overviewHeader(unsigned long uid,OVERVIEW *ov)
{
  fprintf (stderr,"OVERVIEW: %.25s (%ld chars)\n",
	   ov->subject ? ov->subject : " ",ov->optional.octets);
}

//-----------------------------------------------------------------
mailEnvelope::mailEnvelope()
{
		my_env = mail_newenvelope();
		envelope_destroy = true;
//	fprintf(stderr,"mailEnvelope() : %p\n",this);
}
//-----------------------------------------------------------------
mailEnvelope::mailEnvelope(ENVELOPE *env)
{
		my_env = env;
		envelope_destroy = false;
//	fprintf(stderr,"mailEnvelope(ENVELOPE) : %p\n",this);
}
//-----------------------------------------------------------------
mailEnvelope::~mailEnvelope()
{
		if(envelope_destroy) mail_free_envelope(&my_env);
//	fprintf(stderr,"~mailEnvelope() : %p\n",this);
}
//-----------------------------------------------------------------
bool mailEnvelope::isEmpty()
{
		return (my_env == NULL);
}
//-----------------------------------------------------------------
tm *mailEnvelope::messageDate(bool localize)
{
	return parseDate(my_env->date,localize);
}

const char *mailEnvelope::messageDateStr()
{
	return my_env->date;
}

void mailEnvelope::setMessageReturnPath(const char *in_adr)
{
  mail_free_address (&my_env->return_path);
	blockCallback();
  rfc822_parse_adrlist (&my_env->return_path,(char *)in_adr,".UNKNOWN.");
	releaseCallback();
}
//-----------------------------------------------------------------
void mailEnvelope::setMessageFrom(const char *in_adr)
{
  mail_free_address (&my_env->from);
	blockCallback();
  rfc822_parse_adrlist (&my_env->from,(char *)in_adr,".UNKNOWN.");
	releaseCallback();
}
//-----------------------------------------------------------------
void mailEnvelope::setMessageSender(const char *in_adr)
{
  mail_free_address (&my_env->sender);
	blockCallback();
  rfc822_parse_adrlist (&my_env->sender,(char *)in_adr,".UNKNOWN.");
	releaseCallback();
}
//-----------------------------------------------------------------
void mailEnvelope::setMessageDate(const char *aStr)
{
  char line[MAILTMPLEN];
	
	if(aStr) strncpy(line,aStr,MAILTMPLEN-1);
  else rfc822_date (line);
  if(my_env->date)
	{
		fs_give((void **) &my_env->date);
	}
	my_env->date = (char *) fs_get (1+strlen (line));
  strcpy(my_env->date,line);
}
//-----------------------------------------------------------------
void mailEnvelope::setMessageReplyTo(const char *in_adr)
{
  mail_free_address (&my_env->reply_to);
	blockCallback();
  rfc822_parse_adrlist (&my_env->reply_to,(char *)in_adr,".UNKNOWN.");
	releaseCallback();
}
//-----------------------------------------------------------------
void mailEnvelope::setMessageTo(const char *in_adr)
{
  mail_free_address (&my_env->to);
	blockCallback();
  rfc822_parse_adrlist (&my_env->to,(char *)in_adr,".UNKNOWN.");
	releaseCallback();
}
//-----------------------------------------------------------------
void mailEnvelope::setMessageCC(const char *in_adr)
{
  mail_free_address (&my_env->cc);
	blockCallback();
  rfc822_parse_adrlist (&my_env->cc,(char *)in_adr,".UNKNOWN.");
	releaseCallback();
}
//-----------------------------------------------------------------
void mailEnvelope::setMessageBCC(const char *in_adr)
{
  mail_free_address (&my_env->bcc);
	blockCallback();
  rfc822_parse_adrlist (&my_env->bcc,(char *)in_adr,".UNKNOWN.");
	releaseCallback();
}
//-----------------------------------------------------------------
void mailEnvelope::setFollowupTo(const char *in_followup_to)
{
	if(my_env->followup_to)
		fs_give((void **) &my_env->followup_to );
	if(in_followup_to && *in_followup_to) my_env->followup_to = cpystr( in_followup_to );
}
//-----------------------------------------------------------------
void mailEnvelope::setSubject(const char *in_subject)
{
	if(my_env->subject)
		fs_give((void **) &my_env->subject );
	if(in_subject && *in_subject)	my_env->subject = cpystr( in_subject );
}
//-----------------------------------------------------------------
void mailEnvelope::setNewsgroups(const char *in_newsgroups)
{
	if(my_env->newsgroups)
		fs_give((void **) &my_env->newsgroups );
	if(in_newsgroups && *in_newsgroups)	my_env->newsgroups = cpystr( in_newsgroups );
}
//-----------------------------------------------------------------
const char *mailEnvelope::messageReturnPath(enum AddrOutput type)
{
	return emailAddrAsText(my_env->return_path,type);
}
//-----------------------------------------------------------------
const char *mailEnvelope::messageFrom(enum AddrOutput type)
{
	return emailAddrAsText(my_env->from,type);
}
//-----------------------------------------------------------------
const char *mailEnvelope::messageSender(enum AddrOutput type)
{
	return emailAddrAsText(my_env->sender,type);
}
//-----------------------------------------------------------------
const char *mailEnvelope::messageReplyTo(enum AddrOutput type)
{
	return emailAddrAsText(my_env->reply_to,type);
}
//-----------------------------------------------------------------
const char *mailEnvelope::messageTo(enum AddrOutput type)
{
	return emailAddrAsText(my_env->to,type);
}
//-----------------------------------------------------------------
const char *mailEnvelope::messageCC(enum AddrOutput type)
{
	return emailAddrAsText(my_env->cc,type);
}
//-----------------------------------------------------------------
const char *mailEnvelope::messageBCC(enum AddrOutput type)
{
	return emailAddrAsText(my_env->bcc,type);
}
//-----------------------------------------------------------------
const char *mailEnvelope::messageSubject()
{
	if(my_env->subject)
	return my_env->subject;
	return "";
}
//-----------------------------------------------------------------
const char *mailEnvelope::messageNewsgroups()
{
	return my_env->newsgroups;
}
//-----------------------------------------------------------------
const char *mailEnvelope::messageFollowupTo()
{
	return my_env->followup_to;
}

//-----------------------------------------------------------------
void mailEnvelope::logNotify (char *errmsg, long int flag)
{
	fprintf(stderr,"mailEnvelope : [%ld] %s\n",flag,errmsg);
}
//-----------------------------------------------------------------
const char *mailEnvelope::emailAddrAsText(ADDRESS *adr,enum AddrOutput type)
{
	static char *buffer= NULL;
	char real[256];
	char personal[256];
	char *lt;
	char *gt;
	char *sep;
	ADDRESS *tmp;
	unsigned int buf_len = 0;

	delete buffer;
	buffer = NULL;
	buffer = (char *)malloc(2048);
	buf_len = 2047;

	strcpy(buffer,"");

	if(type == textNONE && !adr) return NULL;
	if(type == textNONE && adr) return buffer;
	if(!adr) return ""; // safer for the rest of the world (instead of NULL)
	blockCallback();
	if(type == textPlain)
	{
//	    		rfc822_write_address_full (buffer,adr,NULL);
		lt = "<";
		gt = ">";
		sep = ", ";
	} else {
		lt = "&lt;";
		gt = "&gt;";
		sep = "<BR>\n";
	}
	while(adr) {
		strcpy(personal,"");
		strcpy(real,"");
		tmp = adr->next;
		adr->next = NULL;
		rfc822_write_address_full (real,adr,NULL);
		for(unsigned int i=0;i<strlen(real);i++)
		{
			if(real[i] == '\'') real[i] = '|';
		}
		adr->next = tmp;
//  	snprintf (real,sizeof(real)-1,"%s@%s",&adr->mailbox[0],&adr->host[0]);

  	if (adr->personal && *adr->personal)
		{
			snprintf(personal,sizeof(personal)-1,"%s",adr->personal);
		} else {
      snprintf (personal,sizeof(personal)-1,"%s%s@%s%s",lt,&adr->mailbox[0],&adr->host[0],gt);
		}

		switch(type) {
			case textNONE:
			case textPlain :
				strncat(buffer,real,buf_len-strlen(buffer));
				break;
			case textHTML :
				strncat(buffer,personal,buf_len-strlen(buffer));
				break;
			case textAnchor :
				strncat(buffer,"<A HREF='MAILTO:",buf_len-strlen(buffer));
				strncat(buffer,real,buf_len-strlen(buffer));
				strncat(buffer,"'>",buf_len-strlen(buffer));
				strncat(buffer,personal,buf_len-strlen(buffer));
				strncat(buffer,"</A>",buf_len-strlen(buffer));
				break;
		}

		if(adr->next) strncat(buffer,sep,buf_len-strlen(buffer));
		adr = adr->next;
		if(buf_len - strlen(buffer) < 256)
		{
			char *temp;
			buf_len = buf_len + 2048;
			temp = (char *)malloc(buf_len);
			strcpy(temp,buffer);
			delete buffer;
			buffer = temp;
		}
	}
	releaseCallback();
  return buffer;
}
//-----------------------------------------------------------------
const char *mailEnvelope::dateStr(const char *templ,bool localize)
{
	static char date[128];
	if(!templ) templ = "%a, %d %b %Y";
 	strftime(date, sizeof(date), templ, messageDate(localize));

	return date;
}
//-----------------------------------------------------------------
void mailEnvelope::extraHeader(const char *add)
{
	fs_give((void **)&my_env->remail);
	if(add && *add) my_env->remail = cpystr(add);
}

//-----------------------------------------------------------------
//-----------------------------------------------------------------
mailBody::mailBody()
{
	my_part = mail_newbody_part();
	my_part->next = NULL;
	my_body = &my_part->body;
	body_destroy = true;
//	fprintf(stderr,"mailBody() : %p\n",this);
}
//-----------------------------------------------------------------
mailBody::mailBody(PART *in_body)
{
	my_part = in_body;
	my_body = &my_part->body;
	body_destroy = false;
//	fprintf(stderr,"mailBody(PART) : %p\n",this);
}
//-----------------------------------------------------------------
mailBody::mailBody(BODY *in_body)
{
	my_body = in_body;
	my_part = NULL;
	body_destroy = false;
//	fprintf(stderr,"mailBody(BODY) : %p\n",this);
}
//-----------------------------------------------------------------
mailBody::~mailBody()
{
	if(body_destroy)
	{
		//we MUST not destroy any part but ourself!!
		my_body->nested.part = NULL;
		my_part->next = NULL;
		mail_free_body_part(&my_part);
	}
//	fprintf(stderr,"~mailBody() : %p\n",this);
}
//-----------------------------------------------------------------
bool mailBody::isEmpty()
{
	return (my_body == NULL);
}
//-----------------------------------------------------------------

mailBody *mailBody::searchPart(const char *what,char *pfx,long i)
{
  char tmp[MAILTMPLEN];
	mailBody *result = NULL;
  mailBody *part;			/* multipart doesn't have a row to itself */
	fprintf(stderr,"mailBody::searchPart(const char *%s,char *%s,long %ld)\n",what,pfx,i);
  if (bodyEType() == TypeMULTIPART) {
				/* if not first time, extend prefix */
    if (pfx) sprintf (tmp,"%s%ld.",pfx,++i);
    else tmp[0] = '\0';
    for (i = 0,part = nestedPart(); (part && !part->isEmpty()); part->nextPart())
		{
			if(!result)
	      result = part->searchPart(what,tmp,i++);
		}
		delete part;
//    for (i = 0,part = body->nested.part; part; part = part->next)
//      display_body (&part->body,tmp,i++);
  }
  else {			/* non-multipart, output oneline descriptor */
    if (!pfx) pfx = "";		/* dummy prefix if top level */
				/* encapsulated message? */
	    sprintf (tmp,"%s%ld",pfx,++i);
	    fprintf(stderr,"part is %s %s\n",tmp,bodyFullType());
			if(!strcmp(what,tmp)) result = new mailBody(my_body);
		mailMessage *aMsg = NULL;
    if ((bodyEType() == TypeMESSAGE) && !strcmp (bodySubType(),"RFC822") &&
				(aMsg = nestedMsg())) {
			part = aMsg->body();
      if (part->bodyEType() == TypeMULTIPART)
      {
	      if(!result)
	      	result = part->searchPart(what,pfx,i-1);
      } else {			/* build encapsulation prefix */
				sprintf (tmp,"%s%ld.",pfx,i);
				if(!result)
					result = part->searchPart(what,tmp,(long) 0);
      }
			delete part;
      delete aMsg;
    }
  }
	return result;
}
//-----------------------------------------------------------------
void mailBody::forceDESTROY()
{
		body_destroy = true;
}
//-----------------------------------------------------------------
const char *mailBody::bodyDecoded(unsigned long *out_len )
{
	PARTTEXT *field;
	char *out_str=NULL;
	unsigned long temp;

	fprintf(stderr,"bodyDecoded(%p)\n",my_body);

	if(!out_len) out_len = &temp;

	field = &my_body->contents;

		switch(my_body->encoding) {
			case ENCBASE64 :
				out_str = (char *)rfc822_base64(field->text.data,field->text.size,out_len);
				break;
			case ENCQUOTEDPRINTABLE :
				out_str = (char *)rfc822_qprint(field->text.data,field->text.size,out_len);
				break;
			default:
  			out_str = (char *)malloc(field->text.size+1);
  			memcpy(out_str,(char *)field->text.data,field->text.size);
  			((char *)out_str)[field->text.size] = 0x00;  // tie off the buffer
  			*out_len = field->text.size;
  			break;
				
				
//				out_str = cpystr((char *)field->text.data);
//				*out_len = field->text.size;
//				break;
		}

	return out_str;
}
//-----------------------------------------------------------------
void mailBody::bodyEncoded(const char *in_str,unsigned long in_size)
{
	unsigned long out_size=0;
	void *out_str = NULL;

	switch(my_body->encoding) {
		case ENCBASE64 :
			out_str = rfc822_binary((unsigned char *)in_str,in_size,&out_size);
//			fs_give( (void **)&in_str);
			break;
		case ENCQUOTEDPRINTABLE :
			out_str = rfc822_8bit((unsigned char *)in_str,in_size,&out_size);
//			fs_give( (void **)&in_str);
			break;
		default :
			out_str = malloc(in_size+1);
			memcpy(out_str,in_str,in_size);
			((char *)out_str)[in_size] = 0x00;  // tie off the buffer
			out_size = in_size;
			break;
	}

	if(my_body->contents.text.data  && out_str)
	{
		free (my_body->contents.text.data);
	}
	if(out_str)
	{
		my_body->contents.text.size = out_size;
		my_body->contents.text.data = (unsigned char *)out_str;
		my_body->size.bytes = out_size;
	}
}
//-----------------------------------------------------------------
void mailBody::bodyEncoded(const char *in_str)
{
	bodyEncoded(in_str,strlen(in_str));
}
//-----------------------------------------------------------------
unsigned long mailBody::bodySize()
{
	return my_body->size.bytes;
}
//-----------------------------------------------------------------
const char *mailBody::bodyMIME()
{
	return (const char *)my_body->mime.text.data;
}
//-----------------------------------------------------------------
const char *mailBody::bodyRAW()
{
	return (const char *)my_body->contents.text.data;
}
//-----------------------------------------------------------------
const char *mailBody::bodyFullType()
{
	static char *retval = NULL;
	unsigned int len;
	if(retval) fs_give((void **)&retval);
	retval = NULL;

	if(bodySubType()) {
		len = strlen(bodyType()) + strlen(bodySubType()) + 2;
		retval = (char *)malloc(len);
		snprintf(retval,len,"%s/%s",bodyType(),bodySubType());
	}

	return retval;
}

//-----------------------------------------------------------------
enum MimeBodyTypes mailBody::bodyEType()
{
	switch (my_body->type) {
		case TYPETEXT:
			return(TypeTEXT);
			break;
		case TYPEMULTIPART:
			return(TypeMULTIPART);
			break;
		case TYPEMESSAGE:
			return(TypeMESSAGE);
			break;
		case TYPEAPPLICATION:
			return(TypeAPPLICATION);
			break;
		case TYPEAUDIO:
			return(TypeAUDIO);
			break;
		case TYPEIMAGE:
			return(TypeIMAGE);
			break;
		case TYPEVIDEO:
			return(TypeVIDEO);
			break;
		case TYPEMODEL:
			return(TypeMODEL);
			break;
		default:
			return(TypeUNKNOWN);
	}
}

//-----------------------------------------------------------------
const char *mailBody::bodyType()
{
	const char *ret;

	switch (my_body->type) {
		case TYPETEXT:
			ret = "TEXT";
			break;
		case TYPEMULTIPART:
			ret = "MULTIPART";
			break;
		case TYPEMESSAGE:
			ret = "MESSAGE";
			break;
		case TYPEAPPLICATION:
			ret = "APPLICATION";
			break;
		case TYPEAUDIO:
			ret = "AUDIO";
			break;
		case TYPEIMAGE:
			ret = "IMAGE";
			break;
		case TYPEVIDEO:
			ret = "VIDEO";
			break;
		case TYPEMODEL:
			ret = "MODEL";
			break;
		default:
			ret = "UNKNOWN";
	}
	return ret;
}

//-----------------------------------------------------------------
void mailBody::setEType(enum MimeBodyTypes new_type)
{
	switch(new_type) {
		case TypeTEXT:
			my_body->type =  TYPETEXT;
			break;
		case TypeMULTIPART:
			my_body->type =  TYPEMULTIPART;
			break;
		case TypeMESSAGE:
			my_body->type =  TYPEMESSAGE;
			break;
		case TypeAPPLICATION:
			my_body->type =  TYPEAPPLICATION;
			break;
		case TypeAUDIO:
			my_body->type =  TYPEAUDIO;
			break;
		case TypeIMAGE:
			my_body->type =  TYPEIMAGE;
			break;
		case TypeVIDEO:
			my_body->type =  TYPEVIDEO;
			break;
		case TypeMODEL:
			my_body->type =  TYPEMODEL;
			break;
		case TypeUNKNOWN:
			my_body->type =  TYPEOTHER;
			break;
	}
}

//-----------------------------------------------------------------
void mailBody::setFullType(const char *in_type)
{
		char *type = ucase( cpystr(in_type) );
		char *tmp;
		tmp = strtok(type,"/");
		if(tmp) {
			setType(tmp);
			tmp = strtok(NULL,"/");
			if(tmp) setSubType(tmp);
		}
		fs_give((void **) &type );
}

//-----------------------------------------------------------------
void mailBody::setType(const char *in_type)
{
		char *type = ucase( cpystr(in_type) );
		if(!strcmp(type,"TEXT"))
		my_body->type =  TYPETEXT;
		else if(!strcmp(type,"MULTIPART"))
		my_body->type =  TYPEMULTIPART;
		else if(!strcmp(type,"MESSAGE"))
		my_body->type =  TYPEMESSAGE;
		else if(!strcmp(type,"APPLICATION"))
		my_body->type =  TYPEAPPLICATION;
		else if(!strcmp(type,"AUDIO"))
		my_body->type =  TYPEAUDIO;
		else if(!strcmp(type,"IMAGE"))
		my_body->type =  TYPEIMAGE;
		else if(!strcmp(type,"VIDEO"))
		my_body->type =  TYPEVIDEO;
		else if(!strcmp(type,"MODEL"))
		my_body->type =  TYPEMODEL;
		else
		my_body->type =  TYPEOTHER;
		fs_give((void **) &type );
}
//-----------------------------------------------------------------
const char *mailBody::bodySubType()
{
	return my_body->subtype;
}
//-----------------------------------------------------------------
void mailBody::setSubType(const char *subtype)
{
  if (my_body->subtype) fs_give ((void **) &my_body->subtype);
	my_body->subtype = ucase(cpystr(subtype));
}
//-----------------------------------------------------------------
const char *mailBody::bodyDispositionParameter(const char *what)
{
	PARAMETER *par=NULL;
	const char *ret = NULL;

	if (my_body && (par = my_body->disposition.parameter)) do
	{
		if(strcmp(par->attribute,what) == 0) ret = par->value;
	}
  while ((par = par->next));
	return ret;
}
//-----------------------------------------------------------------
void mailBody::setDispositionParameter(const char *in_par,const char* in_value)
{
	PARAMETER *par=NULL;

	if (my_body && (par = my_body->disposition.parameter)) do
	{
		if(strcmp(par->attribute,in_par) == 0) break;
	}
  while ((par = par->next));
	if(!par)
	{
		par = mail_newbody_parameter();
		par->next = my_body->disposition.parameter;
		my_body->disposition.parameter = par;
	}
	if(!par->attribute) par->attribute = cpystr(in_par);
	if(par->value) fs_give((void **)&par->value);
	par->value = cpystr(in_value);
}
//-----------------------------------------------------------------
const char *mailBody::bodyParameter(const char *what)
{
	PARAMETER *par=NULL;
	const char *ret = NULL;

	if (my_body && (par = my_body->parameter)) do
	{
		if(par->attribute && strcmp(par->attribute,what) == 0) ret = par->value;
	}
  while ((par = par->next));
	return ret;
}
//-----------------------------------------------------------------
void mailBody::setParameter(const char *in_par,const char *in_value)
{
	PARAMETER *par=NULL;
	PARAMETER *pre=NULL;
	
	if (my_body && (par = my_body->parameter)) do
	{
		if(strcmp(par->attribute,in_par) == 0) break;
		pre = par;
	}
  while ((par = par->next));
	//create a new parameter
	if(!par && in_value && *in_value)
 	{
 		fprintf(stderr,"mailBody::setParameter creating new\n");
 		par = mail_newbody_parameter();
 		par->next = my_body->parameter;
 		my_body->parameter = par;
 	}
	//fill in parameter or delete it
	if(par && in_value && *in_value)
	{
 		fprintf(stderr,"mailBody::setParameter setting value\n");
  	if(!par->attribute) par->attribute = cpystr(in_par);
  	if(par->value) fs_give((void **)&par->value);
  	par->value = cpystr(in_value);
	} else {
		if(par)
		{
			//see which one is the predecessor (maybe my_body->parameter)
			if(pre)
			{
				pre->next = par->next;
			} else {
				my_body->parameter = par->next;
			}
			par->next = NULL;
			mail_free_body_parameter(&par);
	 		fprintf(stderr,"mailBody::setParameter deleting\n");
		}
	}
}
//-----------------------------------------------------------------
const char *mailBody::bodyFilename()
{
	return bodyDispositionParameter("FILENAME");
}
//-----------------------------------------------------------------
void mailBody::setFilename(const char *value)
{
	setDispositionParameter("FILENAME",value);
}
//-----------------------------------------------------------------
const char *mailBody::bodyName()
{
	return bodyParameter("NAME");
}
//-----------------------------------------------------------------
void mailBody::setBodyName(const char *value)
{
	setParameter("NAME",value);
}
//-----------------------------------------------------------------
const char *mailBody::bodyID()
{
	return my_body->id;
}
//-----------------------------------------------------------------
void mailBody::setID(const char *id)
{
  if (my_body->id) fs_give ((void **) &my_body->id);
	my_body->id = cpystr(id);	
}
//-----------------------------------------------------------------
const char *mailBody::bodyDisposition()
{
	return my_body->disposition.type;
}
//-----------------------------------------------------------------
void mailBody::setDisposition(const char *disposition)
{
  if (my_body->disposition.type) fs_give ((void **) &my_body->disposition);
	my_body->disposition.type = cpystr(disposition);
}
//-----------------------------------------------------------------
const char *mailBody::bodyEncoding()
{
	const char *ret;

	switch( my_body->encoding) {
		case ENC7BIT :
			ret = "7BIT";
			break;
		case ENC8BIT :
			ret = "8BIT";
			break;
		case ENCBINARY :
			ret = "BINARY";
			break;
		case ENCBASE64 :
			ret = "BASE64";
			break;
		case ENCQUOTEDPRINTABLE :
			ret = "QUOTEDPRINTABLE";
			break;
		case ENCOTHER :
			ret = "UNKNOWN";
			break;
		default:
			ret = NULL;
	}
	return ret;
}
//-----------------------------------------------------------------
void mailBody::setEncoding(const char *in_encoding)
{
	char *encoding = ucase( cpystr(in_encoding));

	if(!strcmp(encoding,"7BIT"))
		my_body->encoding = ENC7BIT ;
	else if(!strcmp(encoding,"8BIT"))
		my_body->encoding = ENC8BIT ;
	else if(!strcmp(encoding,"BINARY"))
		my_body->encoding = ENCBINARY ;
	else if(!strcmp(encoding,"BASE64"))
		my_body->encoding = ENCBASE64 ;
	else if(!strcmp(encoding,"QUOTEDPRINTABLE"))
		my_body->encoding = ENCQUOTEDPRINTABLE ;
	else if(!strcmp(encoding,"QUOTED-PRINTABLE"))
		my_body->encoding = ENCQUOTEDPRINTABLE ;
	else
		my_body->encoding = ENCOTHER ;
	fs_give( (void **) &encoding);
}
//-----------------------------------------------------------------
const char *mailBody::bodyLanguage()
{
	if(my_body->language)
		return (const char *)my_body->language->text.data;
	else
		return NULL;
}
//-----------------------------------------------------------------
void mailBody::setLanguage(const char *language)
{
	STRINGLIST *stl;

	stl = mail_newstringlist();
	stl->text.data = (unsigned char *) ucase (cpystr (language));
	stl->text.size = strlen ((char *) stl->text.data);
	stl->next = my_body->language;
	my_body->language = stl;
}
//-----------------------------------------------------------------
const char *mailBody::bodyCharset()
{
	return bodyParameter("CHARSET");
}

//-----------------------------------------------------------------
void mailBody::setCharset(const char *value)
{
	setParameter("CHARSET",value);
}

//-----------------------------------------------------------------
const char *mailBody::bodyDescription()
{
	return my_body->description;
}
//-----------------------------------------------------------------
void mailBody::setDescription(const char *description)
{
  if (my_body->description) fs_give ((void **) &my_body->description);
	my_body->description = cpystr(description);
}
//-----------------------------------------------------------------
void mailBody::setSize(unsigned long in_size)
{
	my_body->size.bytes = in_size;
}
//-----------------------------------------------------------------
mailBody *mailBody::copyPart()
{
	if(!my_part)	return new mailBody(my_body);

	return new mailBody(my_part);
}
//-----------------------------------------------------------------
mailMessage *mailBody::nestedMsg()
{
	mailMessage *retval = NULL;
	if(my_body->nested.msg && my_body->nested.msg->body && my_body->nested.msg->env)
	{
	  retval = new mailMessage(my_body->nested.msg->env,my_body->nested.msg->body);
		retval->header_len = my_body->nested.msg->header.text.size;
		retval->header = (char *)my_body->nested.msg->header.text.data;
	
	}
	return retval;
}
//-----------------------------------------------------------------
mailBody *mailBody::nestedPart()
{
	if(my_body->nested.part)
	return new mailBody(my_body->nested.part);
	return NULL;
}
//-----------------------------------------------------------------
void mailBody::setNestedPart(mailBody *in_body)
{
	my_body->nested.part = in_body->my_part;
}
//-----------------------------------------------------------------
void mailBody::nextPart()
{
	if(my_part && my_part->next) {
		my_part = my_part->next;
		my_body = &my_part->body;
	} else {
		my_part = NULL;
		my_body = NULL;
	}
}
//-----------------------------------------------------------------
void mailBody::setNextPart(mailBody *in_body)
{
	my_part->next = in_body->my_part;
}

//-----------------------------------------------------------------
//-----------------------------------------------------------------
/*MESSAGE *newMESSAGE()
{
	MESSAGE *msg;
	msg = mail_newmsg();
	if(msg)
	{
		msg->env = mail_newenvelope();
		msg->body = mail_newbody();
	}
	return msg;
} */


//-----------------------------------------------------------------
//-----------------------------------------------------------------
mailMessage::mailMessage() : mailEnvelope(), mailBody()
{
	header = NULL;
	header_len = 0;
//	fprintf(stderr,"mailMessage() : %p\n",this);
}
//-----------------------------------------------------------------
mailMessage::mailMessage(ENVELOPE *env,BODY *body) : mailEnvelope(env), mailBody(body)
{
	header = NULL;
	header_len = 0;
//	fprintf(stderr,"mailMessage(ENVELOPE,BODY) : %p\n",this);
}
//-----------------------------------------------------------------
mailMessage::mailMessage(MESSAGE *msg) : mailEnvelope(msg->env), mailBody(msg->body)
{
	if(msg->header.text.size && msg->header.text.data)
	{
		header_len = msg->header.text.size;
		header = cpystr((const char *)msg->header.text.data);
	}
//	fprintf(stderr,"mailMessage(MESSAGE) : %p\n",this);
}
//-----------------------------------------------------------------
mailMessage::~mailMessage()
{
	if(header) fs_give((void **)&header);
//	fprintf(stderr,"~mailMessage : %p\n",this);
}
//-----------------------------------------------------------------
void mailMessage::fullFetch(MailStream *client,unsigned long msgno)
{
	if(client && client->blockCallback())
	{
		blocknotify_t bn = (blocknotify_t) mail_parameters (NIL,GET_BLOCKNOTIFY,NIL);
		(*bn) (PROGRESS_START,(void *)"Message");
//		mm_progress(START,0,"Message");
		(*bn) (PROGRESS_SET_MAX,(void *)bodySize());
//		mm_progress(SET_MSG,bodySize(),NULL);
		if(!header)
		{
			char *temp;
			temp = client->fetchHeaderContent(msgno,"",&header_len);
			header = (char *)malloc((header_len) + 1);
			if(header)
  		{
  			memcpy(header,temp,header_len);
  			header[header_len] = 0x00;
  		}
  	}
		mFullFetch(client,msgno,this,NULL,0);
		(*bn) (PROGRESS_STOP,(void *)"Opening");
//		mm_progress(STOP,0,NULL);
		client->releaseCallback();
	}
}
//-----------------------------------------------------------------
unsigned long mailMessage::messageSize()
{
	return 0;	
}
//-----------------------------------------------------------------
bool mailMessage::isEmpty()
{
	return (mailEnvelope::isEmpty() && mailBody::isEmpty());
}
//-----------------------------------------------------------------
mailBody *mailMessage::body()
{
	return new mailBody(my_body);
}
//-----------------------------------------------------------------
mailEnvelope *mailMessage::envelope()
{
	return new mailEnvelope(my_env);
}
//-----------------------------------------------------------------
void r822new_header (char *header,ENVELOPE *env,BODY *body)
{
	if(true == false)
	{
	} else {
		rfc822_header(header,env,body);
	}
}

void r822new_write_body_header (char **dst,BODY *body)
{
  if(body->mime.text.data && body->mime.text.size)
  {
		*dst += strlen (*dst);
		strcpy(*dst,(char *)body->mime.text.data);
		*dst += strlen (*dst)-2;
		**dst=0x00;
	} else {
		rfc822_write_body_header(dst,body);
	}
	return;
}

long r822new_output_body (BODY *body,soutr_t f,void *s)
{
  PART *part;
  PARAMETER *param;
  char *cookie = NIL;
  char tmp[MAILTMPLEN];
  char *t;
  switch (body->type) {
  case TYPEMULTIPART:		/* multipart gets special handling */

    t = (char *) body->contents.text.data;
    if (t && *t && !((*f) (s,t) && (*f) (s,"\015\012"))) return NIL;

    part = body->nested.part;	/* first body part */
				/* find cookie */
    for (param = body->parameter; param && !cookie; param = param->next)
      if (!strcmp (param->attribute,"BOUNDARY")) cookie = param->value;
    if (!cookie) cookie = "-";	/* yucky default */
    do {			/* for each part */
				/* build cookie */
      sprintf (t = tmp,"--%s\015\012",cookie);
				/* append mini-header */
      r822new_write_body_header (&t,&part->body);
      strcat (t,"\015\012");	/* write terminating blank line */
				/* output cookie, mini-header, and contents */
      if (!((*f) (s,tmp) && r822new_output_body (&part->body,f,s))) return NIL;
    } while ((part = part->next));/* until done */
				/* output trailing cookie */
    sprintf (t = tmp,"--%s--",cookie);
    break;
  default:			/* all else is text now */
    t = (char *) body->contents.text.data;
    break;
  }
				/* output final stuff */
  if (t && *t && !((*f) (s,t) && (*f) (s,"\015\012"))) return NIL;
  return LONGT;
}
long r822new (char *t,ENVELOPE *env,BODY *body,soutr_t f,void *s,
		    long ok8bit)
{
	if(env) //the headers must be in the buffer already if !env
	{
		char *remail = env->remail;
		if(remail) (*f) (s,remail);
		env->remail = NULL;
	  if (ok8bit) rfc822_encode_body_8bit (env,body);
	  else rfc822_encode_body_7bit (env,body);
	  r822new_header (t,env,body);	/* build RFC822 header */
	}
				/* output header and body */
  return (!*t || (*f) (s,t)) && (body ? r822new_output_body (body,f,s) : T);
}


//-----------------------------------------------------------------
mailMessage *mailMessage::makeReply(const char *prefix,const char *indent)
{
	char buffer[1024];

	mailMessage *nMsg = new mailMessage;

	//set recipient
	nMsg->setMessageTo( messageReplyTo() );
	if(!messageTo())
		nMsg->setMessageTo( messageFrom() );
	nMsg->my_env->in_reply_to = cpystr(my_env->message_id );

	//make new subject
	strncpy( buffer,"Re: ",sizeof(buffer) -1);
	strncat( buffer,messageSubject(),sizeof(buffer) - strlen(buffer) - 1 );
	nMsg->setSubject( buffer );

	//quote old message
	char *temp;
	temp = quoteMessage(prefix,indent);
	if(temp)
	{
		nMsg->bodyEncoded( temp );
		delete temp;
	}

	return nMsg;
}
//-----------------------------------------------------------------
mailMessage *mailMessage::makeReplyAll(const char *prefix,const char *indent)
{
	char buffer[8192];

	mailMessage *nMsg = new mailMessage;

	//build new recipients
	strcpy( buffer,"  "); // so the check wont run into any problems
	strncat( buffer,messageFrom(),sizeof(buffer) - strlen(buffer) - 1 );
	if( buffer[strlen(buffer)-1] != ' ')
	strncat( buffer,", ",sizeof(buffer) - strlen(buffer) - 1 );
	strncat( buffer,messageTo(),sizeof(buffer) - strlen(buffer) - 1 );
	if( buffer[strlen(buffer)-1] != ' ')
	strncat( buffer,", ",sizeof(buffer) - strlen(buffer) - 1 );
	strncat( buffer,messageCC(),sizeof(buffer) - strlen(buffer) - 1 );
	if( buffer[strlen(buffer)-1] != ' ')
	strncat( buffer,", ",sizeof(buffer) - strlen(buffer) - 1 );
	strncat( buffer,messageBCC(),sizeof(buffer) - strlen(buffer) - 1 );
	buffer[strlen(buffer)-2] = 0x00;	
	nMsg->setMessageTo( buffer );

	//make subject
	strncpy( buffer,"Re: ",sizeof(buffer) -1);
	strncat( buffer,messageSubject(),sizeof(buffer) - strlen(buffer) - 1 );
	nMsg->setSubject( buffer );
	nMsg->my_env->in_reply_to = cpystr(my_env->message_id );

	//quote old message
	char *temp;
	temp = quoteMessage(prefix,indent);
	if(temp)
	{
		nMsg->bodyEncoded( temp );
		delete temp;
	}

	return nMsg;
}
//-----------------------------------------------------------------
mailMessage *mailMessage::makeForwardInline(const char *prefix,const char *indent)
{
	mailMessage *nMsg = new mailMessage;

	//quote old message
	char *temp;
	temp = quoteMessage(prefix,indent);
	if(temp)
	{
		nMsg->bodyEncoded( temp );
		delete temp;
	}
	return nMsg;
}
//-----------------------------------------------------------------
char *mailMessage::quoteMessage(const char *prefix,const char *indent)
{
	char *in_buffer;
	if(my_body->type == TYPETEXT)
		in_buffer = (char *)bodyDecoded();
	else {
		mailBody *temp = new mailBody(&my_body->nested.part->body);
		in_buffer = (char *)temp->bodyDecoded();
		delete temp;
	}

	if(!in_buffer) return NULL;

	char *ptr;
	char *out_ptr;
	char *out_buffer;
	unsigned long lines = 0;
	unsigned long size = 0;
	unsigned long ind_len = 0;
	char last_char;

	ind_len = strlen(indent);
	ptr = (char *)in_buffer;
	while(*ptr)
	{
		if(*ptr == 0x0a) lines++;
		ptr++;
	}
	size = strlen(in_buffer) + lines * ind_len + strlen(prefix)+2;
	out_buffer = (char *)malloc( size+1 );
	strncpy( out_buffer, prefix, size );
	size -= strlen(out_buffer);
	strncat( out_buffer, "\x0a",size);
	size -= 2;
	ptr = (char *)in_buffer;
	out_ptr = &out_buffer[strlen(out_buffer)];
	last_char = 0x0a;
	while(*ptr && size > 0)
	{
		if(last_char == 0x0a)
		{
			strncat(out_ptr,indent,size);
			size -= ind_len;
			out_ptr += ind_len;
		}
		last_char = *ptr++;
		if(last_char == 0x0d) last_char = *ptr++;
		*out_ptr++= last_char;
		size--;
	}
	*out_ptr = 0x00;


	delete in_buffer;

	return out_buffer;
}

void cloneAddress(ADDRESS *dest,ADDRESS *src)
{
	dest->personal = cpystr(src->personal );
	dest->adl = cpystr(src->adl );
	dest->mailbox = cpystr(src->mailbox );
	dest->host = cpystr(src->host );
	dest->error = cpystr(src->error );
}

ADDRESS *cloneAddress(ADDRESS *src)
{
	ADDRESS *retval = NULL;
	ADDRESS *pred = NULL;	

	while(src)
	{
		ADDRESS *temp = mail_newaddr();
		if(!retval) retval = temp;
		if(pred) pred->next = temp;
		pred = temp;
		cloneAddress(temp,src);
		src = src->next;
	}
	return retval;
}

void cloneEnvelope(ENVELOPE *dest,ENVELOPE *src)
{
	dest->ngbogus = src->ngbogus ;
	dest->remail = cpystr(src->remail );
	dest->return_path = cloneAddress(src->return_path);
	dest->date = cpystr(src->date );
	dest->from = cloneAddress(src->from);
	dest->sender = cloneAddress(src->sender);
	dest->reply_to = cloneAddress(src->reply_to);
	dest->subject = cpystr(src->subject );
	dest->to = cloneAddress(src->to);
	dest->cc = cloneAddress(src->cc);
	dest->bcc = cloneAddress(src->bcc);
	dest->in_reply_to = cpystr(src->in_reply_to );
	dest->message_id = cpystr(src->message_id );
	dest->newsgroups = cpystr(src->newsgroups );
	dest->followup_to = cpystr(src->followup_to );
	dest->references = cpystr(src->references );
}

ENVELOPE *cloneEnvelope(ENVELOPE *src)
{
	ENVELOPE *dest = mail_newenvelope();

	cloneEnvelope(dest,src);
	return dest;
}

void cloneParameter(PARAMETER *dest,PARAMETER *src)
{
	dest->attribute = cpystr( src->attribute );
	dest->value = cpystr( src->value );
	dest->next = NULL;
}

PARAMETER *cloneParameter(PARAMETER *src)
{
	PARAMETER *retval = NULL;
	while(src)
	{
		PARAMETER *temp = mail_newbody_parameter();

		temp->next = retval;
		retval = temp;
		cloneParameter(temp,src);
		src = src->next;
	}
	return retval;
}

PART *clonePart(PART *src);
MESSAGE *cloneMessage(MESSAGE *src);

void cloneBody(BODY *dest,BODY *src)
{
	dest->type = src->type;
	dest->encoding = src->encoding;
	dest->subtype = cpystr(src->subtype );
	dest->parameter = cloneParameter( src->parameter );
	dest->id = cpystr(src->id );
	dest->description = cpystr(src->description );
	dest->disposition.type = cpystr(src->disposition.type );
	dest->disposition.parameter = cloneParameter( src->disposition.parameter );

/*	dest->mime.offset = src->mime.offset;
	dest->mime.text.size = src->mime.text.size;
	if(src->mime.text.data)
	{
		dest->mime.text.data = (unsigned char *)malloc( dest->mime.text.size + 1 );
		memcpy( dest->mime.text.data,src->mime.text.data,dest->mime.text.size);
		dest->mime.text.data[dest->mime.text.size] = 0x00;
	} */
	// we dont copy the mime headers anymore since we might
	// change the content encoding later
	
	dest->contents.offset = src->contents.offset;
	dest->contents.text.size = src->contents.text.size;
	if(src->contents.text.data)
	{
		dest->contents.text.data = (unsigned char *)malloc( dest->contents.text.size + 1 );
		memcpy( dest->contents.text.data,src->contents.text.data,dest->contents.text.size);
		dest->contents.text.data[dest->contents.text.size] = 0x00;
	}

	dest->size.bytes = src->size.bytes;
	dest->size.lines = src->size.lines;
	dest->md5 = cpystr(src->md5 );

	if(src->type == TYPEMULTIPART)
	{
		dest->nested.part = clonePart(src->nested.part);
	} else if(src->type == TYPEMESSAGE)
	{
		dest->nested.msg = cloneMessage(src->nested.msg);
	}
}

BODY *cloneBody(BODY *src)
{
	BODY *dest = mail_newbody();

	cloneBody(dest,src);
	return dest;
}

void cloneMessage(MESSAGE *dest,MESSAGE *src)
{
	dest->env = cloneEnvelope(src->env);
	dest->body = cloneBody(src->body);
}

MESSAGE *cloneMessage(MESSAGE *src)
{
	MESSAGE * dest = mail_newmsg();
	cloneMessage(dest,src);
	return dest;
}


PART *clonePart(PART *src)
{
	PART *retval = NULL;
	PART *pred = NULL;	

	while(src)
	{
		PART *temp = mail_newbody_part();
		if(!retval) retval = temp;
		if(pred) pred->next = temp;
		pred = temp;
		cloneBody(&temp->body,&src->body);
		src = src->next;
	}
	return retval;
}

//-----------------------------------------------------------------
mailMessage *mailMessage::makeForwardAttach()
{
	mailMessage *nMsg = new mailMessage;

	cloneEnvelope(nMsg->my_env,my_env);
	if(bodyEType() == TypeMULTIPART) {
		cloneBody(nMsg->my_body,my_body);
	} else {
	  nMsg->setType("multipart");
  	nMsg->setSubType("mixed");
	  nMsg->bodyEncoded("This is a Multipart-Message in MIME format.\n");

		// create bodyPart for editor text.
		mailBody *mMultiPart = new mailBody;
    mMultiPart->setType("text");
    mMultiPart->setSubType("plain");
    nMsg->setNestedPart(mMultiPart);
		mailBody *bodyPart = new mailBody;
		cloneBody(bodyPart->my_body,my_body);
		mMultiPart->setNextPart(bodyPart);

    // Since there is at least one more attachment create another bodypart
/*		mailBody *bodyPart = mMultiPart;
    for (msgPart=mAtmList.first(); msgPart; msgPart=mAtmList.next())
		{
			if(bodyPart) bodyPart->setNextPart(msgPart);
			bodyPart = msgPart;
		} */
  }
	return nMsg;
}

void mFullFetch (MailStream *my_client,unsigned long curMsg,mailBody *in_body,char *pfx,long i)
{
  char tmp[MAILTMPLEN];
	char prefix[16];
  mailBody *part;			/* multipart doesn't have a row to itself */


	if(!my_client || !in_body || in_body->isEmpty()) return;

  if (!strcmp(in_body->bodyType(),"MULTIPART")) {
				/* if not first time, extend prefix */
    if (pfx) sprintf (tmp,"%s%ld.",pfx,++i);
    else tmp[0] = '\0';
    for (i = 0,part = in_body->nestedPart(); (part && !part->isEmpty()); part->nextPart())
		{
      mFullFetch(my_client,curMsg,part,tmp,i++);
		}
		delete part;
  }
  else {			/* non-multipart, output */
    if (!pfx)
		{
			pfx = "";		/* dummy prefix if top level */
		}
		i++;

		snprintf (prefix,sizeof(prefix)-1,"%s%ld",pfx,i);
		my_client->fetchBodyContent(curMsg,prefix);
				/* encapsulated message? */
    if(	!strcmp(in_body->bodyType(), "MESSAGE") &&
				!strcmp(in_body->bodySubType(),"RFC822") )
		{
			mailMessage *aMsg = in_body->nestedMsg();
			BODY *raw = in_body->raw_body();
			mailBody *aBody = aMsg->body();
			unsigned long header_len;
			if(!raw->nested.msg->header.text.data)
			{
				raw->nested.msg->header.text.data = (unsigned char *)my_client->fetchHeaderContent(curMsg,prefix,&header_len);
			}
			fprintf(stderr,"cached %p\n",raw->nested.msg->header.text.data);
      if (!strcmp(aBody->bodyType(), "MULTIPART")) mFullFetch(my_client,curMsg,aBody,pfx,i-1);
      else {			/* build encapsulation prefix */
				sprintf (tmp,"%s%ld.",pfx,i);
				mFullFetch(my_client,curMsg,aBody,tmp,(long) 0);
      }
			delete aMsg;
			delete aBody;
    }
  }
}

unsigned long mEstimate(mailBody *in_body,char *pfx,long i)
{
  char tmp[MAILTMPLEN];
	char prefix[16];
  mailBody *part;			/* multipart doesn't have a row to itself */
	unsigned long estimate = 0;

	if(!in_body || in_body->isEmpty()) return 0;

  if (!strcmp(in_body->bodyType(),"MULTIPART")) {
				/* if not first time, extend prefix */
    if (pfx) sprintf (tmp,"%s%ld.",pfx,++i);
    else tmp[0] = '\0';
    for (i = 0,part = in_body->nestedPart(); (part && !part->isEmpty()); part->nextPart())
		{
      estimate += mEstimate(part,tmp,i++);
		}
		delete part;
  }
  else {			/* non-multipart, output */
    if (!pfx)
		{
			pfx = "";		/* dummy prefix if top level */
		}
		i++;

		snprintf (prefix,sizeof(prefix)-1,"%s%ld",pfx,i);
//action
		estimate += in_body->bodySize();
  }
	in_body->setSize(estimate);	
	return estimate;
}

long MailStream::sendMessage(mailMessage *msg,char **hostlist,unsigned long port)
{
  char line[MAILTMPLEN];
	long retval = 0;
	SENDSTREAM *my_stream=NULL;
	NETDRIVER *io_stream=NULL;

	if(port > 65535) io_stream = &pipedriver;
	else io_stream = NULL;

	if(blockCallback())
	{
		unsigned long estimate = mEstimate(msg,NULL,0);
	  rfc822out_t r822orig = (rfc822out_t) mail_parameters (NIL,GET_RFC822OUTPUT,NIL);
	  mail_parameters (NIL,SET_RFC822OUTPUT,r822new);

	  rfc822_date (line);
	  if(!msg->my_env->date)
		{
			msg->my_env->date = (char *) fs_get (1+strlen (line));
		  strcpy(msg->my_env->date,line);
		}

	  if (msg->my_env->to || msg->my_env->cc || msg->my_env->bcc) {
	    fprintf(stderr,"Sending...%ld bytes\n",estimate);
//			if(port > 65535)
//			{
//    	  if ((retval = pipe_mail (io_stream,hostlist,msg->my_env,msg->my_body))) puts ("[Ok]");
//      	else printf ("[Failed - %s]\n",my_stream->reply);
//			} else {
    	  if ((my_stream = smtp_open_full (io_stream,hostlist,"smtp",port,OP_DEBUG))) {
      	  if ((retval = smtp_mail (my_stream,"MAIL",msg->my_env,msg->my_body))) puts ("[Ok]");
        	else printf ("[Failed - %s]\n",my_stream->reply);
//  	    }
	    }
  	}
	  else if(msg->my_env->newsgroups) {
//    puts ("Posting...");
//    if (stream = nntp_open (newslist,debug)) {
//      if (retval = nntp_mail (stream,env,body)) puts ("[Ok]");
//      else printf ("[Failed - %s]\n",stream->reply);
//    }
	  } else {
			setError("No Recipient specified!");
	  }
	  if (my_stream) smtp_close (my_stream);

	  mail_parameters (NIL,SET_RFC822OUTPUT,r822orig);
		releaseCallback();
	}
	return retval;
}
//-----------------------------------------------------------------
tm *parseDate(const char *in_date,bool localize)
{
	static tm tz;
  time_t tmp;
	long int gmtoff;
		
	MESSAGECACHE selt;
	mail_parse_date(&selt,(char *)in_date);

	tz.tm_wday = 0;
	tz.tm_yday = 0;
	tz.tm_sec = selt.seconds;
	tz.tm_min = selt.minutes;
	tz.tm_hour = selt.hours;
	tz.tm_mday = selt.day;
	tz.tm_mon = selt.month-1;
	tz.tm_year = (selt.year + BASEYEAR) - 1900;
	tz.tm_gmtoff = (selt.zhours * 60 + selt.zminutes ) * 60;
	if(selt.zoccident) tz.tm_gmtoff *= -1;
	gmtoff = tz.tm_gmtoff;
	
	//tz is now in TZ given by message
	if(localize)
	{
		struct tm *tz_out;
		
		tmp = mktime( &tz );
		//tmp is now incorrect in our TZ and tz.tm_gmtoff is our offset
		tmp = tmp - gmtoff + tz.tm_gmtoff;
		//tmp is now properly adapted to our TZ
		tz_out = localtime(&tmp);
		//if we dont copy it here I always get lots of messages on Mar 15 (really)
		memcpy(&tz,tz_out,sizeof(struct tm));
	}
	return &tz;
}
