#ifndef CPP_CLIENT_H
#define CPP_CLIENT_H


#include <stdio.h>
#include <time.h>

/* fetch the header files for the 
 * standard c-client
 * the undef's at the end keep
 * our namespace a little bit cleaner
 */
#include "imap-2000/c-client/c-client.h"
#undef write //clean up
#undef NNTP

#define MAXERROR 256
#define MAXBOXLEN NETMAXHOST+NETMAXUSER+NETMAXMBX+NETMAXSRV+50

enum MimeBodyTypes {
	TypeUNKNOWN,
	TypeTEXT,
	TypeMULTIPART,
	TypeMESSAGE,
	TypeAPPLICATION,
	TypeAUDIO,
	TypeIMAGE,
	TypeVIDEO,
	TypeMODEL
};

enum mailProgress {
	mpSTART,				/* setup your progress-bar */
	mpSTOP,				/* kill your progress-bar */
	mpTIMEOUT,			/* called by the timeout handler */
	mpSET_TOTAL,			/* set total max value */
	mpSET_CURRENT,			/* set the current value */
	mpSET_DELTA,			/* relative progress */
	mpSET_STATUS		/* things that just happen */
};

class mailMessage;
class mailBody;
class mailEnvelope;

int mm_timeout(time_t,time_t);
long r822new (char *t,ENVELOPE *env,BODY *body,soutr_t f,void *s,long ok8bit);
unsigned long mEstimate(mailBody *in_body,char *pfx,long i);
tm *parseDate(const char *,bool localize=false);


class callbackFunctions {
public:
	callbackFunctions();

//here comes our part of the c-client
	virtual int timeoutHandler(time_t,time_t);
	virtual int progressNotify (enum mailProgress,unsigned long,const char *);
	virtual void searchedNotify (long unsigned int);
	virtual void existsNotify (MAILSTREAM *stream,long unsigned int);
	virtual void expungedNotify (long unsigned int);
	virtual void flagsNotify (long unsigned int);
	virtual void notifyNotify (char *,long);
	virtual void listNotify (int, char *, long int);
	virtual void lsubNotify (int, char *, long int);
	virtual void statusNotify (char *, MAILSTATUS *);
	virtual void logNotify (char *, long int);
	virtual void dlogNotify (char *);
	virtual void loginNotify (NETMBX *, char *, char *, long int);
	virtual void criticalNotify ();
	virtual void nocriticalNotify ();
	virtual void diskerrorNotify (long int, long int);
	virtual void fatalNotify (char *);
	virtual void overviewHeader(unsigned long uid,OVERVIEW *ov);

//return the last error message
	const char *getError();
//set the last error message
	void setError(const char *);
	
	bool blockCallback();	
	bool releaseCallback();	

	char lastError[MAXERROR];


//the Callback Functions are now arranged as a Stack
//so we can interrupt ourselfes for more important tasks
	void setPred(callbackFunctions *);
	callbackFunctions *getPred();

private:
	callbackFunctions *myPredecessor;
	unsigned int currentUsage;
};

class MailStream : public callbackFunctions {
public:
//constructor which accepts a standard mailbox
//specification 
	MailStream(const char *box = NULL);
	
//destructor
	virtual ~MailStream();

//set the retry value false if the user
//selects cancel anywhere during login
	void setRetry(bool);
	unsigned long numMsgs();

//set a standard mailbox as current mailbox
	void setBox(const char *,bool readOnly = false);
	const char *getBox(){ return currentBox; }
	void closeBox();
	void reOpen(bool readOnly = false);

//Append text message
	bool appendMessage(const char *,const char *,unsigned long,const char *flag=NULL,const char *date=NULL);

//Move Message
	bool moveMessage(int,const char *);

//expunge current mailbox
	void expungeBox(void);

//check if open succeded
	bool isOpen();

//get parameters
	void *getParameter(unsigned long,void *ptr=NULL);

//get a cache entry
MESSAGECACHE *getCache(unsigned long);

//perform a check
	void Check();

//get status
	long Status(const char *folder=NULL);

	void Ping();

//return the R/W status of the mailbox
	bool getReadWrite();
	bool isLocked();

//set the skip status if you want to ignore mutiple updates
//during overview fetch
	void setSkip(bool);

//get a msgno from a uid
//	unsigned long getMsgno(unsigned long);

//set the message flags
void flagMsg(const char *,const char *);
void unflagMsg(const char *,const char *);
void flagMsg(int,const char *);
void unflagMsg(int,const char *);
int getflagMsg(int);
void toggleflagMsg(int,const char *);

//fetch the overview structure on a given
//message number 
ENVELOPE *fetchStructure (long msgno,BODY **);
mailMessage *fetchStructure (long msgno);

//fetch the FROM/SUBJECT line of a message
//if you dont specify where to put it
//these will use a static buffer
const char *fetchFrom (long msgno,char *str=0,long size=0);
const char *fetchSubject (long msgno,char *str=0,long size=0);

//fetch the actual body-content of a specified part
//and put it in the body-structure
void fetchBodyContent(long msgno,const char *prefix);

//fetch the actual header-content of a specified part
//destroy the buffer after use
char *fetchHeaderContent(long msgno,const char *prefix,unsigned long *);

//fetch the full-content of a specified msg
//destroy the Field after use
void fetchMessage(long msgno,MESSAGE *msg);

//fetch the body structure of a specified part
BODY *fetchBody(long msgno,const char *prefix);

//give us an overview of the mailbox in the selected range
//calls overviewHeader for each entry
void fetchOverview(char *);

virtual void searchSimple(const char *);

//parse a date into a MESSAGECACHE structure
MESSAGECACHE *parseDate(char *);

void setUnSeen(unsigned long);
unsigned long getUnSeen(void);

void listFolders(const char *);

unsigned long uid2msgno(unsigned long uid);
unsigned long msgno2uid(unsigned long msgno);

unsigned long getTimeOut();
void setTimeOut(unsigned long);

long sendMessage(mailMessage *msg,char **hostlist,unsigned long port);

protected:
	char currentBox[MAXBOXLEN];
	MAILSTREAM *my_stream;
	bool Retry;
	bool firstTry;
	unsigned long unSeen;

public:

	virtual void notifyNotify (char *,long);
	virtual void loginNotify (NETMBX *, char *, char *, long int);
	virtual void overviewHeader(unsigned long uid,OVERVIEW *ov);
	virtual void listNotify (int, char *, long int);
	virtual void lsubNotify (int, char *, long int);
	virtual void statusNotify (char *, MAILSTATUS *);
};


//-----------------------------------------------------------------
class mailEnvelope : public callbackFunctions {
friend class MailSender;
friend class MailStream;
friend class mailMessage;
public:
	enum AddrOutput {
		textNONE,
		textPlain,
		textHTML,
		textAnchor
	};

	mailEnvelope();
	mailEnvelope(ENVELOPE *);
	virtual ~mailEnvelope();

	bool isEmpty();
	tm *messageDate(bool localize=false);
	const char *messageDateStr();
	void setMessageReturnPath(const char *in_adr);
	void setMessageDate(const char *aStr=NULL);
	void setMessageFrom(const char *in_adr);
	void setMessageSender(const char *in_adr);
	void setMessageReplyTo(const char *in_adr);
	void setMessageTo(const char *in_adr);
	void setMessageCC(const char *in_adr);
	void setMessageBCC(const char *in_adr);
	void setSubject(const char *);
	void setNewsgroups(const char *);
	void setFollowupTo(const char *);

	const char *messageReturnPath(enum AddrOutput=textPlain);
	const char *messageFrom(enum AddrOutput=textPlain);
	const char *messageSender(enum AddrOutput=textPlain);
	const char *messageReplyTo(enum AddrOutput=textPlain);
	const char *messageTo(enum AddrOutput=textPlain);
	const char *messageCC(enum AddrOutput=textPlain);
	const char *messageBCC(enum AddrOutput=textPlain);
	const char *messageSubject(void);
	const char *messageNewsgroups(void);
	const char *messageFollowupTo(void);
	const char *dateStr(const char *templ=NULL,bool localize=false);
	void extraHeader(const char *);

	virtual void logNotify (char *, long int);

	ENVELOPE *raw_env() { return my_env; }

private:
	const char *emailAddrAsText(ADDRESS *,enum AddrOutput);

protected:
	ENVELOPE *my_env;
	bool envelope_destroy;
};

//-----------------------------------------------------------------
class mailBody {
friend class MailSender;
friend class MailStream;
friend class mailMessage;
public:
	mailBody();
	mailBody(BODY *);
	mailBody(PART *);
	~mailBody();

	bool isEmpty();
	mailBody *searchPart(const char *what,char *pfx=NULL,long i=0);
	void forceDESTROY();
	const char *bodyDecoded(unsigned long *len=NULL);
	void bodyEncoded(const char *,unsigned long);
	void bodyEncoded(const char *);

	const char *bodyType();
	enum MimeBodyTypes bodyEType();
	const char *bodySubType();
	const char *bodyFullType();
	const char *bodyParameter(const char *);
	const char *bodyDispositionParameter(const char *);
	const char *bodyFilename();
	const char *bodyName();
	const char *bodyID();
	const char *bodyDisposition();
	const char *bodyEncoding();
	const char *bodyLanguage();
	const char *bodyCharset();
	const char *bodyDescription();
	unsigned long bodySize();
	const char *bodyRAW();
	const char *bodyMIME();

	void setType(const char *);
	void setEType(enum MimeBodyTypes);
	void setSubType(const char *);
  void setFullType(const char *);
	void setParameter(const char *,const char *);
	void setDispositionParameter(const char *,const char *);
	void setFilename(const char *);
	void setBodyName(const char *);
	void setID(const char *);
	void setDisposition(const char *);
	void setEncoding(const char *);
	void setLanguage(const char *);
	void setCharset(const char *);
	void setDescription(const char *);
	void setSize(unsigned long);
	mailBody *copyPart();
	mailMessage *nestedMsg();
	mailBody *nestedPart();
	void setNestedPart(mailBody *);
	void nextPart();
	void setNextPart(mailBody *);

	BODY *raw_body() { return my_body; }

protected:
	BODY *my_body;
	PART *my_part;
	bool body_destroy;
};

class mailMessage :
	public mailEnvelope, public mailBody
{
friend class MailSender;
friend class MailStream;
friend class mailBody;
public:
	mailMessage();
	mailMessage(ENVELOPE *,BODY *);
	mailMessage(MESSAGE *);
	~mailMessage();

	bool isEmpty();
	mailBody *body();
	mailEnvelope *envelope();
	mailMessage *makeReply(const char *,const char *);
	mailMessage *makeReplyAll(const char *,const char *);
	mailMessage *makeForwardInline(const char *,const char *);
	mailMessage *makeForwardAttach();

	unsigned long messageSize();
	void fullFetch(MailStream *client,unsigned long msgno);

	char *quoteMessage(const char *prefix,const char *indent);

	long sendSMTP(char **hostlist,unsigned long port);
	char *header;
protected:
	unsigned long header_len;
};

#endif












