/***************************************************************************
                          kimap.h  -  description
                             -------------------
    begin                : Sun Dec 19 15:50:22 MET 1999
    copyright            : (C) 1999 by Sven Carstens
    email                : s.carstens@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KIMAP_H
#define KIMAP_H

#include <kapp.h>
#include <ktmainwindow.h>
#include <qwidget.h>

class MailStream;
class KImapView;
class KAccel;
class NMClient;

class KImap : public KTMainWindow
{
  Q_OBJECT 
  public:
    /** construtor */
    KImap(const char *name=NULL);
    /** destructor */
    ~KImap();

	void readConfig();
	void writeConfig(bool withSync=true);
	void checkForQueuedMsg();
	void checkForQueuedMsg(const char *);
	KImapView *getView();
//	static void initPreMenuBar(class KTMainWindow *);
//	static void initPostMenuBar(class KTMainWindow *);
		
signals:
	void signalSendQueuedMessages();
	void signalMessageNext();
	void signalMessageNextUnread();
	void signalMessagePrev();
	void signalMessagePrevUnread();
	void signalMessageReply();
	void signalMessageReplyAll();
	void signalMessageForwardInline();
	void signalMessageForwardAttached();
	void signalMessageDelete();
	void signalMessageAnswer();
	void signalMessageSeen();
	void signalMessageFlag();
	void signalMessageMoveFile();
	void signalMessageMoveFolder();
	void signalMMove(int);

	void signalGroupCreate();
	void signalGroupClear();
	void signalGroupDelete();
	void signalGroupUndelete();
	void signalGroupPrint();
	void signalGroupMoveFile();
	void signalGroupMoveFolder(int);

	void signalMakeMenu(QPopupMenu *,const QObject *,const char *);
	void signalMenuText(int,char **);

 public slots:

	void slotUrlClicked(const char*, int);
	
//	void slotCheckMail();
	void slotMakeMenu(QPopupMenu *,const QObject *,const char *);
	void slotMenuText(int,char **);

	void slotAddrBook();
	void slotCompose();

	void slotMoveFolder(int);
	void slotGroupMoveFolder(int);

//	void slotExists(MailStream *,unsigned long);
  /** switch argument for slot selection by menu or toolbar ID */
  void commandCallback(int id_);
  /** switch argument for Statusbar help entries on slot selection. Add your ID's help here for toolbars and menubar entries. */
  void statusCallback(int id_);
  /** open a new application window by creating a new instance of NewmailApp */
//  void slotFileNewWindow();
  /** clears the document in the actual view to reuse it as the new document */
  void slotFolderNew();
  /** open a file and load it into the document*/
//  void slotFileOpen();
	/** opens a file from the recent files menu */
//	void slotFileOpenRecent(int id_);
  /** save a document */
//  void slotFileSave();
  /** save a document by a new filename*/
//  void slotFileSaveAs();
  /** asks for saving if the file is modified, then closes the actual file and window*/
//  void slotFileClose();
  /** print the actual file */
  void slotFilePrint();
  /** closes all open windows by calling close() on each memberList item until the list is empty, then quits the application.
  	* If queryClose() returns false because the user canceled the saveModified() dialog, the closing breaks.
  	*/
  void slotFileQuit();
  /** put the marked text/object into the clipboard and remove
    *	it from the document
    */
//  void slotEditCut();
  /** put the marked text/object into the clipboard
  	*/
  void slotPreferences();
  /** put the marked text/object into the clipboard
  	*/
//  void slotEditCopy();
  /** paste the clipboard into the document
  	*/
//  void slotEditPaste();
  /** expunge the folder contents
  	*/
  void slotFolderExpunge();
  /** status on all Folders
  	*/
//  void slotFolderStatus();
  /** toggles the toolbar
  	*/
//  void slotViewToolBar();
  /** toggles the statusbar
  	*/
//  void slotViewStatusBar();
  /** changes the statusbar contents for the standard label permanently, used to indicate current actions.
  	* @param text the text that is displayed in the statusbar
  	*/
  void slotStatusMsg(const char *text);
  /** changes the status message of the whole statusbar for two seconds, then restores the last status. This is used to display
		* statusbar messages that give information about actions for toolbar icons and menuentries.
  	* @param text the text that is displayed in the statusbar
   	*/
  void slotStatusHelpMsg(const char *text);

protected:

  KImapView *view;
	bool docked;

	virtual bool event(QEvent *);
//	virtual void timerEvent(QTimerEvent *);
	/** the configuration object of the application */
	KConfig* config;
	/** the key accelerator container */
	KAccel* key_accel;
  /** file_menu contains all items of the menubar entry "File" */
  QPopupMenu *file_menu;
	/** the ... */
	QPopupMenu *move_folder_menu;
  /** edit_menu contains all items of the menubar entry "Edit" */
//  QPopupMenu *edit_menu;
  /** folder_menu contains all items of the menubar entry "Folder" */
  QPopupMenu *folder_menu;
  /** message_menu contains all items of the menubar entry "Message" */
  QPopupMenu *message_menu;
  /** group_menu contains all items of the menubar entry "Group" */
  QPopupMenu *group_menu;
	QPopupMenu *group_folder_menu;
  /** view_menu contains all items of the menubar entry "View" */
  QPopupMenu *view_menu;
  /** help_menu contains all items of the menubar entry "Help" */
  QPopupMenu *help_menu;
  /** view is the main widget which represents your working area. The View
    * class should handle all events of the view widget.  It is kept empty so
    * you can create your view according to your application's needs by
    * changing the view class. */
//	NMClient *my_client;

protected:
 	/** save general Options like all bar positions and status as well as the geometry and the recent file list to the configuration
		* file
		*/ 	
	void saveOptions();
	/** read general Options again and initialize all variables like the recent file list
	  */
	void readOptions();
	/** initKeyAccel creates the keyboard accelerator items for the available slots and changes the menu accelerators.
		* @see KAccel
		*/
	void initKeyAccel();
  /** initMenuBar creates the menubar and inserts the menupopups as well as creating the helpMenu.
  	* @see KApplication#getHelpMenu
  	*/
  void initMenuBar();
  /** this creates the toolbars.
    */
  void initToolBar();
  /** sets up the statusbar for the main window by initialzing a statuslabel.
  	*/
  void initStatusBar();
  /** saves the window properties for each open window during session end to the session config file, including saving the currently
		* opened file by a temporary filename provided by KApplication.
   	* @see KTMainWindow#saveProperties
   	*/
  virtual void saveProperties(KConfig* );
  /** reads the session config file and restores the application's state including the last opened files and documents by reading the
		* temporary files saved by saveProperties()
   	* @see KTMainWindow#readProperties
  	*/
  virtual void readProperties(KConfig* );
  /** creates the centerwidget of the KTMainWindow instance and sets it as the view
  	*/
  void initView(const char *);

  void enableCommand(int id_);
  /** disables menuentries/toolbar items
  	*/
  void disableCommand(int id_);

private:
	class NMStatusBar *mStatusBar;
};

#endif
