/***************************************************************************
                          kimapview.cpp  -  description
                             -------------------
    begin                : Sun Dec 19 1999
    copyright            : (C) 1999 by Sven Carstens
    email                : s.carstens@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kimapview.h"
#include <qsplitter.h>
#include "nmlistfolders.h"
#include "nmfoldercontent.h"
#include "nmmailview.h"
#include "kmcomposewin.h"
#include "extern.h"
#include "nmglobalfolders.h"
#include "nmstatusbar.h"
#include <ktmainwindow.h>

KImapView::KImapView(QWidget *parent, const char *box ) : QWidget(parent)
{
	setBackgroundMode( PaletteBase );

	lowersplitter = new QSplitter(QSplitter::Vertical,this);
	uppersplitter = new QSplitter(QSplitter::Horizontal,lowersplitter);
	if(topLevelWidget() && topLevelWidget()->inherits("KTMainWindow"))
	{
		if(		  ((KTMainWindow *)(topLevelWidget()))->statusBar() == NULL
				|| !((KTMainWindow *)(topLevelWidget()))->statusBar()->inherits("NMStatusBar") )
		{
			delete ((KTMainWindow *)(topLevelWidget()))->statusBar();
			((KTMainWindow *)(topLevelWidget()))->
			setStatusBar(new NMStatusBar(topLevelWidget(),"KImap_statusbar"));
			debug("Generated new status bar for parent");
		} else {
			debug("Parent brought his own status bar %p",((KTMainWindow *)(topLevelWidget()))->statusBar());
			debug("and it is a %s",((KTMainWindow *)(topLevelWidget()))->statusBar()->className());
		}
		
	} else debug("ClassName is : %s",topLevelWidget()->className());
	
	
	if(box)
	{
		QString caption;
		caption.sprintf("%s : %s",kapp->getCaption(),box);
		folders = new NMListFolders(NULL,"Folders");
		setCaption(caption);
		debug("Showing box : %s",box);
	} else {
		folders = new NMListFolders(uppersplitter,"Folders");
		debug("Showing all");
	}	
	overview = new NMFolderContent(uppersplitter,"Folder Content");
	mailview = new NMMailView(lowersplitter,"Mail Content");
	overview->setDefaultViewer(mailview);

//signal from folder to parent
	QObject::connect( overview,SIGNAL( signalStatusMsg( const char *)),
										parent,SLOT( slotStatusMsg( const char *)) );

//signal from mailview to parent
	QObject::connect( mailview,SIGNAL( signalStatusMsg( const char *)),
										parent,SLOT( slotStatusHelpMsg( const char *)) );

//signal from this to parent
//	QObject::connect( this,SIGNAL( signalStatusMsg( const char *)),
//										parent,SLOT( slotStatusMsg( const char *)) );

//signal from folder content to parent
	QObject::connect( overview,SIGNAL( signalStatusMsg( const char *)),
										parent,SLOT( slotStatusMsg( const char *)) );

//signal from parent to folder
	QObject::connect( parent,SIGNAL( signalMessageNext( void )),
										overview,SLOT( slotMessageNext( void )) );

//signal from parent to folder
	QObject::connect( parent,SIGNAL( signalMessageNextUnread( void )),
										overview,SLOT( slotMessageNextUnread( void )) );

//signal from parent to folder
	QObject::connect( parent,SIGNAL( signalMessagePrev( void )),
										overview,SLOT( slotMessagePrev( void )) );

//signal from parent to folder
	QObject::connect( parent,SIGNAL( signalMessagePrevUnread( void )),
										overview,SLOT( slotMessagePrevUnread(void )) );

//signal from parent to mailview
	QObject::connect( parent,SIGNAL( signalMessageReply( void )),
										overview,SLOT( slotReply(void )) );

//signal from parent to mailview
	QObject::connect( parent,SIGNAL( signalMessageReplyAll( void )),
										overview,SLOT( slotReplyAll(void )) );

//signal from parent to mailview
	QObject::connect( parent,SIGNAL( signalMessageForwardInline( void )),
										overview,SLOT( slotForwardInline(void )) );

//signal from parent to mailview
	QObject::connect( parent,SIGNAL( signalMessageForwardAttached( void )),
										overview,SLOT( slotForwardAttach(void )) );

//signal from parent to folder
	QObject::connect( parent,SIGNAL( signalMessageDelete( void )),
										overview,SLOT( slotMDelete(void )) );

//signal from parent to folder
	QObject::connect( parent,SIGNAL( signalMessageFlag( void )),
										overview,SLOT( slotMFlag(void )) );

//signal from parent to folder
	QObject::connect( parent,SIGNAL( signalMessageAnswer( void )),
										overview,SLOT( slotMAnswered(void )) );

//signal from parent to folder
	QObject::connect( parent,SIGNAL( signalMessageSeen( void )),
										overview,SLOT( slotMSeen(void )) );

//signal from parent to folder
//	QObject::connect( parent,SIGNAL( signalMessageMoveFile( void )),
//										overview,SLOT( slotMMove(void )) );

//signal from parent to folder
	QObject::connect( parent,SIGNAL( signalGroupCreate( void )),
										overview,SLOT( slotGCreate(void )) );

//signal from parent to folder
	QObject::connect( parent,SIGNAL( signalGroupClear( void )),
										overview,SLOT( slotGClear(void )) );

//signal from parent to folder
	QObject::connect( parent,SIGNAL( signalGroupPrint( void )),
										overview,SLOT( slotGPrint(void )) );

//signal from parent to folder
	QObject::connect( parent,SIGNAL( signalGroupDelete( void )),
										overview,SLOT( slotGDelete(void )) );

//signal from parent to folder
	QObject::connect( parent,SIGNAL( signalGroupUndelete( void )),
										overview,SLOT( slotGUndelete(void )) );

//signal from parent to folder
//	QObject::connect( parent,SIGNAL( signalGroupMoveFile( void )),
//										overview,SLOT( slotGMoveFile(void )) );

//signal from parent to folder
	QObject::connect( parent,SIGNAL( signalGroupMoveFolder( int )),
										overview,SLOT( slotGMoveFolder(int )) );

//signal from parent to folder
	QObject::connect( parent,SIGNAL( signalMMove( int )),
										overview,SLOT( slotMMove( int )) );

	if(box && *box)
	{
		debug("Showing box : %s",box);
//		if(overview) overview->slotFolder(box);
//		emit signalFolder(client,box);
	} else {
		debug("Showing all");
		folders->setFocus();
	}	

}

KImapView::~KImapView(){
	QObject *parent = this->parent();
	
	QObject::disconnect(overview);
	QObject::disconnect(mailview);
	
	if(gFolders) gFolders->slotSignOff(this);
	if(overview) overview->slotFolder(-1);
}

void KImapView::readConfig()
{
	if(mailview) mailview->readConfig();
	if(overview) overview->readConfig();
}

void KImapView::writeConfig(bool withSync)
{
	if(mailview) mailview->writeConfig(withSync);
	if(overview) overview->writeConfig(withSync);
}

void KImapView::reOpen()
{
	if(overview) overview->reOpen();
}
//-----------------------------------------------------------------------------
void KImapView::slotFolderUp()
{
	folders->scrollUp();
}

//-----------------------------------------------------------------------------
void KImapView::slotFolderDown()
{
	folders->scrollDown();
}

//-----------------------------------------------------------------------------
void KImapView::slotFolderExpand()
{
	folders->expand();
}

//-----------------------------------------------------------------------------
void KImapView::slotFolderCollapse()
{
	folders->collapse();
}

//-----------------------------------------------------------------------------
void KImapView::slotMessageUp()
{
	overview->scrollUp();
}

//-----------------------------------------------------------------------------
void KImapView::slotMessageDown()
{
	overview->scrollDown();
}

//-----------------------------------------------------------------------------
void KImapView::slotMessageExpand()
{
	overview->expand();
}

//-----------------------------------------------------------------------------
void KImapView::slotMessageCollapse()
{
	overview->collapse();
}

void KImapView::resizeEvent( QResizeEvent * )
{
	lowersplitter->setGeometry(0,0,width(),height());
}

void KImapView::slotFolder(int aNum)
{
	if(overview) overview->slotFolder(aNum);
}

/*void KImapView::slotFolder(QListViewItem *item)
{
	const char *text = item->text(4);

	if(text)
	{
		slotFolder(text);
	}	
}*/

void KImapView::slotFolder(const char *what)
{
	if(what)
	{
		QString caption;
		
		caption.sprintf("%s : %s",kapp->getCaption(),what);
		setCaption(caption);
		
		folders->setEnabled(false);
//		if(overview) overview->slotFolder(what);
//		emit signalFolder(client,text);
		folders->setEnabled(true);
	}	
}

void KImapView::print(QPrinter* m_pPrinter)
{
	mailview->printMsg(m_pPrinter);
}

void KImapView::closeEvent(QCloseEvent *aEvent)
{
	QWidget::closeEvent(aEvent);
	debug("CLOSING FOLDER");
	if(overview) overview->slotFolder(-1);
	debug("CLOSING FOLDER");
	if(gFolders) gFolders->slotSignOff(this);
	delete this;	
}
