// kmaddrbookdlg.cpp
// Author: Stefan Taferner <taferner@kde.org>

#include "kmaddrbookdlg.h"
#include "kmaddrbook.h"
#include "extern.h"
#include <assert.h>
#include <kapp.h>
#include <kfiledialog.h>
#include <kmsgbox.h>

//-----------------------------------------------------------------------------
KMAddrBookSelDlg::KMAddrBookSelDlg(KMAddrBook* aAddrBook, const char* aCap):
  KMAddrBookSelDlgInherited(NULL, aCap, TRUE), mGrid(this, 3, 2),
  mListBox(this),
  mBtnOk(i18n("OK"),this), 
  mBtnCancel(i18n("Cancel"),this),
	mCombo(this)
{
  initMetaObject();
  setCaption(aCap ? aCap : i18n("Addressbook"));

  assert(aAddrBook != NULL);
  mAddrBook = aAddrBook;
  mAddress  = 0;

  mBtnOk.adjustSize();
  mBtnOk.setMinimumSize(mBtnOk.size());
  mBtnOk.setMaximumHeight(mBtnOk.height());
  mBtnCancel.adjustSize();
  mBtnCancel.setMinimumSize(mBtnCancel.size());
  mBtnCancel.setMaximumHeight(mBtnCancel.height());

	mCombo.insertItem(i18n("default"));
	readConfig();
	mCombo.adjustSize();
	mCombo.setMinimumSize(mCombo.size());
	mCombo.setMaximumHeight(mCombo.height());

  mGrid.addWidget(&mCombo, 0, 1);
  mGrid.addWidget(new QLabel(&mCombo,i18n("Select addressbook"),this), 0, 0);
  mGrid.addMultiCellWidget(&mListBox, 1, 1, 0, 1);
  mGrid.addWidget(&mBtnOk, 2, 0);
  mGrid.addWidget(&mBtnCancel, 2, 1);

  mGrid.setRowStretch(0,10);
  mGrid.setRowStretch(1,10);
  mGrid.setRowStretch(2,10);
  mGrid.setColStretch(0,10);
  mGrid.setColStretch(1,10);
  mGrid.setColStretch(2,10);
  mGrid.activate();

  connect(&mBtnOk, SIGNAL(clicked()), SLOT(slotOk()));
  connect(&mListBox, SIGNAL(selected(int)), SLOT(slotOk()));
  connect(&mBtnCancel, SIGNAL(clicked()), SLOT(slotCancel()));
  connect(&mCombo, SIGNAL(highlighted(int)), SLOT(slotChange(int)));

	setBook(mAddrBook);

	mListBox.setMultiSelection(true);
}


//-----------------------------------------------------------------------------
KMAddrBookSelDlg::~KMAddrBookSelDlg()
{
}

//-----------------------------------------------------------------------------
void KMAddrBookSelDlg::setBook(KMAddrBook* mAddrBook)
{
  const char* addr;

	mListBox.clear();
  for (addr=mAddrBook->first(); addr; addr=mAddrBook->next())
  {
    mListBox.insertItem(addr);
  }
	mListBox.setSelected(mListBox.currentItem(),false);
	mListBox.repaint();
}

//-----------------------------------------------------------------------------
void KMAddrBookSelDlg::readConfig(void)
{
  KConfig *config = kapp->getConfig();
	KEntryIterator *entryIterator;
	const char *data;
	char *key;

  config->setGroup("Folders");
	entryIterator = config->entryIterator("Addressbook");
	if(entryIterator) {
  	while(entryIterator->current()) {
			data = entryIterator->current()->aValue.data();
			key = (char *)entryIterator->currentKey();
			if(strcmp(key,"default") && *data)
			 mCombo.insertItem( key );
		  entryIterator->operator++();
  	}
    delete entryIterator;
  }
}


//-----------------------------------------------------------------------------
void KMAddrBookSelDlg::slotOk()
{
  int idx = mListBox.currentItem();

  if (idx>=0) mAddress = mListBox.text(idx);
  else mAddress = 0;

  accept();
}


//-----------------------------------------------------------------------------
void KMAddrBookSelDlg::slotCancel()
{
  mAddress = 0;
  reject();
}

//-----------------------------------------------------------------------------
void KMAddrBookSelDlg::slotChange(int what)
{
  KConfig *config = kapp->getConfig();

	const char *data;
	const char *key;

	key = mCombo.text(what);
	if(what == 0) key = "default";
  config->setGroup("Addressbook");
	data = config->readEntry(key);
	fprintf(stderr,"Select: [%d-%s] %s\n",what,key,data);
	KMAddrBook *mAddrBook = new KMAddrBook();
	mAddrBook->load(data);
	setBook(mAddrBook);
	delete mAddrBook;
}


//=============================================================================
//
//  Class KMAddrBookEditDlg
//
//=============================================================================

KMAddrBookEditDlg::KMAddrBookEditDlg(KMAddrBook* aAddrBook, const char* aCap):
  KMAddrBookEditDlgInherited(NULL, NULL, TRUE), mGrid(this, 4, 4),
  mListBox(this), mEdtAddress(this),
  mBtnOk(i18n("OK"),this), 
  mBtnCancel(i18n("Cancel"),this),
  mBtnAdd(i18n("Add"),this), 
  mBtnRemove(i18n("Remove"),this),
  mBtnBAdd(i18n("Add Book"),this),
  mBtnBRemove(i18n("Remove Book"),this),
	mCombo(this)
{
  initMetaObject();
  setCaption(aCap ? aCap : i18n("Addressbook Manager"));

  mIndex = -1;

  mEdtAddress.adjustSize();
  mEdtAddress.setMinimumSize(mEdtAddress.size());
  mBtnOk.adjustSize();
  mBtnOk.setMinimumSize(mBtnOk.size());
  mBtnCancel.adjustSize();
  mBtnCancel.setMinimumSize(mBtnCancel.size());
  mBtnAdd.adjustSize();
  mBtnAdd.setMinimumSize(mBtnAdd.size());
  mBtnRemove.adjustSize();
  mBtnRemove.setMinimumSize(mBtnRemove.size());
  mBtnBAdd.adjustSize();
  mBtnBAdd.setMinimumSize(mBtnBAdd.size());
  mBtnBRemove.adjustSize();
  mBtnBRemove.setMinimumSize(mBtnBRemove.size());

	mCombo.insertItem(i18n("default"));
	readConfig();
	mCombo.adjustSize();
	mCombo.setMinimumSize(mCombo.size());

  mGrid.addMultiCellWidget(&mCombo, 0, 0, 0, 1);
  mGrid.addWidget(&mBtnBAdd, 0, 2);
  mGrid.addWidget(&mBtnBRemove, 0, 3);
  mGrid.addMultiCellWidget(&mListBox, 1, 1, 0, 3);
  mGrid.addMultiCellWidget(&mEdtAddress, 2, 2, 0, 3);
  mGrid.addWidget(&mBtnOk, 3, 0);
  mGrid.addWidget(&mBtnCancel, 3, 1);
  mGrid.addWidget(&mBtnAdd, 3, 2);
  mGrid.addWidget(&mBtnRemove, 3, 3);

  mGrid.setRowStretch(0,0);
  mGrid.setRowStretch(1,10);
  mGrid.setRowStretch(2,0);
  mGrid.setRowStretch(3,0);
  mGrid.setColStretch(0,10);
  mGrid.setColStretch(1,10);
  mGrid.setColStretch(2,10);
  mGrid.setColStretch(3,10);
  mGrid.activate();

  connect(&mListBox, SIGNAL(highlighted(const char*)),
	  SLOT(slotLbxHighlighted(const char*)));
  connect(&mListBox, SIGNAL(selectionChanged()), SLOT(setSelected()));
  connect(&mBtnOk, SIGNAL(clicked()), SLOT(slotOk()));
  connect(&mBtnCancel, SIGNAL(clicked()), SLOT(slotCancel()));
  connect(&mBtnAdd, SIGNAL(clicked()), SLOT(slotAdd()));
  connect(&mBtnRemove, SIGNAL(clicked()), SLOT(slotRemove()));
  connect(&mBtnBAdd, SIGNAL(clicked()), SLOT(slotBookAdd()));
  connect(&mBtnBRemove, SIGNAL(clicked()), SLOT(slotBookRemove()));
  connect(&mCombo, SIGNAL(highlighted(int)), SLOT(slotChange(int)));
	mListBox.setMultiSelection(true);
	setBook(aAddrBook);

	oAddrBook = NULL;
}


//-----------------------------------------------------------------------------
KMAddrBookEditDlg::~KMAddrBookEditDlg()
{
	delete oAddrBook;
}


//-----------------------------------------------------------------------------
void KMAddrBookEditDlg::setSelected()
{
	mListBox.setSelected(mIndex,true);
}

//-----------------------------------------------------------------------------
void KMAddrBookEditDlg::slotLbxHighlighted(const char* aItem)
{
	mListBox.setSelected(mIndex,false);
  if (mIndex>=0)
    mListBox.changeItem(mEdtAddress.text(), mIndex);
  mEdtAddress.setText(aItem);
  mIndex = mListBox.currentItem();
	mListBox.setSelected(mIndex,true);
}


//-----------------------------------------------------------------------------
void KMAddrBookEditDlg::applyChanges()
{
  int idx, num;
  const char* addr = mEdtAddress.text();

  if (mIndex>=0)
    mListBox.changeItem(addr, mIndex);
  else if (addr && *addr)
    mListBox.insertItem(mEdtAddress.text(), mListBox.currentItem());

  mAddrBook->clear();
  num = mListBox.count();
  for(idx=0; idx<num; idx++)
  {
    addr = mListBox.text(idx);
    mAddrBook->insert(addr);
  }
  if(mAddrBook->store() == IO_FatalError)
    {
      KMsgBox::message(0,i18n("KImap error"),
		       i18n("Storing addressbook failed"));
    }
}

//-----------------------------------------------------------------------------
void KMAddrBookEditDlg::slotOk()
{
	applyChanges();
  accept();
}


//-----------------------------------------------------------------------------
void KMAddrBookEditDlg::slotCancel()
{
  reject();
}


//-----------------------------------------------------------------------------
void KMAddrBookEditDlg::slotBookAdd()
{
  char *addr = "# kimap addressbook file\n";
  KConfig *config = kapp->getConfig();
	QString fileName
    = kapp->localkdedir()+"/share/apps/kimap/";

  fileName = KFileDialog::getSaveFileName(fileName.data(), "*", this);
  if(fileName.isEmpty()) return;



  QFile file(fileName);


  //assert(fname != NULL);
  if(!fileName)
    return;

  if (!file.open(IO_ReadWrite|IO_Truncate)) return;

  file.writeBlock(addr,strlen(addr));
  file.close();


	QString key;
	key = fileName.right(fileName.length() - fileName.findRev("/")-1);
  config->setGroup("Addressbook");
	config->writeEntry(key,fileName);
	mCombo.insertItem( key );

	mCombo.setCurrentItem( mCombo.count()-1);
  slotChange(mCombo.count()-1);
}

//-----------------------------------------------------------------------------
void KMAddrBookEditDlg::slotBookRemove()
{
  KConfig *config = kapp->getConfig();
	QString key = mCombo.currentText();
	mCombo.removeItem(mCombo.currentItem());
  config->setGroup("Addressbook");

	if(QFile::remove( config->readEntry(key) )) config->writeEntry(key,"");
}

//-----------------------------------------------------------------------------
void KMAddrBookEditDlg::slotAdd()
{
  const char* addr = mEdtAddress.text();

  if (!addr || !*addr) return;
  mIndex = -1;
  mListBox.insertItem(addr, mListBox.currentItem());
}


//-----------------------------------------------------------------------------
void KMAddrBookEditDlg::slotRemove()
{
  int idx = mListBox.currentItem();
  mIndex = -1;
  if (idx >= 0) mListBox.removeItem(idx);
  if (idx >= (int)mListBox.count()) idx--;
  mListBox.setCurrentItem(idx);
}

//-----------------------------------------------------------------------------
void KMAddrBookEditDlg::setBook(KMAddrBook* aAddrBook)
{
  const char* addr;

	mAddrBook = aAddrBook;
	mListBox.clear();
  for (addr=mAddrBook->first(); addr; addr=mAddrBook->next())
  {
    mListBox.insertItem(addr);
  }
//	mListBox.clearSelection();
//	mListBox.setCurrentItem(0);
//	mListBox.repaint();
}

//-----------------------------------------------------------------------------
void KMAddrBookEditDlg::readConfig(void)
{
  KConfig *config = kapp->getConfig();
	KEntryIterator *entryIterator;
	const char *data;
	char *key;

  config->setGroup("Folders");
	entryIterator = config->entryIterator("Addressbook");
	if(entryIterator) {
  	while(entryIterator->current()) {
			data = entryIterator->current()->aValue.data();
			key = (char *)entryIterator->currentKey();
			if(strcmp(key,"default") && *data)
			mCombo.insertItem( key );
		  entryIterator->operator++();
  	}
    delete entryIterator;
  }
}

//-----------------------------------------------------------------------------
void KMAddrBookEditDlg::slotChange(int what)
{
  KConfig *config = kapp->getConfig();

	const char *data;
	const char *key;

	applyChanges();
	key = mCombo.text(what);
	delete oAddrBook;
	if(what == 0)
	{
		oAddrBook = NULL;
		setBook(addrBook);
	} else {
	  config->setGroup("Addressbook");
		data = config->readEntry(key);
		fprintf(stderr,"Select: [%d-%s] %s\n",what,key,data);
		oAddrBook = new KMAddrBook();
		oAddrBook->load(data);
		setBook(oAddrBook);
	}
	mCombo.setCurrentItem(what); // if we got an error message the combo wont show the right entry
}

//-----------------------------------------------------------------------------
