#include "kmdisplayvcard.h"
#include <qframe.h>
#include <qlabel.h>
#include <qlayout.h>
#include <klocale.h>
#include <qmultilinedit.h>
#include <qbutton.h>

KMDisplayVCard::KMDisplayVCard(VCard *vc, QWidget *parent, const char *name) : KTabCtl(parent, name) {
  _vc = vc;
  BuildInterface();
}


KMDisplayVCard::~KMDisplayVCard() {
  delete _vc;
}
 

void KMDisplayVCard::setVCard(VCard *vc) {
  if (_vc) delete _vc;
  _vc = vc;
}
  

void KMDisplayVCard::BuildInterface() {
  addTab(getFirstTab(), i18n("Name"));
  addTab(getSecondTab(), i18n("Address"));
  addTab(getThirdTab(), i18n("Misc"));
  // now add a button bar for the following:
  // [Save to Disk]     [Import to Address Book]         [Close]
  // QButton *save = new QButton(mainWidget, i18n("&Save"));
  // QButton *import = new QButton(mainWidget, i18n("&Import"));
  // QButton *close = new QButton(mainWidget, i18n("&Close"));
  
}


QWidget *KMDisplayVCard::getFirstTab() {
QFrame *page = new QFrame(this);
QGridLayout *grid = new QGridLayout(page, 7, 2);
QLabel *name;
QLabel *value;
QString tmpstr;
QList<QString> values;

  //
  // Display the Name: field
  tmpstr = _vc->getValue(VCARD_FN);
  if (tmpstr.length() == 0) {
    values = _vc->getValues(VCARD_N);
    tmpstr = "";
    if (values.count() >= 4) {
      tmpstr += *values.at(3);
      tmpstr += " ";
    }
    if (values.count() >= 2) {
      tmpstr += *values.at(1);
      tmpstr += " ";
    }
    if (values.count() >= 3) {
      tmpstr += *values.at(2);
      tmpstr += " ";
    }
    tmpstr += *values.at(0);
    if (values.count() >= 5) {
      tmpstr += " ";
      tmpstr += *values.at(4);
    }
  }
  QString nn = _vc->getValue(VCARD_NICKNAME);
  if (nn != "") {
    tmpstr += " (";
    tmpstr += nn;
    tmpstr += ")";
  }
  name = new QLabel(i18n("Name:"), page);
  value = new QLabel(tmpstr, page);
  grid->addWidget(name, 0, 0);
  grid->addWidget(value, 0, 1);

  //
  // Display the Organization: field
  tmpstr = _vc->getValue(VCARD_ORG);
  name = new QLabel(i18n("Organization:"), page);
  value = new QLabel(tmpstr, page);
  grid->addWidget(name, 1, 0);
  grid->addWidget(value, 1, 1);

  //
  // Display the Title: field
  tmpstr = _vc->getValue(VCARD_TITLE);
  name = new QLabel(i18n("Title:"), page);
  value = new QLabel(tmpstr, page);
  grid->addWidget(name, 2, 0);
  grid->addWidget(value, 2, 1);

  //
  // Display the E-Mail: field
  tmpstr = _vc->getValue(VCARD_EMAIL, VCARD_EMAIL_INTERNET);
  name = new QLabel(i18n("E-Mail:"), page);
  value = new QLabel(tmpstr, page);
  grid->addWidget(name, 3, 0);
  grid->addWidget(value, 3, 1);

  //
  // Display the URL: field
  tmpstr = _vc->getValue(VCARD_URL);
  name = new QLabel(i18n("URL:"), page);
  value = new QLabel(tmpstr, page);
  grid->addWidget(name, 4, 0);
  grid->addWidget(value, 4, 1);

  //
  // Display the Birthday: field
  tmpstr = _vc->getValue(VCARD_BDAY);
  name = new QLabel(i18n("Birthday:"), page);
  // FIXME: this needs to be formatted
  value = new QLabel(tmpstr, page);
  grid->addWidget(name, 5, 0);
  grid->addWidget(value, 5, 1);

return page;
}



#define DO_PHONENUMBER(X,Y)  thisnum = _vc->getValue(VCARD_TEL, X);    \
                             if (thisnum.length() > 0) {               \
                               grid->expand(row+2,0);                  \
                               name = new QLabel(i18n(Y), page);       \
                               name->setMinimumSize(name->sizeHint()); \
                               grid->addWidget(name, row, 0);          \
                               value = new QLabel(thisnum, page);      \
                               grid->addWidget(value, row++, 1);       \
                             }

#define RIGHTSIZE(X)	X->setMinimumSize(X->sizeHint());

QWidget *KMDisplayVCard::getSecondTab() {
QFrame *page = new QFrame(this);
QGridLayout *grid = new QGridLayout(page, 5, 2);
QLabel *name;
QLabel *value;
QString tmpstr;
QList<QString> values;

  //
  // Add the Address[es]
  name = new QLabel(i18n("Address:"), page);
	RIGHTSIZE(name)
  grid->addWidget(name, 0, 0);
  mAddrList = new QListBox(page);
//  mAddrList->setSelectionMode(QListBox::Single);
  mAddrList->setMultiSelection(false);
  grid->addWidget(mAddrList, 0, 1);
  QFrame *addrPage = new QFrame(page);
  QGridLayout *addrGrid = new QGridLayout(addrPage, 5, 4);
  name = new QLabel(i18n("PO BOX:"), addrPage); 
	RIGHTSIZE(name)
  addrGrid->addWidget(name, 0, 0);
  mAddr_PO = new QLabel("", addrPage); 
  addrGrid->addMultiCellWidget(mAddr_PO, 0, 0, 1, 3);
  name = new QLabel(i18n("Address 1:"), addrPage); 
	RIGHTSIZE(name)
  addrGrid->addWidget(name, 1, 0);
  mAddr_addr1 = new QLabel("", addrPage); 
  addrGrid->addMultiCellWidget(mAddr_addr1, 1, 1, 1, 3);
  name = new QLabel(i18n("Address 2:"), addrPage); 
	RIGHTSIZE(name)
  addrGrid->addWidget(name, 2, 0);
  mAddr_addr2 = new QLabel("", addrPage); 
  addrGrid->addMultiCellWidget(mAddr_addr2, 2, 2, 1, 3);
  name = new QLabel(i18n("City:"), addrPage); 
	RIGHTSIZE(name)
  addrGrid->addWidget(name, 3, 0);
  mAddr_city = new QLabel("", addrPage); 
  addrGrid->addWidget(mAddr_city, 3, 1);
  name = new QLabel(i18n("Province:"), addrPage); 
	RIGHTSIZE(name)
  addrGrid->addWidget(name, 3, 2);
  mAddr_prov = new QLabel("", addrPage); 
  addrGrid->addWidget(mAddr_prov, 3, 3);
  name = new QLabel(i18n("Country:"), addrPage); 
	RIGHTSIZE(name)
  addrGrid->addWidget(name, 4, 0);
  mAddr_country = new QLabel("", addrPage); 
  addrGrid->addWidget(mAddr_country, 4, 1);
  name = new QLabel(i18n("Postal Code:"), addrPage); 
	RIGHTSIZE(name)
  addrGrid->addWidget(name, 4, 2);
  mAddr_postal = new QLabel("", addrPage); 
  addrGrid->addWidget(mAddr_postal, 4, 3);
  
  // set the types that are available
  if (!_vc->getValues(VCARD_ADR).isEmpty()) {
    mAddrList->insertItem(i18n("Default")); 
    mAddresses.append(new QList<QString>(_vc->getValues(VCARD_ADR)));
  }
  if (!_vc->getValues(VCARD_ADR, VCARD_ADR_DOM).isEmpty()) {
    mAddrList->insertItem(i18n("Domestic")); 
    mAddresses.append(new QList<QString>(_vc->getValues(VCARD_ADR, VCARD_ADR_DOM)));
  }
  if (!_vc->getValues(VCARD_ADR, VCARD_ADR_INTL).isEmpty()) {
    mAddrList->insertItem(i18n("International")); 
    mAddresses.append(new QList<QString>(_vc->getValues(VCARD_ADR, VCARD_ADR_INTL)));
  }
  if (!_vc->getValues(VCARD_ADR, VCARD_ADR_POSTAL).isEmpty()) {
    mAddrList->insertItem(i18n("Postal")); 
    mAddresses.append(new QList<QString>(_vc->getValues(VCARD_ADR, VCARD_ADR_POSTAL)));
  }
  if (!_vc->getValues(VCARD_ADR, VCARD_ADR_HOME).isEmpty()) {
    mAddrList->insertItem(i18n("Home")); 
    mAddresses.append(new QList<QString>(_vc->getValues(VCARD_ADR, VCARD_ADR_HOME)));
  }
  if (!_vc->getValues(VCARD_ADR, VCARD_ADR_WORK).isEmpty()) {
    mAddrList->insertItem(i18n("Work")); 
    mAddresses.append(new QList<QString>(_vc->getValues(VCARD_ADR, VCARD_ADR_WORK)));
  }
  if (!_vc->getValues(VCARD_ADR, VCARD_ADR_PREF).isEmpty()) {
    mAddrList->insertItem(i18n("Preferred")); 
    mAddresses.append(new QList<QString>(_vc->getValues(VCARD_ADR, VCARD_ADR_PREF)));
  }

  mAddrList->setSelected(0,true);
  mAddrList->setMinimumSize(mAddrList->sizeHint());
  connect(mAddrList, SIGNAL(selectionChanged()), SLOT(slotChangeAddress()));
  slotChangeAddress();   // set the first one

	addrPage->setMinimumSize(addrPage->sizeHint());
  grid->addWidget(addrPage, 1, 1);
  name = new QLabel(i18n("Phone Numbers"), page);
	RIGHTSIZE(name)
  grid->addWidget(name, 2, 0);
  // try to add each type of phone number if we have them
  QString thisnum;
  int row = 3;
  DO_PHONENUMBER(VCARD_TEL_HOME, "Home");
  DO_PHONENUMBER(VCARD_TEL_WORK, "Work");
  DO_PHONENUMBER(VCARD_TEL_PREF, "Preferred");
  DO_PHONENUMBER(VCARD_TEL_VOICE, "Voice");
  DO_PHONENUMBER(VCARD_TEL_FAX, "Fax");
  DO_PHONENUMBER(VCARD_TEL_MSG, "Message");
  DO_PHONENUMBER(VCARD_TEL_CELL, "Cellular");
  DO_PHONENUMBER(VCARD_TEL_PAGER, "Pager");
  DO_PHONENUMBER(VCARD_TEL_BBS, "BBS");
  DO_PHONENUMBER(VCARD_TEL_MODEM, "Modem");
  DO_PHONENUMBER(VCARD_TEL_CAR, "Car");
  DO_PHONENUMBER(VCARD_TEL_ISDN, "ISDN");
  DO_PHONENUMBER(VCARD_TEL_VIDEO, "Video");
  DO_PHONENUMBER(VCARD_TEL_PCS, "PCS");
return page;
}

#undef DO_PHONENUMBER

#define ADDENTRY(X,Y)  tmpstr = _vc->getValue(X);            \
                       if (tmpstr.length() > 0) {            \
                         name = new QLabel(i18n(Y), page);   \
                         name->setMinimumSize(name->sizeHint()); \
                         value = new QLabel(tmpstr, page);   \
                         grid->addWidget(name, row, 0);      \
                         grid->addWidget(value, row++, 1);   \
                       }

QWidget *KMDisplayVCard::getThirdTab() {
QFrame *page = new QFrame(this);
QGridLayout *grid = new QGridLayout(page, 9, 2);
QLabel *name;
QLabel *value;
QString tmpstr;
QList<QString> values;
int row = 0;

  // So far we provide:
  // mailer
  // version
  // rev
  // class
  // role
  // uid
  // geo
  // timezone
  // note
  ADDENTRY(VCARD_MAILER, "Mailer:");
  ADDENTRY(VCARD_PRODID, "vCard made by:");
  ADDENTRY(VCARD_VERSION, "vCard Version:");
  ADDENTRY(VCARD_REV, "Last revised:");
  ADDENTRY(VCARD_CLASS, "Class:");
  ADDENTRY(VCARD_ROLE, "Role:");
  ADDENTRY(VCARD_UID, "UID:");
  values = _vc->getValues(VCARD_GEO);
  if (!values.isEmpty()) {
    tmpstr.sprintf("(%s, %s)", values.at(0)->data(), values.at(1)->data());
    name = new QLabel(i18n("Location:"), page);
    value = new QLabel(tmpstr, page);
    grid->addWidget(name, row, 0);
    grid->addWidget(value, row++, 1);
  }
 
  values = _vc->getValues(VCARD_TZ);
  if (!values.isEmpty()) {
    tmpstr = "";
    for (unsigned int i = 0; i < values.count(); i++) {
      tmpstr += *values.at(i);
      if (i != values.count()-1) tmpstr += ", ";
    }
    name = new QLabel(i18n("Timezone:"), page);
    value = new QLabel(tmpstr, page);
    grid->addWidget(name, row, 0);
    grid->addWidget(value, row++, 1);
  }

  name = new QLabel(i18n("Note:"), page);
  QMultiLineEdit *qtb = new QMultiLineEdit(page);
  qtb->setReadOnly(true);
  qtb->setText(_vc->getValue(VCARD_NOTE));
  qtb->setMinimumSize(qtb->sizeHint());
  grid->addWidget(name, row, 0);
  grid->addWidget(qtb, row++, 1);

return page;
}


#undef ADDENTRY


void KMDisplayVCard::slotChangeAddress() {
unsigned int c = mAddrList->currentItem();
unsigned int d = mAddresses.count();

	if(d>c);
  if (mAddresses.count() > c) {
		if(mAddresses.at(c)->at(0))
    mAddr_PO->setText(*mAddresses.at(c)->at(0));
    mAddr_PO->setMinimumSize(mAddr_PO->sizeHint());

		if(mAddresses.at(c)->at(2))
    mAddr_addr1->setText(*mAddresses.at(c)->at(2));
    mAddr_addr1->setMinimumSize(mAddr_addr1->sizeHint());

		if(mAddresses.at(c)->at(1))
    mAddr_addr2->setText(*mAddresses.at(c)->at(1));
    mAddr_addr2->setMinimumSize(mAddr_addr2->sizeHint());

		if(mAddresses.at(c)->at(3))
    mAddr_city->setText(*mAddresses.at(c)->at(3));
    mAddr_city->setMinimumSize(mAddr_city->sizeHint());

		if(mAddresses.at(c)->at(4))
    mAddr_prov->setText(*mAddresses.at(c)->at(4));
    mAddr_prov->setMinimumSize(mAddr_prov->sizeHint());

		if(mAddresses.at(c)->at(6))
	    mAddr_country->setText(*mAddresses.at(c)->at(6));
    mAddr_country->setMinimumSize(mAddr_country->sizeHint());

		if(mAddresses.at(c)->at(5))
	    mAddr_postal->setText(*mAddresses.at(c)->at(5));
    mAddr_postal->setMinimumSize(mAddr_postal->sizeHint());
  }
}

