// kmsettings.cpp

#include <qdir.h>

//#include "kmacctlocal.h"
//#include "kmacctmgr.h"
//#include "kmacctpop.h"
//#include "kmacctseldlg.h"
//#include "kmfolder.h"
//#include "kmfoldermgr.h"
//#include "kmglobal.h"
#include "kmidentity.h"
//#include "kmmainwin.h"
//#include "kmsender.h"
//#include "kmmessage.h"
#include "kpgp.h"
#include "kfontdialog.h"
#include "kfontutils.h"
//#include "kmtopwidget.h"
#include "extern.h"
#include "kimap.h"

#include <kapp.h>
#include <kfiledialog.h>
#include <ktablistbox.h>
#include <qbuttongroup.h>
#include <qfiledialog.h>
#include <qframe.h>
#include <qgroupbox.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qcheckbox.h>
#include <klocale.h>
#include <kconfig.h>

#ifdef HAVE_PATHS_H
#include <paths.h>
#endif

#ifndef _PATH_SENDMAIL
#define _PATH_SENDMAIL  "/usr/sbin/sendmail"
#endif

//------
//#include "kmsettings.moc"
#include "kmsettings.h"

//-----------------------------------------------------------------------------
KMSettings::KMSettings(QWidget *parent, const char *name) :
  QTabDialog(parent, name, TRUE)
{
  initMetaObject();

  setCaption(i18n("Settings"));
  //resize(500,600);
  setOKButton(i18n("OK"));
  setCancelButton(i18n("Cancel"));

  connect(this, SIGNAL(applyButtonPressed()), this, SLOT(doApply()));
  connect(this, SIGNAL(cancelButtonPressed()), this, SLOT(doCancel()));

  createTabIdentity(this);
  createTabNetwork(this);
  createTabAppearance(this);
  createTabComposer(this);
  createTabMisc(this);
  createTabPgp(this);
}


//-----------------------------------------------------------------------------
KMSettings::~KMSettings()
{
  debug("~KMSettings");
//  accountList->clear();
}


//-----------------------------------------------------------------------------
// Create an input field with a label and optional detail button ("...")
// The detail button is not created if detail_return is NULL.
// The argument 'label' is the label that will be left of the entry field.
// The argument 'text' is the text that will show up in the entry field.
// The whole thing is placed in the grid from row/col to the right.
static QLineEdit* createLabeledEntry(QWidget* parent, QGridLayout* grid,
				     const char* aLabel,
				     const char* aText, 
				     int gridy, int gridx,
				     QPushButton** detail_return=NULL)
{
  QLabel* label = new QLabel(parent);
  QLineEdit* edit = new QLineEdit(parent);
  QPushButton* sel;

  label->setText(aLabel);
  label->adjustSize();
  label->resize((int)label->sizeHint().width(),label->sizeHint().height() + 6);
  label->setMinimumSize(label->size());
  grid->addWidget(label, gridy, gridx++);

  if (aText) edit->setText(aText);
  edit->setMinimumSize(100, label->height()+2);
  edit->setMaximumSize(1000, label->height()+2);
  grid->addWidget(edit, gridy, gridx++);

  if (detail_return)
  {
    sel = new QPushButton("...", parent);
    sel->setFocusPolicy(QWidget::NoFocus);
    sel->setFixedSize(sel->sizeHint().width(), label->height());
    grid->addWidget(sel, gridy, gridx++);
    *detail_return = sel;
  }

  return edit;
}

//-----------------------------------------------------------------------------
// Create an combo box with a label
// The argument 'label' is the label that will be left of the entry field.
// The whole thing is placed in the grid from row/col to the right.
static QComboBox* createLabeledCombo(QWidget* parent, QGridLayout* grid,
				     const char* aLabel,
				     const char* aText,
				     int gridy, int gridx )
{
  QLabel* label = new QLabel(parent);
  QComboBox* cbox = new QComboBox(parent);

  label->setText(aLabel);
  label->adjustSize();
  label->resize((int)label->sizeHint().width(),label->sizeHint().height() + 6);
  label->setMinimumSize(label->size());
  grid->addWidget(label, gridy, gridx++);

  cbox->setMinimumSize(100, label->height()+2);
  cbox->setMaximumSize(1000, label->height()+2);
  grid->addWidget(cbox, gridy, gridx++);

  return cbox;
}

//-----------------------------------------------------------------------------
// Add a widget with a label and optional detail button ("...")
// The detail button is not created if detail_return is NULL.
// The argument 'label' is the label that will be left of the entry field.
// The whole thing is placed in the grid from row/col to the right.
static void addLabeledWidget(QWidget* parent, QGridLayout* grid,
			     const char* aLabel, QWidget* widg,
			     int gridy, int gridx,
			     QPushButton** detail_return=NULL)
{
  QLabel* label = new QLabel(parent);
  QPushButton* sel;

  label->setText(aLabel);
  label->adjustSize();
  label->resize((int)label->sizeHint().width(),label->sizeHint().height() + 6);
  label->setMinimumSize(label->size());
  grid->addWidget(label, gridy, gridx++);

  widg->setMinimumSize(100, label->height()+2);
  //widg->setMaximumSize(1000, label->height()+2);
  grid->addWidget(widg, gridy, gridx++);

  if (detail_return)
  {
    sel = new QPushButton("...", parent);
    sel->setFocusPolicy(QWidget::NoFocus);
    sel->setFixedSize(sel->sizeHint().width(), label->height());
    grid->addWidget(sel, gridy, gridx++);
    *detail_return = sel;
  }
}


//-----------------------------------------------------------------------------
QPushButton* KMSettings::createPushButton(QWidget* parent, QGridLayout* grid,
					  const char* label, 
					  int gridy, int gridx)
{
  QPushButton* button = new QPushButton(parent, label);
  button->setText(i18n(label));
  button->adjustSize();
  button->setMinimumSize(button->size());
  grid->addWidget(button, gridy, gridx);

  return button;
}


//-----------------------------------------------------------------------------
void KMSettings::createTabIdentity(QWidget* parent)
{
  QWidget* tab = new QWidget(parent);
  QGridLayout* grid;
  QBoxLayout* box = new QBoxLayout(tab, QBoxLayout::TopToBottom, 8);
  QPushButton* button;
  QButtonGroup*   bgrp;
	QGroupBox *grp;

  grp = new QGroupBox(i18n("Writing Mail"), tab);
  box->addWidget(grp);
  grid = new QGridLayout(grp, 6, 3, 20, 6);

  nameEdit = createLabeledEntry(grp, grid, i18n("Name:"),
				identity->fullName(), 0, 0);
  orgEdit = createLabeledEntry(grp, grid, i18n("Organization:"),
			       identity->organization(), 1, 0);
  emailEdit = createLabeledEntry(grp, grid, i18n("Email Address:"),
				 identity->emailAddr(), 2, 0);
  pgpEdit = createLabeledEntry(grp, grid, i18n("PGP Default Key:"),
				 identity->pgpDefaultKey(), 3, 0);
  replytoEdit = createLabeledEntry(grp, grid,
				   i18n("Reply-To Address:"),
				   identity->replyToAddr(), 4, 0);
  sigEdit = createLabeledEntry(grp, grid, i18n("Signature File:"),
			       identity->signatureFile(), 5, 0, &button);
  connect(button,SIGNAL(clicked()),this,SLOT(chooseSigFile()));

  grid->setColStretch(0,0);
  grid->setColStretch(1,1);
  grid->setColStretch(2,0);
  grid->setRowStretch(6, 100);

  addTab(tab, i18n("Identity"));
  grid->activate();

  bgrp = new QButtonGroup(i18n("Sending Mail (default)"), tab);
  box->addWidget(bgrp);
  grid = new QGridLayout(bgrp, 6, 4, 20, 4);

  sendmailRadio = new QRadioButton(bgrp);
  sendmailRadio->setMinimumSize(sendmailRadio->size());
  sendmailRadio->setText(i18n("Sendmail"));
  bgrp->insert(sendmailRadio);
  grid->addMultiCellWidget(sendmailRadio, 0, 0, 0, 3);

  sendmailLocationEdit = createLabeledEntry(bgrp, grid,
					    i18n("Location:"),
					    NULL, 1, 1, &button);
  connect(button,SIGNAL(clicked()),this,SLOT(chooseSendmailLocation()));

  smtpRadio = new QRadioButton(bgrp);
  smtpRadio->setText(i18n("SMTP"));
  smtpRadio->setMinimumSize(smtpRadio->size());
  bgrp->insert(smtpRadio);
  grid->addMultiCellWidget(smtpRadio, 2, 2, 0, 3);

  smtpServerEdit = createLabeledEntry(bgrp, grid, i18n("Server:"),
				      NULL, 3, 1);
  smtpPortEdit = createLabeledEntry(bgrp, grid, i18n("Port:"),
				    NULL, 4, 1);
  saveSend = createLabeledCombo(bgrp,grid,i18n("Save Send Messages to:"),NULL,5,1);
	saveSend->setInsertionPolicy( QComboBox::NoInsertion );
  grid->setColStretch(0,4);
  grid->setColStretch(1,1);
  grid->setColStretch(2,10);
  grid->setColStretch(3,0);
  grid->setRowStretch(5, 100);
  grid->activate();
  bgrp->adjustSize();


  KConfig *config = kapp->getConfig();
	KEntryIterator *entryIterator;
	const char *data;
	char *key;
	QString mSaveSend = identity->saveSend();

  config->setGroup("Folders");
	entryIterator = config->entryIterator("Folders");
	if(entryIterator) {
  	while(entryIterator->current()) {
			data = entryIterator->current()->aValue.data();
			key = (char *)entryIterator->currentKey();
			if(key[0] == '\\') key++;
			if(data && *data) //ignore empty entries. see writeFolders for details
			{				
				saveSend->insertItem( key);
				debug("Adding: %s (%s)",key,mSaveSend.data());
				if(!strcmp(mSaveSend.data(),data)) saveSend->setCurrentItem(saveSend->count()-1);
			}
		  entryIterator->operator++();
  	}
    delete entryIterator;
  }

  if (identity->method()==KMIdentity::smMail)
    sendmailRadio->setChecked(TRUE);
  else if (identity->method()==KMIdentity::smSMTP)
    smtpRadio->setChecked(TRUE);

  sendmailLocationEdit->setText(identity->mailer());
  smtpServerEdit->setText(identity->smtpHost());
  QString tmp;
  smtpPortEdit->setText(tmp.setNum(identity->smtpPort()));

}


//-----------------------------------------------------------------------------
void KMSettings::createTabNetwork(QWidget* parent)
{
  QWidget* tab = new QWidget(parent);
  QBoxLayout* box = new QBoxLayout(tab, QBoxLayout::TopToBottom, 4);
  QGridLayout* grid;
  QGroupBox*   grp;
//  QButtonGroup*   bgrp;
//  QPushButton* button;
//  QLabel* label;
//  KMAccount* act;
  QString str;

  //---- group: sending mail
/*  bgrp = new QButtonGroup(i18n("Sending Mail (default)"), tab);
  box->addWidget(bgrp);
  grid = new QGridLayout(bgrp, 6, 4, 20, 4);

  sendmailRadio = new QRadioButton(bgrp);
  sendmailRadio->setMinimumSize(sendmailRadio->size());
  sendmailRadio->setText(i18n("Sendmail"));
  bgrp->insert(sendmailRadio);
  grid->addMultiCellWidget(sendmailRadio, 0, 0, 0, 3);

  sendmailLocationEdit = createLabeledEntry(bgrp, grid, 
					    i18n("Location:"),
					    NULL, 1, 1, &button);
  connect(button,SIGNAL(clicked()),this,SLOT(chooseSendmailLocation()));

  smtpRadio = new QRadioButton(bgrp);
  smtpRadio->setText(i18n("SMTP"));
  smtpRadio->setMinimumSize(smtpRadio->size());
  bgrp->insert(smtpRadio);
  grid->addMultiCellWidget(smtpRadio, 2, 2, 0, 3);

  smtpServerEdit = createLabeledEntry(bgrp, grid, i18n("Server:"),
				      NULL, 3, 1);
  smtpPortEdit = createLabeledEntry(bgrp, grid, i18n("Port:"),
				    NULL, 4, 1);
  grid->setColStretch(0,4);
  grid->setColStretch(1,1);
  grid->setColStretch(2,10);
  grid->setColStretch(3,0);
  grid->setRowStretch(5, 100);
  grid->activate();
  bgrp->adjustSize();

  if (identity->method()==KMIdentity::smMail)
    sendmailRadio->setChecked(TRUE);
  else if (identity->method()==KMIdentity::smSMTP)
    smtpRadio->setChecked(TRUE);

  sendmailLocationEdit->setText(identity->mailer());
  smtpServerEdit->setText(identity->smtpHost());
  QString tmp;
  smtpPortEdit->setText(tmp.setNum(identity->smtpPort())); */


  //---- group: incoming mail
  grp = new QGroupBox(i18n("Mail Folders"), tab);
  box->addWidget(grp);
  grid = new QGridLayout(grp, 5, 2, 20, 8);

/*  label = new QLabel(grp);
  label->setText(i18n("Accounts:   (add at least one account!)"));
  label->setMinimumSize(label->size());
  grid->addMultiCellWidget(label, 0, 0, 0, 1);

  accountList = new KTabListBox(grp, "LstAccounts", 3);
  accountList->setColumn(0, i18n("Name"), 150);
  accountList->setColumn(1, i18n("Type"), 60);
  accountList->setColumn(2, i18n("Folder"), 80);
  accountList->setMinimumSize(50, 50);
  connect(accountList,SIGNAL(highlighted(int,int)),
	  this,SLOT(accountSelected(int,int)));
  connect(accountList,SIGNAL(selected(int,int)),
	  this,SLOT(modifyAccount(int,int)));
  grid->addMultiCellWidget(accountList, 1, 4, 0, 0); */

  addButton = createPushButton(grp, grid, i18n("Add..."), 1, 0);
  connect(addButton,SIGNAL(clicked()),kimap,SLOT(slotFolderNew()));

/*  modifyButton = createPushButton(grp, grid, i18n("Modify..."),2,1);
  connect(modifyButton,SIGNAL(clicked()),this,SLOT(modifyAccount2()));
  modifyButton->setEnabled(FALSE);

  removeButton = createPushButton(grp, grid, i18n("Delete"), 3, 1);
  connect(removeButton,SIGNAL(clicked()),this,SLOT(removeAccount()));
  removeButton->setEnabled(FALSE); */

  grid->setColStretch(0, 10);
  grid->setColStretch(1, 0);

  grid->setRowStretch(1, 2);
  grid->setRowStretch(4, 2);
  grid->activate();
  grp->adjustSize();

/*  accountList->clear(); */
/*  for (act=acctMgr->first(); act; act=acctMgr->next())
    accountList->insertItem(tabNetworkAcctStr(act), -1); */

  addTab(tab, i18n("Network"));
  box->addStretch(100);
  box->activate();
  tab->adjustSize();
}


//-----------------------------------------------------------------------------
void KMSettings::createTabAppearance(QWidget* parent)
{
  QWidget* tab = new QWidget(parent);
  QBoxLayout* box = new QBoxLayout(tab, QBoxLayout::TopToBottom, 4);
  QGridLayout* grid;
  QGroupBox* grp;
  KConfig* config = kapp->getConfig();
  QPushButton* btn;
  QFont fnt;

  //----- group: fonts
  grp = new QGroupBox(i18n("Fonts"), tab);
  box->addWidget(grp);
  grid = new QGridLayout(grp, 5, 4, 20, 4);

  bodyFontLabel = new QLabel(grp);
  addLabeledWidget(grp, grid, i18n("Message Body Font:"), bodyFontLabel, 
		   2, 0, &btn);
  connect(btn,SIGNAL(clicked()),this,SLOT(slotBodyFontSelect()));

  fixedFontLabel = new QLabel(grp);
  addLabeledWidget(grp, grid, i18n("Fixed Font:"), fixedFontLabel,
		   3, 0, &btn);
  connect(btn,SIGNAL(clicked()),this,SLOT(slotFixedFontSelect()));

  listFontLabel = new QLabel(grp);
  addLabeledWidget(grp, grid, i18n("Message List Font:"), listFontLabel,
		   4, 0, &btn);
  connect(btn,SIGNAL(clicked()),this,SLOT(slotListFontSelect()));

  grid->setColStretch(0,1);
  grid->setColStretch(1,10);
  grid->setColStretch(2,0);
  grid->activate();

  // set values
  config->setGroup("Fonts");

  fnt = kstrToFont(config->readEntry("body-font", "helvetica-medium-r-12"));
  bodyFontLabel->setAutoResize(TRUE);
  bodyFontLabel->setText(kfontToStr(fnt));
  bodyFontLabel->setFont(fnt);

  fnt = kstrToFont(config->readEntry("fixed-font", "helvetica-medium-r-12"));
  fixedFontLabel->setAutoResize(TRUE);
  fixedFontLabel->setText(kfontToStr(fnt));
  fixedFontLabel->setFont(fnt);

  fnt = kstrToFont(config->readEntry("list-font", "helvetica-medium-r-12"));
  listFontLabel->setAutoResize(TRUE);
  listFontLabel->setText(kfontToStr(fnt));
  listFontLabel->setFont(fnt);

  //----- group: layout
  grp = new QGroupBox(i18n("Layout"), tab);
  box->addWidget(grp);
  grid = new QGridLayout(grp, 2, 2, 20, 4);

  longFolderList = new QCheckBox(
	      i18n("Long folder list"), grp);
  longFolderList->adjustSize();
  longFolderList->setMinimumSize(longFolderList->sizeHint());
  grid->addMultiCellWidget(longFolderList, 0, 0, 0, 1);
  grid->activate();

  //----- group: preferred type
  grp = new QGroupBox(i18n("MIME Handling"), tab);
  box->addWidget(grp);
  grid = new QGridLayout(grp, 2, 2, 20, 4);

  config->setGroup("MIME");
  preferredType = createLabeledEntry(grp, grid, i18n("Preferred Type:"),
				config->readEntry("PreferredType", "text/html"), 0, 0);

  grid->activate();

  // set values
  config->setGroup("Geometry");
  longFolderList->setChecked(config->readBoolEntry("longFolderList", false));

  //----- activation
  box->addStretch(100);
  addTab(tab, i18n("Appearance"));
}


//-----------------------------------------------------------------------------
void KMSettings::createTabComposer(QWidget *parent)
{
  QWidget *tab = new QWidget(parent);
  QBoxLayout* box = new QBoxLayout(tab, QBoxLayout::TopToBottom, 4);
  QGridLayout* grid;
  QGroupBox* grp;
  QLabel* lbl;
  KConfig* config = kapp->getConfig();
  QString str;
  int i;

  //---------- group: phrases
  grp = new QGroupBox(i18n("Phrases"), tab);
  box->addWidget(grp);
  grid = new QGridLayout(grp, 7, 3, 20, 4);

  lbl = new QLabel(i18n(
        "The following placeholders are supported in the reply phrases:\n"
        "%D=date, %S=subject, %F=sender, %%=percent sign, %_=space"), grp);
  lbl->adjustSize();
  lbl->setMinimumSize(100,lbl->size().height());
  grid->setRowStretch(0,10);
  grid->addMultiCellWidget(lbl, 0, 0, 0, 2);

  phraseReplyEdit = createLabeledEntry(grp, grid, 
				       i18n("Reply to sender:"),
				       NULL, 2, 0);
  phraseReplyAllEdit = createLabeledEntry(grp, grid, 
					  i18n("Reply to all:"),
					  NULL, 3, 0);
  phraseForwardEdit = createLabeledEntry(grp, grid, 
					 i18n("Forward:"),
					 NULL, 4, 0);
  indentPrefixEdit = createLabeledEntry(grp, grid, 
					i18n("Indentation:"),
					NULL, 5, 0);
  grid->setColStretch(0,1);
  grid->setColStretch(1,10);
  grid->setColStretch(2,0);
  grid->activate();
  //grp->adjustSize();


  // set the values
  config->setGroup("Message");
  str = config->readEntry("phrase-reply");
  if (str.isEmpty()) str = i18n("On %D, you wrote:");
  phraseReplyEdit->setText(str);

  str = config->readEntry("phrase-reply-all");
  if (str.isEmpty()) str = i18n("On %D, %F wrote:");
  phraseReplyAllEdit->setText(str);

  str = config->readEntry("phrase-forward");
  if (str.isEmpty()) str = i18n("Forwarded Message");
  phraseForwardEdit->setText(str);

  indentPrefixEdit->setText(config->readEntry("indent-prefix", ">%_"));

  //---------- group appearance
  grp = new QGroupBox(i18n("Appearance"), tab);
  box->addWidget(grp);
  grid = new QGridLayout(grp, 8, 3, 20, 4);

  autoAppSignFile=new QCheckBox(
	      i18n("Automatically append signature"), grp);
  autoAppSignFile->adjustSize();
  autoAppSignFile->setMinimumSize(autoAppSignFile->sizeHint());
  grid->addMultiCellWidget(autoAppSignFile, 0, 0, 0, 1);

  pgpAutoSign=new QCheckBox(
	      i18n("Automatically sign messages using PGP"), grp);
  pgpAutoSign->adjustSize();
  pgpAutoSign->setMinimumSize(pgpAutoSign->sizeHint());
  grid->addMultiCellWidget(pgpAutoSign, 1, 1, 0, 1);

  wordWrap = new QCheckBox(i18n("Word wrap at column:"), grp);
  wordWrap->adjustSize();
  wordWrap->setMinimumSize(autoAppSignFile->sizeHint());
  grid->addWidget(wordWrap, 2, 0);

  wrapColumnEdit = new QLineEdit(grp);
  wrapColumnEdit->adjustSize();
  wrapColumnEdit->setMinimumSize(50, wrapColumnEdit->sizeHint().height());
  grid->addWidget(wrapColumnEdit, 2, 1);

  monospFont = new QCheckBox(i18n("Use monospaced font") +
			     QString(" (still broken)"), grp);
  monospFont->adjustSize();
  monospFont->setMinimumSize(monospFont->sizeHint());
  grid->addMultiCellWidget(monospFont, 3, 3, 0, 1);

  grid->setColStretch(0,1);
  grid->setColStretch(1,1);
  grid->setColStretch(2,10);
  grid->activate();

  //---------- group sending
  grp = new QGroupBox(i18n("When sending mail"), tab);
  box->addWidget(grp);
  grid = new QGridLayout(grp, 4, 3, 20, 4);
  lbl = new QLabel(i18n("Default sending:"), grp);
  lbl->setMinimumSize(lbl->sizeHint());
  grid->addWidget(lbl, 0, 0);
  sendNow = new QRadioButton(i18n("send now"), grp);
  sendNow->setMinimumSize(sendNow->sizeHint());
  connect(sendNow,SIGNAL(clicked()),SLOT(slotSendNow()));
  grid->addWidget(sendNow, 0, 1);
  sendLater = new QRadioButton(i18n("send later"), grp);
  sendLater->setMinimumSize(sendLater->sizeHint());
  connect(sendLater,SIGNAL(clicked()),SLOT(slotSendLater()));
  grid->addWidget(sendLater, 0, 2);

  lbl = new QLabel(i18n("Send messages:"), grp);
  lbl->setMinimumSize(lbl->sizeHint());
  grid->addWidget(lbl, 1, 0);
  allow8Bit = new QRadioButton(i18n("Allow 8-bit"), grp);
  allow8Bit->setMinimumSize(allow8Bit->sizeHint());
  connect(allow8Bit,SIGNAL(clicked()),SLOT(slotAllow8Bit()));
  grid->addWidget(allow8Bit, 1, 1);
  quotedPrintable = new QRadioButton(i18n(
			  "MIME Compilant (Quoted Printable)"), grp);
  quotedPrintable->setMinimumSize(quotedPrintable->sizeHint());
  connect(quotedPrintable,SIGNAL(clicked()),SLOT(slotQuotedPrintable()));
  grid->addWidget(quotedPrintable, 1, 2);
  grid->activate();

  // set values
  config->setGroup("Composer");
  autoAppSignFile->setChecked(stricmp(config->readEntry("signature"),"auto")==0);
  wordWrap->setChecked(config->readBoolEntry("word-wrap",true));
  wrapColumnEdit->setText(config->readEntry("break-at","80"));
  monospFont->setChecked(stricmp(config->readEntry("font","variable"),"fixed")==0);
  pgpAutoSign->setChecked(config->readBoolEntry("pgp-auto-sign",false));

  i = identity->sendImmediate();
  sendNow->setChecked(i);
  sendLater->setChecked(!i);

  i = identity->sendQuotedPrintable();
  allow8Bit->setChecked(!i);
  quotedPrintable->setChecked(i);

  //---------- re we g
  box->addStretch(10);
  box->activate();
 
  addTab(tab, i18n("Composer"));
}



//-----------------------------------------------------------------------------
void KMSettings::createTabMisc(QWidget *parent)
{
  QWidget *tab = new QWidget(parent);
  QBoxLayout* box = new QBoxLayout(tab, QBoxLayout::TopToBottom, 4);
  QGridLayout* grid;
  QGroupBox* grp;

  KConfig* config = kapp->getConfig();
  QString str;

  //---------- group: folders
	int countRow = 0;
  grp = new QGroupBox(i18n("Folders"), tab);
  box->addWidget(grp);
  grid = new QGridLayout(grp, 7, 3, 20, 4);

  checkMailEvery=new QLineEdit(grp);
  checkMailEvery->setMinimumSize(checkMailEvery->sizeHint());
  grid->addMultiCellWidget(checkMailEvery, countRow, countRow, 2, 2);
	QLabel *checkLabel = new QLabel(checkMailEvery,i18n("Check Mail every (minutes)"),grp);
  checkLabel->setMinimumSize(checkLabel->sizeHint());
  grid->addMultiCellWidget(checkLabel, countRow, countRow, 0, 1);
	countRow++;

  checkMailAction=new QLineEdit(grp);
  checkMailAction->setMinimumSize(checkMailAction->sizeHint());
  grid->addMultiCellWidget(checkMailAction, countRow, countRow, 2, 2);
	checkLabel = new QLabel(checkMailAction,i18n("Check Mail action"),grp);
  checkLabel->setMinimumSize(checkLabel->sizeHint());
  grid->addMultiCellWidget(checkLabel, countRow, countRow, 0, 1);
	countRow++;

  sendOnCheck = new QCheckBox(i18n("Send Mail in outbox Folder on Check"),grp);
  sendOnCheck->setMinimumSize(sendOnCheck->sizeHint());
  grid->addMultiCellWidget(sendOnCheck,countRow,countRow,0,2);
	sendOnCheck->setEnabled(false);
	countRow++;

  sendReceipts = new QCheckBox(i18n("Automatically send receive- and read confirmations"),grp);
  sendReceipts->setMinimumSize(sendReceipts->sizeHint());
  grid->addMultiCellWidget(sendReceipts, countRow, countRow, 0, 2);
	sendReceipts->setEnabled(false);
	countRow++;

  fastFetch = new QCheckBox(i18n("Fetch only unread messages from Server"),grp);
  fastFetch->setMinimumSize(fastFetch->sizeHint());
  grid->addMultiCellWidget(fastFetch, countRow, countRow, 0, 2);
	countRow++;

  fastPop = new QCheckBox(i18n("Dont fetch overview from POP-Server"),grp);
  fastPop->setMinimumSize(fastPop->sizeHint());
  grid->addMultiCellWidget(fastPop, countRow, countRow, 0, 2);
	countRow++;

  keepAlive = new QCheckBox(i18n("Keep Connection Alive"),grp);
  keepAlive->setMinimumSize(keepAlive->sizeHint());
  grid->addMultiCellWidget(keepAlive, countRow, countRow, 0, 2);
	countRow++;

  grid->activate();

  //---------- group: folders
  grp = new QGroupBox(i18n("Mail"), tab);
  box->addWidget(grp);
  grid = new QGridLayout(grp, 2, 4, 20, 4);

  localDateformat=new QLineEdit(grp);
  localDateformat->setMinimumSize(localDateformat->sizeHint());
  grid->addMultiCellWidget(localDateformat, 0, 0, 2, 2);
	checkLabel = new QLabel(localDateformat,i18n("Date format : "),grp);
  checkLabel->setMinimumSize(checkLabel->sizeHint());
  grid->addMultiCellWidget(checkLabel, 0, 0, 0, 1);

  localizeTime = new QCheckBox(i18n("Timezone adaption"),grp);
  localizeTime->setMinimumSize(localizeTime->sizeHint());
  grid->addMultiCellWidget(localizeTime, 1, 1, 0, 2);

  grid->activate();

  //---------- set values
  config->setGroup("General");
  checkMailEvery->setText(config->readEntry("check-mail-interval","0"));
  checkMailAction->setText(config->readEntry("check-mail-action",""));
  sendOnCheck->setChecked(config->readBoolEntry("sendOnCheck",false));
  sendReceipts->setChecked(config->readBoolEntry("send-receipts", true));
  fastFetch->setChecked(config->readBoolEntry("fast-fetch", true));
  fastPop->setChecked(config->readBoolEntry("fast-pop", false));
  keepAlive->setChecked(config->readBoolEntry("keep-alive", false));
  localDateformat->setText(config->readEntry("local-date-format","%c %z"));
  localizeTime->setChecked(config->readBoolEntry("localize-time", false));

  //---------- here we go
  box->addStretch(10);
  box->activate();
 
  addTab(tab, i18n("Misc"));
}

// ----------------------------------------------------------------------------
void KMSettings::createTabPgp(QWidget *parent)
{
  pgpConfig = new KpgpConfig(parent);
  addTab(pgpConfig, i18n("PGP"));
}

//-----------------------------------------------------------------------------
/*const QString KMSettings::tabNetworkAcctStr(const KMAccount* act) const
{
  QString str;
  
  str = "";
  str += act->name();
  str += "\n";
  str += act->type();
  str += "\n";
  if (act->folder()) str += act->folder()->name();

  return str;
} */


//-----------------------------------------------------------------------------
void KMSettings::slotBodyFontSelect()
{
  QFont font;
  KFontDialog dlg(this, i18n("Message Body Font"), TRUE);

  font = kstrToFont(bodyFontLabel->text());
  dlg.getFont(font);
  bodyFontLabel->setFont(font);
  bodyFontLabel->setText(kfontToStr(font));
  bodyFontLabel->adjustSize();
}

//-----------------------------------------------------------------------------
void KMSettings::slotFixedFontSelect()
{
  QFont font;
  KFontDialog dlg(this, i18n("Fixed Font"), TRUE);

  font = kstrToFont(fixedFontLabel->text());
  dlg.getFont(font);
  fixedFontLabel->setFont(font);
  fixedFontLabel->setText(kfontToStr(font));
  fixedFontLabel->adjustSize();
}


//-----------------------------------------------------------------------------
void KMSettings::slotListFontSelect()
{
  QFont font;
  KFontDialog dlg(this, i18n("Message List Font"), TRUE);

  font = kstrToFont(listFontLabel->text());
  dlg.getFont(font);
  listFontLabel->setFont(font);
  listFontLabel->setText(kfontToStr(font));
  listFontLabel->adjustSize();
}


//-----------------------------------------------------------------------------
void KMSettings::slotAllow8Bit()
{
  allow8Bit->setChecked(TRUE);
  quotedPrintable->setChecked(FALSE);
}


//-----------------------------------------------------------------------------
void KMSettings::slotQuotedPrintable()
{
  allow8Bit->setChecked(FALSE);
  quotedPrintable->setChecked(TRUE);
}


//-----------------------------------------------------------------------------
void KMSettings::slotSendNow()
{
  sendNow->setChecked(TRUE);
  sendLater->setChecked(FALSE);
}


//-----------------------------------------------------------------------------
void KMSettings::slotSendLater()
{
  sendNow->setChecked(FALSE);
  sendLater->setChecked(TRUE);
}


//-----------------------------------------------------------------------------
void KMSettings::accountSelected(int,int)
{
  modifyButton->setEnabled(TRUE);
  removeButton->setEnabled(TRUE);
}

//-----------------------------------------------------------------------------
void KMSettings::addAccount()
{
/*  KMAcctSelDlg acctSel(this, i18n("Select Account"));
  KMAccount* acct;
  KMAccountSettings* acctSettings;
  const char* acctType = NULL;

  if (!acctSel.exec()) return;

  switch(acctSel.selected())
  {
  case 0:
    acctType = "local";
    break;
  case 1:
    acctType = "pop";
    break;
  default:
    fatal("KMSettings: unsupported account type selected");
  }

  acct = acctMgr->create(acctType, i18n("Unnamed"));
  assert(acct != NULL);

  acct->init(); // fill the account fields with good default values

  acctSettings = new KMAccountSettings(this, NULL, acct);

  if (acctSettings->exec())
    accountList->insertItem(tabNetworkAcctStr(acct), -1);
  else
    acctMgr->remove(acct);

  delete acctSettings; */
}

//-----------------------------------------------------------------------------
void KMSettings::chooseSendmailLocation()
{
  KFileDialog dlg("/", "*", this, NULL, TRUE);
  dlg.setCaption(i18n("Choose Sendmail Location"));

  if (dlg.exec()) sendmailLocationEdit->setText(dlg.selectedFile());
}

//-----------------------------------------------------------------------------
void KMSettings::chooseSigFile()
{
  KFileDialog *d=new KFileDialog(QDir::homeDirPath(),"*",this,NULL,TRUE);
  d->setCaption(i18n("Choose Signature File"));
  if (d->exec()) sigEdit->setText(d->selectedFile());
  delete d;
}

//-----------------------------------------------------------------------------
void KMSettings::modifyAccount(int index,int)
{
/*  KMAccount* acct;
  KMAccountSettings* d;

  if (index < 0) return;

  acct = acctMgr->find(accountList->text(index,0));
  assert(acct != NULL);

  d = new KMAccountSettings(this, NULL, acct);
  d->exec();
  delete d;

  accountList->changeItem(tabNetworkAcctStr(acct), index);
  accountList->setCurrentItem(index); */
}

//-----------------------------------------------------------------------------
void KMSettings::modifyAccount2()
{
//  modifyAccount(accountList->currentItem(),-1);
}

//-----------------------------------------------------------------------------
void KMSettings::removeAccount()
{
/*  QString acctName;
  KMAccount* acct;
  int idx = accountList->currentItem();

  acctName = accountList->text(idx, 0);
  acct = acctMgr->find(acctName);
  if (!acct) return;

  if(!acctMgr->remove(acct))
    return;
  accountList->removeItem(idx);
  if (!accountList->count())
  {
    modifyButton->setEnabled(FALSE);
    removeButton->setEnabled(FALSE);
  }
  if (idx >= (int)accountList->count()) idx--;
  if (idx >= 0) accountList->setCurrentItem(idx);

  accountList->update(); */
}

//-----------------------------------------------------------------------------
void KMSettings::setDefaults()
{
  sigEdit->setText(QString(QDir::home().path())+"/.signature");
  sendmailRadio->setChecked(TRUE);
  sendmailLocationEdit->setText(_PATH_SENDMAIL);
  smtpRadio->setChecked(FALSE);
  smtpPortEdit->setText("25");
}

//-----------------------------------------------------------------------------
void KMSettings::doCancel()
{
  identity->readConfig();
	kimap->readConfig();
//  acctMgr->readConfig();
}

//-----------------------------------------------------------------------------
void KMSettings::doApply()
{
  KConfig* config = kapp->getConfig();

  //----- identity
  identity->setFullName(nameEdit->text());
  identity->setOrganization(orgEdit->text());
  identity->setEmailAddr(emailEdit->text());
  identity->setPgpDefaultKey(pgpEdit->text());
  identity->setReplyToAddr(replytoEdit->text());
  identity->setSignatureFile(sigEdit->text());
  identity->writeConfig(FALSE);
  
  //----- sending mail
  if (sendmailRadio->isChecked())
    identity->setMethod(KMIdentity::smMail);
  else
    identity->setMethod(KMIdentity::smSMTP);
  
  identity->setMailer(sendmailLocationEdit->text());
  identity->setSmtpHost(smtpServerEdit->text());
  identity->setSmtpPort(atoi(smtpPortEdit->text()));
  identity->setSendImmediate(sendNow->isChecked());
  identity->setSendQuotedPrintable(quotedPrintable->isChecked());

	{				
	  KConfig *config = kapp->getConfig();
	  config->setGroup("Folders");
		identity->setSaveSend(config->readEntry(saveSend->currentText()));
	}
  	
  identity->writeConfig(FALSE);
  
  //----- incoming mail
//  acctMgr->writeConfig(FALSE);

  //----- fonts
  config->setGroup("Fonts");
  config->writeEntry("body-font", bodyFontLabel->text());
  config->writeEntry("fixed-font", fixedFontLabel->text());
  config->writeEntry("list-font", listFontLabel->text());

	//----- MIME
	config->setGroup("MIME");
	config->writeEntry("PreferredType",preferredType->text());

  //----- layout
  config->setGroup("Geometry");
  config->writeEntry("longFolderList", longFolderList->isChecked());

  //----- composer phrases
  config->setGroup("Message");
  config->writeEntry("phrase-reply", phraseReplyEdit->text());
  config->writeEntry("phrase-reply-all", phraseReplyAllEdit->text());
  config->writeEntry("phrase-forward", phraseForwardEdit->text());
  config->writeEntry("indent-prefix", indentPrefixEdit->text());

  //----- composer appearance
  config->setGroup("Composer");
  config->writeEntry("signature",autoAppSignFile->isChecked()?"auto":"manual");
  config->writeEntry("word-wrap",wordWrap->isChecked());
  config->writeEntry("break-at", atoi(wrapColumnEdit->text()));
  config->writeEntry("font", monospFont->isChecked()?"fixed":"variable");
  config->writeEntry("pgp-auto-sign", pgpAutoSign->isChecked());

  //----- misc
  config->setGroup("General");
  config->writeEntry("check-mail-interval", atoi(checkMailEvery->text()));
  config->writeEntry("check-mail-action", checkMailAction->text() );
  config->writeEntry("first-start", false);
  config->writeEntry("sendOnCheck",sendOnCheck->isChecked());
  config->writeEntry("send-receipts", sendReceipts->isChecked());
  config->writeEntry("fast-fetch", fastFetch->isChecked());
  config->writeEntry("fast-pop", fastPop->isChecked());
  config->writeEntry("keep-alive", keepAlive->isChecked());
  config->writeEntry("local-date-format", localDateformat->text());
  config->writeEntry("localize-time", localizeTime->isChecked());

  //-----
  config->sync();
  pgpConfig->applySettings();

	// set the new timer values and kill the old one
	kimap->killTimers();
	if(atoi(checkMailEvery->text()))
		kimap->startTimer(atoi(checkMailEvery->text()) * 60 * 1000);

	kimap->readConfig();

//  KMTopLevelWidget::forEvery(&KMTopLevelWidget::readConfig);
}


