/***************************************************************************
                          kpasswdconfig.cpp  -  description
                             -------------------
    begin                : Sat Dec 25 1999
    copyright            : (C) 1999 by Sven Carstens
    email                : s.carstens@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kpasswdconfig.h"
#include "qpasswdfile.h"
#include <qmessagebox.h>
#include <qfileinfo.h>
#include <kapp.h>

KPasswdConfig::KPasswdConfig(const char *pFile,const char *pEncoder) : KSimpleConfig("/dev/null") {
	debug("Setting passwd file : %s",pFile);
  if( pFile )
	{
	  // the file should exist in any case
	  QFileInfo info( pFile );
	  if( !info.exists() )
		{
		  QFile file( pFile );
		  file.open( IO_WriteOnly );
		  file.close();
		}

	  // we use the global app config file to save the filename
	  filename = pFile;
	}
	encoder = pEncoder;
	openFailed = false;
	parseConfigFiles();
}

KPasswdConfig::~KPasswdConfig(){
	sync();
}

void KPasswdConfig::parseConfigFiles()
{
  QPasswdFile aFile( filename,encoder );
	if(aFile.open( IO_ReadWrite ))
	{
	  parseOneConfigFile( aFile, 0L );
	  aFile.close();
	} else openFailed = true;
}

void KPasswdConfig::setKey(const char *nKey)
{
	encoder = nKey;
}

void KPasswdConfig::sync()
{
	int ret;
	if(encoder.isEmpty()) {
		ret = QMessageBox::warning(0,i18n("Warning"),
			       i18n("There is no Pgp-key defined for\n"
				    "the encryption of your passwords.\n"
				    "Save Passwords anyway ?"),
				     i18n("Continue"), i18n("Cancel"));
  	if(ret == 1) return;
	}
	if(openFailed) {
		ret = QMessageBox::warning(0,i18n("Warning"),
			       i18n("Your Passwordfile was not read\n"
				    "on startup.Continuing will overwrite\n"
				    "your old Passwordfile."),
				     i18n("Continue"), i18n("Cancel"));
  	if(ret == 1) return;
	}
  QPasswdFile aFile( filename,encoder );
  writeConfigFile( aFile, false );
}

