/***************************************************************************
                          main.cpp  -  description
                             -------------------
    begin                : Sun Dec 19 15:50:22 MET 1999
    copyright            : (C) 1999 by Sven Carstens
    email                : s.carstens@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "cpp-client.h"
#include "kimap.h"
#include <kstdaccel.h>
#include "kmidentity.h"
#include "kpasswdconfig.h"
#include "kbusyptr.h"
#include "kmaddrbook.h"
#include "kfileio.h"
#include <unistd.h>
#include "kmcomposewin.h"
#include <assert.h>
#include "nmdocking.h"
#include "nmglobalfolders.h"
#include <kwm.h>

#ifndef _PATH_TMP
#define _PATH_TMP "/tmp/"
#endif

#ifndef _IPC_FILE
#define _IPC_FILE ".kmail"
#endif

static void writePid(bool ready);
static void checkMessage(void);

KApplication *app=NULL;
KBusyPtr *kbp=NULL;
KMAddrBook* addrBook = NULL;
KStdAccel* keys = NULL;
KMIdentity* identity = NULL;
KImap *kimap = NULL;
KPasswdConfig *passwd = NULL;
DockWidget *dock = NULL;
NMGlobalFolders *gFolders = NULL;
bool brokenPipe = false;

class KStatusBar *mStatusBar = NULL;
class QProgressBar *mProgress = NULL;
class QPushButton *mCancelButton = NULL;

const char* aboutText =
    "KImap [" VERSION "] by\n\n"
		"Sven Carstens <s.carstens@gmx.de>\n"
    "based on the work of:\n"
    "Stefan Taferner <taferner@kde.org>,\n"
    "Markus Wbben <markus.wuebben@kde.org>\n\n"
    "Lynx <lynx@topaz.hknet.com>,\n"
    "Stephan Meyer <Stephan.Meyer@pobox.com>,\n"
    "and the above authors.\n\n"
    "This program is covered by the GPL.\n\n"
    "Please send bugreports to s.carstens@gmx.de";


void cleanup()
{
	delete dock;
	dock = NULL;
	delete addrBook;
	addrBook = NULL;
	if(gFolders) gFolders->writeFolders(kapp);
	if(gFolders) gFolders->checkForQueuedMessages();
	delete gFolders;
	gFolders = NULL;
	delete kbp;
	kbp = NULL;
	delete passwd;
	passwd = NULL;
	delete identity;
	identity = NULL;
	
  char nlck[80];
  snprintf (nlck,sizeof(nlck), "%s%s%d.lck", _PATH_TMP,_IPC_FILE, getuid());
  unlink (nlck);
}

//--- Sven's pseudo IPC&locking start ---
void serverReady(bool flag)
{
  writePid(flag);
  if (flag) // are we ready now?
    checkMessage(); //check for any pending mesages
}

static void writePid (bool ready)
{
  FILE* lck;
  char nlck[80];
  sprintf (nlck, "%s%s%d.lck", _PATH_TMP, _IPC_FILE, getuid());
  unlink (nlck); // Security - in case it was a socket, link, device...
  lck = fopen (nlck, "w");
  if (!ready)
    fprintf (lck, "%d", 0-getpid());
  else
    fprintf (lck, "%d", getpid());
  fclose (lck);
}

static void checkMessage()
{
  char lf[80];
	fprintf(stderr,"Checking pending messages\n");

  sprintf (lf, "%s%s%d.msg", _PATH_TMP, _IPC_FILE, getuid());
  if (access(lf, F_OK) != 0)
  {
    debug ("No message for me");
    return;
  }
  QString cmd;
  QString delcmd;
  cmd = kFileToString(lf);
  delcmd.sprintf("%s%s%d.msg", _PATH_TMP, _IPC_FILE, getuid());
  unlink (delcmd.data()); // unlink
  //system (delcmd.data()); // delete message if any
  // find a KMMainWin

	fprintf(stderr,"Read message\n");

  if (kapp->topWidget() && kapp->topWidget()->isA("KImap"))
    kimap = (KImap *) kapp->topWidget();

	fprintf(stderr,"Found main widget %p\n",kimap);
  if (cmd.find ("show") == 0)
  {
    //printf ("show();\n");
    if (!kimap)
    {
			fprintf(stderr,"Generating widget\n");
			if(!gFolders) gFolders = new NMGlobalFolders();
			gFolders->readFolders(kapp);
      kimap = new KImap;
      kimap->show(); // thanks, Patrick!
			fprintf(stderr,"Generated widget\n");
    }
    else
      KWM::activate(kimap->winId());
  }
  else if (cmd.find ("check") == 0)
  {
    //printf ("check();\n");
    if (!kimap)
    {
			fprintf(stderr,"Generating widget\n");
			if(!gFolders) gFolders = new NMGlobalFolders();
			gFolders->readFolders(kapp);
      kimap = new KImap;
      kimap->show(); // thanks, jbb!
			fprintf(stderr,"Generated widget\n");
    }
    else
      KWM::activate(kimap->winId());

		fprintf(stderr,"Calling checkmail\n");
		gFolders->slotStatusLookup(false);
//    kimap->slotCheckMail();
  }
  else
  {
    int j;
    int i = cmd.find ("To: ", 0, true);
    if (i==-1)
      return; // not a mailto command

    QString to = "";
    QString cc = "";
    QString bcc = "";
    QString subj = "";

    j = cmd.find('\n', i);
    to = cmd.mid(i+4,j-i-4);

    i = cmd.find ("Cc: ", j, true);
    if (i!=-1)
    {
      j = cmd.find('\n', i);
      cc= cmd.mid(i+4,j-i-4);
    }

    i = cmd.find ("Bcc: ", j, true);
    if (i!=-1)
    {
      j = cmd.find('\n', i);
      bcc= cmd.mid(i+5,j-i-5);
    }
    i = cmd.find ("Subject: ", j, true);
    if (i!=-1)
    {
      j = cmd.find('\n', i);
      subj= cmd.mid(i+9,j-i-9);
    }
    //printf ("To: %s\nCc: %s\nBcc: %s\nSubject: %s\n",
    //        to.data(), cc.data(), bcc.data(), subj.data());

		fprintf(stderr,"Building message\n");
    KMComposeWin* win;
    mailMessage* msg = new mailMessage;
		fprintf(stderr,"Finished message\n");

//    msg->initHeader();
    if (!cc.isEmpty()) msg->setMessageCC(cc);
    if (!bcc.isEmpty()) msg->setMessageBCC(bcc);
    if (!subj.isEmpty()) msg->setSubject(subj);
    if (!to.isEmpty()) msg->setMessageTo(to);

		fprintf(stderr,"Generating new Composer\n");
    win = new KMComposeWin(msg);
    assert(win != NULL);
    win->show();
		fprintf(stderr,"Generated new Composer\n");
  }

}
//--- Sven's pseudo IPC&locking end ---
//-----------------------------------------------------------------------------
static void setSignalHandler(void (*handler)(int))
{
  //--- Sven's pseudo IPC&locking start ---
  signal(SIGUSR1,  handler);
  signal(SIGUSR2,  handler);
  signal(SIGPIPE,  handler);
  //--- Sven's pseudo IPC&locking end ---
}

static void signalHandler(int sigId)
{
  //--- Sven's pseudo IPC&locking start ---
  if (sigId == SIGUSR1)
  {
    fprintf(stderr, "*** KImap got message\n");
    checkMessage();
    setSignalHandler(signalHandler);
  } else if (sigId == SIGUSR2)
  {
		fprintf(stderr,"Lockfile signal\n");
    setSignalHandler(signalHandler);
  } else if (sigId == SIGPIPE)
  {
		fprintf(stderr,"Pipe signal\n");
		brokenPipe = true;
    setSignalHandler(signalHandler);
  } else {
		fprintf(stderr,"Signal %d\n",sigId);
	}
  return;
}

void version() {

 printf("%s\n",aboutText);

}
//-----------------------------------------------------------------------------
static void init(int& argc, char *argv[])
{
  QString  acctPath, foldersPath;
//  KConfig* cfg;
  //--- Sven's pseudo IPC&locking start ---
  if (!app) // because we might have constructed it before to remove KDE args
  //--- Sven's pseudo IPC&locking end ---
  app = new KApplication(argc, argv, "kimap");
  kbp = new KBusyPtr;
	if(!gFolders) gFolders = new NMGlobalFolders();
	gFolders->readFolders(app);

//  keys = new KStdAccel(cfg);

	fprintf(stderr,"Generating application directory\n");
  QDir dir;
  QString d = KApplication::localkdedir();

  d += "/share/apps";
  dir.setPath(d.data());
  if(!dir.exists()){
    dir.mkdir(d.data());
    chmod(d.data(), S_IRWXU);
  }

  d += "/kimap";
  dir.setPath(d.data());
  if(!dir.exists()){
    dir.mkdir(d.data());
    chmod(d.data(), S_IRWXU);
  }

	fprintf(stderr,"Generating accelerators\n");
  keys = new KStdAccel( app->getConfig() );
	fprintf(stderr,"Generated accelerators\n");

	fprintf(stderr,"Generating identity\n");
  identity = new KMIdentity;
	fprintf(stderr,"Generated identity\n");

  addrBook  = new KMAddrBook;
  addrBook->readConfig();
  if(addrBook->load() == IO_FatalError)
    {
      KMsgBox::message(0,i18n("KImap error"),
		       i18n("Loading addressbook failed"));
    }
	fprintf(stderr,"Generated Addressbook\n");
	fprintf(stderr,"Setting signal handler\n");

  setSignalHandler(signalHandler);
	fprintf(stderr,"Signal handler set\n");
}

int main(int argc, char *argv[])
{
	char *openfolder = NULL;
	bool mailto = false;
	bool checkNewMail = false;

    QString to = "";
    QString cc = "";
    QString bcc = "";
    QString subj = "";
		mailMessage *msg = new mailMessage();

//  keys = new KStdAccel( app->getConfig() );
//  identity = new KMIdentity;
  
  //--- Sven's pseudo IPC&locking start ---
  app=0;
  {
    int pId;

    char lf[80];
    sprintf (lf, "%s%s%d.lck", _PATH_TMP, _IPC_FILE, getuid());
    if (access (lf, F_OK) != 0)
      writePid(true); // we are server and ready
    else
    {
			fprintf(stderr,"Lock file found\n");
      FILE *lock, *msg;
      lock = fopen (lf, "r");
      fscanf (lock, "%d", &pId);
      fclose (lock);
			fprintf(stderr,"Lock file pid is %d\n",pId);
      // Check if pid is 0 - this would kill everything
      if (pId == 0)
      {
        debug ("\nAccording to %s%s%d.lck there is existing kmail/kimap", _PATH_TMP, _IPC_FILE,
               getuid());
        debug ("process with pid 0, which is wrong. Please close running kmail/kimap");
        debug ("(if any), and delete this file like this:\n rm -f %s%s*", _PATH_TMP, _IPC_FILE);
        debug ("Then restart kmail/kimap");
        exit (0);
      }
			fprintf(stderr,"Generating message file\n");
      sprintf (lf, "%s%s%d.msg", _PATH_TMP, _IPC_FILE, getuid());
      unlink (lf); // In case of socket, link...
      msg = fopen (lf, "w");
      int i;
			fprintf(stderr,"Generating application\n");
      app = new KApplication(argc, argv, "kimap"); // clear arg list
      argc--;
      argv++;

      // process args:
      QString to, cc, bcc, subj;

			fprintf(stderr,"Processing args\n");
      for (i=0; i<argc; i++)
      {
        if (strcmp(argv[i],"-s")==0)
        {
          if (i<argc-1) subj = argv[++i];
          mailto = TRUE;
        }
				else if (strcmp(argv[i],"-v")==0)
				{
				  version();
 				  exit(0);
				}
        else if (strcmp(argv[i],"-c")==0)
        {
          if (i<argc-1) cc = argv[++i];
          mailto = TRUE;
        }
        else if (strcmp(argv[i],"-b")==0)
        {
          if (i<argc-1) bcc = argv[++i];
          mailto = TRUE;
        }
        else if (strcmp(argv[i],"-check")==0)
          checkNewMail = TRUE;
        else if (argv[i][0]=='-')
        {
          warning(i18n("Unknown command line option: %s"), argv[i]);
          // unknown parameter
        }
        else
        {
          if (!to.isEmpty()) to += ", ";
          if (strncasecmp(argv[i],"mailto:",7)==0) to += argv[i]+7;
          else to += argv[i];
          mailto = TRUE;
        }
      }

			fprintf(stderr,"Writing message\n");
      if (checkNewMail)
        fprintf (msg, "check");
      else if (mailto)
      {
        if (!to.isEmpty()) fprintf (msg, "To: %s\n", to.data());
        if (!cc.isEmpty()) fprintf (msg, "Cc: %s\n", cc.data());
        if (!bcc.isEmpty()) fprintf (msg, "Bcc: %s\n", bcc.data());
        if (!subj.isEmpty()) fprintf (msg, "Subject: %s\n",subj.data());
      }
      else
        fprintf (msg, "show");
      fclose (msg);                   //message written
			fprintf(stderr,"Finished message\n");


      if (pId < 0)                    // server busy?
      {
				fprintf(stderr,"Lock file says server is busy\n");
        if (kill(0-pId, 0) != 0)      // try if it lives at all
        {
          debug ("Server died while busy");
          writePid(true);             // he died and left his pid uncleaned
        }
        else
        {
          debug ("Server is busy - message pending");
          exit (0);                   // ok he lives but is busy
        }
      }
      else                            // if not busy
      {
				fprintf(stderr,"Lock file says server is ready\n");
        if (kill (pId, SIGUSR1) != 0) // Dead?
        {
          debug ("Server died while ready");
          writePid(true);             // then we are server
        }
        else
        {
          debug ("Server is ready - message sent");
          exit (0);
        }
      }
    }
    debug ("We are starting normaly");
    sprintf(lf, "%s%s%d.msg", _PATH_TMP, _IPC_FILE, getuid());
    unlink(lf); // clear old mesage
		fprintf(stderr,"Message file cleared\n");

  }
  //--- Sven's pseudo IPC&locking end ---

  if(!app)
	{
		fprintf(stderr,"Generating application\n");
		app = new KApplication(argc, argv, "kimap");
    argc--;
    argv++;
	}

	fprintf(stderr,"Processing args\n");
  for (int i=0; i<argc; i++)
  {
      if (strcmp(argv[i],"-s")==0)
      {
          if (i<argc-1) subj = argv[++i];
          mailto = TRUE;
      }
      else if (strcmp(argv[i],"-c")==0)
      {
          if (i<argc-1) cc = argv[++i];
          mailto = TRUE;
      }
      else if (strcmp(argv[i],"-b")==0)
      {
          if (i<argc-1) bcc = argv[++i];
          mailto = TRUE;
      }
      else if (strcmp(argv[i],"-h")==0)
      {
          if (i<argc-1) {
							int x;
              QString headerString = argv[++i];
              if( (x = headerString.find( '=' )) != -1 )
                  msg->extraHeader( headerString.left( x ) + ": " + headerString.right( headerString.length()-x-1 ) );
          }
          mailto = TRUE;
      }
      else if(strcmp(argv[i],"-msg")==0)
      {
          if(i<argc-1)
              msg->bodyEncoded( argv[++i] );
          mailto = TRUE;
      }
      else if (strcmp(argv[i],"-check")==0)
			{
          checkNewMail = TRUE;
			}
      else if (strcmp(argv[i],"-open")==0)
			{
				openfolder = argv[++i];
			}
      else if (argv[i][0]=='-')
      {
          warning(i18n("Unknown command line option: %s"), argv[i]);
          // unknown parameter
      }
      else
      {
          if (!to.isEmpty()) to += ", ";
          if (strncasecmp(argv[i],"mailto:",7)==0) to += argv[i]+7;
          else to += argv[i];
          mailto = TRUE;
      }
  }

	fprintf(stderr,"Generating widget\n");

	if(!gFolders) gFolders = new NMGlobalFolders();
	gFolders->readFolders(kapp);
  kimap = new KImap(openfolder);
	fprintf(stderr,"Calling init function\n");
	init(argc,argv);
	fprintf(stderr,"Check for pending messages\n");
	checkMessage();

	fprintf(stderr,"About to set mainwidget : %p\n",kimap);
  app->setMainWidget(kimap);
	if(!mailto)
	  kimap->show();
  else
	{
    if (!cc.isEmpty()) msg->setMessageCC(cc);
    if (!bcc.isEmpty()) msg->setMessageBCC(bcc);
    if (!subj.isEmpty()) msg->setSubject(subj);
    if (!to.isEmpty()) msg->setMessageTo(to);

		fprintf(stderr,"Generating new Composer\n");
    KMComposeWin *win = new KMComposeWin(msg);
    assert(win != NULL);
    win->show();
		fprintf(stderr,"Generated new Composer\n");
	}
	fprintf(stderr,"Starting event handler\n");
	app->exec();
	
	fprintf(stderr,"Application finished starting cleanup\n");
	cleanup();
	fprintf(stderr,"The End\n");
	return 0;
}
