/***************************************************************************
                          nmclient.cpp  -  description
                             -------------------
    begin                : Fri Oct 22 1999
    copyright            : (C) 1999 by Sven Carstens
    email                : s.carstens@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#include "nmclient.h"

#include "extern.h"
#include "resource.h"
#include "kmimemagic.h"
#include <qcheckbox.h>
#include <qstack.h>
#include <qprogressdialog.h>
#include <qprogressbar.h>
#include <qmessagebox.h>
#include "kbusyptr.h"
#include "kpasswdconfig.h"
#include "nmstatusbar.h"

#include <kapp.h>
#include <kstatusbar.h>
#include "kimap.h"
#include <mimelib/string.h>
#include <mimelib/utility.h>
#include <mimelib/message.h>
#include <ctype.h>
#include <qregexp.h>

extern "C" {
#include "imap-2000/c-client/utf8.h"
}

static KMimeMagic* sMagic = NULL;

NMClient::NMClient() : MailStream()
{
	cancelPressed = false;
	foundMsgno = "";
}

NMClient::~NMClient(){
}

// you must free the result afterwards
//-----------------------------------------------------------------
const char *emailAddrAsText(ADDRESS *adr)
{
	char address[200];
	const char *retval;

	address[0] = 0x00;

	if(!adr) return NULL;

 	if (adr->personal && *adr->personal)
  	snprintf(address,sizeof(address)-1,"<%s@%s>",adr->mailbox,adr->host);
	else
  	snprintf(address,sizeof(address)-1,"%s <%s@%s>",adr->personal,adr->mailbox,adr->host);

	retval = cpystr(address);

  return retval;
}

QString magicSetType(mailBody *body)
{
  QString mimetype;
	QString work;
	QString bod;
	const char *bod_raw;
	unsigned long bod_len;

	int sep;
  if (!sMagic)
  {
    // initialize mime magic
    sMagic = new KMimeMagic(kapp->kde_mimedir() + "/magic");
    sMagic->setFollowLinks(TRUE);
  }

	bod_raw = body->bodyDecoded(&bod_len);
	bod = QString(bod_raw,bod_len);
	delete bod_raw;	

  mimetype = sMagic->findBufferType(bod, bod.size()-1)->getContent();
	work = mimetype;
	sep = work.find('/');
	body->setType(work.left(sep) );
	body->setSubType(work.right( work.length() - sep -1 ));
	return mimetype;
}

QString magicSetType(QString *data)
{
  QString mimetype;

  if (!sMagic)
  {
    // initialize mime magic
    sMagic = new KMimeMagic(kapp->kde_mimedir() + "/magic");
    sMagic->setFollowLinks(TRUE);
  }

  mimetype = sMagic->findBufferType(data->data(), data->size()-1)->getContent();
	return mimetype;
}

int NMClient::timeoutHandler(time_t t1,time_t t2)
{
	return progressNotify(mpTIMEOUT,t1,NULL);
}

int NMClient::progressNotify(mailProgress function,unsigned long value,const char *txt)
{
	bool result = true;
	
//	class NMStatusBar *aStatusBar = NULL;
//	if(kimap)	aStatusBar = (NMStatusBar *)kimap->statusBar();
	
//	if(aStatusBar) aStatusBar->progressNotify(function,value,txt,result);
	emit signalProgressNotify(function,value,txt,result);
	if(result) return 1;
	return 0;
}

void NMClient::loginNotify(NETMBX *mb,char *user,char *pwd,long trial)
{
  NMPasswdDialog* dlg;
	QString host;
	QString entry;
	QString pass;

	host.sprintf("%s",mb->orighost);
  passwd->setGroup("Passwords");

//replace the invalid given password from the c-client
	if(firstTry) pwd[0] = 0x00;
	strcpy(user,mb->user);
	entry = QString(user) + "@" + host;
  pass = passwd->readEntry(entry);
	strcpy(pwd,pass.data());
	if(pwd && *pwd && firstTry)
	{
		//use cached passwords only for the first try
	} else {
		kbp->idle();
    dlg = new NMPasswdDialog(NULL, NULL, this, mb,
  			   user, pwd, "");
		dlg->setSavePassword((pwd && *pwd));
    if (!dlg->exec()) return;
		kbp->busy();
  	if(dlg->getSavePassword()) passwd->writeEntry(entry,pwd,false);
    delete dlg;
	}
	firstTry = false;
	return;
}



//-----------------------------------------------------------------
//use the signal slot mechanism to notify all members
//-----------------------------------------------------------------
void NMClient::flagsNotify (unsigned long number)
{ emit signalFlags(this,number); }

void NMClient::notifyNotify (char *string,long errflg)
{
	MailStream::notifyNotify(string,errflg);
	fprintf(stderr,"mm_notify - ");
	logNotify(string,errflg);
}

void NMClient::listNotify (int delimiter,char *mailbox,long attributes)
{ emit signalList(this,delimiter,mailbox,attributes); }

void NMClient::lsubNotify(int delimiter,char *mailbox,long attributes)
{
  fprintf(stderr,"mm_lsub\n");
  putchar (' ');
  if (delimiter) putchar (delimiter);
  else fputs ("NIL",stdout);
  putchar (' ');
  fputs (mailbox,stdout);
  if (attributes & LATT_NOINFERIORS) fputs (", no inferiors",stdout);
  if (attributes & LATT_NOSELECT) fputs (", no select",stdout);
  if (attributes & LATT_MARKED) fputs (", marked",stdout);
  if (attributes & LATT_UNMARKED) fputs (", unmarked",stdout);
  putchar ('\n');
}

void NMClient::statusNotify (char *mailbox,MAILSTATUS *status)
{	emit signalStatus(this,mailbox,status); }

void NMClient::logNotify (char *string,long errflg)
{
  fprintf(stderr,"mm_log: ");
  switch ((short) errflg) {
  case NIL:
    fprintf (stderr,"[%s]\n",string);
    break;
  case PARSE:
  case WARN:
    fprintf (stderr,"%%%s\n",string);
    break;
  case ERROR:
    fprintf (stderr,"?%s\n",string);
    break;
	default:
    fprintf (stderr,"DUNO %s\n",string);
		break;
  }
	emit signalStatusMsg(string);
	kapp->processEvents();
}

void NMClient::existsNotify (MAILSTREAM *stream,long unsigned int num)
{ emit signalExists(this,num); }

void NMClient::expungedNotify (long unsigned int num)
{ emit signalExpunged(this,num); }

void NMClient::dlogNotify (char *string)
{ fprintf(stderr,"mm_dlog : %s\n",string); }

void NMClient::criticalNotify ()
{ fprintf(stderr,"mm_critical\n"); }

void NMClient::nocriticalNotify ()
{	fprintf(stderr,"mm_nocritical\n"); }

void NMClient::fatalNotify (char *string)
{ fprintf(stderr,"mm_fatal : %s\n",string); }

void NMClient::overviewHeader(unsigned long uid,OVERVIEW *ov)
{ emit signalOverview(this,uid,ov); }

void NMClient::searchedNotify(unsigned long number)
{
	char buffer[16];
	
	snprintf(buffer,sizeof(buffer),"%ld ",number);
	foundMsgno += buffer;
}

void NMClient::searchSimple(const char *what)
{
	foundMsgno = "";
	MailStream::searchSimple(what);
}

const char *NMClient::searchResult()
{
	return foundMsgno.data();
}

//=============================================================================
//
//  Class  NMPasswdDialog
//
//=============================================================================

NMPasswdDialog::NMPasswdDialog(QWidget *parent, const char *name,MailStream *in_client,
			       NETMBX *account , char *in_user, char *in_pwd, const char *caption)
  :QDialog(parent,name,true)
{
  // This function pops up a little dialog which asks you
  // for a new username and password if one of them was wrong or not set.

	QString caption;
	act = account;
	pwd = in_pwd;
	user = in_user;
	int error_offset;
	QLabel *error;
	client = in_client;

//  kbp->idle();

	if(client->getError()) {
    QLabel *error = new QLabel(this);
		QString errorText = QString(i18n(client->getError()));
		int sep=40;

		sep = errorText.findRev(' ',sep);
		if(sep > -1) errorText.insert(sep,'\n');
    error->setText(errorText);
    error->resize(error->sizeHint());
    error->move(20,130);
		error_offset = error->height()+30;
	} else {
		error = NULL;
		error_offset = 0;
	}

  setMaximumSize(300,200+error_offset);
  setMinimumSize(300,200+error_offset);

	caption.sprintf(i18n("User %s on %s[%s]"),account->user,account->host,account->service);
  setCaption(caption);

  usernameLEdit = new QLineEdit(this,"NULL");
  usernameLEdit->setText(account->user);
  usernameLEdit->setGeometry(100,27,150,25);

  QLabel *label = new QLabel(usernameLEdit,"",this);
  label->setText(i18n("Login &Name :"));
  label->resize(label->sizeHint());
  label->move(20,30);

  passwdLEdit = new QLineEdit(this,"NULL");
  passwdLEdit->setEchoMode(QLineEdit::Password);
  passwdLEdit->setText(pwd);
  passwdLEdit->setGeometry(100,76,150,25);
  connect(passwdLEdit,SIGNAL(returnPressed()),SLOT(slotOkPressed()));

  QLabel *label1 = new QLabel(passwdLEdit,"",this);
  label1->setText(i18n("&Password:"));
  label1->resize(label1->sizeHint());
  label1->move(20,80);

  savePassword=new QCheckBox(
	      i18n("&Save Password"), this);
	savePassword->adjustSize();
	savePassword->move(20,120+error_offset);

  ok = new QPushButton(i18n("OK") ,this,"NULL");
  ok->setGeometry(55,150+error_offset,70,25);
  connect(ok,SIGNAL(pressed()),this,SLOT(slotOkPressed()));

  cancel = new QPushButton(i18n("Cancel"), this);
  cancel->setGeometry(180,150+error_offset,70,25);
  connect(cancel,SIGNAL(pressed()),this,SLOT(slotCancelPressed()));
	passwdLEdit->setFocus();
}

//-----------------------------------------------------------------------------
void NMPasswdDialog::slotOkPressed()
{
  //kbp->busy();
	if(act) {
  strcpy(user,usernameLEdit->text());
  strcpy(pwd,passwdLEdit->text());
  }
  done(1);
}

//-----------------------------------------------------------------------------
void NMPasswdDialog::slotCancelPressed()
{
  //kbp->busy();
  strcpy(pwd,"");
	client->setRetry(false);
  done(0);
}

//-----------------------------------------------------------------------------
void NMPasswdDialog::setSavePassword(bool save)
{
	savePassword->setChecked( save );
}

//-----------------------------------------------------------------------------
bool NMPasswdDialog::getSavePassword()
{
	return savePassword->isChecked();
}

//---------------------------------------------------------------------------------
//typedef long (*soutr_t) (void *stream,char *string);

long streamQString(void *stream,char *string)
{
	long len = 0;

	if(!stream) return len;

	len = ((QString *)stream)->length();
	((QString *)stream)->append(string);
	len = ((QString *)stream)->length() - len;

//	fprintf(stderr,"APPEND: '%s'\n",string);
	return len;	
}

//-----------------------------------------------------------------
extern long r822new (char *,ENVELOPE *,BODY *,soutr_t,void *,long);


const QString bodyQString(mailMessage *msg,bool preserve)
{
	static QString message;
  char tmp[8*MAILTMPLEN];
	BODY *body = msg->raw_body();
	ENVELOPE *env = msg->raw_env();

  rfc822out_t r822orig = (rfc822out_t) mail_parameters (NIL,GET_RFC822OUTPUT,NIL);
  mail_parameters (NIL,SET_RFC822OUTPUT,r822new);
	message = "";
	if(msg->header && preserve)
	{
		env = NULL;
		message += msg->header;
	}
	tmp[0] = 0x00;
	rfc822_output(tmp,env,body,(soutr_t)streamQString,(void *)&message,1);
  mail_parameters (NIL,SET_RFC822OUTPUT,r822orig);
	return message;
}

QString i18ndate(const char *in)
{
	QString retval;
	QString inval = in;
  int sep=0;

	while(sep >= 0)
	{
		int space,comma;

		space = inval.find(' ');
		comma = inval.find(',');
		sep = (space < comma) ? space : comma;
		if(sep >= 2)
			retval += QString(i18n(inval.left(sep)));
		else
			retval += inval.left(sep);
		if(sep != -1) retval += (space < comma) ? " " : ",";
		inval = inval.right( inval.length() - sep -1);
	}
	return retval;	
}

const QString decode_MIME2(const QString encoded)
{
	static QString decoded;
	SIZEDTEXT source,destination;
	
	source.data = (unsigned char *)encoded.data();
	source.size = encoded.length();
	destination.size=0;
	destination.data=NULL;
	utf8_mime2text(&source,&destination);
	decoded.resize(destination.size+1);
	if(destination.data)
		memcpy(decoded.data(),destination.data,destination.size);
	decoded.data()[destination.size] = 0x00;
	
	return decoded;
}

//-----------------------------------------------------------------------------
const QString &decodeMIME2(const QString aStr)
{
  static QString result;
  char *pos, *dest, *beg, *end, *mid;
  char *str;
	unsigned long str_len;
	char *tmp;
  char encoding, ch;
  bool valid;
  int i;

//	if(aStr.isEmpty) return "";
//	strncpy(result,aStr,sizeof(result)-1);
	result = aStr;
	tmp = aStr.data();
	i = 0;
	while(tmp && *tmp)
	{
		if(tmp[0] == '=' && tmp[1] == '?') i = 1;
		tmp++;
	}
	if(!i) return result; //found encoded string ?

//  result.truncate(aStr.length());
  for (pos=aStr.data(), dest=result.data(); *pos; pos++)
  {
    if (pos[0]!='=' || pos[1]!='?')
    {
      *dest++ = *pos;
      continue;
    }
    beg = pos+2;
    end = beg;
    valid = true;
    // parse charset name
    for (i=2,pos+=2; (*pos!='?'&&(ispunct(*pos)||isalnum(*pos))); i++)
      pos++;
    if (*pos!='?' || i<4) valid = false;
    else
    {
      // get encoding and check delimiting question marks
      encoding = toupper(pos[1]);
      if (pos[2]!='?' || (encoding!='Q' && encoding!='B'))
	valid = false;
      pos+=3;
      i+=3;
    }
    if (valid)
    {
      mid = pos;
      // search for end of encoded part
      while (*pos && !(*pos=='?' && *(pos+1)=='='))
      {
	i++;
	pos++;
      }
			end = pos;
			if(*end) end++;
			if(*end) end++;//end now points to the first char after the encoded string or at the end
      //end = pos+2;
//      if (i>=maxLen || !*pos) valid = false;
//      if (i>=maxLen) valid = false;
    }
    if (valid)
    {
      ch = *pos;
      *pos = '\0';
      str = mid;
			str_len = (int)(pos - mid);
      if (encoding == 'Q')
      {
	// decode quoted printable text
	for (i=str_len-1; i>=0; i--)
	  if (str[i]=='_') str[i]=' ';
	str = (char *)rfc822_qprint((unsigned char *)str,str_len,&str_len);
      }
      else
      {
	// decode base64 text
	str = (char *)rfc822_base64((unsigned char *)str,str_len,&str_len);
      }
      *pos = ch;
      for (i=0; i < str_len; i++)
				*dest++ = str[i];
			fs_give((void **)&str);
      pos = end -1;
    }
    else
    {
      pos = beg - 2;
      *dest++ = *pos++;
      *dest++ = *pos;
    }
  }
  *dest = '\0';
	//get those double quotes right
  result.replace(QRegExp("\x22\x27",false), "\x22");
  result.replace(QRegExp("\x27\x22",false), "\x22");
  return result;
}
