/***************************************************************************
                          nmclient.h  -  description
                             -------------------
    begin                : Fri Oct 22 1999
    copyright            : (C) 1999 by Sven Carstens
    email                : s.carstens@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#ifndef NMCLIENT_H
#define NMCLIENT_H

#include <cpp-client.h>
#include <qdialog.h>
#include <qstring.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qcheckbox.h>
/**
  *@author Sven Carstens
  */

const QString &decodeMIME2(const QString);
const char *emailAddrAsText(ADDRESS *);
QString i18ndate(const char *in);
//const QString bodyDecoded(BODY *);
void bodyEncoded(BODY *,QString);
QString magicSetType(mailBody *);
QString magicSetType(QString *);

const QString bodyQString(mailMessage *msg,bool preserve=false);
void sendSMTP(mailMessage *msg,char **hostlist,unsigned long port);

class NMClient :  public QWidget ,public MailStream{
	Q_OBJECT
public:
	NMClient();
	~NMClient();

	virtual void searchSimple(const char *);
	const char *searchResult();
//const QString bodyDecoded(int,const char *);
//our virtual overrides
virtual int progressNotify(enum mailProgress,unsigned long,const char *);
virtual int timeoutHandler(time_t,time_t);
virtual void existsNotify (MAILSTREAM *,long unsigned int);
virtual void expungedNotify (long unsigned int);
virtual void flagsNotify (long unsigned int);
virtual void notifyNotify (char *,long);
virtual void listNotify (int, char *, long int);
virtual void lsubNotify (int, char *, long int);
virtual void statusNotify (char *, MAILSTATUS *);
virtual void logNotify (char *, long int);
virtual void dlogNotify (char *);
virtual void loginNotify (NETMBX *, char *, char *, long int);
virtual void criticalNotify ();
virtual void nocriticalNotify ();
virtual void fatalNotify (char *);
virtual void searchedNotify (unsigned long);
virtual void overviewHeader(unsigned long uid,OVERVIEW *ov);

//public slots:
//	void cancelWasPressed();

signals:
	void signalOverview(NMClient *,unsigned long uid,OVERVIEW *ov);
	void signalFlags(NMClient *,unsigned long number);
	void signalExists(NMClient *,unsigned long);
	void signalExpunged(NMClient *,unsigned long);
	void signalStatus(NMClient *,char *,MAILSTATUS *);
	void signalList(NMClient *,int,char *,long);

	void signalStatusMsg(const char *);
	void signalProgressNotify(enum mailProgress,unsigned long,const char *,bool &);

protected:
	bool cancelPressed;

private:
	QString foundMsgno;
};

class NMPasswdDialog : public QDialog {
	Q_OBJECT
public:
	NMPasswdDialog(QWidget *parent, const char *name, MailStream *,
			       NETMBX *account , char *user, char * pwd, const char *caption);
	void setSavePassword(bool);
	bool getSavePassword();

private:
	MailStream *client;
	NETMBX *act;
	char *user;
	char *pwd;
  QLineEdit *usernameLEdit;
  QLineEdit *passwdLEdit;
  QPushButton *ok;
  QPushButton *cancel;
	QCheckBox *savePassword;
  void header(MAILSTREAM *stream,long msgno);

private slots:
  void slotOkPressed();
  void slotCancelPressed();
};


#endif
