/***************************************************************************
                          nmdocking.cpp  -  description
                             -------------------
    begin                : Wed Dec 29 1999
    copyright            : (C) 1999 by Sven Carstens
    email                : s.carstens@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/*
 *
 * $Id: docking.cpp,v 1.10 1998/12/20 20:45:56 porten Exp $
 *
 *              Copyright (C) 1997 Bernd Johannes Wuebben
 *                      wuebben@math.cornell.edu
 *
 * This file was contributed by Harri Porten <porten@tu-harburg.de>
 *
 */

#include <qtooltip.h>
#include <qdir.h>
#include <kwm.h>
#include <kapp.h>
#include <qmessagebox.h>
#include "nmdocking.h"
#include "kimap.h"
#include "extern.h"
#include <unistd.h>

#include <sys/wait.h>
#include <errno.h>



// static member
DockWidget *DockWidget::dock_widget = 0;

DockWidget::DockWidget(const char *name)
  : QWidget(0, name, 0) {

  docked = false;
	working = false;
	recentWaiting = false;
  QString pixdir = KApplication::kde_datadir() + "/kimap/pics/";
  QString tmp;
	
#define PMERROR(pm) \
  tmp.sprintf(i18n("Could not load %s !"), pm); \
  QMessageBox::warning(this, i18n("Error"), tmp);

  // load pixmaps
  if (!dock_none_pixmap.load(pixdir + "mini-nomail.xpm")){
    PMERROR("mini-nomail.xpm");
  }
  if (!dock_new_pixmap.load(pixdir + "mini-newmail.xpm")){
    PMERROR("mini-newmail.xpm");
  }
  if (!dock_old_pixmap.load(pixdir + "mini-oldmail.xpm")){
    PMERROR("mini-oldmail.xpm");
  }
  if (!dock_work_pixmap.load(pixdir + "mini-working.xpm")){
    PMERROR("mini-working.xpm");
  }

  // popup menu for right mouse button
  popup_m = new QPopupMenu();
  toggleID = popup_m->insertItem(i18n("Restore"),
				 this, SLOT(toggle_window_state()));
  checkID = popup_m->insertItem(i18n("Check Mail"));
  popup_m->insertSeparator();

  DockWidget::dock_widget = this;
	default_pixmap = &dock_work_pixmap;
}


DockWidget::~DockWidget() {
  DockWidget::dock_widget = 0;
}


void DockWidget::dock() {
  if (!docked) {
    // prepare panel to accept this widget
    KWM::setDockWindow (this->winId());

    // that's all the space there is
    this->setFixedSize(24, 24);

    // finally dock the widget
    this->show();

    docked = true;
  }
}


void DockWidget::undock() {
  if (docked) {
    // the widget's window has to be destroyed in order
    // to undock from the panel. Simply using hide() is
    // not enough (seems to be necessary though).
    hide();
    destroy(true, true);

    // recreate window for further dockings
    this->create(0, true, false);

    docked = false;
  }
}


const bool DockWidget::isDocked() {
  return docked;
}

void DockWidget::setWorking(bool in) {
	working = in;
	paintIcon();	
}


void DockWidget::paintEvent (QPaintEvent *) {
  paintIcon();
}


void DockWidget::paintIcon () {
	if(!working)	
	  bitBlt(this, 0, 0, default_pixmap,0,0,-1,-1,CopyROP,true);
	else
	  bitBlt(this, 0, 0, &dock_work_pixmap,0,0,-1,-1,CopyROP,true);
}

void DockWidget::none() {
	recentWaiting = false;
	default_pixmap = &dock_none_pixmap;
	paintIcon();
}

void DockWidget::unseen() {
	recentWaiting = false;
	default_pixmap = &dock_old_pixmap;
	paintIcon();
}

void DockWidget::recent() {
  int status;
	int i,rc;
	
	KConfig *config=kapp->getConfig();

  config->setGroup("General");
  actionUnseen = config->readEntry("check-mail-action");
	
	if(!recentWaiting) {
	  if(!(i = fork()) && !actionUnseen.isEmpty())
			  execl("/bin/sh","sh","-c",actionUnseen.data(),NULL);
	  rc = waitpid(0/*child_pid*/, &status, 0);
	  if (rc==-1) fprintf(stderr,"waitpid: %s\n", strerror(errno));
	}
	recentWaiting = true;
	default_pixmap = &dock_new_pixmap;
	paintIcon();
}

void DockWidget::mousePressEvent(QMouseEvent *e) {
  // open/close connect-window on right mouse button
  if ( e->button() == LeftButton ) {
    toggle_window_state();
  }

  // open popup menu on left mouse button
  if ( e->button() == RightButton ) {
    int x = e->x() + this->x();
    int y = e->y() + this->y();

    QString text;
    if(kimap->isVisible())
      text = i18n("Minimize");
    else
      text = i18n("Restore");

		popup_m->connectItem(checkID,(QObject *)gFolders,SLOT(slotStatusLookup()));
    popup_m->changeItem(text, toggleID);
    popup_m->popup(QPoint(x, y));
    popup_m->exec();
		//disconnect here to avoid duplicate calls
		popup_m->disconnectItem(checkID,(QObject *)gFolders,SLOT(slotStatusLookup()));
  }

}

void DockWidget::toggle_window_state() {
  // restore/hide connect-window
  if(kimap != 0L)  {
    if (kimap->isVisible())
      kimap->hide();
    else {
      kimap->show();
      KWM::activate(kimap->winId());
    }
  }
}
