/***************************************************************************
                          nmfoldercontent.cpp  -  description
                             -------------------
    begin                : Sat Oct 16 1999
    copyright            : (C) 1999 by Sven Carstens
    email                : s.carstens@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include <stdlib.h>
#include <qdialog.h>
#include <qlabel.h>
#include <qpixmap.h>
#include <qheader.h>
#include <qmessagebox.h>
#include <qprinter.h>
#include "kfileio.h"
#include "kfontutils.h"
#include "kfiledialog.h"
#include "kmcomposewin.h"
#include "kmidentity.h"
#include "kbusyptr.h"
#include <kconfig.h>
#include <kapp.h>
#include <kiconloader.h>
#include <qdir.h>
#include <qpopupmenu.h>
#include <qmessagebox.h>
#include "nmfoldercontent.h"
#include "nmsearch.h"
#include "nmmailview.h"
#include "extern.h"
#include "nmglobalfolders.h"
#include "nmstatusbar.h"
#include "resource.h"

NMFolderContent::NMFolderContent(QWidget *parent, const char *name ) : QListView(parent,name) {

  KIconLoader* loader = app->getIconLoader();

	subRecent = 0;
	subUnseen = 0;
	lastMessageNo = 0;
	lastFolderSelected = 0;
	
	addColumn("");
	addColumn(i18n("Date"));
	addColumn(i18n("From"));
	addColumn(i18n("Subject"));
	addColumn(i18n("Size"));
/*	addColumn("Number");
	addColumn("Sortdate");
	addColumn("Sortflags");
	addColumn("test");
	addColumn("test"); */
	setSorting(0,true);
	setSorting(MSGNO_POSITION,true);
	statusView = NULL;
	mFastFetch = false;
	mFastPop = false;
	isSaveSend = false;
	setColumnAlignment(SIZE_POSITION,2);
	setAllColumnsShowFocus(true);
	setFocusPolicy( QWidget::StrongFocus );

		int width = 0;
		for(int i=0;i<QListView::header()->count();i++) {
			width += columnWidth( i );
		}
		setMinimumWidth( width );

  pixNew   = loader->loadIcon("kmmsgnew.xpm");
	pixUnseen = loader->loadIcon("kmmsgunseen.xpm" );
	pixRead = loader->loadIcon("kmmsgold.xpm" );
	pixReplied = loader->loadIcon("kmmsgreplied.xpm" );
	pixDeleted = loader->loadIcon("kmmsgdel.xpm" );
	pixFlagged = loader->loadIcon("kmmsgflagged.xpm" );
	pixQueued = loader->loadIcon("kmmsgqueued.xpm" );
	pixSend = loader->loadIcon("kmmsgsent.xpm" );
	
	my_client = NULL;
	my_viewer = NULL;
	
	currentPopup = NULL;
	currentMail = NULL;
	QObject::connect( this,	SIGNAL( rightButtonClicked( QListViewItem *,const QPoint &,int) ),
													SLOT( slotRightButtonClicked( QListViewItem *,const QPoint &,int) ) );
	QObject::connect( this,	SIGNAL( doubleClicked( QListViewItem *) ),
													SLOT( slotDoubleClicked( QListViewItem *) ) );
	QObject::connect( this,	SIGNAL( returnPressed( QListViewItem *) ),
													SLOT( slotDoubleClicked( QListViewItem *) ) );
	
	QObject::connect( this,	SIGNAL( signalSubStatus(int,int,int) ),
										gFolders, SLOT( slotSubStatus(int,int,int) ) );
	
	QObject::connect( this,	SIGNAL( signalSignOn(int,NMFolderContent *) ),
										gFolders, SLOT( slotSignOn(int,NMFolderContent *) ) );
	
	QObject::connect( this,	SIGNAL( signalSignOff(int,NMFolderContent *) ),
										gFolders, SLOT( slotSignOff(int,NMFolderContent *) ) );
	
	QObject::connect( this,SIGNAL( signalMakeMenu(QPopupMenu *,const QObject *,const char *)),
										gFolders,SLOT( slotMakeMenu(QPopupMenu *,const QObject *,const char *)) );
										
	
	setAccelerators();	
	readConfig();
}

NMFolderContent::~NMFolderContent(){
	if(gFolders) gFolders->slotSignOff(this);
	emit signalSignOff(currentFolder,this); // useless: signals are already disconnected
	delete my_client;
	my_client = NULL;
}

void NMFolderContent::closeEvent(QCloseEvent *aEvent)
{	
	QWidget::closeEvent(aEvent);
	if(gFolders) gFolders->slotSignOff(this);
	emit signalSignOff(currentFolder,this);
	debug("NMFolderContet::closeEvent()");
	if(my_client) my_client->closeBox();
	debug("NMFolderContet::closeEvent()");
	delete this;
}

void NMFolderContent::writeConfig(bool withSync)
{
}

void NMFolderContent::readConfig()
{
	KConfig *config=kapp->getConfig();
	QString mListFont;

  config->setGroup("Fonts");
  mListFont = config->readEntry("list-font", "helvetica-medium-r-12");
  setFont(kstrToFont(mListFont));
  config->setGroup("General");
	mFastFetch = config->readBoolEntry("fast-fetch",false);
	mFastPop = config->readBoolEntry("fast-pop",false);
	timerEvent(NULL);
}

// sorting is inverted on every second click on the header
void NMFolderContent::setSorting(int column,bool increasing)
{
	static int old_column = 0;

	if(column == old_column) increasing = !increasing;
	old_column = column;

//	debug("Set sorting: %d %s\n",column,increasing ? "YES" : "NO");
	QListView::setSorting(column,increasing);
}



void NMFolderContent::slotRefresh()
{
	if(my_client && !my_client->isLocked())
	{
		my_client->Ping(); // should get the new exists message
		if((lastMessageNo < my_client->numMsgs()) && my_client->isOpen()) {
			QString spec;

			spec.sprintf("%ld:%ld",my_client->msgno2uid(lastMessageNo+1),my_client->msgno2uid(my_client->numMsgs()));
			emit signalStatusMsg(i18n("Reading new messages"));
			kbp->busy();
			debug("Reading new Messsages (slotRefresh): %s\n",spec.data());
	  	my_client->fetchOverview(spec.data());
			my_client->Status(); // this will notify nmlistfolders
			emit signalStatusMsg(i18n("Ready."));
			kbp->idle();
		}
	}
}

void NMFolderContent::timerEvent(QTimerEvent *ev)
{
	KConfig *config=kapp->getConfig();
	
	killTimers();
	if(ev)
		slotRefresh();

  config->setGroup("General");
	if(config->readBoolEntry("keep-alive",false))
		startTimer( 60000 );
//	debug("NMFolderContent::timerEvent(%p)",ev);
}

void NMFolderContent::slotReRead()
{
		killTimers(); // kill the timer so it doesnt interrupt us
		QString spec = "1:*";
  	clear();
		currentMail = NULL;
		subUnseen = 0;
		subRecent = 0;
		lastMessageNo = 0;
  	if(my_client->isOpen()) {
			const char *status;
			if(my_client->isLocked()) return;

			emit signalStatusMsg(i18n("Opening Folder"));
			kbp->busy();
			hide();
//			my_client->Check();
			if(mFastFetch && my_client->getUnSeen())
				spec.sprintf("%ld:*",my_client->msgno2uid(my_client->getUnSeen()));
			if(!mFastPop || !gFolders->getAt(currentFolder)->getProtocol().contains(QRegExp("pop",false)))
			{
				debug("Reading new Messsages (slotReRead): %s\n",spec.data());
	    	my_client->fetchOverview(spec.data());
			} else {
				lastMessageNo = my_client->numMsgs();
				for(unsigned long i=1;i<=my_client->numMsgs();i++)
				{
					char msg[16],size[16];

					snprintf(msg,sizeof(msg)-1,"%014ld",i);
					snprintf(size,sizeof(size)-1,"%ld",(my_client->getCache(i))->rfc822_size);
				 	NMFolderContentItem *item = new NMFolderContentItem(this, "","","","",size,msg,"");
					item->setSelectable(false);
				}
			}

			my_client->Status(); // this will notify nmlistfolders
			kbp->idle();
			show();
			if(!currentItem())
			{
				QListViewItem *tmp = firstChild();
				while(tmp && tmp->nextSibling()) tmp = tmp->nextSibling();
				setCurrentItem(tmp);
			}
			else emit currentChanged(currentItem());
			ensureItemVisible(currentItem());
			setSelected(currentItem(),TRUE);
			currentMail = currentItem();

			if(!my_client->getReadWrite()) status = i18n("Read/Write");
			else status = i18n("Read only");
			emit signalStatusMsg(status);

    } else {
			currentFolder = -1;
    }
	emit doubleClicked(NULL);
	timerEvent(NULL); //restart the timer
}

void NMFolderContent::setDefaultViewer(NMMailView *in_viewer)
{
	my_viewer = in_viewer;
	
	if(my_viewer) {
//inbound signal from folder content
  	QObject::connect( this,SIGNAL( doubleClicked( QListViewItem *)),
  										SLOT( slotMail( QListViewItem *)) );
  	QObject::connect( this,SIGNAL( returnPressed( QListViewItem *)),
  										SLOT( slotMail( QListViewItem *)) );
	}
}

void NMFolderContent::slotMail(QListViewItem *item)
{
	if(item)
	{
		unsigned long curmsg = atoi(item->text(MSGNO_POSITION));
//		debug("NMFolderContent::slotMail() my_client - %p my_viewer - %p",my_client,my_viewer);
		if(my_viewer) my_viewer->slotMail(my_client,curmsg);
	} else {
		if(my_viewer) my_viewer->slotMail(NULL);
	}
}

void NMFolderContent::slotFolder(int folder)
{
	NMClient *client = new NMClient;

	if(my_client)
	{
		if(my_client->isLocked()) return;
		emit signalStatusMsg(i18n("Closing"));
		my_client->closeBox();
		if(gFolders) gFolders->slotSignOff(this);
		emit signalSignOff(currentFolder,this);
	}
	delete my_client;
	my_client = NULL;		
		
//signals from client to folder content
//used by ourself
	QObject::connect( client,SIGNAL( signalOverview( NMClient *,unsigned long,OVERVIEW *)),
										this,SLOT( slotOverview( NMClient *,unsigned long,OVERVIEW *)) );

//used by ourself
	QObject::connect( client,SIGNAL( signalFlags( NMClient *,unsigned long)),
										this,SLOT( slotFlags( NMClient *,unsigned long)) );

//used by ourself
	QObject::connect( client,SIGNAL( signalExpunged( NMClient *,unsigned long)),
										this,SLOT( slotExpunged( NMClient *,unsigned long)) );

//relayed to globalFolders
	QObject::connect( client,SIGNAL( signalStatus(NMClient *,char *,MAILSTATUS *)),
										gFolders,SLOT( slotStatus(NMClient *,char *,MAILSTATUS *)) );

//relayed to upper widgets
	QObject::connect( client,SIGNAL( signalStatusMsg(const char *)),
										this,SLOT( slotStatusMsg(const char *)) );

	if(folder) NMStatusBar::connectToStatusBar(client,this);
		
	slotFolder(client,folder);
}

void NMFolderContent::slotFolderExpunge()
{
	if(my_client && !my_client->isLocked())
	{
	  emit signalStatusMsg(i18n("Expunging Folder contents..."));
	  subRecent = 0;
	  subUnseen = 0;
		my_client->expungeBox();
		my_client->Status();
  	emit signalStatusMsg(i18n("Ready."));
  }
}

void NMFolderContent::slotFolder(NMClient *client,int aNum)
{
	debug("Setting folder %d",aNum);
	realFolder *mFolder = gFolders->getAt(aNum);

  if(mFolder && client) {
		my_client = client;
		if(aNum != currentFolder) {
			KMIdentity *myID = new KMIdentity();
			myID->setIdentityFor(mFolder->getVirtual());
			
			setCaption(mFolder->assembleURIFolder());
			if(strcmp(myID->saveSend().data(),mFolder->assembleURIFolder()))
			{
				isSaveSend = false;
				setColumnText(FROM_POSITION,i18n("From"));
			} else {
				isSaveSend = true;
				setColumnText(FROM_POSITION,i18n("To"));
			}
			kbp->busy();
			client->setTimeOut(1);
			
			emit signalSignOn(aNum,this);
			
			client->setBox(mFolder->assembleUWIFolder());
			timerEvent(NULL);
			kbp->idle();
			if(client->getError())
			{
				QMessageBox::warning(0,i18n("ERROR opening Folder"),client->getError(),QMessageBox::Ok,QMessageBox::Default);
			}
			currentFolder = aNum;
			slotReRead();
	  } else {
			debug("NMFolderContent::slotFolder - same Folder");
    }
		resizeEvent(NULL);
  } else {
		debug("NMFolderContent::slotFolder - no client or no folder");
  }
	setFocus();
}

void NMFolderContent::slotExpunged (NMClient *client,unsigned long which)
{
//add progressnotify	
	QListViewItem *item = firstChild();
	while(item)
	{
		unsigned long tmp = atol(item->text(MSGNO_POSITION));
		QListViewItem *removeit = item;
		
		if(tmp > which)
		{
			char buffer[16];
			snprintf(buffer,sizeof(buffer),"%014ld",tmp-1);
			item->setText(MSGNO_POSITION,buffer);
		}
		item = item->nextSibling();
		if(tmp == which) {
			delete(removeit);
//			debug("After remove  %p curent %p ? %p",removeit,currentItem(),currentMail);
			//the client is busy, so we cant set the new currentMail
			if(removeit == currentMail)  { currentMail = currentItem();	emit doubleClicked(NULL);}
		}
	}
	debug("NMFolderContent::slotExpunged(%ld)",which);
	subUnseen = 0;
	subRecent = 0;
}

void NMFolderContent::slotFlags (NMClient *client,unsigned long which)
{
	slotFlags(client,which,NULL);
}

void NMFolderContent::slotFlags (NMClient *client,unsigned long which,QListViewItem *item)
{
	char newflags[16];
	char flags[6];
  unsigned long i = 0;
  MESSAGECACHE *elt;
	if(!client) return;
	QPixmap *ret=NULL;
	elt = client->getCache(which);

  if(!ret) ret = elt->deleted ? &pixDeleted : NULL;
  if(!ret) ret = elt->recent ? (elt->seen ? NULL : &pixNew) : NULL;
  if(!i) i = elt->recent ? (elt->seen ? 4 : 6) : 0;
  flags[0] = elt->recent ?  'R' : '_';
//  flags[0] = elt->recent ? (elt->seen ? 'R': 'N') : '_';
  if(!ret) ret = (elt->recent | elt->seen) ? NULL : &pixUnseen;
  if(!i) i = (elt->recent | elt->seen) ? 0 : 5;
  flags[1] = elt->seen ? '_' : 'U';
//  flags[1] = (elt->recent | elt->seen) ? '_' : 'U';
  if(!ret) ret = elt->flagged ? &pixFlagged : NULL;
  if(!i) i = elt->flagged ? 3 : 0;
  flags[2] = elt->flagged ? 'F' : '_';
  if(!ret) ret = elt->answered ? &pixReplied : NULL;
  if(!i) i = elt->answered ? 0 : 0;
  flags[3] = elt->answered ? 'A' : '_';
  if(!i) i = elt->deleted ? 0 : 1;
  flags[4] = elt->deleted ? 'D' : '_';
	if(!ret) ret = &pixRead;
	flags[5] = '\0';

	if(isSaveSend) {
		if(elt->answered)
		{
			ret = &pixSend;
			i = 2;
		} else {
			ret = &pixQueued;
			i = 1;
		}
		if(elt->deleted)
		{
			ret = &pixDeleted;
			i = 0;
		}
	}
	snprintf(newflags,sizeof(newflags)-1,"%1.1ld%s",i,flags);
/*  if ((i = elt->user_flags)) {
    strcat (flags,"{");
    while (i) {
      strcat (flags,mailstream->user_flags[find_rightmost_bit (&i)]);
      if (i) strcat (flags," ");
    }
    strcat (flags,"} ");
  } */

  if(!item ) item = currentPopup;
  if(!item ) item = currentMail;
  if(!item || which != (unsigned)atol(item->text(MSGNO_POSITION))) item = currentItem();
  if(item && which != (unsigned)atol(item->text(MSGNO_POSITION)))
  {
		item = findItem(firstChild(),MSGNO_POSITION,which);
  }

  if(item && which == (unsigned)atol(item->text(MSGNO_POSITION)))
	{
		const char *oldflags;
		int recent=0;
		int unseen=0;

		oldflags = item->text(SORTFLAG_POSITION);
		if(oldflags)
		{
//			fprintf(stderr,"Oldflags : %c%c, Newflags : %c%c\n",oldflags[1],oldflags[2],newflags[1],newflags[2]);
//			fprintf(stderr,"Oldflags : '%s', Newflags : '%s'\n",oldflags,newflags);
			if(oldflags[1] < newflags[1]) { recent--;subRecent--; }//fprintf(stderr,"Recent --\n");
			if(oldflags[1] > newflags[1]) { recent++;subRecent++; }//fprintf(stderr,"Recent ++\n");
			if(oldflags[2] < newflags[2])
			{
				unseen--; //fprintf(stderr,"Unseen --\n");
				subUnseen--;
				if(oldflags[1] != '_') { recent--; subRecent--;} //fprintf(stderr,"Recent --\n");
			}
			if(oldflags[2] > newflags[2])
			{
				unseen++; //fprintf(stderr,"Unseen ++\n");
				subUnseen++;
				if(oldflags[1] != '_') { recent++; subRecent++;}//fprintf(stderr,"Recent ++\n");
			}
//			fprintf(stderr,"subRecent : %d, subUnseen : %d\n",subRecent,subUnseen);

			if(recent || unseen) emit signalSubStatus(currentFolder,recent,unseen);
		}
		item->setText(SORTFLAG_POSITION,newflags);	
  	item->setPixmap(FLAG_POSITION,*ret);
	}
}

//find a specified item
//-----------------------------------------------------------------------------
QListViewItem *NMFolderContent::findItem(QListViewItem *item,int column,long what)
{
	QListViewItem *retval = NULL;
	while(item && !retval)
	{
		retval = findItem(item->firstChild(),column,what);
		if(what == atol(item->text(column)))
		{
//			fprintf(stderr,"FOUND: '%ld' - %p\n",what,item);
			retval = item;
		}

		item=item->nextSibling();
	}
	return retval;
}

void NMFolderContent::slotReadSoFar()
{
	fprintf(stderr,"I say subRecent : %d, subUnseen : %d\n",subRecent,subUnseen);
	emit signalSubStatus(currentFolder,subRecent,subUnseen);
}

void NMFolderContent::slotStatus(NMClient *client,char *box,MAILSTATUS *status)
{
	emit signalStatus(client,box,status); // relay the status call
}

void NMFolderContent::slotStatusMsg(const char *in_msg)
{
	emit signalStatusMsg(in_msg); // relay the status call
}

void NMFolderContent::slotOverview (NMClient *client,unsigned long uid,OVERVIEW *ov)
{
//	fprintf(stderr,"OVERVIEW_HEADER : %ld\n",uid);
	char date[32];
	char from[128];
	char to[128];
	char size[32];
	char msg[32];
  char sortdate[32];
	QString subject = ov->subject;
	char *t;
  ADDRESS *adr;
  unsigned long msgno = client->uid2msgno(uid);

	if(!client || !ov) return;

	snprintf(msg,sizeof(msg),"%014ld",msgno);

  if(ov->date)
	{
	  MESSAGECACHE *selt;
		struct tm *tz = parseDate(ov->date,true);
				
		selt = client->parseDate(ov->date);
	  if(selt) {
//	     snprintf (sortdate,sizeof(sortdate),"%04d%02d%02d%02d%02d%02d",
//		     selt->year + BASEYEAR,selt->month,selt->day,selt->hours,selt->minutes,selt->seconds );
	     snprintf (sortdate,sizeof(sortdate),"%04d%02d%02d%02d%02d%02d",
		     tz->tm_year + BASEYEAR-70,tz->tm_mon+1,tz->tm_mday,tz->tm_hour,tz->tm_min,tz->tm_sec );
//	     snprintf (date,sizeof(date),"%02d.%02d.%04d",selt->day,selt->month,selt->year + BASEYEAR );
	     strftime (date,sizeof(date),"%d.%m.%Y",tz );
		} else  {
			strcpy(date,"??");
			strcpy(sortdate,"??");
		}
	} else {
		strcpy(date,"??");
		strcpy(sortdate,"??");
	}

				/* get first from address from envelope */
  for (adr = ov->from; adr && !adr->host; adr = adr->next);
  if (adr) {			/* if a personal name exists use it */
    if (!(t = adr->personal)) {
      snprintf (from,sizeof(from),"<%s@%s>",adr->mailbox,adr->host);
		} else {
			QString personal = adr->personal;
      snprintf (from,sizeof(from),"%s",personal.data(),adr->mailbox,adr->host);
		}
  } else {
		from[0] = 0x00;
	}

				/* get first to address from envelope */
  for (adr = ov->to; adr && !adr->host; adr = adr->next);
  if (adr) {			/* if a personal name exists use it */
    if (!(t = adr->personal)) {
      snprintf (to,sizeof(to),"<%s@%s>",adr->mailbox,adr->host);
		} else {
			QString personal = adr->personal;
      snprintf (to,sizeof(to),"%s",personal.data(),adr->mailbox,adr->host);
		}
  } else {
		to[0] = 0x00;
	}

  snprintf (size,sizeof(size),"%ld", ov->optional.octets);
	NMFolderContentItem *item;
	if(isSaveSend)
		item = new NMFolderContentItem(this, "",date,decodeMIME2(to),decodeMIME2(subject),size,msg,sortdate);
	else
		item = new NMFolderContentItem(this, "",date,decodeMIME2(from),decodeMIME2(subject),size,msg,sortdate);
	if((unsigned)atol(msg) == client->getUnSeen()) setCurrentItem(item);
	slotFlags(client,msgno,item);
//	if(isSaveSend && !ov->date)
//  	item->setPixmap(FLAG_POSITION,pixFlagged);
	
	item->setSelectable( false );
	if(lastMessageNo < (unsigned)atol(msg)) lastMessageNo = (unsigned)atol(msg);
}

NMFolderContentItem::NMFolderContentItem( QListView * parent, const char *c1, const char *c2 = 0, const char *c3 = 0, const char *c4 = 0, const char *c5 = 0, const char *c6 = 0, const char *c7 = 0, const char *c8 = 0 )
 : QListViewItem(parent,c1,c2,c3,c4,c5,c6,c7,c8) {
}

NMFolderContentItem::~NMFolderContentItem() {};

const char *NMFolderContentItem::key(int column,bool order) const
{
	QString append;

	switch (column) {
		case SIZE_POSITION :
			append.sprintf("%9.9ld",atol(QListViewItem::text(column)));
			break;
    case DATE_POSITION :
			column = SORTDATE_POSITION;
		  append.setStr(QListViewItem::key(column,order));
			break;
    case FLAG_POSITION :
			column = SORTFLAG_POSITION;
		  append.setStr(QListViewItem::key(column,order));
			break;
    default:
		  append.setStr(QListViewItem::key(column,order));
      break;
  }
	append += QListViewItem::text(SORTDATE_POSITION);
	((QListViewItem *)this)->setText(TEMPSORT_POSITION,append);
	
	return QListViewItem::text(TEMPSORT_POSITION);
}

void NMFolderContent::resizeEvent(QResizeEvent *event)
{
		int width = 0;

		setColumnWidth( FROM_POSITION, 0);
		setColumnWidth( SUBJECT_POSITION, 0);
		for(int i=0;i<QListView::header()->count();i++) {
			width += columnWidth( i );
		}
		width = this->width() - width;
		if(verticalScrollBar() && verticalScrollBar()->isVisible()) width -= verticalScrollBar()->width();
		setColumnWidth( FROM_POSITION, width * 0.33);
		setColumnWidth( SUBJECT_POSITION, width - columnWidth( FROM_POSITION));

		width = this->width();
		if(verticalScrollBar() && verticalScrollBar()->isVisible()) width -= verticalScrollBar()->width();
		resizeContents(width,contentsHeight());
		QListView::resizeEvent( event );
}

void NMFolderContent::slotRightButtonClicked(QListViewItem *item,const QPoint &point,int column)
{
//  BODY msgPart;
  QPopupMenu *menu;
	QPopupMenu *folder;
	QPopupMenu *mark;

	if(my_client->isLocked()) return;
	currentPopup = item;
  {
    menu = new QPopupMenu();
    folder = new QPopupMenu();
    mark = new QPopupMenu();
		lastFolderSelected = -1;
//		emit signalMakeMenu( folder ,this,SLOT(slotMMove(int)));
		emit signalMakeMenu( folder ,this,SLOT(slotFolderSelected(int)));

//	  connect( folder, SIGNAL(activated(int)), SLOT(slotMMove(int)) );
	  connect( folder, SIGNAL(activated(int)), SLOT(slotFolderSelected(int)) );


	  mark->setCheckable(true);
    mark->insertItem(i18n("Deleted"), this, SLOT(slotMDelete()),0,1);
    mark->insertItem(i18n("Flagged"), this, SLOT(slotMFlag()),0,2);
		if(isSaveSend)
	    mark->insertItem(i18n("Send"), this, SLOT(slotMAnswered()),0,3);
	  else
  	  mark->insertItem(i18n("Answered"), this, SLOT(slotMAnswered()),0,3);
    mark->insertItem(i18n("Unseen"), this, SLOT(slotMSeen()),0,4);
		{
      const char *flags = item->text(SORTFLAG_POSITION);
      if(flags) {
	  		if(flags[2] == 'U') mark->setItemChecked(4, TRUE);
	  		if(flags[3] == 'F') mark->setItemChecked(2, TRUE);
	  		if(flags[4] == 'A') mark->setItemChecked(3, TRUE);
	  		if(flags[5] == 'D') mark->setItemChecked(1, TRUE);
	  	}
		}
    menu->insertItem(i18n("Mark Message"),mark );
    menu->insertItem(i18n("Move to ..."),folder );
		if(isSaveSend) {
    	menu->insertItem(i18n("Open in Composer"), this, SLOT(slotOpenComposer()));
    }
    menu->insertItem(i18n("Open in Viewer"), this, SLOT(slotOpenViewer()));
		menu->exec(point,0);
		delete menu;
		delete folder;
		delete mark;
		// if we call slotMMove instead of slotFolderSelected the KFileDialog
		// will popup and disappear immediately
		if(lastFolderSelected >= 0) slotMMove(lastFolderSelected);
		lastFolderSelected = -1;
  }
	currentPopup = NULL;
}

void NMFolderContent::slotMDelete()
{
	QListViewItem *item = currentPopup;
	if(!item) item = currentItem();
	if(!item) return;

	if(my_client->isLocked()) return;
	setFocus();
	if(my_client)
	{
		int msgno;
		msgno = atoi( item->text(MSGNO_POSITION) );
		my_client->toggleflagMsg(msgno,"\\DELETED");
	}
	if(!currentPopup && item->itemBelow())
	{
		setCurrentItem(item->itemBelow());
		ensureItemVisible(currentItem());
	}
}

void NMFolderContent::slotMFlag()
{
	QListViewItem *item = currentPopup;
	if(!item) item = currentItem();
	if(!item) return;

	if(my_client->isLocked()) return;
	setFocus();
	if(my_client)
	{
		int msgno;
		msgno = atoi( item->text(MSGNO_POSITION) );
		my_client->toggleflagMsg(msgno,"\\FLAGGED");
	}
	if(!currentPopup && item->itemBelow())
	{
		setCurrentItem(item->itemBelow());
		ensureItemVisible(currentItem());
	}
}

void NMFolderContent::slotMAnswered()
{
	QListViewItem *item = currentPopup;
	if(!item) item = currentItem();
	if(!item) return;

	if(my_client->isLocked()) return;
	setFocus();
	if(my_client)
	{
		int msgno;
		msgno = atoi( item->text(MSGNO_POSITION) );
		my_client->toggleflagMsg(msgno,"\\ANSWERED");
	}
	if(!currentPopup && item->itemBelow())
	{
		setCurrentItem(item->itemBelow());
		ensureItemVisible(currentItem());
	}
}

void NMFolderContent::slotMSeen()
{
	QListViewItem *item = currentPopup;
	if(!item) item = currentItem();
	if(!item) return;

	if(my_client->isLocked()) return;
	setFocus();
	if(my_client)
	{
		int msgno;
		msgno = atoi( item->text(MSGNO_POSITION) );
		my_client->toggleflagMsg(msgno,"\\SEEN");
	}
	if(!currentPopup && item->itemBelow())
	{
		setCurrentItem(item->itemBelow());
		ensureItemVisible(currentItem());
	}
}

//search folder
/*void NMFolderContent::slotFSearch()
{
	QListViewItem *item = currentPopup;
	if(!item) item = currentItem();
	if(!item) return;

	if(my_client->isLocked()) return;
	setFocus();
	if(my_client)
	{
		NMSearch search;
		enum NMSearch::eActions todo;
		
		search.show();
		
		int i=0;
		QString results;
		my_client->searchSimple(search.getSearchString());
		todo = search.getAction();
		results = ((NMClient *)my_client)->searchResult();
		while(!results.isEmpty())
		{
			i = results.find(" ");
			
			switch(todo) {
				case NMSearch::doFlag :
					my_client->flagMsg(results.left(i).data(),"\\FLAGGED");
					break;
				case NMSearch::doUnFlag :
					my_client->unflagMsg(results.left(i).data(),"\\FLAGGED");
					break;
				case NMSearch::doToggleFlag :
					my_client->toggleflagMsg(atol(results.left(i).data()),"\\FLAGGED");
					break;
				case NMSearch::doDelete :
					my_client->flagMsg(results.left(i).data(),"\\DELETED");
					break;
			}
			results.remove(0,i+1);
		}
	}			
	return;
} */

bool NMFolderContent::moveMsg(unsigned long msg,const char *select)
{
// this works only if the destination belongs to the same user
//	my_client->moveMessage(atol(item->text(MSGNO_POSITION)),select);
	mailMessage *mMsg = my_client->fetchStructure(msg);
	mMsg->fullFetch(my_client,msg);

	QString body = bodyQString(mMsg,true);
	QString flag_txt;
	
	int flags = my_client->getflagMsg(msg);

 	if(!(flags & fSEEN)) flag_txt += "\\UNSEEN ";
 	if(flags & fFLAGGED) flag_txt += "\\FLAGGED ";
 	if(flags & fANSWERED) flag_txt += "\\ANSWERED ";
 	if(flags & fDELETED) flag_txt += "\\DELETED";

	if(my_client->appendMessage(select,body.data(),body.length(),flag_txt.data(), mMsg->dateStr("%a, %d %b %Y %T %z",false )))
	{
		my_client->flagMsg(msg,"\\DELETED");
		if( !strcmp(select,my_client->getBox()) ) slotReRead();
		return true;
	}
	return false;
}

void NMFolderContent::slotFolderSelected(int value)
{
	lastFolderSelected = value;
}


void NMFolderContent::slotMMove(int value)
{
//	QString fileName;
//	QString select;
	QListViewItem *item = currentPopup;
	if(!item) item = currentItem();
	if(!item) return;

	if(my_client->isLocked()) return;
	setFocus();
	
	if(moveMsg(atol(item->text(MSGNO_POSITION)),gFolders->getMovePath(value)))
		if(!currentPopup && item->itemBelow())
		{
			setCurrentItem(item->itemBelow());
			ensureItemVisible(currentItem());
		}
}

void NMFolderContent::slotDoubleClicked(QListViewItem *item)
{
	if(my_client->isLocked()) return;
	if(currentMail)
	{
		currentMail->setSelected( false );
		currentMail->repaint();
	}
	currentMail = item;	
	if(currentMail)
	{
		setCurrentItem( currentMail );
		ensureItemVisible( currentMail );
		currentMail->setSelected( true );
		currentMail->repaint();
	}
}

void NMFolderContent::reOpen()
{
	if(currentMail) slotDoubleClicked(currentMail);
  if(currentMail) emit doubleClicked(currentMail);
}
void NMFolderContent::slotMessageNext()
{
	QListViewItem *item = NULL;

	if(currentMail) item = currentMail->itemBelow();
	if(!item && !currentMail) item = firstChild();
	if(!item) item = currentMail;
	if(item) slotDoubleClicked(item);
  if(item) emit doubleClicked(item);
}

void NMFolderContent::slotMessageNextUnread()
{
	QListViewItem *item = NULL;
	const char *flag;

	if(currentMail)
	{
 		if(currentMail->text(SORTFLAG_POSITION)[2] != '_' ) item = currentMail;
 		if(currentMail->text(SORTFLAG_POSITION)[2] == '_' ) item = currentMail->itemBelow();
	}
	if(!item) item = firstChild();
	while(item)
	{
		flag = item->text(SORTFLAG_POSITION);
		if(flag && flag[2] != '_') break;
		item = item->itemBelow();
	}
	if(!item)
	{
		//go find the next folder with unread messages
		 item = currentMail;
//		 emit signalMoveToUnreadFolder();
		if(gFolders)	gFolders->moveToUnseenFolder();
	} else {
		slotDoubleClicked(item);
  	emit doubleClicked(item);
  }
}

void NMFolderContent::slotMessagePrev()
{
	QListViewItem *item = NULL;

	if(currentMail) item = currentMail->itemAbove();
	if(!item && !currentMail)
	{
		item = firstChild();
		while(item && item->itemBelow()) item = item->itemBelow();
	}
	if(!item) item = currentMail;
	if(item) slotDoubleClicked(item);
  if(item) emit doubleClicked(item);
}

void NMFolderContent::slotMessagePrevUnread()
{
	QListViewItem *item = NULL;
	const char *flag;

	if(currentMail) item = currentMail->itemAbove();
	if(!item)
	{
		item = firstChild();
		while(item && item->itemBelow()) item = item->itemBelow();
	}
	while(item)
	{
		flag = item->text(SORTFLAG_POSITION);
		if(flag && flag[2] != '_') break;
		item = item->itemAbove();
	}
	if(!item) item = currentMail;
	if(item) slotDoubleClicked(item);
  if(item) emit doubleClicked(item);
}

void NMFolderContent::scrollUp()
{
	QListViewItem *item = currentItem();
	
	if(item && item->itemAbove())
	{
		setCurrentItem(item->itemAbove());
		ensureItemVisible(currentItem());
	}	
	setFocus();
}

void NMFolderContent::scrollDown()
{
	QListViewItem *item = currentItem();
	
	if(item && item->itemBelow())
	{
		setCurrentItem(item->itemBelow());
		ensureItemVisible(currentItem());
	}	
	setFocus();
}

void NMFolderContent::collapse()
{
	QListViewItem *item = currentMail;
	if(!item) item = currentItem();

	if(item)
	{
		if(item->isExpandable() || (item->childCount() > 0)) item->setOpen(false);
	}
	setFocus();
}

void NMFolderContent::expand()
{
	QListViewItem *item = currentMail;
	if(!item) item = currentItem();

	if(item)
	{
		if(item->isExpandable() || (item->childCount() > 0)) item->setOpen(true);
	}
	setFocus();
}

void NMFolderContent::slotReply()
{
	QListViewItem *item = currentItem();
	if(item && my_client)
	{
		unsigned long curmsg = atoi(item->text(MSGNO_POSITION));
		mailMessage *mMsg = my_client->fetchStructure(curmsg);
		mMsg->fullFetch(my_client,curmsg);
		KConfig *config = kapp->getConfig();
		config->setGroup("Message");
		QString sReplyStr = formatString(config->readEntry("phrase-reply",i18n("On %D, you wrote:")),mMsg);
		QString sIndent = formatString(config->readEntry("indent-prefix",i18n(">%_")),mMsg);

		KMComposeWin *composer = new KMComposeWin( mMsg->makeReply(sReplyStr,sIndent) );

		my_client->flagMsg(item->text(MSGNO_POSITION),"\\ANSWERED");
		delete mMsg;
		composer->show();
	}
}
//-----------------------------------------------------------------------------
void NMFolderContent::slotReplyAll()
{
	QListViewItem *item = currentItem();
	if(item && my_client)
	{
		unsigned long curmsg = atoi(item->text(MSGNO_POSITION));
		mailMessage *mMsg = my_client->fetchStructure(curmsg);
		mMsg->fullFetch(my_client,curmsg);
		KConfig *config = kapp->getConfig();
		config->setGroup("Message");

		QString sReplyStr = formatString(config->readEntry("phrase-reply-all",i18n("On %D, %F wrote:")),mMsg);
		QString sIndent = formatString(config->readEntry("indent-prefix",i18n(">%_")),mMsg);
		KMComposeWin *composer = new KMComposeWin( mMsg->makeReplyAll(sReplyStr,sIndent) );

		my_client->flagMsg(item->text(MSGNO_POSITION),"\\ANSWERED");
		delete mMsg;
		composer->show();
	}
}
//-----------------------------------------------------------------------------
void NMFolderContent::slotForwardInline()
{
	QListViewItem *item = currentItem();
	if(item && my_client)
	{
		unsigned long curmsg = atoi(item->text(MSGNO_POSITION));
		mailMessage *mMsg = my_client->fetchStructure(curmsg);
		mMsg->fullFetch(my_client,curmsg);
		KConfig *config = kapp->getConfig();
		config->setGroup("Message");

		QString sReplyStr = formatString(config->readEntry("phrase-forward",i18n("Forwarded Message:")),mMsg);
		QString sIndent = formatString(config->readEntry("indent-prefix",i18n(">%_")),mMsg);
		KMComposeWin *composer = new KMComposeWin( mMsg->makeForwardInline(sReplyStr,sIndent) );

		delete mMsg;
		composer->show();
	}
}
//-----------------------------------------------------------------------------
void NMFolderContent::slotForwardAttach()
{
	QListViewItem *item = currentItem();
	if(item && my_client)
	{
		unsigned long curmsg = atoi(item->text(MSGNO_POSITION));
		mailMessage *mMsg = my_client->fetchStructure(curmsg);
		mMsg->fullFetch(my_client,curmsg);

		KMComposeWin *composer = new KMComposeWin( mMsg->makeForwardAttach() );
		delete mMsg;
		composer->show();
	}
}

//-----------------------------------------------------------------------------
void NMFolderContent::slotOpenComposer()
{
	QListViewItem *item = currentPopup;
	
	if(item && my_client)
	{
		unsigned long curmsg = atoi(item->text(MSGNO_POSITION));
		mailMessage *mMsg = my_client->fetchStructure(curmsg);
		mMsg->fullFetch(my_client,curmsg);

		KMComposeWin *composer = new KMComposeWin( mMsg );
		composer->show();
	}
}

//-----------------------------------------------------------------------------
void NMFolderContent::slotOpenViewer()
{
	QListViewItem *item = currentPopup;
	
	if(item && my_client)
	{
		QString caption;
		
		unsigned long curmsg = atoi(item->text(MSGNO_POSITION));
		mailMessage *mMsg = my_client->fetchStructure(curmsg);
		mMsg->fullFetch(my_client,curmsg);
		caption.sprintf("%s : %ld",my_client->getBox(),curmsg);
	
	
 		KTMainWindow *master = new KTMainWindow();			  		
		NMGlobalFolders::initPreMenuBar(master);			  		
		NMMailView *new_view = new NMMailView(master);
		new_view->slotMail(mMsg,caption.data());
 		master->setView(new_view);
		NMGlobalFolders::initPostMenuBar(master);			  		
 		master->show();
		new_view->show();
	
	}
}

//-----------------------------------------------------------------------------
const QString NMFolderContent::formatString(const QString aStr,mailEnvelope *env) const
{
  QString result, str;
  const char* pos;
  char ch, cstr[64];
  tm *time;
  int i;

  if (aStr.isEmpty()) return aStr;

  for (pos=aStr.data(); *pos; )
  {
    ch = *pos++;
    if (ch=='%')
    {
      ch = *pos++;
      switch (ch)
      {
      case 'D':
	time = env->messageDate();
	strftime(cstr, 63, i18n("%a, %d %b %Y"), time);
	result += i18ndate(cstr);
	break;
      case 'F':
	result += env->messageFrom();
	break;
      case 'f':
	str = env->messageFrom();
	for (i=0; str[i]>' '; i++)
	  ;
	for (; i < (int)str.length() && str[i] <= ' '; i++)
	  ;
	result += str[0];
	if (str[i]>' ') result += str[i];
	else if (str[1]>' ') result += str[1];
	break;
      case 'S':
	result += env->messageSubject();
	break;
      case '_':
	result += ' ';
	break;
      case '%':
	result += '%';
	break;
      default:
	result += '%';
	result += ch;
	break;
      }
    }
    else result += ch;
  }
  return result;
}

void NMFolderContent::slotGCreate()
{
//add progressnotify	
	QListViewItem *item = currentPopup;
	if(!item) item = currentItem();
	if(!item) return;

	if(my_client->isLocked()) return;
	setFocus();
	if(my_client)
	{
		NMSearch search;
		
		search.show();
		if(!search.result()) return;		
		int i=0;
		QString results;
//		debug("NMFolderContent::slotGCreate searching for %s",search.getSearchString());
		my_client->searchSimple(search.getSearchString());
		results = ((NMClient *)my_client)->searchResult();
//		debug("NMFolderContent::slotGCreate found %s",results.data());
		while(!results.isEmpty())
		{
			i = results.find(" ");
//add progressnotify			
			debug("Processing '%s'",results.left(i).data());
			my_client->flagMsg(results.left(i).data(),"\\FLAGGED");
			results.remove(0,i+1);
		}
	}			
}

void NMFolderContent::slotGClear()
{
//add progressnotify	
	QListViewItem *iterator = firstChild();
	
	while(iterator)
	{
		if(iterator->text(SORTFLAG_POSITION)[3] == 'F')
			my_client->unflagMsg(iterator->text(MSGNO_POSITION),"\\FLAGGED");
		iterator = iterator->nextSibling();
	}
}

void NMFolderContent::slotGDelete()
{
//add progressnotify	
	QListViewItem *iterator = firstChild();
	
	while(iterator)
	{
		if(iterator->text(SORTFLAG_POSITION)[3] == 'F')
			my_client->flagMsg(iterator->text(MSGNO_POSITION),"\\DELETED");
		iterator = iterator->nextSibling();
	}
}

void NMFolderContent::slotGUndelete()
{
//add progressnotify	
	QListViewItem *iterator = firstChild();
	
	while(iterator)
	{
		if(iterator->text(SORTFLAG_POSITION)[3] == 'F')
			my_client->unflagMsg(iterator->text(MSGNO_POSITION),"\\DELETED");
		iterator = iterator->nextSibling();
	}
}

void NMFolderContent::slotGPrint()
{
//add progressnotify	
	NMMailView *myView = new NMMailView();
	QListViewItem *iterator = firstChild();
	
	QPrinter printer;
  if ( printer.setup( 0 ) )
  {
	
		while(iterator)
		{
			if(iterator->text(SORTFLAG_POSITION)[3] == 'F')
			{
				myView->slotMail(my_client,atoi(iterator->text(MSGNO_POSITION)));
//				printer.setup( 0 );
				myView->printMsg(&printer);
			}
			iterator = iterator->nextSibling();
		}
	}
	
	delete(myView);
}

void NMFolderContent::slotGMoveFile()
{debug("NMFolderContent::slotGMoveFile() is deprecated\n");}

void NMFolderContent::slotGMoveFolder(int value)
{
//add progressnotify	
	QListViewItem *iterator = firstChild();
	
	QString select = gFolders->getMovePath(value);
	fprintf(stderr,"Group selected [%d] : %s\n",value,select.data());
	
	while(iterator)
	{
		if(iterator->text(SORTFLAG_POSITION)[3] == 'F')
			moveMsg(atol(iterator->text(MSGNO_POSITION)),select);
		iterator = iterator->nextSibling();
	}
}

void NMFolderContent::setAccelerators()
{
	QAccel *my_Accel = new QAccel( topLevelWidget() );

	my_Accel->insertItem( Key_N, ID_MESSAGE_NEXT );
	my_Accel->connectItem( ID_MESSAGE_NEXT, this,	SLOT( slotMessageNext( void )) );
  my_Accel->insertItem( Key_Plus, ID_MESSAGE_NEXTUNREAD );
	my_Accel->connectItem( ID_MESSAGE_NEXTUNREAD, this, SLOT( slotMessageNextUnread( void )) );
  my_Accel->insertItem( Key_P, ID_MESSAGE_PREV );
	my_Accel->connectItem( ID_MESSAGE_PREV, this, SLOT( slotMessagePrev( void )) );
  my_Accel->insertItem( Key_Minus, ID_MESSAGE_PREVUNREAD );
	my_Accel->connectItem(ID_MESSAGE_PREVUNREAD	,this,SLOT( slotMessagePrevUnread(void )) );


  my_Accel->insertItem( Key_R, ID_MESSAGE_REPLY );
	my_Accel->connectItem( ID_MESSAGE_REPLY, this,SLOT( slotReply(void )) );
  my_Accel->insertItem( Key_A, ID_MESSAGE_REPLYALL );
	my_Accel->connectItem(ID_MESSAGE_REPLYALL, this,SLOT( slotReplyAll(void )) );
  my_Accel->insertItem(  Key_I ,ID_MESSAGE_FORWARDINLINE );
	my_Accel->connectItem(ID_MESSAGE_FORWARDINLINE, this,SLOT( slotForwardInline(void )) );
  my_Accel->insertItem( Key_T , ID_MESSAGE_FORWARDATTACHED );
	my_Accel->connectItem(ID_MESSAGE_FORWARDATTACHED, this,SLOT( slotForwardAttach(void )) );

  my_Accel->insertItem( Key_D , ID_MESSAGE_DELETE );
	my_Accel->connectItem(ID_MESSAGE_DELETE, this,SLOT( slotMDelete(void )) );
  my_Accel->insertItem( Key_F ,ID_MESSAGE_FLAG );
	my_Accel->connectItem(ID_MESSAGE_FLAG, this,SLOT( slotMFlag(void )) );
  my_Accel->insertItem( Key_W , ID_MESSAGE_ANSWER );
	my_Accel->connectItem(ID_MESSAGE_ANSWER, this,SLOT( slotMAnswered(void )) );
  my_Accel->insertItem( Key_S , ID_MESSAGE_SEEN );
	my_Accel->connectItem(ID_MESSAGE_SEEN, this,SLOT( slotMSeen(void )) );

  my_Accel->insertItem( Key_X, ID_FOLDER_EXPUNGE );
	my_Accel->connectItem(ID_FOLDER_EXPUNGE, this, SLOT( slotFolderExpunge(void) ) );

  QPopupMenu *message_menu = new QPopupMenu();
  message_menu->insertItem( i18n("&Next"), ID_MESSAGE_NEXT );
	message_menu->setAccel(Key_N ,ID_MESSAGE_NEXT);
  message_menu->insertItem( i18n("Next unread"), ID_MESSAGE_NEXTUNREAD );
	message_menu->setAccel(Key_Plus ,ID_MESSAGE_NEXTUNREAD);
  message_menu->insertItem( i18n("&Previous"), ID_MESSAGE_PREV );
	message_menu->setAccel(Key_P ,ID_MESSAGE_PREV);
  message_menu->insertItem( i18n("Previous unread"), ID_MESSAGE_PREVUNREAD );
	message_menu->setAccel(Key_Minus ,ID_MESSAGE_PREVUNREAD);
	message_menu->insertSeparator();
  message_menu->insertItem(Icon("fileprint.xpm"), i18n("&Print..."), ID_FILE_PRINT );
	message_menu->insertSeparator();
  message_menu->insertItem( i18n("&Reply"), ID_MESSAGE_REPLY );
	message_menu->setAccel(Key_R ,ID_MESSAGE_REPLY);
  message_menu->insertItem( i18n("Reply to &all"), ID_MESSAGE_REPLYALL );
	message_menu->setAccel(Key_A ,ID_MESSAGE_REPLYALL);
  message_menu->insertItem( i18n("Forward &Inline"), ID_MESSAGE_FORWARDINLINE );
	message_menu->setAccel(Key_I ,ID_MESSAGE_FORWARDINLINE);
  message_menu->insertItem( i18n("Forward A&ttached"), ID_MESSAGE_FORWARDATTACHED );
	message_menu->setAccel(Key_T ,ID_MESSAGE_FORWARDATTACHED);
	message_menu->insertSeparator();
  message_menu->insertItem( i18n("&Delete"), ID_MESSAGE_DELETE );
	message_menu->setAccel(Key_D ,ID_MESSAGE_DELETE);
  message_menu->insertItem( i18n("&Flag"), ID_MESSAGE_FLAG );
	message_menu->setAccel(Key_F ,ID_MESSAGE_FLAG);
  message_menu->insertItem( i18n("Ans&wer"), ID_MESSAGE_ANSWER );
	message_menu->setAccel(Key_W ,ID_MESSAGE_ANSWER);
  message_menu->insertItem( i18n("&Seen"), ID_MESSAGE_SEEN );
	message_menu->setAccel(Key_S ,ID_MESSAGE_SEEN);
	message_menu->insertSeparator();

  move_folder_menu = new QPopupMenu();
  connect( move_folder_menu, SIGNAL(activated(int)), SLOT(slotMMove(int)) );
  message_menu->insertItem(i18n("&Move to ..."), move_folder_menu, ID_MESSAGE_MOVEFOLDER );
  ///////////////////////////////////////////////////////////////////
  // menuBar entry group_menu
	QPopupMenu *group_menu = new QPopupMenu();
  group_menu->insertItem(i18n("Create"), ID_GROUP_CREATE);
  group_menu->insertItem(i18n("Clear"), ID_GROUP_CLEAR);
	group_menu->insertSeparator();
  group_menu->insertItem(i18n("Delete"), ID_GROUP_DELETE);
  group_menu->insertItem(i18n("Undelete"), ID_GROUP_UNDELETE);
  group_menu->insertItem(i18n("Print"), ID_GROUP_PRINT);

  group_folder_menu = new QPopupMenu();
  connect( group_folder_menu, SIGNAL(activated(int)), SLOT(slotGMoveFolder(int)) );
  group_menu->insertItem(i18n("&Move to ..."), group_folder_menu, ID_GROUP_MOVEFOLDER );
	
	if(topLevelWidget()->isA("KTMainWindow")
	|| topLevelWidget()->isA("KImap"))
	{
		((KTMainWindow *)topLevelWidget())->menuBar()->insertItem(i18n("&Message"), message_menu);
		((KTMainWindow *)topLevelWidget())->menuBar()->insertItem(i18n("&Group"), group_menu);
	}

  connect(group_menu,SIGNAL(activated(int)),SLOT(commandCallback(int)));
  connect(group_menu,SIGNAL(highlighted(int)),SLOT(statusCallback(int)));

  connect(message_menu,SIGNAL(activated(int)),SLOT(commandCallback(int)));
  connect(message_menu,SIGNAL(highlighted(int)),SLOT(statusCallback(int)));

	NMStatusBar::connectToStatusBar(this,this);	
}
/*
const QString &NMFolderContent::getMovePath(int id)
{
	static QString fileName;
	
	realFolder *selected = gFolders->getAt(id);
	if(!selected)
	{
	  fileName = KFileDialog::getSaveFileName(QDir::currentDirPath(), "*", NULL);
		if(!fileName.isEmpty())
		{
			if(QFile::exists(fileName))
			{
				QString str;
	      str.sprintf(i18n( "File %s exists."),fileName.data());
				
	      int rc = QMessageBox::information(NULL, i18n("Information"),
		   	str, i18n("&Append"), i18n("&Overwrite"), i18n("&Cancel"), 0, 2);
      	debug("QMessageBox::result %d",rc);
      	switch(rc)
      	{
      		case 1:
					{      		
      			QFile file(fileName);
      		  if (file.open(IO_Raw|IO_WriteOnly|IO_Truncate))
					  {
						  file.writeBlock("", 0);
						} else {
							debug("Error opening %s for writing!",fileName.data());
						}
					}
      		case 0:
      			break;
					default:
						fileName = "";
						break;
				}
			} else {
				{      		
     			QFile file(fileName);
     		  if (file.open(IO_Raw|IO_WriteOnly))
				  {
					  file.writeBlock("", 0);
					} else {
						debug("Error opening %s for writing!",fileName.data());
					}
				}
			}
		} else {
			fileName = "";
		}
	} else {
		fileName = selected->assembleUWIFolder();
	}
	return fileName;
}
*/
void NMFolderContent::statusCallback(int id_){
//	debug("NMFolderContent::statusCallback(%d)",id_);
  switch (id_){
		case ID_GROUP_MOVEFOLDER:
			emit signalMakeMenu( group_folder_menu ,this,SLOT(slotGMoveFolder(int)));
			break;
		case ID_MESSAGE_MOVEFOLDER:
			emit signalMakeMenu( move_folder_menu ,this,SLOT(slotMMove(int)));
			break;
  }
}

void NMFolderContent::commandCallback(int id_){
//	debug("NMFolderContent::commandCallback(%d)",id_);
  switch (id_){
		case ID_FOLDER_EXPUNGE:
			slotFolderExpunge();
			break;
		case ID_FOLDER_STATUS:
			if(gFolders) gFolders->slotStatusLookup(true);
			break;
		case ID_FOLDER_CHECK:
			if(gFolders) gFolders->slotStatusLookup(false);
			break;


		case ID_MESSAGE_NEXT:
			slotMessageNext();
			break;
		case ID_MESSAGE_NEXTUNREAD:
			slotMessageNextUnread();
			break;
		case ID_MESSAGE_PREV:
			slotMessagePrev();
			break;
		case ID_MESSAGE_PREVUNREAD:
			slotMessagePrevUnread();
			break;
		case ID_MESSAGE_REPLY:
			slotReply();
			break;
		case ID_MESSAGE_REPLYALL:
			slotReplyAll();
			break;
		case ID_MESSAGE_FORWARDINLINE:
			slotForwardInline();
			break;
		case ID_MESSAGE_FORWARDATTACHED:
			slotForwardAttach();
			break;
		case ID_MESSAGE_DELETE:
			slotMDelete();
			break;
		case ID_MESSAGE_FLAG:
			slotMFlag();
			break;
		case ID_MESSAGE_SEEN:
			slotMSeen();
			break;
		case ID_MESSAGE_ANSWER:
			slotMAnswered();
			break;
		case ID_MESSAGE_MOVEFOLDER:
//			slotMMove(currentMail);
			break;

		case ID_GROUP_CREATE:
			slotGCreate();
			break;
		case ID_GROUP_CLEAR:
			slotGClear();
			break;
		case ID_GROUP_DELETE:
			slotGDelete();
			break;
		case ID_GROUP_UNDELETE:
			slotGUndelete();
			break;
		case ID_GROUP_PRINT:
			slotGPrint();
			break;
		case ID_GROUP_MOVEFOLDER:
			//handled by the submenus
			break;
  }
}

void NMFolderContent::checkForQueuedMessages()
{
	debug("NMFolderContent::checkForQueuedMessages - start");
}
