/***************************************************************************
                          nmfoldercontent.h  -  description
                             -------------------
    begin                : Sat Oct 16 1999
    copyright            : (C) 1999 by Sven Carstens
    email                : s.carstens@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#ifndef NMFOLDERCONTENT_H
#define NMFOLDERCONTENT_H

#include <qpixmap.h>
#include <qwidget.h>
#include <qlistview.h>
#include "nmmailview.h"
#include "nmclient.h"

/**
  *@author Sven Carstens
  */
//#include <qdialog.h>
//#include <qlabel.h>
//#include <qpushbutton.h>

#define FLAG_POSITION 0
#define DATE_POSITION 1
#define FROM_POSITION 2
#define SUBJECT_POSITION 3
#define SIZE_POSITION 4
#define MSGNO_POSITION 5
#define SORTDATE_POSITION 6
#define SORTFLAG_POSITION 7
#define TEMPSORT_POSITION 8

class QPixmap;
class QPushButton;
class QLabel;
class QDialog;
class QPopupMenu;

class NMFolderContentItem : public QListViewItem {
public:
	NMFolderContentItem( QListView * parent, const char *, const char * = 0, const char * = 0, const char * = 0, const char * = 0, const char * = 0, const char * = 0, const char * = 0 );
  ~NMFolderContentItem();
  const char *key(int,bool) const;
//  const char *text(int) const;
};

class NMFolderContent : public QListView  {
   Q_OBJECT
public: 
	NMFolderContent(QWidget *parent=0, const char *name=0);
	~NMFolderContent();
	void setDefaultViewer(NMMailView *);
	void header(NMClient *stream,long msgno);
	void displayFlags (unsigned long which=0);
	void reOpen();

	virtual void setSorting(int column,bool increasing = true);

signals:
  void currentChanged(QListViewItem *);
	void signalStatusMsg(const char *);
	void signalSubStatus(int,int,int);
	void signalMakeMenu(QPopupMenu *,const QObject *,const char *);
	void signalMenuText(int,char **);

	void signalProgressNotify(enum mailProgress,unsigned long,const char *,bool &);	
//	void signalSignOn(const char *,NMFolderContent *);
//	void signalSignOff(const char *,NMFolderContent *);
	void signalSignOn(int,NMFolderContent *);
	void signalSignOff(int,NMFolderContent *);
	
signals: // relayed signals
//	void signalOverview(NMClient *,unsigned long uid,OVERVIEW *ov);
//	void signalFlags(NMClient *,unsigned long number);
//	void signalExists(NMClient *,unsigned long);
//	void signalExpunged(NMClient *,unsigned long);
	void signalStatus(NMClient *,char *,MAILSTATUS *);
//	void signalList(NMClient *,int,char *,long);
//	void signalStatusMsg(const char *);

protected:
	void closeEvent(QCloseEvent *);
	
private slots:	
	void slotFolder(NMClient *,int);

public slots:
	void statusCallback(int);
	void commandCallback(int);
	void checkForQueuedMessages();	
	void slotReadSoFar();
	void slotDoubleClicked(QListViewItem *);
	void slotReRead();
	void slotRefresh();
	void slotFolderExpunge();
	void slotStatusMsg(const char *); //relay
//	void slotStatus(NMClient *);
	void slotStatus(NMClient *,char *,MAILSTATUS *); //relay
	void slotFolder(int);
	void slotOverview(NMClient *,unsigned long,OVERVIEW *);
	void slotFlags(NMClient *,unsigned long,QListViewItem *which);
	void slotFlags(NMClient *,unsigned long);
	void slotExpunged(NMClient *,unsigned long);
	void slotMail(QListViewItem *);
	
	void slotRightButtonClicked(QListViewItem *,const QPoint &,int);
	void slotMDelete(void);
	void slotMFlag(void);
	void slotMAnswered(void);
	void slotMSeen(void);
//	void slotFSearch(void);
	void slotMMove(int);
	void slotFolderSelected(int);
	void slotMessageNext();
	void slotMessageNextUnread();
	void slotMessagePrev();
	void slotMessagePrevUnread();

	void slotReply();
	void slotReplyAll();
	void slotForwardInline();
	void slotForwardAttach();
	void slotOpenComposer();
	void slotOpenViewer();
	
	void slotGCreate();
	void slotGClear();
	void slotGDelete();
	void slotGUndelete();
	void slotGPrint();
	void slotGMoveFile();
	void slotGMoveFolder(int);
	
	void readConfig();
	void writeConfig(bool withSync);

	void scrollDown();
	void scrollUp();
	void expand();
	void collapse();

protected:
	virtual void timerEvent(QTimerEvent *);
	void resizeEvent(QResizeEvent *);
	const QString formatString(const QString aStr,mailEnvelope *env) const;
	QListViewItem *findItem(QListViewItem *,int,long);
	bool moveMsg(unsigned long,const char *);
	void setAccelerators();
	
private:
	QPopupMenu *group_folder_menu,*move_folder_menu;
	NMClient *my_client;
	NMMailView *my_viewer;
	int currentFolder;
	QString lastError;
	QListViewItem *statusView;
	bool mFastFetch;
	bool mFastPop;
	bool isSaveSend;
	QPixmap pixNew;
	QPixmap pixUnseen;
	QPixmap pixRead;
	QPixmap pixReplied;
	QPixmap pixDeleted;
	QPixmap pixFlagged;
	QPixmap pixSend;
	QPixmap pixQueued;
	QListViewItem *currentPopup;
	QListViewItem *currentMail;
	int subUnseen;
	int subRecent;
	unsigned int lastMessageNo;
	int lastFolderSelected;
};

#endif
















