/***************************************************************************
                          nmglobalfolders.h  -  description
                             -------------------
    begin                : Sat May 27 2000
    copyright            : (C) 2000 by Sven Carstens
    email                : s.carstens@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef NMGLOBALFOLDERS_H
#define NMGLOBALFOLDERS_H

#include <qstrlist.h>
#include <qlistview.h>
#include <qstring.h>
#include <qobject.h>
#include "nmclient.h"
#include "nmfoldercontent.h"
/**
  *@author Sven Carstens
  */

class realFolder {
public:
	realFolder();
	~realFolder();

	char *assembleUWIFolder(void);
	char *assembleURIFolder(void);
	bool parseFolder(const char *);
	const QString &getHost() { return host; }
	const QString &getPort() { return port; }
	const QString &getUser() { return user; }
	const QString &getPath() { return path; }
	const QString &getProtocol() { return protocol; }
	const QString &getVirtual() { return virtualName; }
	bool getCheck() { return check; }
	bool isDeleted() { return deleted; }
	int getUnseen() { return unseen; }
	int getRecent() { return recent; }
	int getTotal() { return total; }
	NMFolderContent *getView() { return view; }
	void setHost(const char *in) { host = in; }
	void setPort(const char *in) { port = in; }
	void setUser(const char *in) { user = in; }
	void setPath(const char *in) { path = in; }
	void setProtocol(const char *in) { protocol = in; }
	void setVirtual(const char *in) { virtualName = in; }
	void setCheck(bool in) { check = in; }
	void setDeleted(bool in) { deleted = in; }
	void setUnseen(int in) { unseen = in; }
	void setRecent(int in) { recent = in; }
	void setTotal(int in) { total = in; }	
	void setView(class NMFolderContent *in) { view = in; }
	
private:
	bool parseUWIFolder(const char *);
	bool parseURIFolder(const char *);
	QString protocol,host,user,path,port,virtualName;
	bool check,deleted;
	int unseen,recent,total;
	class NMFolderContent *view;
};

bool operator==(realFolder &,realFolder &);

class NMGlobalFolders : public QObject {
	Q_OBJECT

public:
	NMGlobalFolders();
	~NMGlobalFolders();
	
	void readFolders(class KApplication *);
	void writeFolders(class KApplication *);
	void editProperties(realFolder *,bool);	
	void editProperties(const char *);	
	void editProperties(int);	
	const char *renameFolder(realFolder *);	
	const char *renameFolder(const char *);	
	const char *renameFolder(int);	
	void deleteFolder(const char *);
	int getNOF();	
	realFolder *getAt(int);
	const QString &getMovePath(int);		
	const QString &getMovePath(const char *);		

	static void initPreMenuBar(class KTMainWindow *);
	static void initPostMenuBar(class KTMainWindow *);
	
public slots:	
	void checkForQueuedMessages();
	void moveToUnseenFolder();
	void moveToRecentFolder();

  void slotMakeMenu(QPopupMenu *,const QObject *,const char *);
	void slotStatus(NMClient *,char *,MAILSTATUS *);
	void slotNewFolder(const char *);
	void slotFolder(const char *);
	void slotFolder(int);
	void slotFolder(realFolder *);
	void slotNFolder(const char *);
	void slotNFolder(int);
	void slotNFolder(realFolder *);
	
	void slotSignOff(QWidget *);
	void slotSignOff(int,NMFolderContent *);
	void slotSignOn(int,NMFolderContent *);
	void slotSignOn(realFolder *,NMFolderContent *);
	
	void slotStatusLookup();
	void slotStatusLookup(bool);
	void slotStatusLookup(realFolder *);
	void slotStatusLookup(const char *);
	void slotStatusLookup(int);
	
	void slotSubStatus(const char *,int,int);
	void slotSubStatus(int,int,int);
	void slotSubStatus(realFolder *,int,int);

	void slotExpunge(int);
	void slotExpunge(realFolder *);
	
signals:
	void signalMessageNextUnread();
	void signalChanged(int,realFolder *);
	void signalStatus(const char *,int,int,int,int);
	void signalReadSoFar();
	void signalRefresh();
	void signalCurrentFolder(int);
			
protected:
	realFolder *findUnseenFolder(realFolder *startWith=NULL);
	realFolder *findRecentFolder(realFolder *startWith=NULL);
		
	realFolder *findFolder(const char *);
	realFolder *findVirtual(const char *);
	const QString &getMovePath(realFolder *);		
	void doDeleteFolder(const char *);
	void doStatus(bool);
	void statusChange(realFolder *);
	virtual void timerEvent(QTimerEvent *);
	QPopupMenu *findParent(QDict<QPopupMenu> *dict,QString &aVirtual,const QObject *,const char *);

private:
	void writeFolders();
	void listFolders(QStrList *strlist,QListViewItem *item,const char *prefix);
	void deleteFolders(QListViewItem *);
	QDialog *currentDialog;
	int checkUnseen;
	int checkRecent;
	bool dontTellDocking;

private:
	QList<realFolder> mFolders;
	QList<class NMFolderList *> mLists;
};

#endif
