/***************************************************************************
                          nmlistfolders.cpp  -  description
                             -------------------
    begin                : Wed Oct 20 1999
    copyright            : (C) 1999 by Sven Carstens
    email                : s.carstens@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include <unistd.h>
#include <stdlib.h>
#include "nmlistfolders.h"
#include <kapp.h>
#include <qheader.h>
#include <qlistview.h>
#include "extern.h"
#include "kbusyptr.h"
#include "nmstatusbar.h"
#include "nmglobalfolders.h"
#include "resource.h"
#include <kaccel.h>
#include <ktmainwindow.h>
#include <kiconloader.h>

#define COL_VIRTUAL 0
#define COL_SSL 1
#define COL_UNSEEN 2
#define COL_RECENT 3
#define COL_TOTAL 4
#define COL_INTNUM 5

NMListFolders::NMListFolders(QWidget *parent, const char *name ) : QListView(parent,name) {
	
  KIconLoader* loader = kapp->getIconLoader();
  pixmapSSL = loader->loadIcon("mini-lock.xpm");
	
	addColumn(i18n("Folder"));
	addColumn("");
	addColumn(i18n("unseen"));
	addColumn(i18n("recent"));
	addColumn(i18n("total"));
//	addColumn(i18n("int Num"));
	setRootIsDecorated(true);
	readFolders();
	setAllColumnsShowFocus(true);
	setFocusPolicy( QWidget::StrongFocus );

//	setVScrollBarMode( AlwaysOn );
	currentFolder = NULL;
	currentPopup = NULL;
	
	QObject::connect( this,	SIGNAL( rightButtonClicked( QListViewItem *,const QPoint &,int) ),
													SLOT( slotRightButtonClicked( QListViewItem *,const QPoint &,int) ) );

	QObject::connect( this,	SIGNAL( doubleClicked( QListViewItem *) ),
													SLOT( slotDoubleClicked( QListViewItem *) ) );
	QObject::connect( this,	SIGNAL( returnPressed( QListViewItem *) ),
													SLOT( slotDoubleClicked( QListViewItem *) ) );
	
	//tell the global folders object that the user has bothered us
	QObject::connect( this, SIGNAL( signalFolder( const char * ) ),
										gFolders, SLOT( slotFolder( const char * ) ) );	
	
	//tell the global folders object that the user has bothered us
	QObject::connect( this, SIGNAL( signalNFolder( const char * ) ),
										gFolders, SLOT( slotNFolder( const char * ) ) );	
	
	//tell the global folders object that the user has bothered us
	QObject::connect( this, SIGNAL( signalExpunge( int ) ),
										gFolders, SLOT( slotExpunge( int ) ) );	
	
	//global folders object tells us the status of the folders
	QObject::connect( gFolders, SIGNAL( signalStatus( const char *,int,int,int,int ) ),
										this, SLOT( slotStatus( const char *,int,int,int,int ) ) );	
	
	//global folders object tells us the status of the folders
	QObject::connect( gFolders, SIGNAL( signalChanged( int,realFolder * ) ),
										this, SLOT( slotChanged( int,realFolder * ) ) );	

	//global folders object tells us which is the current one
	QObject::connect( gFolders, SIGNAL( signalCurrentFolder( int) ),
										this, SLOT( setCurrentFolder( int) ) );	

	initMenu();
}

NMListFolders::~NMListFolders(){
}

//-----------------------------------------------------------------------------
QListViewItem *NMListFolders::findParent(QString dir,QListViewItem *item)
{
	int sep;
	QString prefix;
  QString suffix;
	QListViewItem *iterator;
	const char *debug;

	iterator = item;
  sep = dir.find('/');
	if(sep == -1) return item;
	if(sep == 0) sep = dir.find('/',1);
	if(!item)
		iterator = this->firstChild();
	else
    iterator = item->firstChild();

  prefix = dir.left(sep);
  suffix = dir.right(dir.length()-sep-1);
	
	while(iterator) {
		debug = iterator->text(0);
		if(!strcmp(prefix,iterator->text(0))) break;
    iterator = iterator->nextSibling();
  }
	if(! iterator) {
		if(item)
      iterator = new QListViewItem(item,prefix.data() );
		else
      iterator = new QListViewItem(this,prefix.data() );
		iterator->setSelectable( false );
  }
  sep = suffix.find('/');
  if(sep!=-1) {
		return findParent(suffix,iterator);
	} else {
  	return iterator;
  }
}

//-----------------------------------------------------------------------------
void NMListFolders::readFolders(void)
{
	clear();
	currentFolder = NULL;
	if(gFolders)
	{
		realFolder *work;
		QListViewItem *item;
		QString intNum;
		
		for(int i=gFolders->getNOF();i>0;i--)
		{
			work = gFolders->getAt(i-1);
			if(work && !work->isDeleted())
			{
				item = insertFolder(work->getVirtual(),work->assembleUWIFolder());
		  	QString protocol = work->getProtocol();
		  	if(protocol[protocol.length()-1 ] == 's' )
					item->setPixmap(COL_SSL,pixmapSSL);
//				debug("Adding: %s -> %s",work->getVirtual().data(),work->assembleURIFolder());
				intNum.sprintf("%d",i-1);
				item->setText(COL_INTNUM,intNum);
			}
		}
	} else { debug("Global folders not present");}
  update();
}

//-----------------------------------------------------------------------------
QListViewItem *NMListFolders::insertFolder(const char *folder,const char *value)
{
		QString suffix;
		QListViewItem *parent,*item;

			suffix = QString(folder);
			suffix = suffix.mid( suffix.findRev('/')+1,64);
			parent = findParent( folder,NULL );
      if(parent)
  			item = new QListViewItem(parent, suffix.data(),"","","","");
      else
			  item = new QListViewItem(this, suffix.data(),"","","","");
			item->setSelectable( false );
			
			return item;
}

//return the full virtual path of a folder
//-----------------------------------------------------------------------------
const char *NMListFolders::fullPath(QListViewItem *item)
{
	static QString path;

	path.setStr("");
	while(item) {
		path = item->text(0) + path;
		if(item->parent()) path = "/" + path;
		item = item->parent();
	}
	return path.data();
}


//find a specified item
//-----------------------------------------------------------------------------
QListViewItem *NMListFolders::findItem(QListViewItem *item,int column,const char *str)
{
	QListViewItem *retval = NULL;
	while(item && !retval)
	{
		retval = findItem(item->firstChild(),column,str);
		if(item->text(COL_INTNUM) && !strcmp(str,item->text(column)) )
		{
//			fprintf(stderr,"FOUND: '%s'\n",str);
			retval = item;
		}

		item=item->nextSibling();
	}
	return retval;
}

void NMListFolders::resizeContents(int width, int h)
{
	int maxW,minW;

	QListView::resizeContents(width,h);
	QListView::update();
//	QListView::resize(width,height());
	if(verticalScrollBar() && verticalScrollBar()->isVisible()) width += verticalScrollBar()->width();
//	QListView::resize(width,height());
	setMaximumWidth( width );
	setMinimumWidth( width );
	setMaximumWidth( 32767 );
	setMinimumWidth( 0 );
	fprintf(stderr,"Content: %d, %d\n",width,h);
}

void NMListFolders::resize( void )
{
		int width = 0;

		for(int i=0;i<QListView::header()->count();i++) {
			width += columnWidth( i );
		}
		if(verticalScrollBar() && verticalScrollBar()->isVisible()) width += verticalScrollBar()->width();
		resizeContents( contentsWidth(), 0 );
//	setMaximumWidth( width );
}

void NMListFolders::resizeEvent(QResizeEvent *event)
{
//	resize();
		int width = 0;
		int oldwidth = 0;
		oldwidth = columnWidth(0);

		setColumnWidth( 0, 0);
		for(int i=0;i<QListView::header()->count();i++) {
			width += columnWidth( i );
		}
		width = this->width() - width;
		if(verticalScrollBar() && verticalScrollBar()->isVisible()) width -= verticalScrollBar()->width();
		setColumnWidth( 0, width);
//		if(width != oldwidth) QListView::resize( contentsWidth(),height() );

	QListView::resizeEvent(event);
}

void NMListFolders::slotRightButtonClicked(QListViewItem *item,const QPoint &point,int column)
{
  QPopupMenu *menu;
	currentPopup = item;
  {
    menu = new QPopupMenu();
    menu->insertItem(i18n("New..."), this, SLOT(slotFNew()));
    menu->insertItem(i18n("Rename..."), this, SLOT(slotFRename()));
    menu->insertItem(i18n("Delete..."), this, SLOT(slotFDelete()));
    menu->insertItem(i18n("Expunge..."), this, SLOT(slotFExpunge()));
    menu->insertItem(i18n("Status..."), this, SLOT(slotFStatus()));
    menu->insertItem(i18n("Open in new Window"), this, SLOT(slotFOpenNew()));
    menu->insertItem(i18n("Properties..."), this, SLOT(slotFProperties()));
    menu->exec(point,0);
  }
	currentPopup = NULL;
}

void NMListFolders::slotFStatus()
{
	if(gFolders && currentPopup)
	{
		if(currentPopup->text(COL_INTNUM))
			gFolders->slotStatusLookup(atoi(currentPopup->text(COL_INTNUM)));
		else
			gFolders->slotStatusLookup(currentPopup->text(0));
	}
	currentPopup = NULL;	
}

void NMListFolders::slotFNew()
{
	if(gFolders)
		if(currentPopup)
		{
			gFolders->slotNewFolder(fullPath(currentPopup));
		}else if(currentPopup->text(COL_INTNUM)) {
			gFolders->slotNewFolder(fullPath(currentPopup->parent()));
		} else {
			gFolders->slotNewFolder(NULL);
		}		
	currentPopup = NULL;	
	return;
}

void NMListFolders::slotFDelete()
{
	if(gFolders && currentPopup) gFolders->deleteFolder(fullPath(currentPopup));
	return;
}

void NMListFolders::slotFProperties()
{
//	if(gFolders && currentPopup) gFolders->editProperties(currentPopup->text(4));
	if(gFolders && currentPopup && currentPopup->text(COL_INTNUM))
		gFolders->editProperties(atoi(currentPopup->text(COL_INTNUM)));
	return;
}

void NMListFolders::slotFExpunge()
{
	if(currentPopup && currentPopup->text(COL_INTNUM))
		emit signalExpunge(atoi(currentPopup->text(COL_INTNUM)));
}

void NMListFolders::slotFOpenNew()
{
	if(currentPopup) emit signalNFolder(fullPath(currentPopup));
}

void NMListFolders::slotFRename()
{
	const char *result;
	if(gFolders)
	{
		if(currentPopup->text(COL_INTNUM))
		{
			result = gFolders->renameFolder(atoi(currentPopup->text(COL_INTNUM)));
		} else {
			result = gFolders->renameFolder(fullPath(currentPopup));
		}
	}
	return;
}

void NMListFolders::slotDoubleClicked(QListViewItem *item)
{
	if(item) emit signalFolder(fullPath(item));
	setCurrentFolder(item);
}

void NMListFolders::setCurrentFolder(QListViewItem *item)
{
	if(currentFolder)
	{
		currentFolder->setSelected( false );
		currentFolder->repaint();
	}
	currentFolder = item;	
	if(currentFolder)
	{
		ensureItemVisible(currentFolder);
		currentFolder->setSelected( true );
		currentFolder->repaint();
	}
}

void NMListFolders::scrollUp()
{
	QListViewItem *item = currentItem();
	
	if(item && item->itemAbove())
	{
		setCurrentItem(item->itemAbove());
		ensureItemVisible(currentItem());
	}	
	setFocus();
}

void NMListFolders::scrollDown()
{
	QListViewItem *item = currentItem();
	
	if(item && item->itemBelow())
	{
		setCurrentItem(item->itemBelow());
		ensureItemVisible(currentItem());
	}	
	setFocus();
}

void NMListFolders::collapse()
{
	QListViewItem *item = currentItem();

	if(item)
	{
		if(item->isExpandable() || (item->childCount() > 0)) item->setOpen(false);
		item->repaint();
	}
	setFocus();
}

void NMListFolders::expand()
{
	QListViewItem *item = currentItem();

	if(item)
	{
		if(item->isExpandable() || (item->childCount() > 0)) item->setOpen(true);
		item->repaint();
	}
	setFocus();
}

void NMListFolders::slotStatus(const char *aVirtual,int aNum,int nRecent, int nUnseen,int nTotal)
{
	QString intNum;
	QListViewItem *item;
	intNum.sprintf("%d",aNum);
	item = findItem(firstChild(),COL_INTNUM,intNum);
	if(item)
	{
		intNum.sprintf("%d",nRecent);
		item->setText(COL_RECENT,intNum);
		intNum.sprintf("%d",nUnseen);
		item->setText(COL_UNSEEN,intNum);
		intNum.sprintf("%d",nTotal);
		item->setText(COL_TOTAL,intNum);
	} else {
		debug("NMListFolders::slotStatus couldnt find iternal Num %d",aNum);
	}
}

void NMListFolders::slotChanged(int aNum,realFolder *aFolder)
{
	QString intNum;
	QListViewItem *item;
	
  if(aNum >= 0)
  {
  	debug("NMListFolders::slotChanged - %d %s",aNum,aFolder->getVirtual().data());
  	intNum.sprintf("%d",aNum);
  	item = findItem(firstChild(),COL_INTNUM,intNum);
  	if(item) item->setText(COL_INTNUM,NULL);
//  	while(item && item->childCount() == 0)
  	while(item && item->childCount() == 0 && item->text(COL_INTNUM) == NULL)
  	{
  		QListViewItem *temp;
  		temp = item->parent();
  		if(item==currentFolder) currentFolder = NULL;
			delete item;
			item = temp;  		
  	}
  	if(!aFolder->isDeleted())
  	{
 	  	QString protocol = aFolder->getProtocol();
 	  	
    	item = insertFolder(aFolder->getVirtual(),aFolder->assembleUWIFolder());
 	  	if(protocol[protocol.length()-1 ] == 's' )
 				item->setPixmap(COL_SSL,pixmapSSL);
    	item->setText(COL_INTNUM,intNum);
    }
  }
}

void NMListFolders::setCurrentFolder(int aNum)
{
	QString intNum;
	QListViewItem *item;
	
  if(aNum >= 0)
  {
  	debug("NMListFolders::setCurrentFolder - %d",aNum);
  	intNum.sprintf("%d",aNum);
  	item = findItem(firstChild(),COL_INTNUM,intNum);
  	setCurrentFolder(item);
  }
}

void NMListFolders::initMenu()
{
  ///////////////////////////////////////////////////////////////////
  // menuBar entry folder_menu
  QPopupMenu *folder_menu = new QPopupMenu();
  folder_menu->insertItem( i18n("&New"), ID_FOLDER_NEW );
  folder_menu->insertItem( i18n("E&xpunge"), ID_FOLDER_EXPUNGE );
	folder_menu->setAccel(Key_X ,ID_FOLDER_EXPUNGE);
  folder_menu->insertItem( i18n("&Check all"), ID_FOLDER_STATUS );
  folder_menu->insertItem( i18n("Check pre&selected"), ID_FOLDER_CHECK );

	if(topLevelWidget()->isA("KTMainWindow")
	|| topLevelWidget()->isA("KImap"))
	{
		((KTMainWindow *)topLevelWidget())->menuBar()->insertItem(i18n("&Folder"), folder_menu);
	}

  connect(folder_menu,SIGNAL(activated(int)),SLOT(commandCallback(int)));
  connect(folder_menu,SIGNAL(highlighted(int)),SLOT(statusCallback(int)));

}

void NMListFolders::commandCallback(int id_){
  switch (id_){
		case ID_FOLDER_NEW:
			if(gFolders) gFolders->slotNewFolder("");
			break;
		case ID_FOLDER_EXPUNGE:
			if(currentFolder && currentFolder->text(COL_INTNUM))
				emit signalExpunge(atoi(currentFolder->text(COL_INTNUM)));
			break;
		case ID_FOLDER_STATUS:
			if(gFolders) gFolders->slotStatusLookup(true);
			break;
		case ID_FOLDER_CHECK:
			if(gFolders) gFolders->slotStatusLookup(false);
			break;
  }
}

void NMListFolders::statusCallback(int id_){
  switch (id_){
    case ID_VIEW_TOOLBAR:
//    	slotStatusHelpMsg(i18n("Enables / disables the actual Toolbar"));
    	break;
    case ID_VIEW_STATUSBAR:
//    	slotStatusHelpMsg(i18n("Enables / disables the actual Statusbar"));
    	break;
		default:
			break;
  }
}
