/***************************************************************************
                          nmlistremote.cpp  -  description
                             -------------------
    begin                : Mon Oct 25 1999
    copyright            : (C) 1999 by Sven Carstens
    email                : s.carstens@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#include "nmlistremote.h"
#include "nmclient.h"
#include "dirview.h"
#include <kapp.h>
#include <qpopupmenu.h>
#include <qmessagebox.h>
#include <qwidget.h>
#include <qlistview.h>
#include <qregexp.h>
#include "nmglobalfolders.h"
#include "nmstatusbar.h"

extern QWidget *kimap;

class RemoteLoc: public QListViewItem
{
public:
    RemoteLoc( QListView * parent, const char * filename,const char *type,const char *location );
    RemoteLoc( QListViewItem * parent, const char * filename,const char *type,const char *location );
//    RemoteLoc( Directory * parent, const char * filename,const char *type,const char *location );

    void setOpen( bool );
};

RemoteLoc::RemoteLoc( QListView * parent, const char * filename,const char *type,const char *location )
:	QListViewItem(parent,filename,type,location)
{
}

RemoteLoc::RemoteLoc( QListViewItem * parent, const char * filename,const char *type,const char *location )
:	QListViewItem(parent,filename,type,location)
{
}

void RemoteLoc::setOpen(bool open)
{
	QListViewItem::setOpen(open);
	
	//see if we need to read from the server
	if( open && !childCount() )
	{
  	//build the uri for this folder
  	QString retVal;
  	QListViewItem *start = this;
  	while(start)
  	{
  		retVal = QString(start->text(0)) + retVal;
  		start = start->parent();
  	}
  	debug("RemoteLoc::setOpen trying to open %s",retVal.data());
  	realFolder tempFolder;
  	tempFolder.parseFolder(retVal);
  	retVal = tempFolder.assembleUWIFolder();
  	retVal += "%"; //only one level at a time
  	
  	NMListRemote *myList;
  	
  	//get our parent
  	myList = (NMListRemote *)listView();
  	if(myList && myList->inherits("NMListRemote"))
  	{
	  	//fetch the list
	  	debug("RemoteLoc::setOpen trying to open %s",retVal.data());
  		myList->listFolders(retVal);
  	}
  }
}

#define BUTTONS 70

NMListRemote::NMListRemote(QWidget *parent, const char *name ) : QListView(parent,name) {
	setMultiSelection(true);		

	int error_offset = 0;
	addColumn(i18n("Folder"));
	addColumn(i18n("Type"));
	addColumn(i18n("Type"));

	setMinimumSize(300,120+error_offset);
}

//---------------------------------------------------------------------------------
NMListRemote::~NMListRemote(){
}

//-----------------------------------------------------------------------------
QListViewItem *NMListRemote::findParent(QString dir,QListViewItem *item)
{
	int sep;
	QString prefix;
	QString suffix;
	QListViewItem *iterator;
	const char *debug;

	iterator = item;
	sep = dir.find('/');
	if(dir[sep+1] == '/') sep = dir.find('/',sep+2);
	if(sep == -1) return item;
	if(!item)
		iterator = this->firstChild();
	else
    iterator = item->firstChild();

  prefix = dir.left(sep+1);
  suffix = dir.right(dir.length() - sep - 1);
	while(iterator) {
		debug = iterator->text(0);
		if(!strcmp(prefix,iterator->text(0))) break;
    iterator = iterator->nextSibling();
  }
	if(! iterator) {
		if(item)
      iterator = new RemoteLoc(item,prefix.data(),"Directory","");
		else
      iterator = new RemoteLoc(this,prefix.data(),"Directory","" );
		iterator->setSelectable( false );
		iterator->setExpandable( true );
  }
  sep = suffix.find('/');
  if(sep!=-1) {
		return findParent(suffix,iterator);
	} else {
  	return iterator;
  }
}

void NMListRemote::listFolders(const char *what)
{
	QString tmp = QString( what );

	NMClient *liststream= new NMClient();
	if( tmp.find('{') == 0 || tmp.find('#') == 0 )
	{
		tmp.replace(QRegExp("\\*"),"%");
		NMStatusBar::connectToStatusBar(liststream,(QWidget *)parent());
		QObject::connect( liststream, SIGNAL( signalList(NMClient *,int,char *,long) ),
											this, SLOT( slotList(NMClient *,int,char *,long) ) );

		debug("listing %s",tmp.data());
		liststream->listFolders(tmp);
		NMStatusBar::disconnectStatusBar(liststream,(QWidget *)parent());
		if(liststream->getError())
			QMessageBox::warning(0,i18n("ERROR accessing Server"),liststream->getError(),QMessageBox::Ok,QMessageBox::Default);
		setOpen(firstChild(),true);
	} else {
		Directory *root;
		int truncate;
		
		truncate = tmp.find('*');
		if(truncate >= 0) tmp = tmp.left(truncate);
		debug("listing %s",tmp.data());
	  root = new Directory( this,tmp );
	  root->setOpen( TRUE );
	  show();
	}
}

void NMListRemote::slotList(NMClient *,int delimiter,char *mailbox,long attributes)
{
	QListViewItem *parent;
//  if (!(attributes & LATT_NOSELECT))
	{
		realFolder newFolder;
		newFolder.parseFolder(mailbox);
		
		QString box = newFolder.assembleURIFolder();
		parent = findParent( box,NULL );
		int sep = box.findRev((char)delimiter);
		if(!(attributes & LATT_NOSELECT))
		{
      if(parent)
    		new QListViewItem(parent, box.right((box.length() - sep - 1)),"Folder",box);
      else
  		  new QListViewItem(this, box.right((box.length() - sep - 1)),"Folder",box);
		}
		if(!(attributes & LATT_NOINFERIORS))
		{
			box += '/';
			parent = findParent( box,NULL );
		}
	}
}

QString NMListRemote::getFullName(QListViewItem *start)
{
	QString retVal;
	while(start)
	{
		retVal += start->text(0);
		start = start->parent();
	}
		
	return retVal;
}