/***************************************************************************
                          nmmailview.cpp  -  description
                             -------------------
    begin                : Sat Oct 16 1999
    copyright            : (C) 1999 by Sven Carstens
    email                : s.carstens@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#include "nmmailview.h"

#include <unistd.h>
#include <sys/stat.h>
#include <errno.h>
#include <ctype.h>
#include <time.h>
#include <assert.h>

#include <qregexp.h>
#include <qclipboard.h>

#include "extern.h"
#include "kmimemagic.h"
#include "ksimpleconfig.h"
#include "kfiledialog.h"
#include <qdir.h>
#include <qsplitter.h>
#include "html.h"
#include "kfontutils.h"
#include "kfileio.h"
#include "kpgp.h"
#include "kcursor.h"
#include "string.h"

#include "nmclient.h"
#include "kbusyptr.h"
#include "nmstatusbar.h"
#include "kmaddrbook.h"
#include "kmmsgpartdlg.h"
#include "kmcomposewin.h"
#include "resource.h"
#include "vcard.h"
#include "kmdisplayvcard.h"
#include <qprinter.h>
#include <mimelib/string.h>
#include <mimelib/uuencode.h>
static KMimeMagic* sMagic = NULL;


//-----------------------------------------------------------------------------
void fillBodyRaw(mailBody *body,QString &string)
{
	string = body->bodyMIME();
	string += body->bodyRAW();
}

//-----------------------------------------------------------------------------
void fillBodyDecoded(mailBody *body,QString &string,const char *aPartNum=NULL)
{
	const char *str_txt;
	unsigned long str_len;

	str_txt = body->bodyDecoded(&str_len);
	string.resize(str_len+1);
//	assert(str_txt != NULL);
	if(str_txt != NULL)
	{
		memcpy(string.data(),str_txt,str_len);
		string[str_len] = 0x00;
	} else {
		string = "(NULL)";
	}
}


void removedir(const char *in_dir)
{
	QDir thisDir( in_dir );
	if ( !thisDir.isReadable() ) {
	    return;
	}

	const QFileInfoList * files = thisDir.entryInfoList();
	if ( files ) {
	    QFileInfoListIterator it( *files );
	    QFileInfo * f;
	    while( (f=it.current()) != 0 ) {
				++it;
				if ( f->fileName() == "." || f->fileName() == ".." )
		    ; // nothing
				else if ( f->isDir() )
				{
					removedir(QString(in_dir) + "/" + f->fileName());
				}
				else
				{
					thisDir.remove( f->fileName() );
				}
	   }
	}
	thisDir.rmdir(".");
}

//-----------------------------------------------------------------------------
NMMailView::~NMMailView()
{
	writeConfig(true);
	if(mMsg) delete mMsg;
	if(mAttachDir) removedir(mAttachDir.data());
//  if (mAutoDelete) delete mMsg;
}

void NMMailView::slotMail(mailMessage *aMsg,const char *caption)
{
	my_client = NULL;
	curMsg = -1;
	mMsg = aMsg;
	if(caption) setCaption(caption);
	parseMsg();
}

void NMMailView::slotMail(NMClient *client,unsigned long in_curMsg)
{
	QString caption;
	
	if(client && client->isLocked()) return;
	if(client == my_client && in_curMsg == curMsg) return; //don't display the same message twice
	if(client && client->getBox()) caption.sprintf("%s : %ld",client->getBox(),in_curMsg);
	setCaption(caption.data());
	my_client = client;
	curMsg = in_curMsg;
	if(mMsg) delete mMsg;
	if(client) {
		mMsg = client->fetchStructure(curMsg);
		if(client->getError())
			QMessageBox::warning(0,i18n("Problems opening Message"),client->getError(),QMessageBox::Ok,QMessageBox::Default);		
	} else {
		mMsg = new mailMessage();
	}
  parseMsg();
	
	fprintf(stderr,"displayMail : %ld - %p\n",curMsg,client);
	setFocus();
}

void NMMailView::displayBody (mailBody *in_body,const char *pfx,long i)
{
	QString str;
	QString subtype;
	QString disposition;
	static QString alternativ; // the preferred alternative part
	static QString siggen;     // protocol for signature generation

  char tmp[MAILTMPLEN];
	char prefix[16];

//	if(!my_client || !in_body || in_body->isEmpty()) return;
	if(!in_body || in_body->isEmpty()) return;

  if (!stricmp(in_body->bodyType(),"multipart")) {
				/* if not first time, extend prefix */
	  mailBody *part;			/* multipart doesn't have a row to itself */
		alternativ = "";
		siggen = "";
		if(!stricmp(in_body->bodySubType(),"alternative"))
		{
		  KConfig *config = kapp->getConfig();

			config->setGroup("MIME");
			alternativ = config->readEntry("PreferredType","text/html");

		} else if(!stricmp(in_body->bodySubType(),"signed")) {
		  siggen = in_body->bodyParameter("PROTOCOL");
		  debug("SIGNED with %s",siggen.data());
		}
    if (pfx) sprintf (tmp,"%s%ld.",pfx,++i);
    else tmp[0] = '\0';
    for (i = 0,part = in_body->nestedPart(); (part && !part->isEmpty()); part->nextPart())
		{
      displayBody(part,tmp,i++);
		}
		delete part;
  }
  else {			/* non-multipart, output */
		bool signedMessage = false;

    if (!pfx)
		{
			pfx = "";		/* dummy prefix if top level */
			alternativ = "";
		}
		i++;

		//get the current part into memory
		snprintf (prefix,sizeof(prefix),"%s%ld",pfx,i);
		
		if(my_client) my_client->fetchBodyContent(curMsg,prefix);

		//do we have a signed message ?
		if(!siggen.isEmpty()) {
			mailBody *next = in_body->copyPart();
			next->nextPart();
			if(next &&!next->isEmpty() && (!stricmp(siggen,next->bodyFullType()))) {
				//the next part is the signature of the current part
				debug("NextPart is signature %s\n",next->bodyFullType());
				//get the next part && rebuild the prefix
				snprintf (prefix,sizeof(prefix),"%s%ld",pfx,i+1);
				if(my_client) my_client->fetchBodyContent(curMsg,prefix);
				snprintf (prefix,sizeof(prefix),"%s%ld",pfx,i);

				//load the part with MIME headers in memory
				//load the signature without MIME headers
				QString text;
				QString sig;

				fillBodyRaw(in_body,text);
				fillBodyDecoded(next,sig);

			  Kpgp* pgp = Kpgp::getKpgp();

				NMStatusBar::connectToStatusBar(pgp,this);		
			
				//save signature to disk (only way for detached signatures!)
    		if (kStringToFile(sig, mAttachDir+"/check.signature", false, false, false)) {
					emit signalStatusMsg(i18n("Calling PGP/GPG"));
					kapp->processEvents(100); //else the status wont display
					kbp->busy();
	  			pgp->setMessage(text);
					if(pgp->verifySig(mAttachDir+"/check.signature")) {

				    if (pgp->isSigned())
				    {
							QString line="";
				      if (pgp->unknownSignature()) sig = i18n("Message maybe signed by");
				      else if (pgp->goodSignature()) sig = i18n("Message was signed by");
				      else sig = i18n("Warning: Bad signature from");

				      /* HTMLize signedBy data */
				      QString sdata=pgp->signedBy();
				      sdata.replace(QRegExp("\""), "&quot;");
				      sdata.replace(QRegExp("<"), "&lt;");
				      sdata.replace(QRegExp(">"), "&gt;");

				      if (sdata.contains(QRegExp("unknown key ID")))
				      {
					      sdata.replace(QRegExp("unknown key ID"), i18n("unknown key ID"));
					      line.sprintf("<B>%s %s</B><BR></BR>\r\n",sig.data(), sdata.data());
				      }
				      else if (sdata.contains(QRegExp("public key not found")))
				      {
					      sdata.replace(QRegExp("public key not found"), i18n("public key not found"));
					      line.sprintf("<B>%s %s</B><BR></BR>\r\n",sig.data(), sdata.data());
				      } else {
								line = "<B>";
								line += sig;
								line += " <A HREF=\"mailto:";
								line += sdata;
								line += "\">";
								line += sdata;
								line += "</A></B><BR></BR>\r\n";
//					      line.sprintf("<B>%s <A HREF=\"mailto:%s\">%s</A></B><BR>\n", sig.data(),
//							   sdata.data(),sdata.data());
							}

//				      delete sdata;
							mViewer->write("<TABLE BORDER=1><TR><TH>");
				      mViewer->write(line);
							mViewer->write("</TH></TR><TR><TD>");
							signedMessage = true;
				    }
					}
					kbp->idle();
				}
				NMStatusBar::disconnectStatusBar(pgp,this);		
			}
			delete next;
		}


		// try to display that stuff
		displayPartContent(in_body,prefix,alternativ);
		if(signedMessage) mViewer->write("</TD></TR></TABLE>");

				/* encapsulated message? */
    if(	(mAttachmentStyle != IconicAttmnt) &&
				(in_body->bodyEType() == TypeMESSAGE) &&
				!strcmp(in_body->bodySubType(),"RFC822") )
		{
			mailMessage *aMsg = in_body->nestedMsg();
			mailEnvelope *aEnv = aMsg->envelope();
			mailBody *aBody = aMsg->body();

			mViewer->write("<TABLE BORDER=\"1\"><TR><TD>");
		  writeMsgHeader(aEnv,prefix);

      if (!strcmp(aBody->bodyType(), "MULTIPART")) displayBody (aBody,pfx,i-1);
      else {			/* build encapsulation prefix */
				sprintf (tmp,"%s%ld.",pfx,i);
				displayBody (aBody,tmp,(long) 0);
      }
			mViewer->write("</TD></TR></TABLE>");
			delete aMsg;
			delete aBody;
			delete aEnv;
    }
  }
}

//-------------------------------------------------------------------------------------------

//-----------------------------------------------------------------------------
NMMailView::NMMailView(QWidget *aParent, const char *aName)
  :QWidget(aParent, aName)
{
  initMetaObject();

	cidMapping = new QDict<QString>(17,true);

	debug("NMMailView::NMMailView(%p,%s)\n",aParent,aName);
	mMsg = new mailMessage();
	curMsg = 0;
	my_client = NULL;
  mPicsDir = app->kde_datadir()+"/kimap/pics/";
  mAutoDelete = FALSE;

  initHtmlWidget();
  readConfig();

  if (mAttachDir.isNull()) makeAttachDir();
	mViewer->setFocusPolicy( QWidget::StrongFocus );

// request a file part if we have a live client somewhere		
	QObject::connect(mViewer,SIGNAL(fileRequest(const char *)),this,SLOT(slotFileRequest(const char *)));
	
// tell the master about it	
	QObject *temp = (QObject *)kimap;
	if(!temp) temp=topLevelWidget();
	QObject::connect( this,SIGNAL( urlClicked( const char *,int)),
										temp,SLOT( slotUrlClicked( const char *,int)) );
//	busy = false;

	initMenu();
	
	NMStatusBar::connectToStatusBar(this,this);
}


//-----------------------------------------------------------------------------
void NMMailView::makeAttachDir(void)
{
  QString str;
  bool ok = true;

  str = MAIL_TMP;
  if (str[0] == '&') str = app->localkdedir() + str.mid(1,1023);
  mAttachDir.sprintf("%skimap%d", (const char*)str, getpid());

  if (access(mAttachDir, W_OK) != 0) // Not there or not writable
  {
    if (mkdir(str, S_IRWXU) != 0 && chmod(str, S_IRWXU) != 0)
      ok=false; //failed create new or chmod existing tmp/
    else if (mkdir(mAttachDir, S_IRWXU) != 0 && chmod(mAttachDir, S_IRWXU) != 0)
      ok=false; //failed create new or chmod exisiting kimap<pid>/
  }

  if (!ok) warning(i18n("Failed to create temporary "
			"attachment directory '%s': %s"),
		   (const char*)mAttachDir, strerror(errno));
}


//-----------------------------------------------------------------------------
void NMMailView::readConfig(void)
{
  KConfig *config = kapp->getConfig();

  config->setGroup("General");
  mTimeformat = config->readEntry("local-date-format","%c %z");
	mLocalizeTime = config->readBoolEntry("localize-time", false);

  config->setGroup("Reader");
  mAtmInline = config->readNumEntry("attach-inline", 100);
  mHeaderStyle = (HeaderStyle)config->readNumEntry("hdr-style", HdrFancy);
  mAttachmentStyle = (AttachmentStyle)config->readNumEntry("attmnt-style",
							SmartAttmnt);
#ifdef KRN
  config->setGroup("ArticleListOptions");
#endif
  QColor c1=QColor("black");
  QColor c2=QColor("blue");
  QColor c3=QColor("red");
  QColor c4=QColor("white");

  mViewer->setDefaultBGColor(config->readColorEntry("BackgroundColor",&c4));
  mViewer->setDefaultTextColors(config->readColorEntry("ForegroundColor",&c1)
                                ,config->readColorEntry("LinkColor",&c2)
                                ,config->readColorEntry("FollowedColor",&c3));
  //moved to #ifdef KRN where it was before (sven):
  //mViewer->setDefaultFontBase(config->readNumEntry("DefaultFontBase",3));

#ifndef KRN
  config->setGroup("Fonts");
  mBodyFont = config->readEntry("body-font", "helvetica-medium-r-12");
  mFixedFont = config->readEntry("fixed-font", "fixed-medium-r-12");
	setViewerFont(mBodyFont);
	int sep;
	sep = mFixedFont.find('-');
	sep = mFixedFont.find('-',sep+1);
	sep = mFixedFont.find('-',sep+1);
	mFixedSize = mFixedFont.right( mFixedFont.length() - sep-1 );
	mFixedSize = mFixedSize.left( mFixedSize.find('-'));
	mFixedFont = mFixedFont.left( mFixedFont.find('-') );
	fprintf(stderr,"Fixed-Font: %s [%s]\n",mFixedFont.data(),mFixedSize.data());

	sep = mBodyFont.find('-');
	sep = mBodyFont.find('-',sep+1);
	sep = mBodyFont.find('-',sep+1);
	mBodySize = mBodyFont.right( mBodyFont.length() - sep-1 );
	mBodySize = mBodySize.left( mBodySize.find('-'));
	mBodyFont = mBodyFont.left( mBodyFont.find('-') );
	fprintf(stderr,"Body-Font: %s [%s]\n",mBodyFont.data(),mBodySize.data());
/*  mViewer->setStandardFont(kstrToFont(mBodyFont).family());
  // --- sven's get them font sizes right! start ---
  int i, fntSize=0, diff;
  fntSize = kstrToFont(mBodyFont).pointSize();
  //debug ("Fontsize: %d", fntSize);

  int fontsizes[7];
  mViewer->resetFontSizes();
  mViewer->getFontSizes(fontsizes);
  diff= fntSize - fontsizes[3];
  if (fontsizes[0]+diff > 0)
  {
    for (i=0;i<7; i++)
    {
      //debug ("Old fontsize #%d: %d", i, fontsizes[i]);
      fontsizes[i]+=diff;
      //debug ("New fontsize #%d: %d", i, fontsizes[i]);
    }
  }
  mViewer->setFontSizes(fontsizes);
  // --- sven's get them font sizes right! end ---
  //mViewer->setFixedFont(mFixedFont); */
#else
  mViewer->setDefaultFontBase(config->readNumEntry("DefaultFontBase",3));
  mViewer->setStandardFont(config->readEntry("StandardFont","helvetica"));
  mViewer->setFixedFont(config->readEntry("FixedFont","courier"));
#endif
  update();
}


//-----------------------------------------------------------------------------
void NMMailView::writeConfig(bool aWithSync)
{
	if(topLevelWidget()->isA("KImap"))
	{
    KConfig *config = kapp->getConfig();

    config->setGroup("Reader");
    config->writeEntry("attach-inline", mAtmInline);
    config->writeEntry("hdr-style", (int)mHeaderStyle);
    config->writeEntry("attmnt-style",(int)mAttachmentStyle);

    // config->setGroup("Fonts");
    // config->writeEntry("body-font", mBodyFont);
    // config->writeEntry("fixed-font", mFixedFont);

    if (aWithSync) config->sync();
  }
}


//-----------------------------------------------------------------------------
void NMMailView::initHtmlWidget(void)
{
//	QSplitter *mSplitter = new QSplitter(this);
//	mSplitter->setOrientation( QSplitter::Vertical );
//  mSplitter->resize(width(), height());
  mViewer = new NMHTMLWidget(this, mPicsDir);
//  mViewer = new NMHTMLWidget(mSplitter, mPicsDir);
  mViewer->resize(width()-16, height()-110);
  mViewer->setURLCursor(KCursor::handCursor());
  mViewer->setDefaultBGColor(QColor("#ffffff"));
  /*
  mViewer->setDefaultBGColor(pal->normal().background());
  mViewer->setDefaultTextColor(app->textColor, app->);
  */
  connect(mViewer,SIGNAL(URLSelected(const char *,int)),this,
	  SLOT(slotUrlOpen(const char *,int)));
  connect(mViewer,SIGNAL(onURL(const char *)),this,
	  SLOT(slotUrlOn(const char *)));
  connect(mViewer,SIGNAL(fileRequest(const char *)),this,
	  SLOT(slotFileRequest(const char *)));
  connect(mViewer,SIGNAL(popupMenu(const char *, const QPoint &)),
	  SLOT(slotUrlPopup(const char *, const QPoint &)));
  connect(mViewer,SIGNAL(textSelected(bool)),
	  SLOT(slotTextSelected(bool)));

  mSbVert = new QScrollBar(0, 110, 12, height()-110, 0,
			   QScrollBar::Vertical, this);
  mSbHorz = new QScrollBar(0, 0, 24, width()-32, 0,
			   QScrollBar::Horizontal, this);	
  connect(mViewer, SIGNAL(scrollVert(int)), SLOT(slotScrollVert(int)));
  connect(mViewer, SIGNAL(scrollHorz(int)), SLOT(slotScrollHorz(int)));
  connect(mSbVert, SIGNAL(valueChanged(int)), mViewer, SLOT(slotScrollVert(int)));
  connect(mSbHorz, SIGNAL(valueChanged(int)), mViewer, SLOT(slotScrollHorz(int)));
  connect(mViewer, SIGNAL(documentChanged()), SLOT(slotDocumentChanged()));
  connect(mViewer, SIGNAL(documentDone()), SLOT(slotDocumentDone()));
}


//-----------------------------------------------------------------------------
void NMMailView::setBodyFont(const QString aFont)
{
  mBodyFont = aFont.copy();
  update();
}


//-----------------------------------------------------------------------------
void NMMailView::setHeaderStyle(NMMailView::HeaderStyle aHeaderStyle)
{
  mHeaderStyle = aHeaderStyle;
  update();
}


//-----------------------------------------------------------------------------
void NMMailView::setAttachmentStyle(int aAttachmentStyle)
{
  mAttachmentStyle = (AttachmentStyle)aAttachmentStyle;
  update();
}

//-----------------------------------------------------------------------------
void NMMailView::setInlineAttach(int aAtmInline)
{
  mAtmInline = aAtmInline;
  update();
}

//-----------------------------------------------------------------------------
void NMMailView::parseMsg(const char *prefix,mailBody *aMsgPart)
{
	QString myPrefix = prefix;

	int postfix = 0;
  //assert(mMsg!=NULL);

	if(!myPrefix.isEmpty()) {
		postfix = atoi( myPrefix.right( myPrefix.length() - myPrefix.findRev('.') - 1) );
		myPrefix = myPrefix.left( myPrefix.findRev('.') + 1);
		postfix--;

		mMsg = aMsgPart->nestedMsg();				
	}

	kbp->busy();
	cidMapping->clear();
  mViewer->begin(mPicsDir);
  mViewer->write(QString("<HTML><HEAD><BASE HREF='") + "file:/" + "'></HEAD><BODY>");

	if(mMsg && !mMsg->isEmpty()) {
//#if defined CHARSETS
	  fprintf(stderr,"Setting viewer charset to %s\n",mMsg->bodyCharset());
	  mViewer->setCharset(mMsg->bodyCharset());
//#endif
		mailEnvelope *mEnv = mMsg->envelope();
		mailBody *mBody = mMsg->body();
	  writeMsgHeader(mEnv,prefix);
		displayBody(mBody,myPrefix.data(),postfix);
		delete mEnv;
		delete mBody;
//  parseMsg("");
  }
  mViewer->write("</BODY></HTML>");
  mViewer->end();
  mViewer->parse();
	kbp->idle();
}
//-----------------------------------------------------------------------------
void NMMailView::parseMsg()
{

	cidMapping->clear();
  mViewer->begin(mPicsDir);
  mViewer->write(QString("<HTML><HEAD><BASE HREF='") + "file:/" + "'></HEAD><BODY>");

	if(mMsg && !mMsg->isEmpty() && curMsg != 0) {
//#if defined CHARSETS
	  fprintf(stderr,"Setting viewer charset to %s\n",mMsg->bodyCharset());
	  mViewer->setCharset(mMsg->bodyCharset());
//#endif

		mailEnvelope *mEnv = mMsg->envelope();
		mailBody *mBody = mMsg->body();
	  writeMsgHeader(mEnv,NULL);
		displayBody(mBody,NULL,0);
		delete mEnv;
		delete mBody;
//  parseMsg("");
	}

  mViewer->write("</BODY></HTML>");
  mViewer->end();
  mViewer->parse();
	kbp->idle();
}

//-----------------------------------------------------------------------------
void NMMailView::writeMsgHeader(mailEnvelope *aMsgEnvelope,const char *prefix)
{
  if (!aMsgEnvelope || aMsgEnvelope->isEmpty()) return;

//Lotus has sometimes no FROM field
	if(!aMsgEnvelope->messageFrom(mailEnvelope::textNONE)) aMsgEnvelope->setMessageFrom(aMsgEnvelope->messageReplyTo(mailEnvelope::textPlain));
	if(!aMsgEnvelope->messageReplyTo(mailEnvelope::textNONE)) aMsgEnvelope->setMessageFrom(aMsgEnvelope->messageReturnPath(mailEnvelope::textPlain));

  QString str;

	if(mHeaderStyle != HdrAll) {
			tablein();
				tablerowin();
					if(mHeaderStyle == HdrFancy)
					tabledata( "<IMG SRC=" + mPicsDir + "kdelogo.xpm>",1,30);
					tabledata( "<B><FONT SIZE=+2>" + strToHtml( decodeMIME2(aMsgEnvelope->messageSubject())) + "</FONT></B>",0);
				tablerowout();

		tablerowin();mViewer->write("<TD>");
		tablein();

    if (mHeaderStyle != HdrBrief) {
				tablerowin();
					tabledata( QString("<B>") + i18n("From: ") + "</B>",0,0,1 );
				tablerowout();
		}
					tabledata( decodeMIME2(aMsgEnvelope->messageFrom(mailEnvelope::textAnchor)) );
    if (mHeaderStyle != HdrBrief) {
				tablerowin();
					tabledata( QString("<B>") + i18n("To: ") + "</B>",0,0,1 );
					tabledata( decodeMIME2(aMsgEnvelope->messageTo(mailEnvelope::textAnchor)) );
				tablerowout();
    if (aMsgEnvelope->messageCC(mailEnvelope::textNONE)) {
				tablerowin();
					tabledata( QString("<B>") + i18n("Cc: ") + "</B>",0,0,1 );
					tabledata( decodeMIME2(aMsgEnvelope->messageCC(mailEnvelope::textAnchor)) );
				tablerowout();
		}
    if (mHeaderStyle == HdrLong && aMsgEnvelope->messageBCC(mailEnvelope::textNONE)) {
				tablerowin();
					tabledata( QString("<B>") + i18n("Bcc: ") + "</B>",0,0,1 );
					tabledata( decodeMIME2(aMsgEnvelope->messageBCC(mailEnvelope::textAnchor)) );
				tablerowout();
		}
    if (mHeaderStyle == HdrLong && aMsgEnvelope->messageReplyTo(mailEnvelope::textNONE)) {
				tablerowin();
					tabledata( QString("<B>") + i18n("Reply to: ") + "</B>",0,0,1 );
					tabledata( decodeMIME2(aMsgEnvelope->messageReplyTo(mailEnvelope::textAnchor)) );
				tablerowout();
		}
				tablerowin();
					tabledata( QString("<B>") + i18n("Date: ") + "</B>",0,0,1 );
					tabledata( strToHtml(i18ndate(aMsgEnvelope->dateStr(mTimeformat,mLocalizeTime))) );
		}
				tablerowout();

			mViewer->write("</TD>");	tablerowout();
			tableout();

			tableout();
	} else {

//		str.sprintf("%s",(char *)&body->mime.text.data[0]);
		if(my_client)
		{
			char *field;
			unsigned long len;
			
			field = my_client->fetchHeaderContent(curMsg,prefix,&len);
			str = QString(field,len+1);
		} else {
			str = mMsg->header;
			
		}
		tablein();
		while(!str.isEmpty() && str[0] != 0x0a && str[0] != 0x0d) {
			tablerowin();
			int left;

			left = str.find(':');
			tabledata( "<B>" + str.left(left+1) + "</B>",0,0,1 );
			str = str.right(str.length() - left - 1);

			left = 0;
			char debug = str[left];
			while( str[left] == 0x20 || str[left] == 0x09 ) {
				left = str.find(0x0a,left) + 1;
				debug = str[left];
			}
	    tabledata( strToHtml(str.left(left)) );
			str = str.right(str.length() - left);

			tablerowout();
		}
		tableout();
	}
  mViewer->write("<BR></BR>\r\n");
}

//-----------------------------------------------------------------------------
void NMMailView::writeBodyHTML(const QString aStr)
{
	QString str = aStr;

            // ---Sven's strip </BODY> and </HTML> from end of attachment start-
            // We must fo this, or else we will see only 1st inlined html attachment
            // It is IMHO enough to search only for </BODY> and put \0 there.
            int i;
            i = str.findRev("</body>", -1, false); //case insensitive
            if (i>0)
              str.truncate(i);
            else // just in case - search for </html>
            {
              i = str.findRev("</html>", -1, false); //case insensitive
              if (i>0) str.truncate(i);
            }
            // ---Sven's strip </BODY> and </HTML> from end of attachment end-
      str.replace(QRegExp("\\\"cid:",false), "\"http://localhost/");
      str.replace(QRegExp("=cid:",false), "=http://localhost/");

            mViewer->write(str);
}


//-----------------------------------------------------------------------------
void NMMailView::writeEnrichedStr(const QString aStr)
{
  QString line(256), sig, htmlStr = "";
//  Kpgp* pgp = Kpgp::getKpgp();
//  assert( (pgp != NULL) );
	if(aStr.isNull()) return;
//  assert(!aStr.isNull());
//  bool pgpMessage = false;

	htmlStr.setNum(mFixedSize.toFloat()/4) + ">";
	mViewer->write("<FONT FACE='");
	mViewer->write(mFixedFont + "' SIZE=" + htmlStr + ">");
	htmlStr = "";
	

/*	emit signalStatusMsg(i18n("Calling PGP/GPG"));
	kbp->busy();
  if (pgp->setMessage(aStr))
  {

    QString str = pgp->frontmatter();
    if(!str.isEmpty()) htmlStr += quotedHTML(str.data());
    htmlStr += "<BR>\n";
    if (pgp->isEncrypted())
    {
      pgpMessage = true;
      if(pgp->decrypt())
      {
				line.sprintf("<B>%s</B><BR>\n",
		     (const char*)i18n("Encrypted message"));
				htmlStr += line;
      }
      else
      {
				line.sprintf("<B>%s</B><BR>\n%s<BR>\n<BR>\n",
		     (const char*)i18n("Cannot decrypt message:"),
		     (const char*)pgp->lastErrorMsg());
				htmlStr += line;
      }
    }
    // check for PGP signing
    if (pgp->isSigned())
    {
      pgpMessage = true;
      if (pgp->unknownSignature()) sig = i18n("Message maybe signed by");
      else if (pgp->goodSignature()) sig = i18n("Message was signed by");
      else sig = i18n("Warning: Bad signature from");

  */    /* HTMLize signedBy data */
   /*   QString *sdata=new QString(pgp->signedBy());
      sdata->replace(QRegExp("\""), "&quot;");
      sdata->replace(QRegExp("<"), "&lt;");
      sdata->replace(QRegExp(">"), "&gt;");

			fprintf(stderr,"PGP says: %s\n",sdata->data());
      if (sdata->contains(QRegExp("unknown key ID")))
      {
      sdata->replace(QRegExp("unknown key ID"), i18n("unknown key ID"));
      line.sprintf("<B>%s %s</B><BR>\n",sig.data(), sdata->data());
      }
      else if (sdata->contains(QRegExp("public key not found")))
      {
      sdata->replace(QRegExp("public key not found"), i18n("public key not found"));
      line.sprintf("<B>%s %s</B><BR>\n",sig.data(), sdata->data());
      }
      else
      line.sprintf("<B>%s <A HREF=\"mailto:%s\">%s</A></B><BR>\n", sig.data(),
		   sdata->data(),sdata->data());

      delete sdata;
      htmlStr += line;
    }
    htmlStr += "<PRE>" + quotedHTML(pgp->message().data()) + "</PRE>";
    if(pgpMessage) htmlStr += QString("<BR>\n<B>") + i18n("End pgp message") + "</B><BR>\n<BR>\n";
    str = pgp->backmatter();
    if(!str.isEmpty()) htmlStr += quotedHTML(str.data());


  }
  else */
	{
		htmlStr = aStr.data();
      htmlStr.replace(QRegExp("<italic>",false), "<I>");
      htmlStr.replace(QRegExp("</italic>",false), "</I>");
      htmlStr.replace(QRegExp("<bold>",false), "<B>");
      htmlStr.replace(QRegExp("</bold>",false), "</B>");
      htmlStr.replace(QRegExp("<underline>",false), "<U>");
      htmlStr.replace(QRegExp("</underline>",false), "</U>");
      htmlStr.replace(QRegExp("<fixed>",false), "<PRE>");
      htmlStr.replace(QRegExp("</fixed>",false), "</PRE>");
      htmlStr.replace(QRegExp("\\r*\\n\\r*\\n",false), "\n");
//      htmlStr.replace(QRegExp("\\r*\\n\\r*\\n",false), "<BR>\n");
      htmlStr.replace(QRegExp("\\r*\\n",false), " ");
      htmlStr.replace(QRegExp(" ",false), "&nbsp;");
      htmlStr.replace(QRegExp("<<",false), "&lt;");
  }
//	kbp->idle();

  mViewer->write(htmlStr);
	mViewer->write("</FONT>");
}

void NMMailView::writeVCardStr(const QString aStr)
{
	int error;
	VCard *aVCard;
  QString line(256), sig, htmlStr = "";
	if(aStr.isNull()) return;

	aVCard = VCard::parseVCard(aStr,&error);
	if(!error)
	{
		KMDisplayVCard *aDisplay = new KMDisplayVCard(aVCard);
		aDisplay->setCaption(i18n("VCard for ") + aVCard->getValue(VCARD_FN));
		aDisplay->show();
//		htmlStr = aVCard->getVCard();
	}	else {
		debug("Parsing VCard %d",error);
	}
  mViewer->write(htmlStr);
}

//-----------------------------------------------------------------------------
void NMMailView::writeBodyStr(QString aStr)
{
  QString line(256), sig, htmlStr = "";
  Kpgp* pgp = Kpgp::getKpgp();

	NMStatusBar::connectToStatusBar(pgp,this);
	
	if(aStr.isNull()) return;
  bool pgpMessage = false;

	htmlStr.setNum(mFixedSize.toFloat()/4) + ">";
	mViewer->write("<FONT FACE='");
	mViewer->write(mFixedFont + "' SIZE=" + htmlStr + ">");
	htmlStr = "";
	
	emit signalStatusMsg(i18n("Calling PGP/GPG"));
	kbp->busy();
  if (pgp->setMessage(aStr))
  {

    QString str = pgp->frontmatter();
    if(!str.isEmpty()) htmlStr += "<PRE>" + quotedHTML(str.data()) + "</PRE>";
    htmlStr += "<BR></BR>\r\n";
    if (pgp->isEncrypted())
    {
      pgpMessage = true;
      if(pgp->decrypt())
      {
				line.sprintf("<B>%s</B><BR></BR>\r\n",
		     (const char*)i18n("Encrypted message"));
				htmlStr += line;
      }
      else
      {
				line.sprintf("<B>%s</B><BR></BR>\r\n%s<BR></BR>\r\n<BR></BR>\r\n",
		     (const char*)i18n("Cannot decrypt message:"),
		     (const char*)pgp->lastErrorMsg());
				htmlStr += line;
      }
    }
    // check for PGP signing
    if (pgp->isSigned())
    {
      pgpMessage = true;
      if (pgp->unknownSignature()) sig = i18n("Message maybe signed by");
      else if (pgp->goodSignature()) sig = i18n("Message was signed by");
      else sig = i18n("Warning: Bad signature from");

      /* HTMLize signedBy data */
      QString sdata=pgp->signedBy();
      QString smail=pgp->signedBy();

      smail.remove(0,smail.find("<")+1);
      smail.truncate(smail.find(">"));
      sdata.replace(QRegExp("\""), "&quot;");
      sdata.replace(QRegExp("<"), "&lt;");
      sdata.replace(QRegExp(">"), "&gt;");

			fprintf(stderr,"PGP says: %s\n",sdata.data());
      if (sdata.contains(QRegExp("unknown key ID")))
      {
      sdata.replace(QRegExp("unknown key ID"), i18n("unknown key ID"));
      line.sprintf("<B>%s %s</B><BR></BR>\r\n",sig.data(), sdata.data());
      }
      else if (sdata.contains(QRegExp("public key not found")))
      {
      sdata.replace(QRegExp("public key not found"), i18n("public key not found"));
      line.sprintf("<B>%s %s</B><BR></BR>\r\n",sig.data(), sdata.data());
      }
      else
      {
      line.sprintf("<B>%s <A HREF=\"mailto:%s\">%s</A></B><BR></BR>\r\n", sig.data(),
		   smail.data(),sdata.data());
			}
      htmlStr += line;
    }
//    htmlStr += "<PRE>" + quotedHTML(pgp->message().data()) + "</PRE>";
    htmlStr += quotedHTML(pgp->message().data());
    if(pgpMessage) htmlStr += QString("<BR></BR>\r\n<B>") + i18n("End pgp message") + "</B><BR></BR>\r\n<BR></BR>\r\n";
    str = pgp->backmatter();
//    if(!str.isEmpty()) htmlStr += "<PRE>" + quotedHTML(str.data())+"</PRE>";
    if(!str.isEmpty()) htmlStr += quotedHTML(str.data());


  }
//  else htmlStr += "<PRE>" + quotedHTML(aStr.data()) + "</PRE>";
  else if(!aStr.isEmpty())
  {
			QString aFileName,aIcon,aHref,aMimeType;
			int startPos;
			QString aAnchor;
			
			if(getUUPart(aStr,&aFileName,&aHref,&aIcon,&aMimeType,&startPos))
			{
  			aStr.insert(startPos,"THANXFORALLTHEFISH");
//  		  htmlStr += "<PRE>" + quotedHTML(aStr.data())+"</PRE>";
  		  htmlStr += quotedHTML(aStr.data());
//  		  aAnchor = "</PRE>";
  		  aAnchor = "</FONT>" + aIcon;
  			if(!strncmp("image",aMimeType,5))
  			{
    		  aAnchor += "<IMG SRC=\"";
    		  aAnchor += aHref;
    		  aAnchor += "\"></IMG>";
  		  }
//  		  aAnchor += "<PRE>";

				QString tempSize;
				tempSize.setNum(mFixedSize.toFloat()/4) + ">";
				aAnchor += "<FONT FACE='" + mFixedFont + "' SIZE=" + tempSize + ">";

  		  htmlStr.replace(QRegExp("THANXFORALLTHEFISH"),aAnchor);
		  } else {
//  		  htmlStr += "<PRE>" + quotedHTML(aStr.data())+"</PRE>";
  		  htmlStr += quotedHTML(aStr.data());
		  }
  }
	
	NMStatusBar::disconnectStatusBar(pgp,this);

	kbp->idle();

	mViewer->write(htmlStr);
	mViewer->write("</FONT>");
}


//-----------------------------------------------------------------------------
QString NMMailView::quotedHTML(char * pos_in)
{
  QString htmlStr, line;
  unsigned char ch,*pos,*beg;
  bool quoted = FALSE;
  bool lastQuoted = FALSE;
  bool atStart = TRUE;

	pos = (unsigned char *)pos_in;
  htmlStr = "";

  // skip leading empty lines
  for (beg=pos; *pos && *pos<=' '; pos++)
  {
    if (*pos=='\n') beg = pos+1;
  }

  pos = beg;
  int tcnt = 0;
  QString tmpStr;

  while (1)
  {
    ch = *pos;
    if (ch=='\n' || ch=='\0')
    {
      tcnt ++;
      *pos = '\0';
      line = strToHtml((char *)beg,TRUE,TRUE);
      *pos = ch;
      if (quoted && !lastQuoted) line.prepend("<I>");
      else if (!quoted && lastQuoted) line.prepend("</I>");
//      tmpStr += line + "\n";
      tmpStr += line + "&nbsp;<BR></BR>\r\n";
      if (!(tcnt % 100)) {
				htmlStr += tmpStr;
				tmpStr.truncate(0);
      }
      beg = pos+1;
      atStart = TRUE;
      lastQuoted = quoted;
      quoted = FALSE;
    }
    else if (ch > ' ' && atStart)
    {
      if (ch=='>' || /*ch==':' ||*/ ch=='|') quoted = TRUE;
      if( quoted && !lastQuoted && !strncmp((char *)pos,">From",5) ) { quoted = FALSE;beg++;}
			atStart = FALSE;
    }
    if (!ch) break;
    pos++;
  }
  htmlStr += tmpStr;
  return htmlStr;
}

//-----------------------------------------------------------------------------
void NMMailView::writePartIcon(mailBody* aMsgPart, const char *aPartNum)
{
	mViewer->write(getPartIcon(aMsgPart,aPartNum));
}

//-----------------------------------------------------------------------------
const QString NMMailView::getPartIcon(mailBody* aMsgPart, const char *aPartNum)
{
  QString iconName, href, label;//, contDisp;
	QString mimetype;
	QString retval;

  if(aMsgPart == NULL) {
    debug("writePartIcon: aMsgPart == NULL\n");
    return retval;
  }

  debug("writePartIcon: PartNum: %s",aPartNum);

	getFileName(aMsgPart,aPartNum,href,label);

	href = QString("file:") + href + "/" + label;
	
	mimetype = aMsgPart->bodyFullType();
	mimetype = mimetype.lower();

  iconName = this->iconName(mimetype);

  retval = "<TABLE><TR><TD><A HREF=\"" + href + "\"><IMG SRC=\"" + iconName + "\" BORDER=0>"
								+ "</A></TD><TD>"+ mimetype + "<BR></BR>\r\n<A HREF=\"" + href + "\">"+ label + "</A></TD></TR>"
								+ "</TABLE>" + aMsgPart->bodyDescription() + "<BR></BR>\r\n";
	return retval;
}

//-----------------------------------------------------------------------------
void NMMailView::getFileName(mailBody* aMsgPart, const char *aMsgNum,QString &dirName,QString &fileName)
{
	fileName = aMsgPart->bodyFilename();
  if (fileName.isEmpty()) fileName = QString( aMsgPart->bodyName());
  if (fileName.isEmpty()) fileName = QString( aMsgPart->bodyDescription());
	if(!fileName.isEmpty())
	{
		int sep;
		fileName = decodeMIME2(fileName);
		sep = fileName.findRev('/');
		fileName = fileName.right( fileName.length() - sep - 1 );

      // remove quotes from the filename so that the shell does not get confused
      int c = 0;
      while ((c = fileName.find('"', c)) >= 0)
				fileName.remove(c, 1);

      c = 0;
      while ((c = fileName.find('\'', c)) >= 0)
				fileName.remove(c, 1);
	} else
		fileName = "unnamed";

  dirName.sprintf("%s/part%s", (const char*)mAttachDir, aMsgNum);
}

//-----------------------------------------------------------------------------
void NMMailView::saveToTmp(mailBody* aMsgPart, const char *aMsgNum,QString &href)
{
//--- Sven's save attachments to /tmp start ---
	QString fileName;
  QString fname;
  bool ok = true;

	getFileName(aMsgPart,aMsgNum,fname,fileName);

	debug("Got Filename : %s / %s\n",(const char *)fname,(const char *)fileName);

  if (access(fname.data(), W_OK) != 0) // Not there or not writable
    if (mkdir(fname.data(), 0) != 0 || chmod (fname.data(), S_IRWXU) != 0)
      ok = false; //failed create

  if (ok)
  {
		QString tmpStr;

		if(my_client) my_client->fetchBodyContent(curMsg,aMsgNum);

    if (fileName.isEmpty())
      fname += "/unnamed";
    else
      fname = fname + "/" + fileName;

		fillBodyDecoded(aMsgPart,tmpStr);
    if (!kStringToFile(tmpStr, fname, false, false, false))
      ok = false;


		QString mimetype = aMsgPart->bodyFullType();
		mimetype = mimetype.lower();

		//sometimes the mimetype is not set properly
		if(mimetype.find("application/octet") == 0) {
			mimetype = magicSetType(tmpStr);
			aMsgPart->setFullType(mimetype);
		}
  }
  if (ok)
  {
    href.sprintf("file:%s", fname.data());
    debug ("Wrote attachment to %s", href.data());
  }
  else
	{
//--- Sven's save attachments to /tmp end ---
    debug ("Failed writing attachment to %s", href.data());
	  href.sprintf("part://%s", aMsgNum);

	}
}

//-----------------------------------------------------------------------------
void NMMailView::displayPartContent(mailBody* aMsgPart, const char *aPartNum, const char *preferred)
{
	bool asIcon=false;
	bool isFirst;
	isFirst = (aPartNum[strlen(aPartNum)-1] == '1');

      if (isFirst && aMsgPart->bodyEType() == TypeTEXT ) asIcon = FALSE;										//dont iconize first text part
      else switch (mAttachmentStyle)
      {
      case IconicAttmnt:
				asIcon=TRUE; break;
      case InlineAttmnt:
				asIcon=FALSE; break;
      case SmartAttmnt:
				asIcon=(QString(aMsgPart->bodyDisposition()).find("INLINE")<0);
				break;
      }

		  if (!isFirst && (!preferred || !*preferred) ) mViewer->write("<BR></BR>\r\n<HR><BR></BR>\r\n");

		debug("PreferredType is '%s'\n",preferred);
		debug("TYPE: %s ICONize: %s\n",aMsgPart->bodyFullType(),asIcon ? "YES" : "NO");

	switch(aMsgPart->bodyEType()) {
		case TypeTEXT:
			{
			if(my_client) my_client->fetchBodyContent(curMsg,aPartNum);

      if(!isFirst ||  (preferred && *preferred) ) writePartIcon(aMsgPart,aPartNum);

			QString str;
			QString subtype = aMsgPart->bodySubType();
			fillBodyDecoded(aMsgPart,str);
      if (!stricmp(subtype, "html"))
      {
				if( (!preferred || !*preferred) || stricmp(preferred,"text/html") == 0 )
		    writeBodyHTML(str);
      } else if (!stricmp(subtype, "plain")) {
				if( (!preferred || !*preferred) || stricmp(preferred,"text/plain") == 0 )
				writeBodyStr(str);
      } else if (!stricmp(subtype, "enriched")) {
				if( (!preferred || !*preferred) || stricmp(preferred,"text/enriched") == 0 )
				writeEnrichedStr(str);
      } else if (!stricmp(subtype, "x-vcard")) {
//				writeVCardStr(str);
      } else {
				if(isFirst)
	      	writePartIcon(aMsgPart,aPartNum);

				if( (!preferred || !*preferred))
				writeBodyStr(str);
			}
			}
			break;
		case TypeMULTIPART:
			writePartIcon(aMsgPart,aPartNum);
			break;
		case TypeMESSAGE:
			writePartIcon(aMsgPart,aPartNum);
			if(!stricmp("delivery-status",aMsgPart->bodySubType()))
			{
				QString str;
				fillBodyDecoded(aMsgPart,str);
				writeBodyStr(str);
			}
			break;
		case TypeAPPLICATION:
			writePartIcon(aMsgPart,aPartNum);
			break;
		case TypeAUDIO:
			writePartIcon(aMsgPart,aPartNum);
			break;
		case TypeIMAGE:
			writePartIcon(aMsgPart,aPartNum);
			if(!asIcon && !aMsgPart->bodyID()) {
				QString href;
				saveToTmp(aMsgPart,aPartNum,href);

			  mViewer->write(	"<A HREF=\"" + href + "\"><IMG SRC=\"" + href + "\" BORDER=0></A>");
			} else if(aMsgPart->bodyID()) {
				QString href,cid;
				mViewer->end();
				mViewer->parse();
				cid = aMsgPart->bodyID();
				cid.replace(QRegExp(">"),"");
				cid.replace(QRegExp("<"),"");
				cid = "http://localhost/" + cid;
				saveToTmp(aMsgPart,aPartNum,href);
				href.replace(QRegExp("file:/"),"/");
//				mViewer->slotFileLoaded(cid,href);
				
				cidMapping->insert(cid,new QString(href));
				debug("Mapping %s to %s\n",(const char *)cid,(const char *)href);
			}
			break;
		case TypeVIDEO:
			writePartIcon(aMsgPart,aPartNum);
			break;
		case TypeMODEL:
			writePartIcon(aMsgPart,aPartNum);
			break;
		case TypeUNKNOWN:
			writePartIcon(aMsgPart,aPartNum);
			break;
	}
};




//-----------------------------------------------------------------------------
const QString NMMailView::strToHtml(const QString aStr, bool aDecodeQP,
				     bool aPreserveBlanks)
{
  QString qpstr, iStr, result;
  char ch, *pos, str[256];
  int i, i1, x, len;
  int maxLen = 30000;
  char htmlStr[maxLen+256];
  char* htmlPos;

/*  if (aDecodeQP) qpstr = decodeRFC1522(aStr.data());
  else*/ qpstr = aStr;

#define HTML_ADD(str,len) strcpy(htmlPos,str),htmlPos+=len

  htmlPos = htmlStr;
	if(qpstr.data() == NULL) return 0;
  for (pos=qpstr.data(),x=0; *pos; pos++,x++)
  {
    if ((int)(htmlPos-htmlStr) >= maxLen)
    {
      *htmlPos = '\0';
      result += htmlStr;
      htmlPos = htmlStr;
    }

    ch = *pos;
    if (aPreserveBlanks)
    {
      if (ch==' ')
      {
        while (*pos==' ')
        {
//          HTML_ADD(" ", 1);
          HTML_ADD("&nbsp;", 6);
          pos++, x++;
        }
        pos--, x--;

        continue;
      }
      else if (ch=='\t')
      {
	do
	{
	  HTML_ADD("&nbsp;", 6);
	  x++;
	}
	while((x&7) != 0);
      }
      // else aPreserveBlanks = FALSE;
    }
    if (ch=='<') HTML_ADD("&lt;", 4);
    else if (ch=='>') HTML_ADD("&gt;", 4);
    else if (ch=='\n') HTML_ADD("<BR></BR>\r\n", 10);
    else if (ch=='\r') ;//HTML_ADD("RETURN",6);
    else if (ch=='&') HTML_ADD("&amp;", 5);
    else if ((ch=='h' && (strncmp(pos,"http:", 5)==0 || strncmp(pos,"https:", 6)==0)) ||
	     (ch=='f' && strncmp(pos,"ftp:", 4)==0) ||
	     (ch=='m' && strncmp(pos,"mailto:", 7)==0))
    {
      for (i=0; *pos && *pos>' ' && i<255 && *pos != '"' && *pos != '>' && *pos != '\''; i++)
			{
//we must escape special characters even here
		    if (*pos=='<') {strcpy(&str[i],"&lt;");i += 3;}
		    else if (*pos=='>') {strcpy(&str[i],"&gt;");i += 3;}
		    else if (*pos=='&') {strcpy(&str[i],"&amp;");i += 4;}
				else str[i] = *pos;
				pos++;
			}
      pos--;
      while (i>0 && ispunct(str[i-1]) && str[i-1]!='/')
      {
	i--;
	pos--;
      }
      str[i] = '\0';
      HTML_ADD("<A HREF=\"", 9);
      HTML_ADD(str, strlen(str));
      HTML_ADD("\">", 2);
      HTML_ADD(str, strlen(str));
      HTML_ADD("</A>", 4);
    }
    else if (ch=='@' && isalnum(*(pos+1)))
    {
      char *startofstring = qpstr.data();
      char *startpos = pos;
      for (i=0; pos >= startofstring && *pos
	     && (isalnum(*pos)
		 || *pos=='@' || *pos=='.' || *pos=='_'||*pos=='-'
		 || *pos=='*' || *pos=='[' || *pos==']')
	     && i<255; i++, pos--)
	{
	}
      i1 = i;
      pos++;
      for (i=0; *pos && (isalnum(*pos)||*pos=='@'||*pos=='.'||
			 *pos=='_'||*pos=='-' || *pos=='*'  || *pos=='[' || *pos==']')
	     && i<255; i++, pos++)
      {
	iStr += *pos;
      }
      pos--;
      len = iStr.length();
      while (len>2 && ispunct(*pos) && (pos > startpos))
      {
	len--;
	pos--;
      }
      iStr.truncate(len);

      htmlPos -= (i1 - 1);
      if (iStr.length()>3)
	iStr = "<A HREF=\"mailto:" + iStr + "\">" + iStr + "</A>";
      HTML_ADD(iStr.data(), iStr.length());
      iStr = "";
    }

    else *htmlPos++ = ch;
  }

  *htmlPos = '\0';
  result += htmlStr;
  return result;
}


//-----------------------------------------------------------------------------
void NMMailView::printMsg(QPrinter *printer)
{
  if (!curMsg) return;
	if(printer)
	  mViewer->printWith(printer);
	else
	{
	  mViewer->print();
	}
}


//-----------------------------------------------------------------------------
const char *NMMailView::msgPartFromUrl(const char* aUrl)
{
	static char buffer[128];
	char *ptr;

	if(!aUrl) return NULL;
  snprintf(buffer,sizeof(buffer)-1,"file:%s/part", mAttachDir.data());
  int s = strlen(buffer);
  if (strncmp(aUrl, buffer, s) == 0)
  {
		strncpy(buffer,&aUrl[s],sizeof(buffer)-1);
		ptr = buffer;
		if(strncmp("uuencoded",&aUrl[s],9)) {
  		while( *ptr && *ptr != '/') ptr++;
  		*ptr = 0x00;
		}
    return buffer;
  }
  return NULL;
}


//-----------------------------------------------------------------------------
const char *NMMailView::addrFromUrl(const char* aUrl)
{
	static char buffer[128];

	if(!aUrl) return NULL;

	if(strncmp("mailto",aUrl,6)) return NULL;
	strncpy(buffer,&aUrl[7],sizeof(buffer)-1);
  return buffer;
}


//-----------------------------------------------------------------------------
void NMMailView::resizeEvent(QResizeEvent *)
{
  mViewer->setGeometry(0, 0, width()-16, height()-16);
  mSbHorz->setGeometry(0, height()-16, width()-16, 16);
  mSbVert->setGeometry(width()-16, 0, 16, height()-16);

  mSbHorz->setSteps( 12, mViewer->width() - 12 );
  mSbVert->setSteps( 12, mViewer->height() - 12 );
}


//-----------------------------------------------------------------------------
void NMMailView::closeEvent(QCloseEvent *e)
{
  QWidget::closeEvent(e);
  writeConfig();
}


//-----------------------------------------------------------------------------
void NMMailView::slotUrlOn(const char* aUrl)
{
  const char *id;
  mailBody *msgPart;
  QString str;

  id = msgPartFromUrl(aUrl);
  if (!id || !my_client)
  {
    if(aUrl && *aUrl )emit signalStatusMsg(aUrl);
  }
  else
  {
		if(my_client)
		  msgPart = new mailBody(my_client->fetchBody(curMsg,id));
		else
			msgPart = mMsg->searchPart(mAtmCurrent);
		
		if(msgPart && !msgPart->isEmpty())
		{			
  		str = QString(msgPart->bodyFilename());
      if (str.isEmpty()) str = QString(msgPart->bodyName());
      if (str.isEmpty()) str = QString(msgPart->bodyDescription());
      if (str.isEmpty()) str = QString(msgPart->bodyType()) + "/" + msgPart->bodySubType();
      str = decodeMIME2(str);
      emit signalStatusMsg(i18n("Attachment: ") + str);
  		delete msgPart;
		}
  }
}


//-----------------------------------------------------------------------------
void NMMailView::slotUrlOpen(const char* aUrl, int aButton)
{
  const char *id;

  id = msgPartFromUrl(aUrl);
  if (id != NULL)
  {
    // clicked onto an attachment
    mAtmCurrent = id;
    slotAtmOpen();
  }
  else emit urlClicked(aUrl, aButton);
}


//-----------------------------------------------------------------------------
void NMMailView::slotUrlPopup(const char* aUrl, const QPoint& aPos)
{
  const char *id;
	const char *addr;
  QPopupMenu *menu;

	if(!curMsg) return;

  id = msgPartFromUrl(aUrl);
	addr = addrFromUrl(aUrl);
  if (id)
	{
    // Attachment popup
    mAtmCurrent = id;
    menu = new QPopupMenu();
    menu->insertItem(i18n("Open..."), this, SLOT(slotAtmOpen()));
    menu->insertItem(i18n("View..."), this, SLOT(slotAtmView()));
    menu->insertItem(i18n("Save as..."), this, SLOT(slotAtmSave()));
    menu->insertItem(i18n("Print..."), this, SLOT(slotAtmPrint()));
    menu->insertItem(i18n("Properties..."), this, SLOT(slotAtmProperties()));
    menu->popup(aPos,0);
  } else if(addr) {
    // mailto popup
    mAtmCurrent = addr;
    menu = new QPopupMenu();
    menu->insertItem(i18n("Addressbook"), this, SLOT(slotAddrBook()));
    menu->popup(aPos,0);
  } else {
    // general popup
//    mAtmCurrent = id;
    mAtmCurrent = "1";
    menu = new QPopupMenu();
    menu->insertItem(i18n("Save as..."), this, SLOT(slotAtmSave()));
    menu->insertItem(i18n("Print..."), this, SLOT(slotPrint()));
    menu->insertItem(i18n("View Source..."), this, SLOT(slotViewSource()));

    menu->insertItem(i18n("Reply..."), this, SLOT(slotReply()));
    menu->insertItem(i18n("Reply to All..."), this, SLOT(slotReplyAll()));
    menu->insertItem(i18n("Forward Inline..."), this, SLOT(slotForwardInline()));
    menu->insertItem(i18n("Forward Attached..."), this, SLOT(slotForwardAttach()));

    menu->insertItem(i18n("Properties..."), this,
		     SLOT(slotAtmProperties()));
    menu->popup(aPos,0);
	}
}


//-----------------------------------------------------------------------------
void NMMailView::atmViewMsg(const char *prefix,mailBody *aMsgPart)
{
	parseMsg(prefix,aMsgPart);
}


//-----------------------------------------------------------------------------
void NMMailView::slotAtmView()
{
  QString str, pname;
  mailBody *msgPart = NULL;
  if(my_client)
  	msgPart = new mailBody(my_client->fetchBody(curMsg,mAtmCurrent));
	else
		msgPart = mMsg->searchPart(mAtmCurrent);
		
  if(msgPart)
  {
		if(!msgPart->isEmpty())
		{
      pname = msgPart->bodyFilename();
      if (pname.isEmpty()) pname= msgPart->bodyName();
      if (pname.isEmpty()) pname= msgPart->bodyDescription();
      if (pname.isEmpty()) pname=msgPart->bodyFullType();
      if (pname.isEmpty()) pname=i18n("unnamed");
    	else pname = decodeMIME2(pname);
    	
    	pname.sprintf("%s : %s : %s",caption(),mAtmCurrent,QString(pname).data());

      kbp->busy();

      NMMailView* win = new NMMailView; //new reader
  		if(curMsg && my_client) mMsg->fullFetch(my_client,curMsg);
  		win->setAttachmentStyle(InlineAttmnt);

      if ( msgPart->bodyEType() == TypeMESSAGE)
      {
  				win->slotMail(msgPart->nestedMsg());
      } else {
          win->mViewer->begin(mPicsDir);
      		win->displayPartContent(msgPart,mAtmCurrent,NULL);
          win->mViewer->end();
          win->mViewer->parse();
      }
      win->setCaption(i18n("View Attachment: ") + pname);
      win->show();

      kbp->idle();
    }
  	delete msgPart;
	}
}


//-----------------------------------------------------------------------------
void NMMailView::slotAtmOpen()
{
  QString href;
	
  mailBody *msgPart = NULL;
  if(my_client)
  	msgPart = new mailBody(my_client->fetchBody(curMsg,mAtmCurrent));
	else
		msgPart = mMsg->searchPart(mAtmCurrent);

	if(msgPart)
	{
		if(!msgPart->isEmpty())
		{
      if (msgPart->bodyEType() == TypeMESSAGE)
      {
        QString pname = msgPart->bodyFilename();
        if (pname.isEmpty()) pname= msgPart->bodyName();
        if (pname.isEmpty()) pname= msgPart->bodyDescription();
        if (pname.isEmpty()) pname=msgPart->bodyFullType();
        if (pname.isEmpty()) pname="unnamed";

        NMMailView* win = new NMMailView; //new reader
    		win->setAttachmentStyle(InlineAttmnt);
    		pname.sprintf("%s : %s : %s",caption(),mAtmCurrent,QString(pname).data());
  			win->slotMail(msgPart->nestedMsg(),pname.data());
  			win->show();
      } else if (msgPart->bodyEType() == TypeTEXT && !stricmp(msgPart->bodySubType(), "x-vcard")) {
  			QString str;
  			
  			fillBodyDecoded(msgPart,str);
  				
  			writeVCardStr(str);
      } else {
      	saveToTmp(msgPart,mAtmCurrent,href);
        href = "kfmclient openURL \'" + href + "\'";
        system(href);
      }
  	} else if(!strncmp("uuencoded",mAtmCurrent,9)) {
  		  href.sprintf("file:%s/part%s", mAttachDir.data(),mAtmCurrent);
        href = "kfmclient openURL \'" + href + "\'";
        system(href);
  	}
	 	delete msgPart;
	}
}


//-----------------------------------------------------------------------------
void NMMailView::slotAtmSave()
{
  QString fileName, str;
  fileName = QDir::currentDirPath();

	mailBody *msgPart = NULL;
	if(my_client)
		msgPart = new mailBody(my_client->fetchBody(curMsg,mAtmCurrent));
	else
		msgPart = mMsg->searchPart(mAtmCurrent);
		
	if(msgPart)
	{
  	if(!msgPart->isEmpty())
  	{
		  fileName.append("/");
      fileName.append( msgPart->bodyName() );

      fileName = KFileDialog::getSaveFileName(fileName.data(), "*", this);
      if(!fileName.isEmpty())
			{
        kbp->busy();
      	fillBodyDecoded(msgPart,str);
        if (!kStringToFile(str, fileName, TRUE))
          warning(i18n("Could not save file"));
        kbp->idle();
      }
  	} else if(!strncmp("uuencoded",mAtmCurrent,9)) {
			fileName.append(&mAtmCurrent[9]);
      fileName = KFileDialog::getSaveFileName(fileName.data(), "*", this);
	    if(!fileName.isEmpty())
	    {
				QString sourceName;
				
        kbp->busy();
  		  sourceName.sprintf("%s/part%s", mAttachDir.data(),mAtmCurrent);
				str = kFileToString(sourceName,false,false);
        if (!kStringToFile(str, fileName, TRUE))
          warning(i18n("Could not save file"));
        kbp->idle();
  	
//        href = "kfmclient openURL \'" + href + "\'";
//        system(href);
			}
    }
  	delete msgPart;
	}
}


//-----------------------------------------------------------------------------
void NMMailView::slotAtmPrint()
{
  mailBody *msgPart = NULL;
  if(my_client)
  	msgPart = new mailBody(my_client->fetchBody(curMsg,mAtmCurrent));
	else
		msgPart = mMsg->searchPart(mAtmCurrent);

	if(msgPart)
	{
  	if(!msgPart->isEmpty())
  	{
    	NMMailView* win = new NMMailView; //new reader

  		if(curMsg && my_client) mMsg->fullFetch(my_client,curMsg);
  		win->setAttachmentStyle(InlineAttmnt);

      if ( msgPart->bodyEType() == TypeMESSAGE)
      {
  				win->slotMail(msgPart->nestedMsg());
      } else {
          win->mViewer->begin(mPicsDir);
      		win->displayPartContent(msgPart,mAtmCurrent,NULL);
          win->mViewer->end();
          win->mViewer->parse();
      }
      win->setCaption(i18n("Printing Attachment"));

    	win->mViewer->print();
    	delete win;
    }
  	delete msgPart;
	}
}

//-----------------------------------------------------------------------------
void NMMailView::slotAtmProperties()
{

	mailBody *msgPart = NULL;
	if(my_client)
		msgPart = new mailBody(my_client->fetchBody(curMsg,mAtmCurrent));
	else
		msgPart = mMsg->searchPart(mAtmCurrent);

	if(msgPart)
	{

  	if(!msgPart->isEmpty())
  	{
	    KMMsgPartDlg  dlg(0,TRUE);
  	  kbp->busy();
  	  dlg.setMsgPart( msgPart);
  	  kbp->idle();
  	  dlg.exec();
  	}
  	delete msgPart;
	}
}

//-----------------------------------------------------------------------------
void NMMailView::slotScrollVert(int _y)
{
  mSbVert->setValue(_y);
}

//-----------------------------------------------------------------------------
void NMMailView::slotScrollHorz(int _x)
{
  mSbHorz->setValue(_x);
}

//-----------------------------------------------------------------------------
void NMMailView::slotScrollUp()
{
  mSbVert->setValue(mSbVert->value() - 10);	
}

//-----------------------------------------------------------------------------
void NMMailView::slotScrollDown()
{
  mSbVert->setValue(mSbVert->value() + 10);	
}

//-----------------------------------------------------------------------------
void NMMailView::slotScrollPrior()
{
  mSbVert->setValue(mSbVert->value() - (int)(height()*0.8));	
}

//-----------------------------------------------------------------------------
void NMMailView::slotScrollNext()
{
  mSbVert->setValue(mSbVert->value() + (int)(height()*0.8));	
}

//-----------------------------------------------------------------------------
void NMMailView::slotDocumentChanged()
{
  if (mViewer->docHeight() > mViewer->height())
    mSbVert->setRange(0, mViewer->docHeight() -
		    mViewer->height());
  else
    mSbVert->setRange(0, 0);

  if (mViewer->docWidth() > mViewer->width())
    mSbHorz->setRange(0, mViewer->docWidth() -
		    mViewer->width());
  else
    mSbHorz->setRange(0, 0);
}


//-----------------------------------------------------------------------------
void NMMailView::slotTextSelected(bool)
{
  QString temp;

  mViewer->getSelectedText(temp);
  kapp->clipboard()->setText(temp);
}


//-----------------------------------------------------------------------------
QString NMMailView::copyText()
{
  QString temp;
  mViewer->getSelectedText(temp);
  return temp;
}


//-----------------------------------------------------------------------------
void NMMailView::slotDocumentDone()
{
  // mSbVert->setValue(0);
}

//-----------------------------------------------------------------------------
void NMMailView::slotPrint()
{
	mViewer->print();
}
//-----------------------------------------------------------------------------
void NMMailView::slotReply()
{
	QString Msg;
	Msg.sprintf("%ld",curMsg);
	KConfig *config = kapp->getConfig();
	config->setGroup("Message");
	QString sReplyStr = formatString(config->readEntry("phrase-reply",i18n("On %D, you wrote:")),mMsg);
	QString sIndent = formatString(config->readEntry("indent-prefix",i18n(">%_")),mMsg);
	KMComposeWin *composer = new KMComposeWin( mMsg->makeReply(sReplyStr,sIndent) );
	if(my_client) my_client->flagMsg(Msg.data(),"\\ANSWERED");
	composer->show();
}
//-----------------------------------------------------------------------------
void NMMailView::slotReplyAll()
{
	QString Msg;
	Msg.sprintf("%ld",curMsg);
	KConfig *config = kapp->getConfig();
	config->setGroup("Message");
	QString sReplyStr = formatString(config->readEntry("phrase-reply-all",i18n("On %D, %F wrote:")),mMsg);
	QString sIndent = formatString(config->readEntry("indent-prefix",i18n(">%_")),mMsg);
	KMComposeWin *composer = new KMComposeWin( mMsg->makeReplyAll(sReplyStr,sIndent) );
	if(my_client) my_client->flagMsg(Msg.data(),"\\ANSWERED");
	composer->show();
}
//-----------------------------------------------------------------------------
void NMMailView::slotForwardInline()
{
	KConfig *config = kapp->getConfig();
	config->setGroup("Message");
	QString sReplyStr = formatString(config->readEntry("phrase-forward",i18n("Forwarded Message:")),mMsg);
	QString sIndent = formatString(config->readEntry("indent-prefix",i18n(">%_")),mMsg);
	KMComposeWin *composer = new KMComposeWin( mMsg->makeForwardInline(sReplyStr,sIndent) );
	composer->show();
}
//-----------------------------------------------------------------------------
void NMMailView::slotForwardAttach()
{
	KMComposeWin *composer = new KMComposeWin( mMsg->makeForwardAttach() );
	composer->show();
}


//-----------------------------------------------------------------------------
void NMMailView::slotAddrBook()
{
  if (!mAtmCurrent || !*mAtmCurrent) return;
  addrBook->insert(mAtmCurrent);
  emit signalStatusMsg(i18n("Address added to addressbook."));
}
//-----------------------------------------------------------------------------

QString NMMailView::magicSetType(QString bod)
{
  QString mimetype;
//  int sep;

  if (!sMagic)
  {
    // initialize mime magic
    sMagic = new KMimeMagic(kapp->kde_mimedir() + "/magic");
    sMagic->setFollowLinks(TRUE);
  }

//  if (aAutoDecode) bod = bodyDecoded();
//  else bod = mBody;

  mimetype = sMagic->findBufferType(bod, bod.size()-1)->getContent();
/*  sep = mimetype.find('/');
  mType = mimetype.left(sep);
  mSubtype = mimetype.mid(sep+1, 64); */
	return mimetype;
}

QString NMMailView::magicSetType(const char *bod,int bod_size)
{
  QString mimetype;

  if (!sMagic)
  {
    // initialize mime magic
    sMagic = new KMimeMagic(kapp->kde_mimedir() + "/magic");
    sMagic->setFollowLinks(TRUE);
  }

  mimetype = sMagic->findBufferType(bod, bod_size)->getContent();
	return mimetype;
}


//-----------------------------------------------------------------------------
const QString NMMailView::iconName(QString mimetype)
{
  QString fileName, icon;
  QDir dir;
	int sep;

	sep = mimetype.findRev('/');
 	QString mType = mimetype.left(sep);
  fileName = KApplication::kde_mimedir() + "/" + mimetype + ".kdelnk";

  if (dir.exists(fileName))
  {
    KConfig config(fileName);
    config.setGroup("KDE Desktop Entry");
    icon = config.readEntry("Icon");
  }
  else //Try harder
  {
//    QString mime;
//    mime = mType + "/" + mSubtype;
	  sep = mimetype.find('/');

    dir.setPath(KApplication::kde_mimedir() + "/" + mType);
    if (dir.exists())
    {
      unsigned int i;
      QString fqn;
      for (i=0; i<dir.count(); i++)
      {
        if (strcmp (dir[i], ".") == 0 || strcmp (dir[i], "..") == 0)
          continue;
        fqn = dir.path();
        fqn += "/";
        fqn += dir[i];
        //debug ("try: %s", fqn.data());
        KSimpleConfig conf(fqn.data(), true);
        conf.setGroup("KDE Desktop Entry");
        if (conf.readEntry ("MimeType") == mimetype)
          icon = conf.readEntry("Icon");
      }
    }
  }

  if(icon.isEmpty()) // If no icon specified.
    if (mType == "message")
      icon = "kimap.xpm";
    else
      icon = "unknown.xpm";
  return KApplication::kde_icondir() + "/" + icon;
}

void NMMailView::setFocus()
{
	mViewer->setFocus();
}

void NMMailView::tablein(int border)
{
	QString table;
	table.sprintf("<TABLE BORDER=%d>",border);
	mViewer->write( table );
}
void NMMailView::tableout()
{
	mViewer->write("</TABLE>");
}

void NMMailView::tablerowin()
{
	mViewer->write("<TR>");
}
void NMMailView::tablerowout()
{
	mViewer->write("</TR>");
}

void NMMailView::tabledata(const char *content,int colspan,int rowspan,int align)
{
	char buffer[8];

	mViewer->write( "<TD VALIGN=TOP" );
	if(align) {
		mViewer->write( " ALIGN=RIGHT" );
	}
	if(colspan) {
		mViewer->write( " COLSPAN=" );
		snprintf(buffer,sizeof(buffer)-1,"%d",colspan);
		mViewer->write( buffer );
	}
	if(rowspan) {
		mViewer->write( " ROWSPAN=" );
		snprintf(buffer,sizeof(buffer)-1,"%d",rowspan);
		mViewer->write( buffer );
	}
	mViewer->write( ">" );
	mViewer->write( content );
	mViewer->write( "</TD>" );
}

//-----------------------------------------------------------------------------
const QString NMMailView::formatString(const QString aStr,mailEnvelope *env) const
{
  QString result, str;
  const char* pos;
  char ch, cstr[64];
  tm *time;
  int i;

  if (aStr.isEmpty()) return aStr;

  for (pos=aStr.data(); *pos; )
  {
    ch = *pos++;
    if (ch=='%')
    {
      ch = *pos++;
      switch (ch)
      {
      case 'D':
	time = env->messageDate();
	strftime(cstr, sizeof(cstr), i18n("%a, %d %b %Y"), time);
	result += i18ndate(cstr);
	break;
      case 'F':
	result += env->messageFrom();
	break;
      case 'f':
	str = env->messageFrom();
	for (i=0; str[i]>' '; i++)
	  ;
	for (; i < (int)str.length() && str[i] <= ' '; i++)
	  ;
	result += str[0];
	if (str[i]>' ') result += str[i];
	else if (str[1]>' ') result += str[1];
	break;
      case 'S':
	result += env->messageSubject();
	break;
      case '_':
	result += ' ';
	break;
      case '%':
	result += '%';
	break;
      default:
	result += '%';
	result += ch;
	break;
      }
    }
    else result += ch;
  }
  return result;
}

void NMMailView::asciiArmor(QString &str)
{
	int pos = 0;
	char debug;
	while(pos >= 0)
	{
		debug = str[pos];
		if(str[pos] == '-') str.insert(pos,"- ");
		pos = str.find(0x0a,pos);
		if(pos >= 0) pos++;
	}
}

//sad but true: urls with cid: dont get here
void NMMailView::slotFileRequest(const char *url)
{
	QString *cidMap = cidMapping->find(url);
	if(cidMap) mViewer->slotFileLoaded(url,cidMap->data());
	fprintf(stderr,"URL-Requested: %s\n",url);
}

void NMMailView::setViewerFont(const char *mFont)
{
  mViewer->setStandardFont(kstrToFont(mFont).family());
  // --- sven's get them font sizes right! start ---
  int i, fntSize=0, diff;
  fntSize = kstrToFont(mFont).pointSize();
  //debug ("Fontsize: %d", fntSize);

  int fontsizes[7];
  mViewer->resetFontSizes();
  mViewer->getFontSizes(fontsizes);
  diff= fntSize - fontsizes[3];
  if (fontsizes[0]+diff > 0)
  {
    for (i=0;i<7; i++)
    {
      //debug ("Old fontsize #%d: %d", i, fontsizes[i]);
      fontsizes[i]+=diff;
      //debug ("New fontsize #%d: %d", i, fontsizes[i]);
    }
  }
  mViewer->setFontSizes(fontsizes);
  // --- sven's get them font sizes right! end ---
  //mViewer->setFixedFont(mFixedFont);
//  mViewer->write("</FONT><FONT FACE='ARIAL'>");
}

NMHTMLWidget::NMHTMLWidget(	QWidget *parent, const char *name, const char *pixdir)
: KHTMLWidget(parent,name,pixdir)
{
}

#define PRINTING_MARGIN		36	// printed margin in 1/72in units
// Print the current document to the printer.
// This currently prints the entire document without releasing control
// to the event loop.  This isn't a problem for small documents, but
// may be annoying for very large documents.  If this is changed in
// the future, it should be noted that this widget CANNOT be redrawn
// while it is printing as its layout is recalculated to suit the paper
// size it is being printed on.
//
void NMHTMLWidget::printWith(QPrinter *printer)
{
    float scalers[] = { 1.1, 1.0, 0.9, 0.75, 0.6, 0.4 };

	bool newPainter = false;
	int pgWidth = 595, pgHeight = 842;

	// we have to wait till the widget has finished parsing
	while(parsing)
	{
		fprintf(stderr,"Waiting for Parsing %p\n",this);
		kapp->processEvents(10);
	}
	
	switch ( printer->pageSize() )
	{
	    case QPrinter::A4:
		pgWidth = 595;
		pgHeight = 842;
		break;

	    case QPrinter::B5:
		pgWidth = 516;
		pgHeight = 729;
		break;

	    case QPrinter::Letter:
		pgWidth = 612;
		pgHeight = 792;
		break;

	    case QPrinter::Legal:
		pgWidth = 612;
		pgHeight = 1008;
		break;

	    case QPrinter::Executive:
		pgWidth = 540;
		pgHeight = 720;
		break;

            default:
	      assert( 0 );
	}

	if ( printer->orientation() == QPrinter::Landscape )
	{
	    int tmp = pgWidth;
	    pgWidth = pgHeight;
	    pgHeight = tmp;
	}

	pgWidth -= ( 2*PRINTING_MARGIN );
	pgHeight -= ( 2*PRINTING_MARGIN );

	pgWidth = (int) ((float)pgWidth / scalers[ settings->fontBaseSize ] );
	pgHeight = (int) ((float)pgHeight / scalers[ settings->fontBaseSize ] );

	int margin = (int)((float)PRINTING_MARGIN /
		scalers[ settings->fontBaseSize ] );

	QPainter prPainter;
	prPainter.begin( printer );

	prPainter.scale( scalers[ settings->fontBaseSize ],
		scalers[ settings->fontBaseSize ] );

	clue->recalcBaseSize( &prPainter );
	clue->reset();
	clue->setMaxWidth( pgWidth );
	clue->calcSize();
	clue->setPos( 0, clue->getAscent() );
	calcAbsolutePos();

	unsigned numBreaks = 1;
	int pos = 0;
	QArray<int> breaks( 10 );
	breaks[0] = 0;

	do
	{
	    debugM( "Break pos = %d\n", pos );
	    pos = clue->findPageBreak( pos + pgHeight );
	    if ( pos >= 0 )
	    {
		breaks[ numBreaks ] =  pos;
		numBreaks++;
		if ( numBreaks == breaks.size() )
		    breaks.resize( numBreaks + 10 );
	    }
	}
	while ( pos > 0 );

	for ( unsigned b = 0; b < numBreaks; b++ )
	{
	    int printHeight;
	    if ( b < numBreaks - 1 )
		printHeight = breaks[b+1] - breaks[b];
	    else
		printHeight = pgHeight;
	    clue->print( &prPainter, 0, breaks[b], pgWidth, printHeight,
	    margin, margin-breaks[b], true );
	    if ( b < numBreaks - 1 )
	    {
		printer->newPage();
	    }
	}

	prPainter.end();

	if ( painter == 0 )
	{
	    painter = new QPainter;
	    painter->begin( this );
	    newPainter = TRUE;
	}

	clue->recalcBaseSize( painter );
	calcSize();
	calcAbsolutePos();

	if ( newPainter )
	{
	    painter->end();
	    delete painter;
	    painter = 0;
	}
}

void NMMailView::initMenu()
{
  ///////////////////////////////////////////////////////////////////
  // menuBar entry view_menu
  view_menu = new QPopupMenu();
  view_menu->setCheckable(true);
  view_menu->insertItem(i18n("&Toolbar"), ID_VIEW_TOOLBAR);
  view_menu->insertItem(i18n("&Statusbar"), ID_VIEW_STATUSBAR );
 	view_menu->insertSeparator();
  view_menu->insertItem(i18n("&Brief Headers"),ID_VIEW_HEADER +
		       NMMailView::HdrBrief);
  view_menu->insertItem(i18n("&Fancy Headers"),ID_VIEW_HEADER +
		       NMMailView::HdrFancy);
  view_menu->insertItem(i18n("&Standard Headers"),ID_VIEW_HEADER +
		       NMMailView::HdrStandard);
  view_menu->insertItem(i18n("&Long Headers"),ID_VIEW_HEADER +
		       NMMailView::HdrLong);
  view_menu->insertItem(i18n("&All Headers"),ID_VIEW_HEADER +
		       NMMailView::HdrAll);
  view_menu->insertSeparator();
  view_menu->insertItem(i18n("Iconic Attachments"),ID_VIEW_ATTACHM + NMMailView::IconicAttmnt);
  view_menu->insertItem(i18n("Smart Attachments"),ID_VIEW_ATTACHM + NMMailView::SmartAttmnt);
  view_menu->insertItem(i18n("Inlined Attachments"),ID_VIEW_ATTACHM + NMMailView::InlineAttmnt);
  view_menu->setItemChecked((int)headerStyle() + ID_VIEW_HEADER, TRUE);
  view_menu->setItemChecked((int)attachmentStyle()+ ID_VIEW_ATTACHM, TRUE);
	
	if(topLevelWidget()->isA("KTMainWindow")
	|| topLevelWidget()->isA("KImap"))
	{
		((KTMainWindow *)topLevelWidget())->menuBar()->insertItem(i18n("&View"), view_menu);
	}

  connect(view_menu,SIGNAL(activated(int)),SLOT(commandCallback(int)));
  connect(view_menu,SIGNAL(highlighted(int)),SLOT(statusCallback(int)));
}

void NMMailView::commandCallback(int id_){
  switch (id_){
    case ID_FILE_SAVE_FILE:
//    	emit signalMMove(-1);
    	break;
    case ID_FILE_COMPOSE:
//    	slotCompose();
    	break;
    case ID_FILE_ADDRESSBOOK:
//    	slotAddrBook();
    	break;
    case ID_FILE_PREFERENCES:
//    	slotPreferences();
    	break;
    case ID_FILE_PRINT:
			printMsg(NULL);
//    	slotFilePrint();
    	break;
    case ID_FILE_QUIT:
//    	slotFileQuit();
    	break;

		case ID_MESSAGE_REPLY:
			slotReply();
			break;
		case ID_MESSAGE_REPLYALL:
			slotReplyAll();
			break;
		case ID_MESSAGE_FORWARDINLINE:
			slotForwardInline();
			break;
		case ID_MESSAGE_FORWARDATTACHED:
			slotForwardAttach();
			break;

    case ID_VIEW_TOOLBAR:
    	if(topLevelWidget()->isA("KTMainWindow")
    	|| topLevelWidget()->isA("KImap"))
    	{
      	if( view_menu->isItemChecked(ID_VIEW_TOOLBAR))
      	  view_menu->setItemChecked(ID_VIEW_TOOLBAR, false);
      	else
      		view_menu->setItemChecked(ID_VIEW_TOOLBAR, true);
      		
				((KTMainWindow *)topLevelWidget())->enableToolBar();
    	}
    	break;
    case ID_VIEW_STATUSBAR:
    	if(topLevelWidget()->isA("KTMainWindow")
    	|| topLevelWidget()->isA("KImap"))
    	{
      	if( view_menu->isItemChecked(ID_VIEW_STATUSBAR))
      	  view_menu->setItemChecked(ID_VIEW_STATUSBAR, false);
      	else
      		view_menu->setItemChecked(ID_VIEW_STATUSBAR, true);

				((KTMainWindow *)topLevelWidget())->enableStatusBar();
    	}
    	break;
    default:
		  switch (10 * (int)(id_ / 10)){
				case ID_VIEW_HEADER:
				  view_menu->setItemChecked((int)headerStyle() + ID_VIEW_HEADER, FALSE);
					setHeaderStyle((enum NMMailView::HeaderStyle)(id_ % 10));
				  view_menu->setItemChecked((int)headerStyle() + ID_VIEW_HEADER, TRUE);
				break;
				case ID_VIEW_ATTACHM:
				  view_menu->setItemChecked((int)attachmentStyle()+ ID_VIEW_ATTACHM, FALSE);
					setAttachmentStyle((enum NMMailView::AttachmentStyle)(id_ % 10));
				  view_menu->setItemChecked((int)attachmentStyle()+ ID_VIEW_ATTACHM, TRUE);
				break;
				default:
				break;
			}
    	break;
  }
}

void NMMailView::statusCallback(int id_){
  switch (id_){
    case ID_FILE_PRINT:
//    	slotStatusHelpMsg(i18n("Prints the current document"));
    	break;
    case ID_FILE_QUIT:{
    	QString caption=kapp->getCaption();
//    	slotStatusHelpMsg(i18n("Exits "+ caption));
    	break;
    }

    case ID_VIEW_TOOLBAR:
//    	slotStatusHelpMsg(i18n("Enables / disables the actual Toolbar"));
    	break;
    case ID_VIEW_STATUSBAR:
//    	slotStatusHelpMsg(i18n("Enables / disables the actual Statusbar"));
    	break;
		default:
			break;
  }
}

/*void NMMailView::slotViewToolBar()
{
  ///////////////////////////////////////////////////////////////////
  // turn Toolbar on or off
	if( view_menu->isItemChecked(ID_VIEW_TOOLBAR))
	  view_menu->setItemChecked(ID_VIEW_TOOLBAR, false);
	else
		view_menu->setItemChecked(ID_VIEW_TOOLBAR, true);
		
  enableToolBar();

  slotStatusMsg(i18n("Ready."));
}

void NMMailView::slotViewStatusBar()
{
  ///////////////////////////////////////////////////////////////////
  //turn Statusbar on or off
	if( view_menu->isItemChecked(ID_VIEW_STATUSBAR))
	  view_menu->setItemChecked(ID_VIEW_STATUSBAR, false);
	else
		view_menu->setItemChecked(ID_VIEW_STATUSBAR, true);

  enableStatusBar();

  slotStatusMsg(i18n("Ready."));
} */

bool NMMailView::getUUPart(QString &aStr,QString *aFilename,QString *aLocation,QString *aIcon,QString *aMimeType,int *aPosition)
{		
		QRegExp uuBegin("\r?\nbegin [0-9][0-9][0-9] [^ \r\n]+\r?\n");
		int posBegin,lenBegin;
		QString uuPart;
				
		posBegin = uuBegin.match(aStr,0,&lenBegin);
		if( posBegin != -1 )
		{
			int curLine = posBegin+lenBegin;
			int lenLine,posLine;
			QRegExp uuLine("M............................................................");
			QRegExp uuLast(".*\r?\n\x60\r?\nend");
			
			debug("NMMailView::getUUPart - matched uudecode begin at %d [%d]",posBegin,lenBegin);
			uuPart = aStr.mid(posBegin,lenBegin);
			posLine = uuLine.match(aStr,curLine,&lenLine);
			while( posLine != -1 )
			{
//				debug("NMMailView::getUUPart - matched uudecode line at %d [%d]",posLine,lenLine);
				curLine = posLine+lenLine;
				posLine = uuLine.match(aStr,curLine,&lenLine);
			}
			posLine = uuLast.match(aStr,curLine,&lenLine);
			if( posLine != -1 )
			{
				debug("NMMailView::getUUPart - matched uudecode end at %d [%d]",posLine,lenLine);
				DwUuencode mEncoder;
				QString aHref,mMimeType;
				mailBody aMsgPart;
				
				mEncoder.SetAsciiChars( DwString( aStr.mid(posBegin,posLine+lenLine-posBegin) ) );
				mEncoder.Decode();
				
        aMsgPart.setBodyName( mEncoder.FileName() );
      	aMsgPart.setEncoding("8bit");
        aMsgPart.bodyEncoded(mEncoder.BinaryChars().data(),mEncoder.BinaryChars().size());
        aMsgPart.setDisposition("uuencoded");
      	aMsgPart.setFilename(mEncoder.FileName() );
				mMimeType = magicSetType(mEncoder.BinaryChars().data(),mEncoder.BinaryChars().size());
				aMsgPart.setFullType(mMimeType);
				saveToTmp(&aMsgPart,"uuencoded",aHref);
				
				aStr.replace(posBegin,posLine+lenLine-posBegin,"");
			  if(aPosition) *aPosition = posBegin;
			  if(aMimeType) *aMimeType = mMimeType;
			  if(aIcon) *aIcon = getPartIcon(&aMsgPart,"uuencoded");
			  if(aLocation) *aLocation = aHref;
			  if(aFilename) *aFilename = mEncoder.FileName();
				debug("NMMailView::getUUPart - %s as %s [%s]",mEncoder.FileName(),aHref.data(),mMimeType.data() );
			  return true;
			} else {
			  return false;
			}
		}
	  return false;
}

void NMMailView::slotViewSource()
{
	if(mMsg && !mMsg->isEmpty())
	{
  	QMultiLineEdit *sourceView;
  	kbp->busy();
		QString str = bodyQString(mMsg,true);
    kbp->idle();
		sourceView = new QMultiLineEdit();
		sourceView->setReadOnly( true );
		sourceView->setText(str);
		sourceView->show();
  }
}