/***************************************************************************
                          nmmailview.h  -  description
                             -------------------
    begin                : Sat Oct 16 1999
    copyright            : (C) 1999 by Sven Carstens
    email                : s.carstens@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

class KHTMLWidget;
class NMHTMLWidget;
class QPrinter;

#ifndef NMMAILVIEW_H
#define NMMAILVIEW_H

#include <qwidget.h>
#include <qlistview.h>
#include <html.h>
#include <cpp-client.h>

/**
  *@author Sven Carstens
  */

class NMClient;


class NMMailView : public QWidget {
   Q_OBJECT
public: 
	NMMailView(QWidget *parent=0, const char *name=0);
	~NMMailView();

public slots:
	void slotMail(NMClient *,unsigned long);
	void slotMail(mailMessage *,const char *caption=NULL);
	
private:
	mailMessage *mMsg;
	MailStream *my_client;
	unsigned long curMsg;
	void displayBody(mailBody *,const char *,long);
	const QString emailAddrAsAnchor(ADDRESS *,bool);
	void initMenu();
	bool getUUPart(QString &aStr,QString *aFilename=NULL,QString *aLocation=NULL,QString *aIcon=NULL,QString *aMimeType=NULL,int *aPosition=NULL);
	
public:

	void displayPartContent(mailBody* aMsgPart,const char *aMsgNum,const char *preferred);

//-------------------------------------------------------------------
  /** Read settings from app's config file. */
  virtual void readConfig(void);

  /** Write settings to app's config file. Calls sync() if withSync is TRUE. */
  virtual void writeConfig(bool withSync=TRUE);

  /** Get/set maximum lines of text for inline attachments. */
  int inlineAttach(void) const { return mAtmInline; }
  virtual void setInlineAttach(int maxLines);

  /** Style of the message header. */
  enum HeaderStyle { HdrFancy=1, HdrBrief=2, HdrStandard=3, HdrLong=4,
                     HdrAll=5 };
  //** Stype of attachments. */
  enum AttachmentStyle {IconicAttmnt=1, SmartAttmnt =2, InlineAttmnt = 3};

  /** Get/set the message header style. */
  HeaderStyle headerStyle(void) const { return mHeaderStyle; }
  virtual void setHeaderStyle(HeaderStyle style);

  /** Get/set the message attachment style. */
  AttachmentStyle attachmentStyle(void) const { return mAttachmentStyle;}
  virtual void setAttachmentStyle(int style);

  /** Returns the current message or NULL if none. */
//  DwMessage* msg(void) const { return mMsg; }

  /** Print current message. */
  virtual void printMsg(QPrinter *printer=NULL);

  /** Return selected text */
  QString copyText();

  /** Get/set auto-delete msg flag. */
  bool autoDelete(void) const { return mAutoDelete; }
  void setAutoDelete(bool f) { mAutoDelete=f; }

  /** Get/set message body font. */
  virtual void setBodyFont(const QString);
  const QString bodyFont(void) const { return mBodyFont; }

  /** Returns path where attachments are kept. Gets set with first
    KMReaderWin that is created. */
  QString attachDir(void) { return mAttachDir; }

	void update(void) { parseMsg(); };

signals:
	void signalProgressNotify(enum mailProgress,unsigned long,const char *,bool &);	

  /** Emitted to show a text on the status line. */
  void signalStatusMsg(const char* text);

  /** The user presses the right mouse button. 'url' may be NULL. */
  void popupMenu(const char* url, const QPoint& mousePos);

  /** The user has clicked onto an URL that is no attachment. */
  void urlClicked(const char* url, int button);

//  /** The user wants to see the attachment which is message */
//  void showAtmMsg (BODY *msg);

public slots:
	void commandCallback(int);
	void statusCallback(int);
  /** HTML Widget scrollbar and layout handling. */
  void slotScrollVert(int _y);
  void slotScrollHorz(int _x);
  void slotScrollUp();
  void slotScrollDown();
  void slotScrollPrior();
  void slotScrollNext();
  void slotDocumentChanged();
  void slotDocumentDone();
  void slotTextSelected(bool);

	void slotPrint();
	void slotViewSource();
	void slotReply();
	void slotReplyAll();
	void slotForwardInline();
	void slotForwardAttach();

  /** An URL has been activate with a click. */
  void slotUrlOpen(const char* url, int button);

  /** The mouse has moved on or off an URL. */
  void slotUrlOn(const char* url);

  /** The user presses the right mouse button on an URL. */
  void slotUrlPopup(const char* url, const QPoint& mousePos);

	void slotAddrBook();

protected slots:
  /** Some attachment operations. */
  void slotAtmOpen();
  void slotAtmView();
  void slotAtmPrint();
  void slotAtmSave();
  void slotAtmProperties();

	void slotFileRequest(const char *);

protected:

	const QString bodyDecoded(const QString,BODY *);
  /** Parse given message and add it's contents to the reader window. */
  virtual void parseMsg();
  virtual void parseMsg(const char *prefix,mailBody *aMsgPart);
//  virtual void parseMsg();

  /** Creates a nice mail header depending on the current selected
    header style. */
  virtual void writeMsgHeader(mailEnvelope *aMsgEnvelope,const char *);

  /** Feeds the HTML widget with the contents of the given message-body
    string. Contains HTML. */
  virtual void writeBodyHTML(const QString bodyString);

  /** Feeds the HTML widget with the contents of the given message-body
    string. May contain body parts. */
  virtual void writeBodyStr(QString bodyString);

  virtual void writeEnrichedStr(const QString bodyString);
	virtual void writeVCardStr(const QString bodyString);
  /** Create a nice icon with comment and name for the given
    body part, appended to the HTML view. Content type and subtype
    are set afterwards if they were not before. */
  virtual void writePartIcon(mailBody* msgPart, const char *partNumber);
  virtual const QString getPartIcon(mailBody* msgPart, const char *partNumber);

	void getFileName(mailBody* aMsgPart, const char *aMsgNum,QString &dirName,QString &fileName);

	/** save the Body Content to Tmp Path */
	void saveToTmp(mailBody *,const char *,QString &);

  /** Convert given string to HTML. Converts blanks and tabs at
    beginning of line to non-breakable spaces if preserveLeadingBlanks
    is TRUE. */
  virtual const QString strToHtml(const QString str,
				  bool decodeQuotedPrintable=TRUE,
				  bool preserveLeadingBlanks=FALSE);

  /** Change the string to `quoted' html (meaning, that the quoted
    part of the message get italized */
  QString quotedHTML(char * pos);

  /** HTML initialization. */
  virtual void initHtmlWidget(void);

  /** Some necessary event handling. */
  virtual void closeEvent(QCloseEvent *);
  virtual void resizeEvent(QResizeEvent *);

  /** Returns id of message part from given URL or -1 if invalid. */
  virtual const char *msgPartFromUrl(const char* url);

	virtual const char *addrFromUrl(const char* aUrl);

  /** View message part of type message/RFC822 in extra viewer window. */
  virtual void atmViewMsg(const char *,mailBody *);

  /** Create directory for attachments */
  virtual void makeAttachDir(void);

	const QString iconName(QString);
	QString magicSetType(QString);
	QString magicSetType(const char *,int);

protected:
	class QPopupMenu *view_menu;
  int mAtmInline;
  const char *mAtmCurrent;
  NMHTMLWidget *mViewer;
  QScrollBar *mSbVert, *mSbHorz;
  QString mPicsDir;
  HeaderStyle mHeaderStyle;
  AttachmentStyle mAttachmentStyle;
  bool mAutoDelete;
  QString mBodyFont;
	QString mFixedFont;
	QString mFixedSize;
	QString mBodySize;
  QString mAttachDir;
	QString mTimeformat;
	bool mLocalizeTime;
	void setFocus(void);

	QDict<QString> *cidMapping;

	void NMMailView::tablein(int border=0);
	void NMMailView::tableout(void);
	void NMMailView::tablerowin(void);
	void NMMailView::tablerowout(void);
	void NMMailView::tabledata(const char *content,int colspan=0,int rowspan=0,int align=0);

	void asciiArmor(QString &);

	const QString formatString(const QString aStr,mailEnvelope *env) const;
	void setViewerFont(const char *);
};

class NMHTMLWidget : public KHTMLWidget {
public:
	NMHTMLWidget ( QWidget *parent = 0L, const char *name = 0L, const char *pixdir = 0L );
	void printWith(QPrinter *);
};
#endif
