/***************************************************************************
                          nmsearch.cpp  -  description
                             -------------------
    begin                : Mon Mar 20 2000
    copyright            : (C) 2000 by Sven Carstens
    email                : s.carstens@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
// kmsettings.cpp

#include <qdir.h>

//#include "kmacctlocal.h"
//#include "kmacctmgr.h"
//#include "kmacctpop.h"
//#include "kmacctseldlg.h"
//#include "kmfolder.h"
//#include "kmfoldermgr.h"
//#include "kmglobal.h"
#include "kmidentity.h"
//#include "kmmainwin.h"
//#include "kmsender.h"
//#include "kmmessage.h"
#include "kpgp.h"
#include "kfontdialog.h"
#include "kfontutils.h"
//#include "kmtopwidget.h"
#include "extern.h"
#include "kimap.h"

#include <kapp.h>
#include <kfiledialog.h>
#include <ktablistbox.h>
#include <qbuttongroup.h>
#include <qfiledialog.h>
#include <qframe.h>
#include <qgroupbox.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qcheckbox.h>
#include <klocale.h>
#include <kconfig.h>

#ifdef HAVE_PATHS_H
#include <paths.h>
#endif

#include "nmsearch.h"

//-----------------------------------------------------------------------------
NMSearch::NMSearch(QWidget *parent, const char *name) :
  QTabDialog(parent, name, TRUE)
{
  initMetaObject();

  setCaption(i18n("Search in Folder"));
  //resize(500,600);
  setOKButton(i18n("OK"));
  setCancelButton(i18n("Cancel"));

  connect(this, SIGNAL(applyButtonPressed()), this, SLOT(doApply()));
  connect(this, SIGNAL(cancelButtonPressed()), this, SLOT(doCancel()));

  createTabSearch(this);
}


//-----------------------------------------------------------------------------
NMSearch::~NMSearch()
{
  debug("~NMSearch");
}


//-----------------------------------------------------------------------------
// Create an input field with a label and optional detail button ("...")
// The detail button is not created if detail_return is NULL.
// The argument 'label' is the label that will be left of the entry field.
// The argument 'text' is the text that will show up in the entry field.
// The whole thing is placed in the grid from row/col to the right.
static QLineEdit* createLabeledEntry(QWidget* parent, QGridLayout* grid,
				     const char* aLabel,
				     const char* aText,
				     int gridy, int gridx,
				     QPushButton** detail_return=NULL)
{
  QLabel* label = new QLabel(parent);
  QLineEdit* edit = new QLineEdit(parent);
  QPushButton* sel;

  label->setText(aLabel);
  label->adjustSize();
  label->resize((int)label->sizeHint().width(),label->sizeHint().height() + 6);
  label->setMinimumSize(label->size());
  grid->addWidget(label, gridy, gridx++);

  if (aText) edit->setText(aText);
  edit->setMinimumSize(100, label->height()+2);
  edit->setMaximumSize(1000, label->height()+2);
  grid->addWidget(edit, gridy, gridx++);

  if (detail_return)
  {
    sel = new QPushButton("...", parent);
    sel->setFocusPolicy(QWidget::NoFocus);
    sel->setFixedSize(sel->sizeHint().width(), label->height());
    grid->addWidget(sel, gridy, gridx++);
    *detail_return = sel;
  }

  return edit;
}


//-----------------------------------------------------------------------------
// Add a widget with a label and optional detail button ("...")
// The detail button is not created if detail_return is NULL.
// The argument 'label' is the label that will be left of the entry field.
// The whole thing is placed in the grid from row/col to the right.
static void addLabeledWidget(QWidget* parent, QGridLayout* grid,
			     const char* aLabel, QWidget* widg,
			     int gridy, int gridx,
			     QPushButton** detail_return=NULL)
{
  QLabel* label = new QLabel(parent);
  QPushButton* sel;

  label->setText(aLabel);
  label->adjustSize();
  label->resize((int)label->sizeHint().width(),label->sizeHint().height() + 6);
  label->setMinimumSize(label->size());
  grid->addWidget(label, gridy, gridx++);

  widg->setMinimumSize(100, label->height()+2);
  //widg->setMaximumSize(1000, label->height()+2);
  grid->addWidget(widg, gridy, gridx++);

  if (detail_return)
  {
    sel = new QPushButton("...", parent);
    sel->setFocusPolicy(QWidget::NoFocus);
    sel->setFixedSize(sel->sizeHint().width(), label->height());
    grid->addWidget(sel, gridy, gridx++);
    *detail_return = sel;
  }
}


//-----------------------------------------------------------------------------
QPushButton* NMSearch::createPushButton(QWidget* parent, QGridLayout* grid,
					  const char* label,
					  int gridy, int gridx)
{
  QPushButton* button = new QPushButton(parent, label);
  button->setText(i18n(label));
  button->adjustSize();
  button->setMinimumSize(button->size());
  grid->addWidget(button, gridy, gridx);

  return button;
}


//-----------------------------------------------------------------------------
void NMSearch::createTabSearch(QWidget* parent)
{
  QWidget* tab = new QWidget(parent);
  QGridLayout* grid;
  QBoxLayout* box = new QBoxLayout(tab, QBoxLayout::TopToBottom, 8);
  QPushButton* button;
  QButtonGroup*   bgrp;
	QGroupBox *grp;

  grp = new QGroupBox(i18n("Searching Folders"), tab);
  box->addWidget(grp);
  grid = new QGridLayout(grp, 6, 1, 20, 6);

  searchEdit = createLabeledEntry(grp, grid, i18n("Search for:"),
				"", 0, 0);
//  sigEdit = createLabeledEntry(grp, grid, i18n("Signature File:"),
//			       identity->signatureFile(), 5, 0, &button);
//  connect(button,SIGNAL(clicked()),this,SLOT(chooseSigFile()));

  grid->setColStretch(0,0);
  grid->setColStretch(1,1);
  grid->setColStretch(2,0);
  grid->setRowStretch(6, 100);

  addTab(tab, i18n("Simple"));

  grp = new QGroupBox(i18n("Known Keywords"), tab);
  box->addWidget(grp);
  grid = new QGridLayout(grp, 6, 3, 20, 6);

  QLabel* label = new QLabel(grp);

  label->setText(	"BEFORE rfc-date\nSINCE rfc-date\nON rfc-date\n"
  								"(UN)ANSWERED\n(UN)DELETED\n(UN)FLAGGED\n(UN)SEEN\n"
  								"RECENT\nNEW\nOLD\n"
  								"FROM adress\nTO adress\nCC adress\nBCC adress\nSUBJECT word\n"
  								"ALL\nBODY string\nKEYWORD string\nTEXT string\n");

  label->adjustSize();
  label->resize((int)label->sizeHint().width(),label->sizeHint().height() + 6);
  label->setMinimumSize(label->size());
  grid->addWidget(label, 0, 0);
//  grid->addMultiCellWidget(new QLabel("BEFORE",grp), 0, 0, 0, 3);
  grid->activate();
  grid->setColStretch(0,0);
  grid->setColStretch(1,1);
  grid->setColStretch(2,0);
  grid->setRowStretch(6, 100);

/*  bgrp = new QButtonGroup(i18n("What to do"), tab);
  box->addWidget(bgrp);
  grid = new QGridLayout(bgrp, 6, 6, 20, 4);

  flagRadio = new QRadioButton(bgrp);
  flagRadio->setMinimumSize(flagRadio->size());
  flagRadio->setText(i18n("Flag"));
  bgrp->insert(flagRadio);
  grid->addMultiCellWidget(flagRadio, 0, 0, 0, 3);

  unflagRadio = new QRadioButton(bgrp);
  unflagRadio->setMinimumSize(unflagRadio->size());
  unflagRadio->setText(i18n("Unflag"));
  bgrp->insert(unflagRadio);
  grid->addMultiCellWidget(unflagRadio, 1, 1, 0, 3);

  toggleflagRadio = new QRadioButton(bgrp);
  toggleflagRadio->setMinimumSize(toggleflagRadio->size());
  toggleflagRadio->setText(i18n("Toggle Flag"));
  bgrp->insert(toggleflagRadio);
  grid->addMultiCellWidget(toggleflagRadio, 2, 2, 0, 3);

  deleteRadio = new QRadioButton(bgrp);
  deleteRadio->setMinimumSize(deleteRadio->size());
  deleteRadio->setText(i18n("Mark Deleted"));
  bgrp->insert(deleteRadio);
  grid->addMultiCellWidget(deleteRadio, 3, 3, 0, 3);

  moveFileRadio = new QRadioButton(bgrp);
  moveFileRadio->setMinimumSize(moveFileRadio->size());
  moveFileRadio->setText(i18n("Move to File"));
  bgrp->insert(moveFileRadio);
  grid->addMultiCellWidget(moveFileRadio, 4, 4, 0, 3);

  moveFolderRadio = new QRadioButton(bgrp);
  moveFolderRadio->setMinimumSize(moveFolderRadio->size());
  moveFolderRadio->setText(i18n("Move to Folder"));
  bgrp->insert(moveFolderRadio);

  grid->setColStretch(0,4);
  grid->setColStretch(1,1);
  grid->setColStretch(2,10);
  grid->setColStretch(3,0);
  grid->setRowStretch(5, 100);
  grid->activate();
  bgrp->adjustSize(); */
}

//-----------------------------------------------------------------------------
void NMSearch::doCancel()
{
}

//-----------------------------------------------------------------------------
void NMSearch::doApply()
{
}

const char *NMSearch::getSearchString()
{
	return searchEdit->text();
}

const char *NMSearch::getActionPath()
{
	return searchEdit->text();
}

enum NMSearch::eActions NMSearch::getAction()
{
//  QRadioButton *flagRadio, *unflagRadio, *toggleflagRadio;
	if(flagRadio->isChecked()) return NMSearch::doFlag;
	if(unflagRadio->isChecked()) return NMSearch::doUnFlag;
	if(toggleflagRadio->isChecked()) return NMSearch::doToggleFlag;
	if(deleteRadio->isChecked()) return NMSearch::doDelete;
	if(moveFileRadio->isChecked()) return NMSearch::doMoveFile;
	if(moveFolderRadio->isChecked()) return NMSearch::doMoveFolder;
	return NMSearch::doNOOP;
}
