/***************************************************************************
                          nmselect.cpp  -  description
                             -------------------
    begin                : Tue Oct 26 1999
    copyright            : (C) 1999 by Sven Carstens
    email                : s.carstens@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include <stdlib.h>

#include "kapp.h"
#include "nmselect.h"
#include "nmlistremote.h"
#include "qpushbutton.h"
#include <qbuttongroup.h>
#include <qradiobutton.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qcheckbox.h>
#include "extern.h"
#include "kbusyptr.h"
#include "cpp-client.h"
#include <ktmainwindow.h>
#include "nmstatusbar.h"

#define BUTTON_SPACING 10
#define BUTTON_WIDTH 70

//NMSelect::NMSelect(QWidget *parent, const char *name ) : QDialog(parent,name,true) {
NMSelect::NMSelect(QWidget *parent, const char *name ) : KTMainWindow() {

// 		KTMainWindow *master = new KTMainWindow();			  		
//	master->setView(this);
	setStatusBar(mStatusBar = new NMStatusBar(this));
	first = NULL;
	second = NULL;
	third = NULL;

	in_host = NULL;
	in_user = NULL;
	in_port = NULL;
	in_pattern = NULL;
	in_useSSL = NULL;
	useSSL = false;

	in_protocol = 1;
	my_result = -1;
  setMinimumSize(BUTTON_SPACING * 5 + BUTTON_WIDTH * 4,200);

  ok = new QPushButton(i18n("OK") ,this,"NULL");
  ok->setGeometry(BUTTON_SPACING,130,70,25);
  connect(ok,SIGNAL(pressed()),this,SLOT(slotOkPressed()));
  connect(ok,SIGNAL(clicked()),this,SLOT(slotOkPressed()));
	ok->setEnabled(false);

  prev = new QPushButton(i18n("&Prev"), this);
  prev->setGeometry(BUTTON_SPACING * 2 + BUTTON_WIDTH,130,70,25);
  connect(prev,SIGNAL(pressed()),this,SLOT(slotPrevPressed()));
	prev->setEnabled(false);

  next = new QPushButton(i18n("&Next"), this);
  next->setGeometry(BUTTON_SPACING * 3 + BUTTON_WIDTH * 2,130,70,25);
  connect(next,SIGNAL(pressed()),this,SLOT(slotNextPressed()));
  connect(next,SIGNAL(clicked()),this,SLOT(slotNextPressed()));

  cancel = new QPushButton(i18n("&Cancel"), this);
  cancel->setGeometry(BUTTON_SPACING * 4 + BUTTON_WIDTH * 3,130,70,25);
  connect(cancel,SIGNAL(pressed()),this,SLOT(slotCancelPressed()));

	setCaption(i18n("New Folder"));
	openFirst();
}

//-----------------------------------------------------------------------------
NMSelect::~NMSelect(){
}

//-----------------------------------------------------------------------------
void NMSelect::slotOkPressed()
{
  //kbp->busy();
  done(1);
}

//-----------------------------------------------------------------------------
void NMSelect::slotPrevPressed()
{
	if(second) {
		closeSecond();
		prev->setEnabled(false);
		openFirst();
	}
	else if(third) {
		closeThird();
		next->setEnabled(true);
		ok->setEnabled(false);
		openSecond();
	}
  //kbp->busy();
}

//-----------------------------------------------------------------------------
void NMSelect::slotNextPressed()
{
	if(first && validFirst()) {
		closeFirst();
		prev->setEnabled(true);
		openSecond();
	}
	else if(second && validSecond()) {
		closeSecond();
		openThird();
		next->setEnabled(false);
		ok->setEnabled(true);
	}
  //kbp->busy();
}

//-----------------------------------------------------------------------------
void NMSelect::slotCancelPressed()
{
  //kbp->busy();
  done(0);
}

//-----------------------------------------------------------------------------
void NMSelect::resizeEvent(QResizeEvent *event)
{
	int button_spacing = 0;

	button_spacing = (width() - (4 * BUTTON_WIDTH)) / 5;
	if(first)
		first->resize( width() - 20, height() - 80 );
	if(second)
		second->resize( width() - 20, height() - 80 );
	if(third)
		third->resize( width(), height() - 80);

	ok->move(button_spacing * 1 + BUTTON_WIDTH * 0,height()- 60);
	prev->move(button_spacing * 2 + BUTTON_WIDTH * 1,height()- 60);
	next->move(button_spacing * 3 + BUTTON_WIDTH * 2,height()- 60);
	cancel->move(button_spacing * 4 + BUTTON_WIDTH * 3,height()- 60);
	mStatusBar->move(0,height()-mStatusBar->height());
	mStatusBar->resize( width(), mStatusBar->height());
}

void NMSelect::openFirst()
{
	first = new QButtonGroup("Type",this);
	first->setGeometry(10,10,width()-120,height()-150);

  QVBoxLayout *vbox = new QVBoxLayout(first, 10);
  vbox->addSpacing( first->fontMetrics().height() );

	QRadioButton *radio;
	radio = new QRadioButton("&POP3",first);
  vbox->addWidget(radio);
	radio = new QRadioButton("&IMAP",first);
  vbox->addWidget(radio);
	radio = new QRadioButton("&MH",first);
  vbox->addWidget(radio);
	radio = new QRadioButton("&Folder",first);
  vbox->addWidget(radio);

	first->setButton(in_protocol-1);
	first->show();
	resizeEvent(NULL);
	next->setDefault(true);
}

bool NMSelect::validFirst()
{
	QButton *btn;
  int i = 0;

	MailStream stream;
	
	in_protocol = -1;
	while((btn = first->find(i++)))
	{
		if(btn->isOn()) in_protocol = i;
	}

	switch( in_protocol ) {
		case 1:
			protocol = "POP3";
			port.sprintf("%ld",(unsigned long)stream.getParameter(GET_POP3PORT));
			break;
		case 2:
			protocol = "IMAP";
			port.sprintf("%ld",(unsigned long)stream.getParameter(GET_IMAPPORT));
			break;
		case 3:
			protocol.setStr( "#mh");
			break;
		default:
			protocol.setStr( "");
			break;
	}

	return ( in_protocol != -1);
}

void NMSelect::closeFirst()
{
	delete first;
	first = NULL;
}

void NMSelect::openSecond()
{
	char *type;
	if( protocol == "IMAP") type = "Network";
	else if( protocol == "POP3") type = "Network";
	else	type = "Local";
	second = new QGroupBox(type,this);
	second->setGeometry(10,10,width()-120,height()-150);

	QHBoxLayout *hbox;
  QVBoxLayout *vboxl,*vboxr;
	QLabel *label;
	QLineEdit *line;
	QCheckBox *check;
	hbox = new QHBoxLayout(second,10);

	vboxl = new QVBoxLayout( 10);
	hbox->addLayout(vboxl,10);
  vboxl->addSpacing( second->fontMetrics().height() );

	vboxr = new QVBoxLayout( 10 );
	hbox->addLayout(vboxr,30);
  vboxr->addSpacing( second->fontMetrics().height());

	line = new QLineEdit(second);
	line->setMaximumHeight( second->fontMetrics().height()+4 );
  vboxr->addWidget(line);
	label = new QLabel(line,"&Host",second);
  vboxl->addWidget(label);
	in_host = line;
	if(in_protocol >= 3) {
		line->setEnabled(false);
		label->setEnabled(false);
	}	else {
		in_host->setText( host.data() );
	}

	line = new QLineEdit(second);
	line->setMaximumHeight( second->fontMetrics().height()+4 );
  vboxr->addWidget(line);
	label = new QLabel(line,"&User",second);
  vboxl->addWidget(label);
	in_user = line;
	if(in_protocol >= 3) {
		line->setEnabled(false);
		label->setEnabled(false);
	}	else {
		in_user->setText( user.data() );
	}

	line = new QLineEdit(second);
	line->setMaximumHeight( second->fontMetrics().height()+4 );
  vboxr->addWidget(line);
	label = new QLabel(line,"P&ort",second);
  vboxl->addWidget(label);
	in_port = line;
	if(in_protocol >= 3) {
		line->setEnabled(false);
		label->setEnabled(false);
	}	else {
		in_port->setText( port.data() );
	}

	line = new QLineEdit(second);
	line->setMaximumHeight( second->fontMetrics().height()+4 );
  vboxr->addWidget(line);
	label = new QLabel(line,"P&ath",second);
  vboxl->addWidget(label);
	in_pattern = line;
	if(in_protocol == 1) {
		line->setEnabled(false);
		label->setEnabled(false);
	}	else {
		if(in_protocol == 2 && pattern.isEmpty()) in_pattern->setText("*");
		else in_pattern->setText( pattern.data() );
	}

	check = new QCheckBox(second);
	check->setMaximumHeight( (second->fontMetrics().height()+4) );
  vboxr->addWidget(check);
	label = new QLabel(check,i18n("Use SSL"),second);
  vboxl->addWidget(label);
	in_useSSL = check;

#ifndef HAVE_OPENSSL
	in_useSSL->setEnabled(false);
	label->setEnabled(false);
#endif
	
	if(in_protocol >= 3) {
		check->setEnabled(false);
		label->setEnabled(false);
	}	else {
		in_useSSL->setChecked( useSSL );
	}

	second->show();
	in_pattern->setFocus();
	in_host->setFocus();
	resizeEvent(NULL);
	next->setDefault(true);
}

bool NMSelect::validSecond()
{
	bool  ret = false;

	host.setStr( in_host->text() );
	user.setStr( in_user->text() );
	port.setStr ( in_port->text() );
	pattern.setStr( in_pattern->text() );
	useSSL = in_useSSL->isChecked();
	
	switch( in_protocol ) {
		case 1:
			if(! host.isEmpty() && ! port.isEmpty() && ! user.isEmpty())
			ret = true;
			break;
		case 2:
			if(! host.isEmpty() && ! port.isEmpty() && ! user.isEmpty() && ! pattern.isEmpty() )
			ret = true;
			break;
		case 3:
			if(! pattern.isEmpty() )
			ret = true;
			break;
		case 4:
			if(! pattern.isEmpty() )
			ret = true;
			break;
	}
	if(host.isEmpty()) in_host->setFocus();
	else if(user.isEmpty()) in_user->setFocus();
	else if(port.isEmpty()) in_port->setFocus();
	else if(pattern.isEmpty()) in_pattern->setFocus();

	return ret;
}

void NMSelect::closeSecond()
{
	delete second;

	in_host = NULL;
	in_user = NULL;
	in_port = NULL;
	in_pattern = NULL;

	second = NULL;
}

void NMSelect::openThird()
{
	QString path;

	if(!host.isEmpty()) {
		path = '{' + host + ':' + port + "/user=" + user + '/' + protocol;
//		if(useSSL) path = path + "/secure";
		if(useSSL) path = path + "/ssl/novalidate-cert";
		path = path + '}' + pattern;
	} else if( protocol.find("#mh") == 0 ) {
		path = protocol + '/' + pattern;
	} else {
		path = pattern;
	}
	third = new NMListRemote(this);

	fprintf(stderr,"Listing %s\n",path.data() );

	third->show();
	third->setFocusPolicy( QWidget::StrongFocus );
	resizeEvent(NULL);
	kbp->busy();
	ok->setEnabled(false);
	prev->setEnabled(false);
	next->setEnabled(false);
	cancel->setEnabled(false);
	third->listFolders( path.data() );
	ok->setEnabled(true);
	prev->setEnabled(true);
	cancel->setEnabled(true);
	kbp->idle();
	QObject::connect( third, SIGNAL( returnPressed(QListViewItem *) ),
										this, SLOT( slotDoubleClicked(QListViewItem *) ) );
	QObject::connect( third, SIGNAL( doubleClicked(QListViewItem *) ),
										this, SLOT( slotDoubleClicked(QListViewItem *) ) );
	QObject::connect( third, SIGNAL( rightButtonClicked(QListViewItem *,const QPoint &,int) ),
										this, SLOT( slotRightButtonClicked(QListViewItem *,const QPoint &,int) ) );
	ok->setDefault(true);
}

void NMSelect::closeThird()
{
	delete third;
	third = NULL;
}


QStrList *NMSelect::getSelected()
{
	QStrList *list = new QStrList(true);
  QListViewItem *item = NULL;
	if(third) item = third->firstChild();
	if(item && result()) iterateList(item,list);

	return list;
}

void NMSelect::iterateList(QListViewItem *item,QStrList *list)
{
	while(item)
	{
		if(item->isSelected() )
		{
			list->append( item->text(2) );
//			fprintf(stderr,"add: %s\n",item->text(2) );
		}
		if(item->firstChild() ) iterateList( item->firstChild() ,list );
		item = item->nextSibling();
	}
}

void NMSelect::slotRightButtonClicked(QListViewItem *item,const QPoint &point,int where)
{
	slotDoubleClicked(item);
}

void NMSelect::slotDoubleClicked(QListViewItem *item)
{
	if(item && item->firstChild())
	{
		selectAll(item->firstChild());
		item->listView()->triggerUpdate();
	}
}

void NMSelect::selectAll(QListViewItem *item)
{
	while(item)
	{
		if(item->isSelectable()) item->setSelected( !item->isSelected() );
		if(item->firstChild()) selectAll(item->firstChild());
		item = item->nextSibling();
	}
}

void NMSelect::done(int i)
{
	my_result = i;
	hide();
}

int NMSelect::result()
{
	return my_result;
}
