/***************************************************************************
                          nmstatusbar.cpp  -  description
                             -------------------
    begin                : Wed May 24 2000
    copyright            : (C) 2000 by Sven Carstens
    email                : s.carstens@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "nmstatusbar.h"
#include <kapp.h>
#include <qprogressbar.h>
#include <qpushbutton.h>
#include "resource.h"
#include <qstack.h>
#include <malloc.h>
#include <ktmainwindow.h>

NMStatusBar::NMStatusBar(QWidget *parent, const char *name ) : KStatusBar(parent,name) {
 	mProgress = new QProgressBar(this);
 	mProgress->adjustSize();
    	
 	mCancelButton = new QPushButton(this);
 	mCancelButton->setText(i18n("Cancel"));
 	mCancelButton->adjustSize();
// 	mCancelButton->hide();
 	mCancelButton->show();

  insertWidget(mProgress,mProgress->width()+50, ID_STATUS_MSG +1 );
	insertWidget(mCancelButton,mCancelButton->width(), ID_STATUS_MSG +2 );
	setInsertOrder(KStatusBar::RightToLeft);

  insertItem(i18n("Ready."), ID_STATUS_MSG );
 	mCancelButton->setEnabled(false);
	mStack.setAutoDelete(true);
	cancelPressed = false;
 	QObject::connect( mCancelButton, SIGNAL( released() ),
 										this, SLOT( cancelWasPressed() ) );
}

NMStatusBar::~NMStatusBar(){
}

void NMStatusBar::progressNotify(enum mailProgress function,unsigned long value,const char *txt,bool &result)
{
	char *clear_txt = NULL;
	unsigned long rate = 0;
	const char * mName = NULL;
	time_t mStart = 0;
	long mTotal = 0;
	long mCurrent = 0;
	
	if(!mStack.isEmpty())
	{
		mTotal = mStack.current()->pTotal;
		mCurrent = mStack.current()->pCurrent;
		mName = mStack.current()->pName;
		mStart = mStack.current()->pStart;
	}
	
	mBuffer[0] = 0x00;
	//transfer rate is calculated every three seconds (if were called that soon)
	if(mTotal && mCurrent && (time(0) - mStart > 3))
	{
		rate = mCurrent / (time(0) - mStart);
//		mStart = time(0);
	}

	switch(function)
	{
		case mpSTART:
			clear_txt = "mpSTART";
			mStack.push((struct progressObject *)malloc(sizeof(struct progressObject)));
			mName = (mStack.current())->pName = txt;
			mStart = (mStack.current())->pStart = time(0);
			mTotal = (mStack.current())->pTotal = 0;
			mCurrent = (mStack.current())->pCurrent = 0;
			cancelPressed = false;
			if(mProgress) mProgress->reset();
			rate = 0;
			if(mCancelButton) mCancelButton->setEnabled(true);
			if(mProgress) mProgress->setEnabled(true);
			if(kapp) kapp->processEvents(10);
			break;
		case mpSTOP:
			clear_txt = "mpSTOP";
			mStack.pop();
			cancelPressed = false;
			if(mStack.isEmpty())
			{
  			if(mCancelButton) mCancelButton->setOn(false);
  			if(mCancelButton) mCancelButton->setEnabled(false);
  			if(mProgress) mProgress->reset();
  			if(mProgress) mProgress->setEnabled(false);
				snprintf(mBuffer,sizeof(mBuffer)-1,"%s",i18n("Ready."));
				rate = 0;
  			repaint();
  		} else	{
    		mTotal = mStack.current()->pTotal;
    		mCurrent = mStack.current()->pCurrent;
    		mName = mStack.current()->pName;
    		mStart = mStack.current()->pStart;
/*      	if(mTotal && (time(0) - mStart > 3))
      	{
      		rate = time(0) - mStart;
      		mStart = time(0);
      	} */
   			if(rate > 0)
   				snprintf(mBuffer,sizeof(mBuffer)-1,i18n("%s %ld of %ld (%ld bps)"),mName,mCurrent,mTotal,rate);
   			else
   				snprintf(mBuffer,sizeof(mBuffer)-1,i18n("%s %ld of %ld"),mName,mCurrent,mTotal);
    	}
			break;
		case mpTIMEOUT:
			clear_txt = "mpTIMEOUT";
			if(value > 3)
				if(mTotal>0)
				 snprintf(mBuffer,sizeof(mBuffer)-1,i18n("%s %ld of %ld (%ld secs idle)"),mName,mCurrent,mTotal,value);
				else
					snprintf(mBuffer,sizeof(mBuffer)-1,i18n("%s ... (%ld secs idle)"),mName,value);
			break;
		case mpSET_TOTAL:
			clear_txt = "mpSET_TOTAL";
			mStart = time(0);
			mCurrent = 0;
			mTotal = value;
			if(mTotal)
			{
				rate = 0;
			}
			break;
		case mpSET_DELTA:
			clear_txt = "mpSET_DELTA";
			value += mCurrent;
		case mpSET_CURRENT:
			if(!clear_txt) clear_txt = "mpSET_CURRENT";
			mCurrent  = value;
 			if(rate > 0)
 				snprintf(mBuffer,sizeof(mBuffer)-1,i18n("%s %ld of %ld (%ld bps)"),mName,mCurrent,mTotal,rate);
 			else
 				snprintf(mBuffer,sizeof(mBuffer)-1,i18n("%s %ld of %ld"),mName,mCurrent,mTotal);
			break;
		case mpSET_STATUS:
			clear_txt = "mpSET_STATUS";
			snprintf(mBuffer,sizeof(mBuffer)-1,"%s",txt);
			break;
		default:
			clear_txt = "unknown";
			break;
	}
	debug("progress: %s %ld %p - %s",clear_txt,value,txt,mName);
	
	if(mTotal)
	{
		if(mProgress->progress() > (int)(100 * mCurrent / mTotal)) mProgress->reset();
		mProgress->setProgress( (int)(100 * mCurrent / mTotal) );
	} else mProgress->reset();

	if(*mBuffer && (time(0) - mStart) > 2) changeItem(mBuffer,ID_STATUS_MSG);
	if(*mBuffer && mStack.isEmpty()) changeItem(mBuffer,ID_STATUS_MSG);

	kapp->processEvents(10);

	if(!mStack.isEmpty())
	{
		mStack.current()->pTotal = mTotal;
		mStack.current()->pCurrent = mCurrent;
//		mStack.current()->pName = mName;
		mStack.current()->pStart = mStart;
	}
	
	result = true;
	if(mCancelButton && mCancelButton->isDown()) result = false;
	if(cancelPressed) result = false;
}

void NMStatusBar::cancelWasPressed()
{
	cancelPressed = true;
}

void NMStatusBar::slotStatusMsg(const char *text)
{
  ///////////////////////////////////////////////////////////////////
  // change status message permanently
  clear();
  changeItem(text, ID_STATUS_MSG );
	kapp->processEvents(1);
}


void NMStatusBar::slotStatusHelpMsg(const char *text)
{
  ///////////////////////////////////////////////////////////////////
  // change status message of whole statusbar temporary (text, msec)
  message(text, 2000);
}

bool NMStatusBar::connectToStatusBar(QObject *fromObject,QWidget *toWidget)
{
	bool retval = false;
	
	if(toWidget->topLevelWidget())
	{
  	if(toWidget->topLevelWidget()->inherits("KTMainWindow"))
  	{
  		if(		 ((KTMainWindow *)(toWidget->topLevelWidget()))->statusBar() == NULL
  				|| ((KTMainWindow *)(toWidget->topLevelWidget()))->statusBar()->inherits("NMStatusBar") )
  		{
  			QObject::connect( fromObject,SIGNAL( signalProgressNotify(enum mailProgress,unsigned long,const char *,bool &) ),
  												((KTMainWindow *)(toWidget->topLevelWidget()))->statusBar(),
  												SLOT( progressNotify(enum mailProgress,unsigned long,const char *,bool &) ) );
  		} else {
				debug("NMStatusBar::connectToStatusBar couldnt connect StatusBar is : %s",((KTMainWindow *)(toWidget->topLevelWidget()))->statusBar()->className());  		}
  	} else debug("NMStatusBar::connectToStatusBar couldnt connect ClassName is : %s",toWidget->topLevelWidget()->className());
  }
	return retval;
}

bool NMStatusBar::disconnectStatusBar(QObject *fromObject,QWidget *toWidget)
{
	bool retval = false;
	
	if(toWidget->topLevelWidget())
	{
  	if(toWidget->topLevelWidget()->inherits("KTMainWindow"))
  	{
  		if(		 ((KTMainWindow *)(toWidget->topLevelWidget()))->statusBar() == NULL
  				|| ((KTMainWindow *)(toWidget->topLevelWidget()))->statusBar()->inherits("NMStatusBar") )
  		{
  			QObject::disconnect( fromObject,SIGNAL( signalProgressNotify(enum mailProgress,unsigned long,const char *,bool &) ),
  												((KTMainWindow *)(toWidget->topLevelWidget()))->statusBar(),
  												SLOT( progressNotify(enum mailProgress,unsigned long,const char *,bool &) ) );
  		} else {
				debug("NMStatusBar::disconnectStatusBar couldnt disconnect StatusBar is : %s",((KTMainWindow *)(toWidget->topLevelWidget()))->statusBar()->className());  		}
  	} else debug("NMStatusBar::disconnectStatusBar couldnt disconnect ClassName is : %s",toWidget->topLevelWidget()->className());
  }
	return retval;
}