/***************************************************************************
                          qpasswdfile.cpp  -  description
                             -------------------
    begin                : Sat Dec 25 1999
    copyright            : (C) 1999 by Sven Carstens
    email                : s.carstens@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "qpasswdfile.h"
#include "kpgp.h"
#include <qmessagebox.h>
#include <kapp.h>

QPasswdFile::QPasswdFile(const char *pFile,const char *pEncoder) : QFile(pFile) {
//	fprintf(stderr,"QPasswdFile::QPasswdFile()\n");
	dirty_buffer = false;
	encoder = pEncoder;
//	if(in_file) open(0);
}

QPasswdFile::~QPasswdFile(){
//	fprintf(stderr,"QPasswdFile::~QPasswdFile()\n");
}

bool QPasswdFile::open(int mode){
	fprintf(stderr,"QPasswdFile::open(int) ");
  Kpgp* pgp = Kpgp::getKpgp();
	bool retval = true;
	QFile::open(mode);

	if(!(mode & IO_Truncate) && isOpen()) {
		decoded_buffer.resize(QFile::size());
		QFile::readBlock(decoded_buffer.data(),decoded_buffer.size());
	  if (pgp->setMessage(decoded_buffer))
		{
	    if (pgp->isEncrypted())
	    {
				int ret = 0;
	      while(!ret && !pgp->decrypt())
				{
				
					ret = QMessageBox::warning(0,i18n("Warning"),
			       i18n("Your Passwordfile could not be\n"
				    "decrypted with the supplied Passphrase."),
				     i18n("Retry"), i18n("Continue"));
				}
				if(ret) retval = false;
				decoded_buffer = pgp->message();
				fprintf(stderr,"\n");
			} else fprintf(stderr,"not encrypted\n");
		} else fprintf(stderr,"setMessage failed\n");
//	fprintf(stderr,"'%s'\n",decoded_buffer.data());
	}
	decoded_at = 0;
	return retval;
}

int QPasswdFile::readLine(char *p,unsigned int maxlen) {
//	fprintf(stderr,"QPasswdFile::readLine()");
	unsigned int del;

	del = decoded_buffer.find( 0x0a,decoded_at )+1;
	if(del == 0) del = decoded_buffer.length();
	del = (del > decoded_at + maxlen) ? decoded_at + maxlen : del;
	del = del - decoded_at;
	strncpy( p,&(decoded_buffer.data()[decoded_at]), del);
	p[del] = 0x00;
	decoded_at += del;
	return (del);
}

bool QPasswdFile::at(int in){
//	fprintf(stderr,"QPasswdFile::at()");
	if(in > (int)decoded_buffer.length()) return false;
	decoded_at = in;
	return true;
}

bool QPasswdFile::atEnd() const
{
//	fprintf(stderr,"QPasswdFile::atEnd()");
	if(decoded_at >= decoded_buffer.length()) return true;
	return false;
}

int QPasswdFile::getch()
{
//	fprintf(stderr,"QPasswdFile::getch()");
	if(decoded_at >= decoded_buffer.length()) return EOF;
	return (decoded_buffer.data())[decoded_at++];
}

void QPasswdFile::close()
{
//	fprintf(stderr,"QPasswdFile::close()\n");
	if(dirty_buffer)
	{
	  Kpgp* pgp = Kpgp::getKpgp();
		QStrList my_self;

		my_self.append(encoder);

		pgp->setMessage(decoded_buffer);
		pgp->setEncryptToSelf(true);
		pgp->encryptFor(my_self,false);
		QString encoded_buffer = pgp->message();
		
		if(encoded_buffer.isEmpty())
			encoded_buffer = decoded_buffer;
	  QFile::writeBlock(encoded_buffer.data(),encoded_buffer.length());
	}
	QFile::close();
}

int QPasswdFile::writeBlock(const char *buffer,unsigned int size)
{
//	fprintf(stderr,"QPasswdFile::writeBlock()\n");
	dirty_buffer = true;
	decoded_buffer += buffer;
	decoded_at = decoded_buffer.length();
	return strlen(buffer);
}

int QPasswdFile::putch(int chr)
{
//	fprintf(stderr,"QPasswdFile::putch()\n");
	dirty_buffer = true;
	decoded_buffer += chr;
	decoded_at = decoded_buffer.length();
	return chr;
}
